/************************************************************************/
/* astrogtk.h								*/
/*                                                                      */
/* Header file for astrogtk library.					*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : May 2006                                               	*/      
/************************************************************************/

#ifndef __ASTRO_CLISERV__
#define __ASTRO_CLISERV__

#ifdef GTK2
#define ASTRO_CONTROL_WIDTH     200
#define ASTRO_INIT_WIDTH	200
#define ASTRO_OBSWAVE_WIDTH     200
#define ASTRO_HEIGHT		30
#else
#define ASTRO_CONTROL_WIDTH     200
#define ASTRO_INIT_WIDTH	200
#define ASTRO_OBSWAVE_WIDTH     200
#define ASTRO_HEIGHT		24
#endif

/* Globals */

struct astro_control_window_data {char *name; int server;};
struct astro_init_window_data {char *name; int server; char get_config;};
struct s_init {int server; char doit; char send_it;};
struct s_star_message {int server; int chara_number; int hd;};
struct s_obswave {int server; char doit;};

extern int chara_number[MAX_SERVERS]; /* Chara number of current star */
extern int reference_cart[MAX_SERVERS];   /* Which reference cart */
extern int last_reference_cart;
extern char scope_active[MAX_SERVERS][NUM_SCOPES];
extern astromod_init_struct beam_config[MAX_SERVERS];
extern double obs_wavelength[MAX_SERVERS];
extern struct s_init init_ok;
extern struct s_init init_cancel;
extern struct s_obswave obs_ok;
extern struct s_obswave obs_cancel;
extern struct s_star_message nostar;
extern struct s_star_message zenith;
extern struct s_star_message internal;
extern struct s_star_message autocoll;
extern struct s_star_message charanum;
extern char got_active_scopes;

/* Prototypes */

/* astrogtk_control_wondow.c */

GtkWidget* create_astrogtk_control_window (GtkButton *button, gpointer *data);
void fill_astrogtk_control_page (GtkWidget *vbox, int server);
void astrogtk_control_set_chara_hd(int server);
void astrogtk_control_ok_callback(GtkButton *button, gpointer user_data);
void astrogtk_setstar_callback(GtkButton *button, gpointer user_data);
void astrogtk_chara_callback(GtkButton *button, gpointer user_data);
void astrogtk_hd_callback(GtkButton *button, gpointer user_data);
bool astrogtk_get_current_star(int server);
void astrogtk_set_server(int server);

/* astrogtk_init_window.c */

GtkWidget* create_astrogtk_init_window (GtkButton *thebutton, gpointer *data);
void fill_astrogtk_beams_pops(GtkWidget *vbox, gpointer *data);
void fill_astrogtk_ref(GtkWidget *vbox, gpointer *data);
void astrogtk_set_beams_pops(int server);
void astrogtk_set_ref(int server);
void astrogtk_set_active_scopes(int server);
void astrogtk_init_ok_callback(GtkButton *button, gpointer doit);
void astrogtk_send_init(GtkButton *button, gpointer data);
bool astrogtk_get_current_config(int server);
bool astrogtk_get_current_reference_cart(int server);
bool astrogtk_get_current_active_scopes(int server);
void set_config_callback(int (*callback)(void));
void unset_config_callback(void);
void get_and_display_star_and_config_callback(GtkWidget *widget, gpointer data);
void send_star_and_config_callback(GtkWidget *widget, gpointer data);

/* astrogtk_messages.c */

int setup_astro_messages(int server, char active_scopes);
int message_astromod_init(int server, struct smessage *mess);
int message_astromod_set_star(int server, struct smessage *mess);
int message_astromod_set_reference_cart(int server, struct smessage *mess);
int message_astromod_set_obs_wavelength(int server, struct smessage *mess);
int message_astromod_set_active_scopes(int server, struct smessage *mess);
void set_active_scopes_callback(void (*callback)(void));
void unset_active_scopes_callback(void);

/* astrogtk_obswave_window.c */

GtkWidget* create_astrogtk_obswave_window (GtkButton *thebutton,gpointer *data);
void astrogtk_obswave_ok_callback(GtkButton *button, gpointer data);
bool astrogtk_get_current_obs_wavelength(int server);

#endif
