/************************************************************************/
/* astrogtk_init_window.c                                               */
/*                                                                      */
/* Routine for initling the basic CHARA astrogtk functions.		*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE                        */
/*                 Based on the SUSI User Interface                     */
/*              In turn based on the CHIP User interface                */
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA                 */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu                                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/* Author : Theo ten Brummelaar                                         */
/* Date   : May   2006                                                  */
/************************************************************************/

#include "../cliserv/include/cliserv.h"
#include "../include/chara_messages.h"
#include "astrogtk.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include "../charalib/chara.h"
#include "../slalib/slalib.h" /* Globals */

int    last_reference_cart = 0;
int chara_number[MAX_SERVERS];
static char beams_pops_filled = FALSE;
static GtkWidget *astrogtk_init_window;
static char astrogtk_init_window_up = FALSE;
static GtkWidget *pop_select_menus[NUM_SCOPES];
static GtkWidget *pop_menu_items[NUM_SCOPES][NUM_POPS];
static GtkWidget *beam_select_menus[NUM_SCOPES];
static GtkWidget *beam_menu_items[NUM_SCOPES][NUM_BEAMS];
static GtkWidget *ref_select_menu;
static GtkWidget *ref_menu_items[NUM_SCOPES];
static int really_destroy = TRUE;
static GtkWidget *pop_menus[NUM_SCOPES] = {NULL, NULL, NULL, NULL, NULL, NULL};
static GtkWidget *beam_menus[NUM_SCOPES] = {NULL, NULL, NULL, NULL, NULL, NULL};
static GtkWidget *scope_buttons[NUM_SCOPES];
static GtkWidget *ref_menu = NULL;
struct s_init init_ok;
struct s_init init_cancel;
struct s_star_message nostar;
struct s_star_message zenith;
struct s_star_message internal;
struct s_star_message autocoll;
struct s_star_message charanum;
int (*config_callback)(void) = NULL;

/************************************************************************/
/* create_astrogtk_init_window()					*/
/*									*/
/* ROutine for creating the astrogtk init window. This should be self*/
/* contained and give you all you need to init the basic astrogtk 	*/
/* functionality. Argumentis the window name.				*/
/************************************************************************/

GtkWidget* create_astrogtk_init_window (GtkButton *thebutton, gpointer *data)
{
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *button;
	struct astro_init_window_data *the_data;

	/* Do nothing if we exist already */

	if (astrogtk_init_window_up) return astrogtk_init_window;

	/* Do nothing if there is no server */

	/* Get the data */

	the_data = (struct astro_init_window_data *)data;

	if (the_data->server < 0) return NULL;

	/* Create the window itself */

	astrogtk_init_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (astrogtk_init_window),
		the_data->name);
	gtk_container_set_border_width (GTK_CONTAINER 
		(astrogtk_init_window), 1);

	/* Box to put things in */

	vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add (GTK_CONTAINER (astrogtk_init_window), vbox);
	gtk_widget_show(vbox);

	/* Put everything in it */

	fill_astrogtk_beams_pops(vbox, data);
	fill_astrogtk_ref(vbox, data);

	/* Make an hbox to put things in */

	hbox = gtk_hbox_new(FALSE, 0);
	gtk_container_add (GTK_CONTAINER (vbox), hbox);
	gtk_widget_show(hbox);

	/* CANCEL button */

	button = gtk_button_new_with_label ("CANCEL");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
	    GTK_SIGNAL_FUNC (astrogtk_init_ok_callback),(gpointer)&init_cancel);
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
        gtk_widget_set_usize(button, ASTRO_INIT_WIDTH/2, ASTRO_HEIGHT);
	gtk_container_set_border_width (GTK_CONTAINER(button),1);
	gtk_widget_show(button);

	/* OK button */

	button = gtk_button_new_with_label ("OK");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
	    GTK_SIGNAL_FUNC (astrogtk_init_ok_callback), (gpointer)&init_ok);
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
        gtk_widget_set_usize(button, ASTRO_INIT_WIDTH/2, ASTRO_HEIGHT);
	gtk_container_set_border_width (GTK_CONTAINER(button),1);
	gtk_widget_show(button);

	/* Now, we can show this window if we like */

	gtk_widget_show(astrogtk_init_window);

	/* Make sure we only get one version of this window */

	astrogtk_init_window_up = TRUE;

	/* That should be all */

	return astrogtk_init_window;

} /* create_astrogtk_init_window() */

/************************************************************************/
/* fill_astrogtk_beams_pops()  						*/
/*									*/
/* Fills out all the stuff you might need to set the POPs and beams.	*/
/************************************************************************/

void fill_astrogtk_beams_pops(GtkWidget *vbox, gpointer *data)
{
	GtkWidget *hbox;
	GtkWidget *label;
	struct astro_init_window_data *the_data;
	int	  i,j;

	/* Get the data */

	the_data = (struct astro_init_window_data *)data;

	if (the_data->server < 0) return;

	/* Setup standard things */

	init_ok.server = the_data->server;
	init_ok.doit = TRUE;
	init_ok.send_it = the_data->get_config;
	init_cancel.server = the_data->server;
	init_cancel.doit = FALSE;
	init_cancel.send_it = FALSE;

	/* Find out what the current configuration is */

	if (the_data->get_config)
	{ 
		if (!astrogtk_get_current_config(the_data->server))
			fprintf(stderr,"Failed to get config from %d\n", 
			the_data->server);
		if (!astrogtk_get_current_reference_cart(the_data->server))
			fprintf(stderr,"Failed to get reference from %d\n", 
			the_data->server);
		if (!astrogtk_get_current_active_scopes(the_data->server))
			fprintf(stderr,"Failed to get active scopes %d\n", 
			the_data->server);
	}

	/* Set up things for all the scopes */

	for(i=0; i<NUM_SCOPES; i++)
	{
		/* Make an hbox to put things in */

		hbox = gtk_hbox_new(FALSE, 0);
		gtk_container_add (GTK_CONTAINER (vbox), hbox);
		gtk_widget_show(hbox);

		/* Now put the label for the scope */

		label = gtk_label_new(scope_types[i]);
		gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
		gtk_widget_show(label);

		/* And the menu for the pops */

		pop_menus[i] = gtk_option_menu_new();
		gtk_widget_show (pop_menus[i]);
		pop_select_menus[i] = gtk_menu_new ();
		gtk_box_pack_start(GTK_BOX(hbox), pop_menus[i], TRUE, TRUE, 0);
		for(j=0; j<NUM_POPS; j++)
		{
			pop_menu_items[i][j] =
			    gtk_menu_item_new_with_label(pop_types[j]);
			gtk_widget_show(pop_menu_items[i][j]);
			gtk_menu_append(GTK_MENU (pop_select_menus[i]),
			    pop_menu_items[i][j]);
		}
		gtk_option_menu_set_menu (GTK_OPTION_MENU (pop_menus[i]),
			pop_select_menus[i]);

		/* And the menu for the beams */

		beam_menus[i] = gtk_option_menu_new();
		gtk_widget_show (beam_menus[i]);
		beam_select_menus[i] = gtk_menu_new ();
		gtk_box_pack_start(GTK_BOX(hbox), beam_menus[i], TRUE, TRUE, 0);
		for(j=0; j<NUM_BEAMS; j++)
		{
			beam_menu_items[i][j] =
			    gtk_menu_item_new_with_label(beam_types[j]);
			gtk_widget_show(beam_menu_items[i][j]);
			gtk_menu_append(GTK_MENU (beam_select_menus[i]),
			    beam_menu_items[i][j]);
		}
		gtk_option_menu_set_menu (GTK_OPTION_MENU (beam_menus[i]),
			beam_select_menus[i]);

		/* And the active scopes bit */

		scope_buttons[i] =
			gtk_check_button_new_with_label(scope_types[i]);
                gtk_box_pack_start(GTK_BOX(hbox),scope_buttons[i],TRUE,TRUE,0);
                gtk_widget_show(scope_buttons[i]);
	}

	beams_pops_filled = TRUE;

	/* Make sure things are up to date */

	astrogtk_set_beams_pops(the_data->server);

	/* Now, we can show this */

	gtk_widget_show(vbox);

} /* fill_astrogtk_beams_pops() */

/************************************************************************/
/* fill_astrogtk_ref()  						*/
/*									*/
/* Fills out all the stuff you might need to set the reference cart.	*/
/************************************************************************/

void fill_astrogtk_ref(GtkWidget *vbox, gpointer *data)
{
	int	j;
	GtkWidget *hbox;
	GtkWidget *label;
	struct astro_init_window_data *the_data;

	/* Get the data */

	the_data = (struct astro_init_window_data *)data;

	if (the_data->server < 0) return;

        /* Make an hbox to put things in */

        hbox = gtk_hbox_new(FALSE, 0);
        gtk_container_add (GTK_CONTAINER (vbox), hbox);
        gtk_widget_show(hbox);

	/* Now put the label for the reference cart */

	label = gtk_label_new("Reference Cart");
	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
	gtk_widget_show(label);

	/* And the menu for the pops */

	ref_menu = gtk_option_menu_new();
	gtk_widget_show(ref_menu);
	ref_select_menu = gtk_menu_new ();
	gtk_box_pack_start(GTK_BOX(hbox), ref_menu, TRUE, TRUE, 0);
	for(j=0; j<NUM_SCOPES; j++)
	{
		ref_menu_items[j] =
		    gtk_menu_item_new_with_label(scope_types[j]);
		gtk_widget_show(ref_menu_items[j]);
		gtk_menu_append(GTK_MENU (ref_select_menu),ref_menu_items[j]);
	}
	gtk_option_menu_set_menu (GTK_OPTION_MENU (ref_menu),
		ref_select_menu);

	/* Make sure things are up to date */

	astrogtk_set_ref(the_data->server);

	/* Now, we can show this */

	gtk_widget_show(vbox);

} /* fill_astrogtk_ref() */

/************************************************************************/
/* astrogtk_set_beam_pops()						*/
/*									*/
/* Update the current configuration.					*/
/************************************************************************/

void astrogtk_set_beams_pops(int server)
{
	int	i;

	if (!beams_pops_filled || server < 0) return;

	for(i=0; i<NUM_SCOPES; i++)
	{
		if (pop_menus[i] != NULL)
		    gtk_option_menu_set_history(GTK_OPTION_MENU (pop_menus[i]),
			beam_config[server].pops[i]);

		if (beam_menus[i] != NULL)
		    gtk_option_menu_set_history(GTK_OPTION_MENU (beam_menus[i]),
			beam_config[server].beams[i]);
	}

} /* astrogtk_set_beams_pops() */

/************************************************************************/
/* astrogtk_set_active_scopes()						*/
/*									*/
/* Update the current configuration.					*/
/************************************************************************/

void astrogtk_set_active_scopes(int server)
{
	int	i;

	if (!beams_pops_filled || server < 0) return;

	for(i=0; i<NUM_SCOPES; i++)
	{
		gtk_toggle_button_set_active(
			(GtkToggleButton *)scope_buttons[i], 
			scope_active[server][i]);
	}

} /* astrogtk_set_active_scopes() */

/************************************************************************/
/* astrogtk_set_ref()							*/
/*									*/
/* Update the current configuration.					*/
/************************************************************************/

void astrogtk_set_ref(int server)
{
	if (ref_menu != NULL)
	{
	    if (server != -1)
		gtk_option_menu_set_history(GTK_OPTION_MENU (ref_menu),
			reference_cart[server]);
	    else
		gtk_option_menu_set_history(GTK_OPTION_MENU (ref_menu),
			last_reference_cart);
	}

} /* astrogtk_set_ref() */

/************************************************************************/
/* astrogtk_init_ok_callback()						*/
/*									*/
/* Callback function for astrogtk init OK button. Stop background	*/
/* processing and destroy astrogtk init window.				*/
/************************************************************************/

void astrogtk_init_ok_callback(GtkButton *button, gpointer data)
{
	struct s_init *action;

	if (!astrogtk_init_window_up) return;

	action = (struct s_init *)data;

	if (action->doit)
	{
		astrogtk_send_init(button, data);
	}

	if (really_destroy)
	{
		gtk_widget_destroy(astrogtk_init_window);
		astrogtk_init_window_up = FALSE;
		beams_pops_filled = FALSE;
	}

} /* astrogtk_init_ok_callback() */

/************************************************************************/
/* astrogtk_send_init()							*/
/*									*/
/* Sends current configuration information to the server.		*/
/************************************************************************/

void astrogtk_send_init(GtkButton *button, gpointer data)
{
	int	i,j;
	GtkWidget *item;
	struct smessage mess;
	struct s_init *action;

	really_destroy = TRUE;

	action = (struct s_init *)data;

	for(i=0; i<NUM_SCOPES; i++)
	{
		item = gtk_menu_get_active((GtkMenu *)pop_select_menus[i]);
		for(j=0; j<NUM_POPS && pop_menu_items[i][j] != item; j++);
			beam_config[action->server].pops[i] = j;

		item = gtk_menu_get_active((GtkMenu *)beam_select_menus[i]);
		for(j=0; j<NUM_BEAMS && beam_menu_items[i][j] != item; j++);
			beam_config[action->server].beams[i] = j;

		scope_active[action->server][i] = gtk_toggle_button_get_active(
                        (GtkToggleButton *)scope_buttons[i]);

	}

	item = gtk_menu_get_active((GtkMenu *)ref_select_menu);
	for(j=0; j<NUM_SCOPES && ref_menu_items[j] != item; j++);
		if (action->server != -1) reference_cart[action->server] = j;
			last_reference_cart = j;

	if (action->send_it)
	{
		mess.type = ASTROMOD_INIT;
		mess.length = sizeof(beam_config[0]);
		mess.data = (unsigned char *)(beam_config+action->server);

		if (!send_message(action->server, &mess))
		{
			if (astrogtk_init_window_up)
				create_message_window(
				"Failed to send ASTROMOD_INIT message.");
			else
				print_status(ERROR,
				"Failed to send ASTROMOD_INIT message.\n");
		}

		mess.type = ASTROMOD_SET_REFERENCE_CART;
		mess.length = sizeof(reference_cart[action->server]);
		mess.data = (unsigned char *)&(reference_cart[action->server]);

		if (!send_message(action->server, &mess))
		{
			if (astrogtk_init_window_up)
				create_message_window(
			"Failed to send ASTROMOD_SET_REFERENCE_CART message.");
			else
				print_status(ERROR,
		"Failed to send ASTROMOD_SET_REFERENCE_CART message.\n");
		}

		mess.type = ASTROMOD_SET_ACTIVE_SCOPES;
		mess.length = NUM_SCOPES*sizeof(char);
		mess.data = (unsigned char *)(scope_active[action->server]);

		if (!send_message(action->server, &mess))
		{
			if (astrogtk_init_window_up)
				create_message_window(
			"Failed to send ASTROMOD_SET_ACTIVE_SCOPES message.");
			else
				print_status(ERROR,
			"Failed to send ASTROMOD_SET_ACTIVE_SCOPES message.\n");
		}
	}

	if (config_callback != NULL) really_destroy = (*config_callback)();

} /* astrogtk_send_init() */

/************************************************************************/
/* astrogtk_get_current_config()					*/
/*									*/
/* Try and find out what the current config is right now.		*/
/* Returns TRUE if things go well.					*/
/************************************************************************/

bool astrogtk_get_current_config(int server)
{
        struct smessage mess;
        time_t  current_time;
	int	old_beam;

        /* The current targets are not valid */

	old_beam = beam_config[server].beams[0];
        beam_config[server].beams[0] = BEAM_NONE;

        /* Send message requesting targets */

        mess.type = ASTROMOD_CURRENT_CONFIG;
        mess.length = 0;
        mess.data = NULL;

        if (!send_message(server, &mess))
        {
		if (astrogtk_init_window_up)
			create_message_window(
                        "Failed to send ASTROMOD_CURRENT_CONFIG message.");
		else
			print_status(ERROR,
                        "Failed to send ASTROMOD_CURRENT_CONFIG message.\n");
		beam_config[server].beams[0] = old_beam;
		return FALSE;
        }

        /* Now wait for a while and see if they are updated */

        current_time = time(0);
        while(beam_config[server].beams[0] == BEAM_NONE)
        {
                process_server_socket(server);
                if (time(0) > current_time + 5)
		{
			beam_config[server].beams[0] = old_beam;
			return FALSE;
		}
        }

        /* That is all */

        return TRUE;

} /* astrogtk_get_current_config() */

/************************************************************************/
/* astrogtk_get_current_reference_cart()				*/
/*									*/
/* Try and find out what the current reference_cart is right now.	*/
/* Returns TRUE if things go well.					*/
/************************************************************************/

bool astrogtk_get_current_reference_cart(int server)
{
        struct smessage mess;
        time_t  current_time;
	int	old_reference_cart;

        /* The current targets are not valid */

	old_reference_cart = reference_cart[server];
        reference_cart[server] = -1;

        /* Send message requesting targets */

        mess.type = ASTROMOD_WHICH_REFERENCE_CART;
        mess.length = 0;
        mess.data = NULL;

        if (!send_message(server, &mess))
        {
		if (astrogtk_init_window_up)
			create_message_window(
                       "Failed to send ASTROMOD_WHICH_REFERENCE_CART message.");
		else
			print_status(ERROR,
	       "Failed to send ASTROMOD_WHICH_REFERENCE_CART message.\n");
		reference_cart[server] = old_reference_cart;
		return FALSE;
        }

        /* Now wait for a while and see if they are updated */

        current_time = time(0);
        while(reference_cart[server] == -1)
        {
                process_server_socket(server);
                if (time(0) > current_time + 5)
		{
			reference_cart[server] = old_reference_cart;
			return FALSE;
		}
        }

        /* That is all */

        return TRUE;

} /* astrogtk_get_current_reference_cart() */

/************************************************************************/
/* astrogtk_get_current_active_scopes()					*/
/*									*/
/* Try and find out what the currentactive scopes are is right now.	*/
/* Returns TRUE if things go well.					*/
/************************************************************************/

bool astrogtk_get_current_active_scopes(int server)
{
        struct smessage mess;
        time_t  current_time;

        /* Send message requesting targets */

        mess.type = ASTROMOD_GET_ACTIVE_SCOPES;
        mess.length = 0;
        mess.data = NULL;
	got_active_scopes = FALSE;

        if (!send_message(server, &mess))
        {
		if (astrogtk_init_window_up)
			create_message_window(
                       "Failed to send ASTROMOD_GET_ACTIVE_SCOPES message.");
		else
			print_status(ERROR,
	       "Failed to send ASTROMOD_GET_ACTIVE_SCOPES message.\n");
		return FALSE;
        }

        /* Now wait for a while and see if they are updated */

        current_time = time(0);
        while(!got_active_scopes)
        {
                process_server_socket(server);
                if (time(0) > current_time + 5) return FALSE;
        }

        /* That is all */

        return TRUE;

} /* astrogtk_get_current_active_scopes() */

/************************************************************************/
/* set_config_callback()						*/
/*									*/
/* Allows calling program to set a callback function for configuration	*/
/* changes.								*/
/************************************************************************/

void set_config_callback(int (*callback)(void))
{
	config_callback = callback;

} /* set_config_callback() */


/************************************************************************/
/* unset_config_callback()						*/
/*									*/
/* Allows calling program to remove a callback function for		*/
/* configuration changes.						*/
/************************************************************************/

void unset_config_callback(void)
{
	config_callback = NULL;

} /* set_config_callback() */

/************************************************************************/
/* get_and_display_star_and_config_callback()                           */
/*                                                                      */
/* Go and find out what the current configuration is.                   */
/* SHould ONLY be used when both control and init have been filled.	*/
/* Normally this is from a paged GUI.					*/
/************************************************************************/

void get_and_display_star_and_config_callback(GtkWidget *widget, gpointer data)
{
	int server;

	server = *((int *)data);

        if (!astrogtk_get_current_star(server))
        {
                print_status(ERROR,"Failed to get current star information.\n");
        }

        astrogtk_control_set_chara_hd(server);

        if (!astrogtk_get_current_config(server))
        {
                print_status(ERROR,"Failed to get current Pops and Beams.\n");
        }

        if (!astrogtk_get_current_reference_cart(server))
        {
                print_status(ERROR,"Failed to get current reference cart.\n");
        }

        if (!astrogtk_get_current_active_scopes(server))
        {
                print_status(ERROR,"Failed to get current active scopes.\n");
        }

        astrogtk_set_beams_pops(server);
        astrogtk_set_ref(server);

} /* get_and_display_star_and_config_callback() */

/************************************************************************/
/* send_star_and_config_callback()                                      */
/*                                                                      */
/* Send both star and pops and beams and reference.			*/
/* SHould ONLY be used when both control and init have been filled.	*/
/* Normally this is from a paged GUI.					*/
/************************************************************************/

void send_star_and_config_callback(GtkWidget *widget, gpointer data)
{
	astrogtk_chara_callback((GtkButton *)widget, NULL);
	astrogtk_send_init((GtkButton *)widget, (gpointer)&init_ok);

} /* send_star_and_config_callback() */
