/************************************************************************/
/* astrogtk_obswave_window.c                                            */
/*                                                                      */
/* Routine for obswaveling the basic CHARA astrogtk functions.		*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE                        */
/*                 Based on the SUSI User Interface                     */
/*              In turn based on the CHIP User interface                */
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA                 */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu                                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/* Author : Theo ten Brummelaar                                         */
/* Date   : March 2008                                                  */
/************************************************************************/

#include "../cliserv/include/cliserv.h"
#include "../include/chara_messages.h"
#include "astrogtk.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include "../charalib/chara.h"
#include "../slalib/slalib.h"

/* Globals */

static GtkWidget *astrogtk_obswave_window;
static char astrogtk_obswave_window_up = FALSE;
static GtkWidget *obswave_entry;

/************************************************************************/
/* create_astrogtk_obswave_window()					*/
/*									*/
/* ROutine for creating the astrogtk obswave window. This should be self*/
/* contained and give you all you need to obswave the basic astrogtk 	*/
/* functionality. Argumentis the window name.				*/
/************************************************************************/

GtkWidget* create_astrogtk_obswave_window (GtkButton *thebutton, gpointer *data)
{
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *button;
	GtkWidget *label;
	char	s[81];
	struct astro_control_window_data *the_data;

	/* Do nothing if we exist already */

	if (astrogtk_obswave_window_up) return astrogtk_obswave_window;

	/* Get the data */

	the_data = (struct astro_control_window_data *)data;

	/* Setup standard things */

	obs_ok.server = the_data->server;
	obs_ok.doit = TRUE;
	obs_cancel.server = the_data->server;
	obs_cancel.doit = FALSE;

	/* Find out what the current observing wavelength is */

	if (!astrogtk_get_current_obs_wavelength(the_data->server))
	{
		create_message_window("Failed to get current OBS wavelength.");
		return NULL;
	}

	/* Create the window itself */

	astrogtk_obswave_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (astrogtk_obswave_window),
		the_data->name);
	gtk_container_set_border_width (GTK_CONTAINER 
		(astrogtk_obswave_window), 1);

	/* Box to put things in */

	vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add (GTK_CONTAINER (astrogtk_obswave_window), vbox);
	gtk_widget_show(vbox);

        /* Make an hbox to put things in */

        hbox = gtk_hbox_new(FALSE, 0);
        gtk_container_add (GTK_CONTAINER (vbox), hbox);
        gtk_widget_show(hbox);


	/* PUt up a place to edit the wavelength */

        label = gtk_label_new("OBS Wave");
        gtk_widget_set_usize (label, ASTRO_OBSWAVE_WIDTH/2, ASTRO_HEIGHT);
        gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
        gtk_widget_show(label);

        obswave_entry = gtk_entry_new ();
        sprintf(s,"%lf", obs_wavelength[the_data->server]);
        gtk_entry_set_text (GTK_ENTRY (obswave_entry),s);
        gtk_widget_set_usize (obswave_entry,ASTRO_OBSWAVE_WIDTH/2,ASTRO_HEIGHT);
        gtk_box_pack_start(GTK_BOX(hbox), obswave_entry,TRUE,TRUE, 0);
        gtk_widget_show(obswave_entry);

        hbox = gtk_hbox_new(FALSE, 0);
        gtk_container_add (GTK_CONTAINER (vbox), hbox);
        gtk_widget_show(hbox);

	/* CANCEL button */

	button = gtk_button_new_with_label ("CANCEL");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
	    GTK_SIGNAL_FUNC (astrogtk_obswave_ok_callback),
		(gpointer)&obs_cancel);
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
        gtk_widget_set_usize(button, ASTRO_OBSWAVE_WIDTH/2, ASTRO_HEIGHT);
	gtk_container_set_border_width (GTK_CONTAINER(button),1);
	gtk_widget_show(button);

	/* OK button */

	button = gtk_button_new_with_label ("OK");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
	    GTK_SIGNAL_FUNC (astrogtk_obswave_ok_callback), (gpointer)&obs_ok);
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
        gtk_widget_set_usize(button, ASTRO_OBSWAVE_WIDTH/2, ASTRO_HEIGHT);
	gtk_container_set_border_width (GTK_CONTAINER(button),1);
	gtk_widget_show(button);

	/* Now, we can show this window if we like */

	gtk_widget_show(astrogtk_obswave_window);

	/* Make sure we only get one version of this window */

	astrogtk_obswave_window_up = TRUE;

	/* That should be all */

	return astrogtk_obswave_window;

} /* create_astrogtk_obswave_window() */

/************************************************************************/
/* astrogtk_obswave_ok_callback()					*/
/*									*/
/* Callback function for astrogtk obswave OK button. Stop background	*/
/* processing and destroy astrogtk obswave window.			*/
/************************************************************************/

void astrogtk_obswave_ok_callback(GtkButton *button, gpointer data)
{
	struct smessage mess;
	struct s_obswave *action;
	char	*entry;

	if (!astrogtk_obswave_window_up) return;

	action = (struct s_obswave *)data;

	if (action->doit)
	{
		entry = (char *)gtk_entry_get_text(GTK_ENTRY(obswave_entry));
		sscanf(entry,"%lf", obs_wavelength+action->server);

		mess.type = ASTROMOD_SET_OBS_WAVELENGTH;
		mess.data = (char unsigned *)(obs_wavelength+(action->server));
		mess.length = sizeof(obs_wavelength[0]);
		if (!send_message(action->server, &mess))
		    create_message_window(
			"Failed to send ASTROMOD_SET_OBS_WAVELENGTH message.");
	}

	gtk_widget_destroy(astrogtk_obswave_window);
	astrogtk_obswave_window_up = FALSE;

} /* astrogtk_obswave_ok_callback() */

/************************************************************************/
/* astrogtk_get_current_obs_wavelength()				*/
/*									*/
/* Try and find out what the current observing wavelength is right now.	*/
/* Returns TRUE if things go well.					*/
/************************************************************************/

bool astrogtk_get_current_obs_wavelength(int server)
{
        struct smessage mess;
        time_t  current_time;
	double  old_obs_wavelength;

        /* The current targets are not valid */

	old_obs_wavelength = obs_wavelength[server];
	obs_wavelength[server] = 0.0;

        /* Send message requesting targets */

        mess.type = ASTROMOD_GET_OBS_WAVELENGTH;
        mess.length = 0;
        mess.data = NULL;

        if (!send_message(server, &mess))
        {
                create_message_window(
                        "Failed to send ASTROMOD_GET_OBS_WAVELENGTH message.");
		obs_wavelength[server] = old_obs_wavelength;
		return FALSE;
        }

        /* Now wait for a while and see if they are updated */

        current_time = time(0);
        while(obs_wavelength[server] == 0.0)
        {
                process_server_socket(server);
                if (time(0) > current_time + 2)
		{
			obs_wavelength[server] = old_obs_wavelength;
			return FALSE;
		}
        }

        /* That is all */

        return TRUE;

} /* astrogtk_get_current_star() */
