/************************************************************************/
/* astromod.h								*/
/*                                                                      */
/* General Header File for astrometric model functions.			*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY CLIENT/SERVER				*/
/*                 Based on the CHARA User Interface			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@@chara.gsu.edu                                       */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                          		*/
/* Date   : April 2006							*/
/************************************************************************/

#ifndef __ASTROMOD__
#define __ASTROMOD__

#include <math.h>

/* Globals */

extern TELESCOPE scopes[MAX_NUMBER_TELESCOPES];
extern SYSTEM	 chara;

/* astromod_calc.c */

int astromod_init(int *beams, int *pops);

double astromod_mjd(int year,             /* Which year */
		    int month,            /* 1..12, the month */
		    int day,              /* 1..31 the day of month */
		    double ut);           /* UT time, 2PI=24hrs */ 

double astromod_atomic_time(double mjd);

int astromod_azel_date(struct s_star star, 	/* Information on the star */
		       int    year, 	/* Which year */
		       int    month,	/* 1..12, the month */
		       int    day, 	/* 1..31 the day of month */
		       double ut, 	/* UT time, 2PI=24hrs */
		       double *az, 	/* Star position in radians */
		       double *el,	/* Star position in radians */
                       double *ref_el,  /* Refracted star position in radians*/
		       double *hour_angle); /* Hour angle */


int astromod_azel_mjd(struct s_star star, 	/* Information on the star */
		      double mjd,	/* Modified Julain Date */
		      double *az, 	/* Star position in radians */
		      double *el,	/* Star position in radians */
                      double *ref_el,   /* Refracted star position in radians*/
		      double *hour_angle); /* Hour angle */

VECTOR astromod_star_vector(double az, double el);

double astromod_cart_position(int cart, int ref_cart, double n_air,
                              double az, double el, int mode, 
			      double *delta_air_path);

int astromod_uv(double dec, double ha, double Bx, double By, double Bz,
                double lambda, double *u, double *v);

void print_chara_config(FILE *fp);
void print_star(STAR_DATA, FILE *fp);

int astromod_twilight(int year, int month, int day, double *start_local,
		double *stop_local, double *start_ut, double *stop_ut);
int     get_tel(int beam);

/* astromod_refract.c */

double astromod_refract(double temp, double pressure, double rel_humid, double CO2,
	double wavelength);

double astromod_compressibility(double temp, double pressure, double CO2);

#endif
