#include <stdio.h>
#include <string.h>
#include <math.h>
#define D_M1BRT_M2BRT 1.0881    // Distance between M1 and M2 of BRT
#define D_V1_M1BRT 0.9775       // Distance between M1(BRT) and beam V1
#define D_V1_VEGA 9.1567       	// Distance between V1 and entrance of VEGA
#define DV 0.0762   			// Distance between two beams after switchyard
#define DF 0.5842   			// Distance between two beams before switchyard for telescopes on the same arms
#define DT 1.2446   			// Distance between beams before switchyard for telescopes on differents arms



// On the basis of Laszlo document, one begins to identify the constant parameter
// of the equation Y=0.045X+cte, where X is the position of the cart and Y is the distance
// of the output pupil P''' with respect to the vertex of the secondary mirror of the BRT

double D_M2BRT_Pupil(tel,pop,x_cart)
char *tel;
char *pop;
double x_cart;
{
double cte;

if (strcmp(tel,"S1")==0) {
	if(strcmp(pop,"POP1")==0) cte=5.225;
	if(strcmp(pop,"POP2")==0) cte=6.049;
	if(strcmp(pop,"POP3")==0) cte=6.872;
	if(strcmp(pop,"POP4")==0) cte=7.696;
	if(strcmp(pop,"POP5")==0) cte=8.447;
}
if (strcmp(tel,"S2")==0) {
	if(strcmp(pop,"POP1")==0) cte=4.460;
	if(strcmp(pop,"POP2")==0) cte=5.283;
	if(strcmp(pop,"POP3")==0) cte=6.107;
	if(strcmp(pop,"POP4")==0) cte=6.931;
	if(strcmp(pop,"POP5")==0) cte=7.682;
}
if (strcmp(tel,"E1")==0) {
	if(strcmp(pop,"POP1")==0) cte=5.458;
	if(strcmp(pop,"POP2")==0) cte=6.281;
	if(strcmp(pop,"POP3")==0) cte=7.105;
	if(strcmp(pop,"POP4")==0) cte=7.928;
	if(strcmp(pop,"POP5")==0) cte=8.680;
}
if (strcmp(tel,"E2")==0) {
	if(strcmp(pop,"POP1")==0) cte=4.070;
	if(strcmp(pop,"POP2")==0) cte=4.893;
	if(strcmp(pop,"POP3")==0) cte=5.717;
	if(strcmp(pop,"POP4")==0) cte=6.540;
	if(strcmp(pop,"POP5")==0) cte=7.292;
}
if (strcmp(tel,"W1")==0) {
	if(strcmp(pop,"POP1")==0) cte=4.166;
	if(strcmp(pop,"POP2")==0) cte=4.990;
	if(strcmp(pop,"POP3")==0) cte=5.813;
	if(strcmp(pop,"POP4")==0) cte=6.637;
	if(strcmp(pop,"POP5")==0) cte=7.388;
}
if (strcmp(tel,"W2")==0) {
	if(strcmp(pop,"POP1")==0) cte=1.973;
	if(strcmp(pop,"POP2")==0) cte=2.796;
	if(strcmp(pop,"POP3")==0) cte=3.620;
	if(strcmp(pop,"POP4")==0) cte=4.443;
	if(strcmp(pop,"POP5")==0) cte=5.195;
}

return(0.045*x_cart+cte);
}




double D_CVT_Pupil(Y,tel,beam)
double Y;
char *tel;
char *beam;
{
	double Z;
	double Path_Switch;

// Constant paths
Z=Y+D_M1BRT_M2BRT+D_V1_M1BRT+D_V1_VEGA;

if (strcmp(tel,"S1")==0) {
	if(strcmp(beam,"V1")==0) Path_Switch=0.;
	if(strcmp(beam,"V2")==0) Path_Switch=DV;
	if(strcmp(beam,"V3")==0) Path_Switch=2*DV;
	if(strcmp(beam,"V4")==0) Path_Switch=3*DV;
}
if (strcmp(tel,"S2")==0) {
	if(strcmp(beam,"V1")==0) Path_Switch=DF;
	if(strcmp(beam,"V2")==0) Path_Switch=DF+DV;
	if(strcmp(beam,"V3")==0) Path_Switch=DF+2*DV;
	if(strcmp(beam,"V4")==0) Path_Switch=DF+3*DV;
}
if (strcmp(tel,"W1")==0) {
	if(strcmp(beam,"V1")==0) Path_Switch=DT+DF;
	if(strcmp(beam,"V2")==0) Path_Switch=DT+DF+DV;
	if(strcmp(beam,"V3")==0) Path_Switch=DT+DF+2*DV;
	if(strcmp(beam,"V4")==0) Path_Switch=DT+DF+3*DV;
}
if (strcmp(tel,"W2")==0) {
	if(strcmp(beam,"V1")==0) Path_Switch=DT+2*DF;
	if(strcmp(beam,"V2")==0) Path_Switch=DT+2*DF+DV;
	if(strcmp(beam,"V3")==0) Path_Switch=DT+2*DF+2*DV;
	if(strcmp(beam,"V4")==0) Path_Switch=DT+2*DF+3*DV;
}
if (strcmp(tel,"E2")==0) {
	if(strcmp(beam,"V1")==0) Path_Switch=2*DT+2*DF;
	if(strcmp(beam,"V2")==0) Path_Switch=2*DT+2*DF+DV;
	if(strcmp(beam,"V3")==0) Path_Switch=2*DT+2*DF+2*DV;
	if(strcmp(beam,"V4")==0) Path_Switch=2*DT+2*DF+3*DV;
}
if (strcmp(tel,"E1")==0) {
	if(strcmp(beam,"V1")==0) Path_Switch=2*DT+3*DF;
	if(strcmp(beam,"V2")==0) Path_Switch=2*DT+3*DF+DV;
	if(strcmp(beam,"V3")==0) Path_Switch=2*DT+3*DF+2*DV;
	if(strcmp(beam,"V4")==0) Path_Switch=2*DT+3*DF+3*DV;
}
Z=Z+Path_Switch;
return(Z*1000.);
}


double D_Pupil(Z,beam)
double Z;
char *beam;
{
	double FL_M3;
	double FL_M4;
	double FL_M5;
	double D;
	double Z1,Z2,Z3;

FL_M3=480.;

if(strcmp(beam,"V1")==0) FL_M4=-6.10;
if(strcmp(beam,"V2")==0) FL_M4=-6.75;
if(strcmp(beam,"V3")==0) FL_M4=-7.55;
if(strcmp(beam,"V4")==0) FL_M4=-8.35;

FL_M5=126;

if(strcmp(beam,"V1")==0) Z=Z+4842.05;
if(strcmp(beam,"V2")==0) Z=Z+4733.15;
if(strcmp(beam,"V3")==0) Z=Z+4624.25;
if(strcmp(beam,"V4")==0) Z=Z+4515.35;

Z=-Z;
// From Z to Z1 through M3_BCP
Z1=1./(1./Z+1./FL_M3);

// From Z1 to Z2 through M4_BCP
Z2=1./(1./(Z1-FL_M3)+1./FL_M4);

// From Z2 to Z3 through M5_BCP
Z3=1./(1./(Z2-FL_M5)+1./FL_M5);

if(strcmp(beam,"V1")==0) D=Z3-971.35;
if(strcmp(beam,"V2")==0) D=Z3-796.25;
if(strcmp(beam,"V3")==0) D=Z3-621.15;
if(strcmp(beam,"V4")==0) D=Z3-446.05;

return(D);
}

// appel main(S1,POP1,V1,10)
main(argc,argv)
int argc;
char *argv[];
{
	double Y,Z,P;

	Y=D_M2BRT_Pupil(argv[1],argv[2],atof(argv[4]));

	Z=D_CVT_Pupil(Y,argv[1],argv[3]);

	P=D_Pupil(Z,argv[3]);
	printf("P=%lf\n",P);

}

