/************************************************************************/
/* baytech.c								*/
/*                                                                      */
/* Test program for Baytech functions.					*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar			                        */
/* Date   : Jan 1999							*/
/************************************************************************/

#include "../charaui/src/charaui.h"
#include "baytech.h"

/* A local global for the serial ports */

struct sserial rpc_port;
struct sserial m4_port;

/* Local for the status */

static unsigned char rpc_status;

/************************************************************************/
/* call_open_baytech_rpc_port()						*/
/*									*/
/* User callable version of open_baytech_rpc_port().			*/
/************************************************************************/

int call_open_baytech_rpc_port(int argc, char **argv)
{
	char	port_name[81];

	socket_test_args(1,"Port");

	if (rpc_port.fd != -1) return error(WARNING,"RPC port already open.");

        /* Check command line */

        if (argc < 2)   /* No port given */
        {
		clean_command_line();
                sprintf(port_name,"/dev/cua0");
                if (quick_edit("Serial port",port_name, port_name,NULL,STRING)==
			KEY_ESC)
                {
                        return NOERROR;
                }
        }
        else
        {
		strcpy(port_name,argv[1]);
	}

	/* Open the port */

	rpc_port = open_baytech_rpc_port(port_name);

	if (rpc_port.fd == -1)
	{
		return ERROR;
	}

	/* Get the initial status */

	rpc_status = baytech_rpc_get_status(rpc_port);
	werase(system_window);
	mvwprintw(system_window,0,0,"RPC status 0x%x.",rpc_status);
	wrefresh(system_window);

	if (rpc_status &0x40) return ERROR;

	return NOERROR;

} /* call_open_baytech_rpc_port() */

/************************************************************************/
/* call_reopen_baytech_rpc_port()					*/
/*									*/
/* User callable version of reopen_baytech_rpc_port().			*/
/************************************************************************/

int call_reopen_baytech_rpc_port(int argc, char **argv)
{
	/* Open the port */

	rpc_status = reopen_baytech_rpc_port(&rpc_port);

	if (rpc_port.fd == -1)
	{
		return error(ERROR,"Failed to reopen port.");
	}

	/* Get the initial status */

	werase(system_window);
	mvwprintw(system_window,0,0,"RPC status 0x%x.",rpc_status);
	wrefresh(system_window);

	if (rpc_status &0x40) return ERROR;

	return NOERROR;

} /* call_reopen_baytech_rpc_port() */

/************************************************************************/
/* call_close_baytech_rpc_port()					*/
/*									*/
/* User callable function to close the serial port for the baytech.	*/
/************************************************************************/

int call_close_baytech_rpc_port(int argc, char **argv)
{
	if (rpc_port.fd == -1) return NOERROR;
	close_serial_port(rpc_port);
	rpc_port.fd = -1;


	return NOERROR;

} /* call_close_baytech_rpc_port */

/************************************************************************/
/* call_baytech_rpc_on()						*/
/*									*/
/* User callable version of baytech_rpc_on().				*/
/************************************************************************/

int call_baytech_rpc_on(int argc, char **argv)
{
	int	channel;
	char	s[81];

	socket_test_args(1,"Channel");

        /* Check command line */

        if (argc < 2)   /* No port given */
        {
                sprintf(s,"1");
		clean_command_line();
                if (quick_edit("Channel",s, s,NULL,INTEGER)== KEY_ESC)
                {
                        return NOERROR;
                }
		sscanf(s,"%d",&channel);
        }
        else
	{
		sscanf(argv[1],"%d",&channel);
	}

	rpc_status = baytech_rpc_on(rpc_port, channel);
	werase(system_window);
	mvwprintw(system_window,0,0,"RPC status 0x%x.",rpc_status);
	wrefresh(system_window);

	if (rpc_status & 0x40) return ERROR;

	return NOERROR;

} /* call_baytech_rpc_on() */

/************************************************************************/
/* call_baytech_rpc_off()						*/
/*									*/
/* User callable version of baytech_rpc_off().				*/
/************************************************************************/

int call_baytech_rpc_off(int argc, char **argv)
{
	int	channel;
	char	s[81];

	socket_test_args(1,"Channel");

        /* Check command line */

        if (argc < 2)   /* No port given */
        {
                sprintf(s,"1");
		clean_command_line();
                if (quick_edit("Channel",s, s,NULL,INTEGER)== KEY_ESC)
                {
                        return NOERROR;
                }
		sscanf(s,"%d",&channel);
        }
        else
	{
		sscanf(argv[1],"%d",&channel);
	}

	rpc_status = baytech_rpc_off(rpc_port, channel);
	werase(system_window);
	mvwprintw(system_window,0,0,"RPC status 0x%x.",rpc_status);
	wrefresh(system_window);

	if (rpc_status & 0x40) return ERROR;

	return NOERROR;

} /* call_baytech_rpc_off() */

/************************************************************************/
/* call_baytech_rpc_all_on()						*/
/*									*/
/* User callable version of baytech_rpc_all_on().			*/
/************************************************************************/

int call_baytech_rpc_all_on(int argc, char **argv)
{
	if (baytech_rpc_all_on(rpc_port) & 0x40) return ERROR;

	return NOERROR;

} /* call_baytech_rpc_all_on */

/************************************************************************/
/* call_baytech_rpc_all_off()						*/
/*									*/
/* User callable versioff of baytech_rpc_all_off().			*/
/************************************************************************/

int call_baytech_rpc_all_off(int argc, char **argv)
{
	if (baytech_rpc_all_off(rpc_port) & 0x40) return ERROR;

	return NOERROR;

} /* call_baytech_rpc_all_off */

/************************************************************************/
/* get_baytech_rpc_get_status()						*/
/*									*/
/* Returns the RPC status.						*/
/************************************************************************/

unsigned char	get_baytech_rpc_get_status(void)
{
	return baytech_rpc_get_status(rpc_port);

} /* get_baytech_rpc_get_status() */

/************************************************************************/
/* call_open_baytech_m4_port()						*/
/*									*/
/* User callable version of open_baytech_m4_port().			*/
/************************************************************************/

int call_open_baytech_m4_port(int argc, char **argv)
{
	char	port_name[81];

	socket_test_args(1,"Port");

	if (m4_port.fd != -1) return error(WARNING,"M4 port already open.");

        /* Check command line */

        if (argc < 2)   /* No port given */
        {
                sprintf(port_name,"/dev/ttyC6");
		clean_command_line();
                if (quick_edit("Serial port",port_name, port_name,NULL,STRING)==
			KEY_ESC)
                {
                        return NOERROR;
                }
        }
        else
        {
		strcpy(port_name,argv[1]);
	}

	/* Open the port */

	m4_port = open_baytech_m4_port(port_name);

	if (m4_port.fd == -1)
	{
		return ERROR;
	}

	return NOERROR;

} /* call_open_baytech_m4_port() */

/************************************************************************/
/* call_close_baytech_m4_port()						*/
/*									*/
/* User callable function to close the serial port for the baytech.	*/
/************************************************************************/

int call_close_baytech_m4_port(int argc, char **argv)
{
	if (m4_port.fd == -1) return NOERROR;
	close_serial_port(m4_port);
	m4_port.fd = -1;


	return NOERROR;

} /* call_close_baytech_m4_port */

/************************************************************************/
/* call_baytech_m4_relay_status()					*/
/************************************************************************/

int call_baytech_m4_relay_status(int argc, char **argv)
{
	werase(system_window);
	mvwprintw(system_window,0,0,"M4 Relay status 0x%x.",
		baytech_m4_relay_status(m4_port));
	wrefresh(system_window);

	return NOERROR;

} /* call_baytech_m4_relay_status() */

/************************************************************************/
/* call_baytech_m4_relay_energize()					*/
/*									*/
/* User callable version of call_baytech_m4_relay_energize().		*/
/************************************************************************/

int call_baytech_m4_relay_energize(int argc, char **argv)
{
	int	channel;
	char	s[81];

	socket_test_args(1,"Channel");

        /* Check command line */

        if (argc < 2)   /* No channel given */
        {
                sprintf(s,"1");
		clean_command_line();
                if (quick_edit("Channel",s, s,NULL,INTEGER)== KEY_ESC)
                {
                        return NOERROR;
                }
		sscanf(s,"%d",&channel);
        }
        else
	{
		sscanf(argv[1],"%d",&channel);
	}

	if (baytech_m4_relay_energize(m4_port, channel) != NOERROR)
	{
	    return error(ERROR,"Sorry, failed to energize relay %d",channel);
	}

	return NOERROR;

} /* call_baytech_m4_relay_energize() */

/************************************************************************/
/* call_baytech_m4_relay_de_energize()					*/
/*									*/
/* User callable version of call_baytech_m4_relay_de_energize().	*/
/************************************************************************/

int call_baytech_m4_relay_de_energize(int argc, char **argv)
{
	int	channel;
	char	s[81];

	socket_test_args(1,"Channel");

        /* Check command line */

        if (argc < 2)   /* No channel given */
        {
                sprintf(s,"1");
		clean_command_line();
                if (quick_edit("Channel",s, s,NULL,STRING)== KEY_ESC)
                {
                        return NOERROR;
                }
		sscanf(s,"%d",&channel);
        }
        else
	{
		sscanf(argv[1],"%d",&channel);
	}

	if (baytech_m4_relay_de_energize(m4_port, channel) != NOERROR)
	{
	    return error(ERROR,"Sorry, failed to de-energize relay %d",channel);
	}

	return NOERROR;

} /* call_baytech_m4_relay_de_energize() */

/************************************************************************/
/* call_baytech_m4_set_dac()						*/
/*									*/
/* User callable version of call_baytech_m4_set_dac().			*/
/************************************************************************/

int call_baytech_m4_set_dac(int argc, char **argv)
{
	int	channel;
	int	value;
	char	s[81];

	socket_test_args(2,"Channel Value");

        /* Check command line */

        if (argc < 2)   /* No channel given */
        {
                sprintf(s,"1");
		clean_command_line();
                if (quick_edit("Channel",s, s,NULL,INTEGER)== KEY_ESC)
                {
                        return NOERROR;
                }
		sscanf(s,"%d",&channel);
        }
        else
	{
		sscanf(argv[1],"%d",&channel);
	}

        if (argc < 3)   /* No value given */
        {
                sprintf(s,"4095");
		clean_command_line();
                if (quick_edit("Value",s, s,NULL,INTEGER)== KEY_ESC)
                {
                        return NOERROR;
                }
		sscanf(s,"%d",&value);
        }
        else
	{
		sscanf(argv[2],"%d",&value);
	}

	if (baytech_m4_set_dac(m4_port, channel, value) != NOERROR)
	{
	    return error(ERROR,"Sorry, failed to set dac %d to %d",
			channel,value);
	}

	return NOERROR;

} /* call_baytech_m4_set_dac() */

/************************************************************************/
/* call_baytech_m4_dac_array()						*/
/*									*/
/* User callable version of call_baytech_m4_dac_array().		*/
/************************************************************************/

int call_baytech_m4_dac_array(int argc, char **argv)
{
	int	channel,i;
	int	values[1000];
	char	s[81];

	socket_test_args(2,"Channel");

        /* Check command line */

        if (argc < 2)   /* No channel given */
        {
                sprintf(s,"1");
		clean_command_line();
                if (quick_edit("Channel",s, s,NULL,INTEGER)== KEY_ESC)
                {
                        return NOERROR;
                }
		sscanf(s,"%d",&channel);
        }
        else
	{
		sscanf(argv[1],"%d",&channel);
	}

	/* Create the array of numbers */

	for (i=0; i<1000; i++)
	{
		if (i%2 == 0) 
			values[i] = 0;
		else
			values[i] = 4095;
	}

	if (baytech_m4_dac_array(m4_port, channel, values, 1000) != NOERROR)
	{
	    return error(ERROR,"Sorry, failed to send array to dac %d",channel);
	}

	return NOERROR;

} /* call_baytech_m4_dac_array() */

/************************************************************************/
/* call_baytech_m4_adc_status()						*/
/************************************************************************/

int call_baytech_m4_adc_status(int argc, char **argv)
{
	float	*result;
	char	s[81],t[81];
	int	i;

	result = baytech_m4_adc_status(m4_port);

	sprintf(s,"%5.1f",result[1]);
	for(i=2;i<=16;i++) 
	{
		sprintf(t,"%5.1f",result[i]);
		strcat(s,t);
	}
	werase(system_window);
	mvwprintw(system_window,0,0,"%s",s);
	wrefresh(system_window);

	return NOERROR;

} /* call_baytech_m4_adc_status() */
