/************************************************************************/
/* test_clock.c								*/
/*                                                                      */
/* Test program for 2.6 kernel clock driver code.			*/
/* Not built automatically. You need to type:				*/
/*									*/
/* make -f Makefile.2.6.18-128.el5 test_clock				*/
/*									*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar			                        */
/* Date   : Oct 2009							*/
/************************************************************************/

#include <math.h>
#include "../charaui/src/charaui.h"
#include "../clock/clock.h"
#include "./bunker_clock.h"
#include "../charalib/chara.h"

/* Prototypes */

int main(int argc, char **argv);
int report_time_next_second(int argc, char **argv);
int test_signal_time(int argc, char **argv);
int linux_time_status(void);
int rt_time_status(void);
void chara_close(void);

void test_offset_callback(void)
{
	message(system_window,"OFFSET TOO LARGE!");
	usleep(20000);
	werase(system_window);
	wrefresh(system_window);
}

int main(int argc, char **argv)
{
	/* Title page. */

	ui_clear_screen();
	put_line("");
	center_line("CHARA 2.6 KERNEL CLOCK TEST PROGRAM");
	put_line("");
	center_line("The CHARA Array");
	center_line("Center for High Angular Resolution Astronomy");
	center_line("Mount Wilson Observatory, CA 91001, USA");
	put_line("");
	center_line("Telephone: 1-626-796-5405");
	center_line("Fax: 1-626-796-6717");
	center_line("email: theo@chara-array.edu");
	center_line("WWW: http://www.chara.gsu.edu");
	put_line("");
	center_line("(C) This executable program is copyright.");
	wait_for_title();

	/* Initialize the user interface */

	TITLE = "CLOCK TEST VERSION 1.0";
	initialise_ui("menus.ini", "help.ini", "test_clock");
        set_user_close_function(chara_close);

	/* Try and open up the FIFOs */

        if (!open_clock_device()) error(FATAL,"Failed to open CBDriver");
        if (!bunker_clock_open()) error(FATAL,"Failed to open BCDriver");

	/* Setup background job(s) */

	background_add(linux_time_status);
	background_add(rt_time_status);
	//background_add(check_clock_offset);
	//set_clock_offset_callback(test_offset_callback);
	//set_max_clock_offset(1000);

        /* Set up the client port */

	setup_standard_clock_messages();

	/* Make sure the background runs */

	background_start(0, NULL);

	/* Let's go! */

	start_ui(); /* Should never return from here. */

	exit(0);

} /* main() */

/************************************************************************/
/* linux_time_status()							*/
/* 									*/
/* Displays the linux system time in the status window.			*/
/************************************************************************/

int linux_time_status(void)
{
	time_t current_time;
	struct tm *now;

	time(&current_time);
	now = localtime(&current_time);
	wstandout(status_window);
	mvwaddstr(status_window,0,0,"Linux Time: ");
	wstandend(status_window);
	wprintw(status_window," %2d:%02d:%02d",
		now->tm_hour,now->tm_min,now->tm_sec);

	return NOERROR;

} /* linux_time_status() */

/************************************************************************/
/* rt_time_status()							*/
/* 									*/
/* Displays the rt system time in the status window.			*/
/************************************************************************/

int rt_time_status(void)
{
	rt_time_struct now;
	ap_time_struct t;
	char	s[20],*p;
	double	st;

	if (get_rt_time(&now) != 0) 
		return error(ERROR,"Failed to get RT time.");

	wstandout(status_window);
	mvwaddstr(status_window,1,0,"CHARA Time: ");
	wstandend(status_window);
	t = rt_to_ap_time(now.time_now);
	wprintw(status_window,"%s",ap_to_str_time(s,t));

	wstandout(status_window);
        mvwaddstr(status_window,2,0,"Sid Time  : ");
        wstandend(status_window);
	st = sidereal_time();
	while (st < 0.0) st += (2.0*M_PI);
        time_to_string(st,s);
        for(p=s; *p!= 0; p++)
        {
                if (p != s && *p == ' ') *p = ':';
                if (*p == '.') *p = 0;
        }
        wprintw(status_window,"%s",s);

	wstandout(status_window); mvwaddstr(status_window,3,0,"Lost Ticks: ");
	wstandend(status_window); wprintw(status_window,"%9d",now.lost_ticks);

	wstandout(status_window);
	mvwaddstr(status_window,4,0,"Lost Secs : ");
	wstandend(status_window);
	wprintw(status_window,"%9d",now.lost_seconds);

	wstandout(status_window);
	mvwaddstr(status_window,5,0,"Tick time : ");
	wstandend(status_window);
	wprintw(status_window,"%9d",now.time_now);

	return NOERROR;

} /* rt_time_status() */

/************************************************************************/
/* report_time_next_second() 						*/
/* 									*/
/* User callable routine to test get_time_next_second().		*/
/************************************************************************/

int report_time_next_second(int argc, char **argv)
{
	message(system_window,
		"Time of next second = %d.",get_time_next_second(TRUE));

	return NOERROR;

} /* report_time_next_second() */

/************************************************************************/
/* report_time() 							*/
/* 									*/
/************************************************************************/

int report_time(int argc, char **argv)
{
	rt_time_struct the_time;

	if (get_rt_time(&the_time) != 0)
		return error(ERROR,"Failed ot get current time.");

	
	message(system_window,
		"Time = %d Lost ticks = %d Lost seconds = %d", 
		the_time.time_now, the_time.lost_ticks, the_time.lost_seconds);

	return NOERROR;

} /* report_time() */

/************************************************************************/
/* chara_close()                                                        */
/*                                                                      */
/* Function called at close time.                                       */
/************************************************************************/
 
void chara_close(void)
{
	close_clock_device();

} /* chara_close() */

/************************************************************************/
/* test_signal_time()							*/
/*									*/
/* Routine for testing how fast tiptilt signals can be sent to us.	*/
/************************************************************************/

int test_signal_time(int argc, char **argv)
{
	struct sserial port;
	CHARA_TIME	start = 0, then, now, last;
	float	dt, dt_mean, dt_mean2, n, period;
	char	c, *p;
	int	nb;
	bool	first = TRUE;
	time_t  start_time;

	/* Open the port */

	port = open_serial_port("/dev/ttyS0", FALSE);

	/* N81 115200 Baud no handshaking */

	 if (set_serial_baud_rate(&port, B115200) != NOERROR)
        {
                close_serial_port(port);
                port.fd = -1;
                return error(ERROR,"Failed to set baud rate");
        }

        if (set_serial_parity(&port, OFF) != NOERROR)
        {
                close_serial_port(port);
                port.fd = -1;
                return error(ERROR,"Failed to set paraty");
        }

        if (set_serial_bitlength(&port, 8) != NOERROR)
        {
                close_serial_port(port);
                port.fd = -1;
                return error(ERROR,"Failed to set bitlength");
        }

        if (set_serial_stopbits(&port, 1) != NOERROR)
        {
                close_serial_port(port);
                port.fd = -1;
                return error(ERROR,"Failed to set stopbits");
        }

        /* No handshaking */

        if (set_serial_xonxoff(&port, OFF) != NOERROR)
        {
                close_serial_port(port);
                port.fd = -1;
                return error(ERROR,"Failed to set xonxoff");
        }

        if (set_serial_hard_handshake(&port, OFF) != NOERROR)
        {
                close_serial_port(port);
                port.fd = -1;
                return error(ERROR,"Failed to set handshaking");
        }

	/* Empty the buffer */

	n = 0.0;
        while(read(port.fd, &c, 1) == 1) n += 1.0;

	/* Try and wait for a full second tick */

	time(&start_time);
	while(start_time == time(0));

	/* OK ... here we go */

	dt_mean = dt_mean2 = n = 0.0;
	last = 0;
	p = (char *)&then;
	nb = 0;
	message(system_window,
		"Flushed %.0f bytes. Waiting for data from tiptilt.",n);
	first = TRUE;

	while(!kbhit())
	{
		/* Read in the signal */

		while (read(port.fd, p, sizeof(*p)) == sizeof(*p))
		{
			p++;
			if (++nb == sizeof(then)) break;
		}

		if (nb < sizeof(then)) continue;

		nb = 0;
		p = (char *)&then;

		now = chara_time_now();

		if (!first)
		{
			dt = (float)(now - then);
			dt_mean += dt;
			dt_mean2 += (dt *dt);
			n += 1.0;
		}

		if (now > last+TICKS_PER_SECOND)
		{
			if (first)
			{
				start = now;
				first = FALSE;
				continue;
			}
			last = now;
			period = (float)(now - start)/(float)TICKS_PER_SECOND;
			message(system_window,
			"dt = %.1f+-%.1f out of %.0f %.2f Secs %.2f Hz",
				dt_mean/n, 
				sqrt(dt_mean2/n - dt_mean/n*dt_mean/n),
				n, period,n/period);
		}
	}
	get_command();

	/* OK, we're done, close the port */

	close_serial_port(port);

	return NOERROR;

} /* test_signal_time() */
