/***************************************************************************
 *   Copyright (C) 2007 by Arnaud Sevin
 *   Arnaud.Sevin@obspm.fr
 * 
 *   With the help of Eric Gendron, Zoltan Hubert, Denis Peret 
 *
 * Copyright (c) 1998, Regents of the University of California
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/
#ifndef CAM_PERSEE_LISTENER_H
#define CAM_PERSEE_LISTENER_H

#include <vector>
#include <sstream>

#include "com_persee.h"
#include "fitsio.h"
#include "moments.h"
#include "ConfigFile.h"
#include "roi.h"
#include "client_FTP.h"

/***************************************************
 *                                                 *
 *          Classe Cam_PERSEE_Listener             *
 *                                                 *
 * - listen data from detector                     *
 * - update monitoring FITS                        *
 * - create acquisition FITS during TopAcq         *
 *                                                 *
 ***************************************************/

/**
 * @brief Classe permettant d'ecouter les ports des electronique de PERSEE
 * @author   Arnaud Sevin <Arnaud.Sevin@obspm.fr>
 * @version  1.0
 * @date     02/06/2008
 */
class ROI_def{
 private:
  string name_ROI;
  string dim_ROI;
  ROI *ROIs;

 public:

  ROI_def(string name_ROI, short unsigned int tab[4]){
    this->name_ROI = name_ROI;
    stringstream out;
    out << tab[0] << "," << tab[1] << "," << tab[2] << "," << tab[3];
    this->dim_ROI = out.str();
    this->ROIs = new ROI(tab);
  }

  ~ROI_def(){
    delete this->ROIs;
  }

  string get_name() {return name_ROI;}
  string get_dim() {return dim_ROI;}
  ROI* get_ROI() {return ROIs;}
/*
  ROI_def& operator= ( const ROI_def& x ) {
    this->name_ROI = x.get_name();
    this->dim_ROI = x.get_dim();
    delete this->ROIs;
    this->ROIs = build_ROI(dim_ROI);
    
    return *this;
  }
*/
};

class Cam_PERSEE_Listener { 
private:
  ///Mode debug ou pas
  unsigned char debug;

  ///acquisition par la NI ?
  unsigned char useNIacq;
  unsigned char useNItrig;

  ///Temps de rafraichissement des HKs
  unsigned int updatetimeHK;

  ///Chemin du port commniquant avec la carte HK
  string pDeviceHK;
  ///Chemin du port commniquant avec la carte Pautron
  string pDevicePautron;
  string pDeviceData;
  ///Vitesse du port commniquant avec la carte Pautron
  int pDeviceDataSpeed;
  ///Chemin du port commniquant avec la carte Zigtron
  string pDeviceZigtron;

  ///Port commniquant avec la carte HK
  Com_PERSEE *pCom_HK;
  ///Port commniquant avec la carte Pautron
  Com_PERSEE *pCom_Pautron;
  Com_PERSEE *pCom_Data;
  ///Port commniquant avec la carte Zigtron
  Com_PERSEE *pCom_Zigtron;

  ///Trame contenant les derniers HKs, elle sert de modele a la trame de donnees
  Frame_PERSEE *pHK;
  ///Buffer Circulaire contenant les nbframes dernieres trames de donnees
  Frame_PERSEE **pCircularBuffer;
  
  ///Taille du buffer circulaire
  unsigned int nbframes;
  ///Position actuel dans le buffer circulaire
  unsigned int index;
  ///Suffix des fichiers d'acquisition
  string storageSuffix;
  ///Emplacement de stockage des fichiers d'acquisition
  string storageDIR;
  unsigned char stopafter1stfits;

  ///Permet de savoir si la camera ecoute le port de la carte HK
  bool listening_HK;
  ///Permet de savoir si la camera ecoute le port de la carte Zygtron
  bool listening_Data;
  ///Permet de savoir si la camera trigg la carte Zygtron
  bool triggering;

  vector<ROI_def*> ROI_tab;
  ///nombre de regions d'interet dans l'image
  //unsigned short int nb_ROI;
  ///Regions d'interet de l'image
  //ROI **ROIs;

  ///Nom des senseurs du banc (HK)
  string name_HK[HK_SIZE];
  ///Nom des regions d'interet de l'image
  //string *name_ROI;
  ///Dimension des regions d'interet de l'image
  //string *dim_ROI;

  ///Nombre d'images sommees
  unsigned short int sommation;
  ///Nombre de division d'horloge utilise
  unsigned short int division;
  ///Temps d'exposition entre de lecture de cible
  unsigned int exposure;
  ///Temps de latence avant la lecture
  unsigned int latency;
  ///Nombre de lecture du pixel par boucle
  unsigned short int nb_read;
  ///Nombre de lecture 
  unsigned short int nb_loop;
  ///Vbias
  unsigned int vbias;
  ///Vreset 
  unsigned int vreset;
  ///Vref
  unsigned int vref;
  ///Vref
  unsigned int triggerTime;
  ///up
  unsigned short int top;
  ///left
  unsigned short int left;

  ///Save all frames or not
  unsigned char save_all;

  ///Check data
  bool checkData;

  ///Current position of the filter
  unsigned short current_filter;

  ///Client FTP pour envoyer les fits sur un serveur distant
  Client_FTP *FTP_client;

  ///FTP user
  string FTP_user;

  ///FTP Passwd
  string FTP_passwd;

  ///FTP host
  string FTP_host;

  ///FTP dir
  string FTP_dir;

public:
  /**
   * Permet de creer le listenner a partir d'un fichier de configuration
   * @param configFile Emplacement du fichier de configuration
   */
  Cam_PERSEE_Listener(const char *configFile);

  ~Cam_PERSEE_Listener();
  
  int getFrames(Frame_PERSEE **tabFrames, int nbFrames);

  /*
   * THEO - This is a cutdown version of getFrames used for JOUFLU
   * Time out is (we think) in milliseconds.
   * Routine will attempt to get as many frames as possible uup to nbFrames
   * until it times out.
   * will return the number of frames it really gets or 0 if none.
   */

  int jouflu_getFrames(Frame_PERSEE **tabFrames, int nbFrames, unsigned int time_out);

  int jouflu_call_clearftdi(void);

  /**
   * Permet de lancer l'ecoute des differents ports
    */
  void start_listen();

  /**
   * Permet de stoper l'ecoute des differents ports
   */
  void stop_listen();

  static void *start_triggering(void *camera_arg);

  /**
   * Permet de lancer l'ecoute de la carte HK
    */
  void start_trigger(unsigned int triggerTime);

  /**
   * Permet de stoper l'ecoute de la carte HK
   */
  void stop_trigger();

  static void *start_listening_HK(void *camera_arg);

  /**
   * Permet de lancer l'ecoute de la carte HK
    */
  void start_listen_HK();

  /**
   * Permet de stoper l'ecoute de la carte HK
   */
  void stop_listen_HK();

  static void *start_listening_Data(void *camera_arg);

  /**
   * Permet de lancer l'ecoute de la carte Zigtron
    */
  void start_listen_Data();

  /**
   * Permet de stoper l'ecoute de la carte Zigtron
   */
  void stop_listen_Data();

  /**
   * Demande l'arret et attend le fin de l'ecoute de la carte Zigtron
   */
  void stop_listen_Data_safe();

  unsigned int get_nb_pix();
  unsigned int get_nb_read() {return nb_read; };
  unsigned int get_nb_loop() {return nb_loop; };

  /**
   * Permet de connaitre le suffix utilise pour la creation des fits
   * @return Le suffix utilise
   */
  string get_suffix()                  { return storageSuffix;          };

  /**
   * Permet de connaitre l'emplacement utilise pour la creation des fits
   * @return L'emplacement utilise
   */
  string get_directory()               { return storageDIR;             };

  /**
   * Permet de definir le suffix utilise pour la creation des fits
   * @param suffix Le suffix a utiliser
   */
  void set_suffix(string suffix);
 
  /**
   * Permet de definir l'emplacement utilise pour la creation des fits
   * @param directory L'emplacement a utiliser
   */
 void set_directory(string directory);

  /**
   * Permet de connaitre le nombre d'image par fits
   * @return Le nombre d'image par fits
   */
  unsigned int get_nbframes()          { return nbframes;               };

  /**
   * Permet de definir le nombre d'image par fits
   * @param nbframes Le nombre d'image par fits a utiliser
   */
  void set_nbframes(unsigned int nbframes);

  /**
   * Permet de connaitre 
   * @return 
   */
  unsigned char get_stop1fit()          { return stopafter1stfits;   };

  /**
   * Permet de definir 
   * @param 
   */
  void set_stop1fit(unsigned char stop1fit);

  /**
   * Permet de connaitre le temps d'exposition
   * @return Exposition
   */
  unsigned int get_exposure()          { return exposure;               };

  int set_sommation(unsigned char sommation);
  int set_division(unsigned char division);
  int set_nbloop(unsigned char nb_loop);
  int set_bias(unsigned char bias, unsigned voltage);
  int set_nbread(unsigned char nb_read);
  /**
   * Permet de connaitre le temps d'exposition
   * @return Exposition
   */
  unsigned int get_parameters(int param [12]){ 
      param[0] = exposure;
      param[1] = nb_read;
      param[2] = sommation;
      param[3] = division;
      param[4] = nb_loop;
      param[5] = vbias;
      param[6] = vreset;
      param[7] = vref;
      param[8] = nbframes;
      param[9] = current_filter;
      param[10] = save_all;
      param[11] = stopafter1stfits;

      return 0;
  };

  /**
   * Permet de definir le temps d'exposition
   * @param exposure le temps d'exposition
   */
  int set_exposure(unsigned int exposure);

  /**
   * Permet de savoir si le serveur fonctionne
   * @return Vrai ou faux
   */
  bool is_working()        { return ( listening_HK || listening_Data ); };

  /**
   * Permet de savoir si le serveur ecoute la carte HK
   * @return Vrai ou faux
   */
  bool is_listening_HK()   { return listening_HK;                       };

  /**
   * Permet de savoir si le serveur ecoute la carte Zigtron
   * @return Vrai ou faux
   */
  bool is_listening_Data() { return listening_Data;                     };

  /**
   * Permet de changer de banque des fenetres
   * @param num_bank Numero de la banque a selectionner
   */
  int set_bank(unsigned char bank);

  /**
   * Permet de changer de banque des fenetres
   * @param num_bank Numero de la banque a selectionner
   */
  int set_ROI(string name_ROI, unsigned short x, unsigned short y, unsigned short sizex, unsigned short sizey, bool sendCmd);

  /**
   * Permet d'initialiser la camera : r�init, freerun, choix de b0, init de fenetres
   */
  int init_cam();

  /**
   * Permet de r�initialiser les ROIs
   */
  int reset_cam();

  /**
   * Permet d'optimiser la lecture de la cible
   */
  int optimize_read();

  /**
   * Permet de sauver toutes les frames ou non
   */
  int save_all_frames(unsigned char save_all);
  
  int set_filter(unsigned short position );
  int set_filter_home( );
  int save_filter(unsigned short position );
  int move_filter(unsigned short position );
  int get_position( );
  int read_ADC(unsigned short position );
  int get_version( );
  int set_cumul(unsigned short cumul );
  int nb_HK(unsigned short nb_HK );
  int set_motors(unsigned short enable );

  /**
   * FTP Connection
   */
  int store_at( string FTP_user, string FTP_passwd, string FTP_host, string FTP_dir ){
    this->FTP_user=FTP_user;
    this->FTP_passwd=FTP_passwd;
    this->FTP_host=FTP_host;
    this->FTP_dir=FTP_dir;
    this->FTP_client = new Client_FTP(FTP_host);

    if(FTP_client->accueil()<0){
      delete FTP_client;
      FTP_client=0L;
      return EXIT_FAILURE;
    }

    if(FTP_client->identification(FTP_user, FTP_passwd)<0){
      delete FTP_client;
      FTP_client=0L;
      return EXIT_FAILURE;
    }

    FTP_client->init();
    return EXIT_SUCCESS;
  }
  void no_store(){
    this->FTP_user="";
    this->FTP_passwd="";
    this->FTP_host="";
    this->FTP_dir="";
    delete FTP_client;
    FTP_client=0L;
  }
  int is_connected() {
    return this->FTP_host.length();
  }


protected :

	  /**
	   * Permet de trigger la Zigtron
	   */
	  void trigg();

	  /**
	   * Permet de recevoir un trame de HK
	   */
	  void receiveHK();

  /**
   * Permet de recevoir un trame de donnees
   */
  void receiveFrames();

  /**
   * Permet d'ecrire un fits a partir d'un tableau de trames PERSEE
   * @param filename Emplacement ou doit etre ecrit le fits
   * @param Frames Tableau de trames PERSEE
   * @param nb_prises Taille du tableau
   * @return Code d'erreur : 0 si ok?
   */
  int writeFits(char *filename, Frame_PERSEE **Frames, unsigned int nb_prises);

  /**
   * Permet de configurer les regions d'interet de l'image
   * @param nameROI Chaine des noms des regions d'interet (separe par des ;)
   * @param dimROI Chaine des dimensions des regions d'interet (separe par des ;)
   * @return nombre de ROIs
   */
  int configure_ROIs(const char *nameROI, const char *dimROI);

};

#endif

