/***************************************************************************
 *   Copyright (C) 2007 by Arnaud Sevin
 *   Arnaud.Sevin@obspm.fr
 * 
 *   With the help of Eric Gendron, Zoltan Hubert, Denis Peret 
 *
 * Copyright (c) 1998, Regents of the University of California
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/
#ifndef CAM_PERSEE_SIMULATOR_H
#define CAM_PERSEE_SIMULATOR_H

#include "com_persee.h"

/***************************************************
 *                                                 *
 *          Classe Cam_PERSEE_Simulator            *
 *                                                 *
 ***************************************************/
/**
 * @brief Classe permettant de simuler l'envoie de données PERSEE
 * @author   Arnaud Sevin <Arnaud.Sevin@obspm.fr>
 * @version  1.0
 * @date     02/06/2008
 */
class Cam_PERSEE_Simulator : public Com_PERSEE{

private:
  ///Permet de savoir si le programme envoie ou pas des données
  bool working;

  ///Permet de tager les trames
  unsigned char c1,c2,c3;

public:
  /**
   * Permet de configurer la communication avec PERSEE
   * @param pDevice Port série de communication
   */
  Cam_PERSEE_Simulator(string pDevice);

  ~Cam_PERSEE_Simulator(){};

  /**
   * Permet de lancer un thead
   * @param PERSEE_Simu lui-même (this)
   */
  static void *start_sending(void *PERSEE_Simu);

  /**
   * Permet de savoir si le programme envoie ou pas des données
   * @return Vrai ou faux
   */
  bool is_working() {return working;}

  /**
   * Permet de lancer l'envoi
   */
  void start_send();
  /**
   * Permet d'arreter l'envoi
   */
  void stop_send();

private:
  /**
   * Permet d'envoyer un trame PERSEE
   */
  void sendFrame();

  /**
   * Permet d'écrire une trame de HKs
   * @param buffer Chaine de caractères dans laquelle on va écrire la trame
   */
  void write_hk( char *buffer );

  /**
   * Permet d'écrire une trame de données
   * @param acqui  définit si la trame est une trame d'acquisition ou pas
   * @param nb_pix nombre de pixels de la trame
   * @param buffer Chaine de caractères dans laquelle on va écrire la trame
   */
  void write_data(bool acqui, int nb_pix, char *buffer);
};

#endif
