////////////////////////////////////////////////////////////////////////////////
//////++++++//////++//////++//////++++++//++//////++//++++++++//////////////////
//////++////++////++//////++////++////////++//////++//++////////////////////////
//////++//////++//++//////++//++//////////++++++++++//++++++////////////////////
//////++//////++//++//////++//++//////////++//////++//++////////////////////////
//////++////++////++//////++////++////////++//////++//++////////////////////////
//////++++++////////++++++////////++++++//++//////++//++++++++// Production ////
////////////////////////////////////////////////////////////////////////////////
//                                                                            //
//      Ecrit par Duche.                                                      //
//      Contact: le_grand_duche@hotmail.com                                   //
//                                                                            //
//      Date de dernière modification : 23/12/2005                            //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////
//  Moment.h  //////////////////////////////////////////////////////////////////

#ifndef _MOMENTS_H
#define _MOMENTS_H

////////////////////////////////////////////////////////////////////////////////
//  FICHIERS JOINTS  ///////////////////////////////////////////////////////////

#include <sys/timeb.h>

////////////////////////////////////////////////////////////////////////////////
//  CONSTANTES  ////////////////////////////////////////////////////////////////

#define YEAR_IN_MILI 3.1556952e10
#define YEAR_IN_SEC 31556952
#define YEAR_IN_MIN 525949.2
#define YEAR_IN_HOUR 8765.82
#define YEAR_IN_DAY  365.2425
#define YEAR_IN_MONTH 12
#define MONTH_IN_MILI 2.629746e10
#define MONTH_IN_SEC 2629746
#define MONTH_IN_MIN 43829.1
#define MONTH_IN_HOUR 730.485
#define MONTH_IN_DAY 30.436875
#define DAY_IN_MILI 86400000
#define DAY_IN_SEC 86400
#define DAY_IN_MIN 1440
#define DAY_IN_HOUR 24
#define HOUR_IN_MILI 3600000
#define HOUR_IN_SEC 3600
#define HOUR_IN_MIN 60
#define MIN_IN_MILI 60000
#define MIN_IN_SEC 60
#define SEC_IN_MILI 1000

#define START 2
#define STOP 1
#define INIT 0

#define ALL -1

////////////////////////////////////////////////////////////////////////////////
//  STRUCTURES UTILISÉE  ///////////////////////////////////////////////////////

class Moment;
class Chrono;

////////////////////////////////////////////////////////////////////////////////
//  CLASSE MOMENT  /////////////////////////////////////////////////////////////

/**
 * @brief Classe permettant de gérer facilement les dates
 * @author   Duche <le_grand_duche@hotmail.com>
 * @version  1.0
 * @date     23/12/2005
 */
class Moment
{
 public:
  Moment() {update();}
  ~Moment() {}

  ///Renvoie l'année
  int year();
  ///Renvoie le mois
  int month();
  ///Renvoie le jour
  int day();
  ///Renvoie le jour de la semaine (0 pour dimanche)
  int wday();
  ///Renvoie l'heure
  int hour();
  ///Renvoie la minute
  int min();
  ///Renvoie la seconde
  int sec();
  ///Renvoie le millième de seconde (approximatif...)
  int milli();
  ///Renvoie le nombre de seconde depuis 01/01/1970
  int realValueTime();
  ///Renvoie le nombre de millièmes associés à realValueTime()
  int realValueMilliTm();

  ///Actualise
  void update();
		
  ///Opérateur "strictement plus petit que"
  bool operator < (const Moment&);
  ///Opérateur "strictement plus grand que"
  bool operator > (const Moment&);
  ///Opérateur "plus petit ou égal"
  bool operator <= (const Moment&);
  ///Opérateur "plus grand ou égal"
  bool operator >= (const Moment&);
  ///Ajoute des millième de seconde
  Moment operator + (int);
  ///Soustrait des millièmes de seconde
  Moment operator - (int);
	
  ///Différence en millièmes de seconde
  friend double operator - (const Moment&,const Moment&);
				
 private:
  int Year;  ///Année
  int Month;  ///Mois
  int Day;  ///Jour
  int Wday;  ///Jour de la semaine
  int Hour;  ///Heure
  int Min;  ///Minute
  int Sec;  ///Seconde
  int Milli;  ///Millième de seconde (relativement approximatif...)
  timeb RealValue;
  void transf();
};

////////////////////////////////////////////////////////////////////////////////
//  CLASSE CHRONO  /////////////////////////////////////////////////////////////

class Chrono
{
 public:
  Chrono(int = 1);  //On lui fournit le nombre de chrono parallèles
  ~Chrono();
  int hour(int = 0);  //On lui fournit l'indice (comme si on avait un vecteur de chronos
  int min(int = 0);  //Idem
  int sec(int = 0);  //Idem
  int milli(int = 0);  //Idem
  int state(int = 0);  //Idem
  
  void reinit(int = 0,int = 0);  //Idem et réinit avec une valeur de départ (nulle par défaut)
  void restart(int = 0);  //Idem
  void update(int = 0);  //Idem
  void stop(int = 0);  //Idem
  
 private:
  int* Milli;
  int* Sec;
  int* Min;
  int* Hour;
  Moment* FirstStartValue;
  int* EffectValue;
  int* State;
  int Nbr;
};

////////////////////////////////////////////////////////////////////////////////
//  FONCTIONS UTILITAIRES DU FICHIER  //////////////////////////////////////////

void TimePause(int,int,int,int);

////////////////////////////////////////////////////////////////////////////////

#endif
