/***************************************************************************
 *   Copyright (C) 2007 by Arnaud Sevin
 *   Arnaud.Sevin@obspm.fr
 * 
 * Copyright (c) 1998, Regents of the University of California
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include "cam_persee_listener.h"
#if USE_NI==1
#include <NIDAQmx.h>
#endif
#include <fstream>

/***************************************************
 *                                                 *
 *          Classe Cam_PERSEE_Listener             *
 *                                                 *
 ***************************************************/
Cam_PERSEE_Listener::Cam_PERSEE_Listener(const char *configFile) {
	/************************
	 *   Initialisation     *
	 ************************/
	pHK = new Frame_PERSEE(0, 1, 1, false);

	/************************
	 * Lecture de la config *
	 ************************/
	//cerr << sizeof(int) << endl;
	ConfigFile cf(configFile);
	try {
		this->debug = cf.ValueI("general", "debug");

		/*************************
		 * Def. des ports de com *
		 *************************/
		pDeviceHK = cf.ValueS("HK", "device");
		pDeviceData = cf.ValueS("Data", "device");
		pDeviceDataSpeed = cf.ValueI("Data", "devicespeed");
		pDevicePautron = cf.ValueS("Data", "pautron");
		pDeviceZigtron = cf.ValueS("Data", "zigtron");

		useNIacq = cf.ValueI("Data", "useNIacq");
		useNItrig = cf.ValueI("Data", "useNItrig");

    //char* serial_list[4]={"FTDZVAS2","FTDW2MMW","FTCVHJ29","FTCVMXUA"};
    //char* board_list[4]={"PAUTRON","ZIG_CTR","ZIG_DATA","HK_BOARD"};

    //BPS921600
		this->pCom_HK = new Com_PERSEE(pDeviceHK);
		if (debug)
			cout << "HK      : " << pCom_HK->AfficherMessageConfigurer();
		if (!useNIacq) {
			this->pCom_Data = new Com_PERSEE(pDeviceData, pDeviceDataSpeed);
			if (debug)
				cout << "Data    : " << pCom_Data->AfficherMessageConfigurer();
		}
		this->pCom_Pautron = new Com_PERSEE(pDevicePautron);
		if (debug)
			cout << "Pautron : " << pCom_Pautron->AfficherMessageConfigurer();
		this->pCom_Zigtron = new Com_PERSEE(pDeviceZigtron);
		if (debug)
			cout << "Zigtron : " << pCom_Zigtron->AfficherMessageConfigurer();

		listening_HK = false;
		listening_Data = false;

		this->current_filter=0;

		/***************************
		 * Def. des param de l'img *
		 ***************************/
		this->sommation = cf.ValueI("Data", "summation");
		this->division = cf.ValueI("Data", "division");
		this->nbframes = cf.ValueI("Data", "nbframes");
		this->storageDIR = cf.ValueS("Data", "directory");
		this->storageSuffix = cf.ValueS("Data", "suffix");
		this->vbias = cf.ValueI("Data", "Vbias");
		this->vreset = cf.ValueI("Data", "Vreset");
		this->vref = cf.ValueI("Data", "Vref");
		this->stopafter1stfits = cf.ValueI("Data", "stop1fit");
		this->checkData = cf.ValueI("Data", "checkData");

		/**************************
		 * Def. des param des HKs *
		 **************************/
		this->updatetimeHK = cf.ValueI("HK", "updatetime") * 1000;
		char hisname[5];
		for (int i = 0; i < HK_SIZE; i++) {
			sprintf(hisname, "HK%02d", i + 1);
			this->name_HK[i] = cf.ValueS("HK", hisname);
		}

		/*****************************
		 * Def. des param de lecture *
		 *****************************/
		this->nb_loop = cf.ValueI("Data", "nb_loop");
		this->nb_read = cf.ValueI("Data", "nb_read");
		this->exposure = cf.ValueI("Data", "exposure");
		this->latency = cf.ValueI("Data", "latency");

		/************************
		 * Creation des ROIs    *
		 ************************/
		//ROI_tab.clear();
		configure_ROIs(cf.ValueS("Data", "NAME").c_str(), cf.ValueS("Data",
				"ROI").c_str());

		if (debug) {
			cout << "nb_ROI  : " << ROI_tab.size() << " with " << get_nb_pix()
					<< " pixels\n";
		}

		save_all = cf.ValueI("Data", "saveall");

		triggering = 0;
	} catch (string msg) {
		cerr << "ERREUR (Server_PERSEE): " << msg << endl;
	}
}

Cam_PERSEE_Listener::~Cam_PERSEE_Listener() {

	delete this->pHK;

	/************************
	 * Effacement des ROIs  *
	 ************************/
	//while(!ROI_tab.empty()) 
	//    ROI_tab.erase(ROI_tab.end());
	//  ROI_tab.pop_back();
	ROI_tab.clear();

	/************************
	 * Effacement des ports *
	 ************************/
	delete this->pCom_HK;
	delete this->pCom_Data;
	delete this->pCom_Pautron;
	delete this->pCom_Zigtron;

}

int Cam_PERSEE_Listener::configure_ROIs(const char *nameROI, const char *dimROI) {
	char buffer[128];
	char *pointeur;
	int i = 0, j = 0;
	vector<string> name_ROI;
	vector<string> dim_ROI;

	/**********************************************
	 * Decomposition du tableau des noms des ROIs *
	 **********************************************/
	strcpy(buffer, nameROI);
	i = 0;
	pointeur = strtok(buffer, ";");
	while (pointeur != NULL) {
		// Cherche les autres separateur
		name_ROI.push_back(pointeur);
		i++;
		pointeur = strtok(NULL, ";");
	}

	/**********************************************
	 * Decomposition du tableau des ROIS en ROIs  *
	 **********************************************/
	strcpy(buffer, dimROI);
	j = 0;
	pointeur = strtok(buffer, ";");
	while (pointeur != NULL) {
		// Cherche les autres separateur
		dim_ROI.push_back(pointeur);
		j++;
		pointeur = strtok(NULL, ";");
	}

	if (i != j)
		cerr << "toutes les ROIs ne sont pas decrites\n";
	//if(pointeur != NULL) cerr << "nb_ROI est trop petit\n";
	int nb_ROI = i;

	/***********************************************
	 * Decomposition des ROIS en tableau d'entiers *
	 ***********************************************/
	//short unsigned int tab[4];
	for (int j = 0; j < nb_ROI; j++) {
		cerr << "store num" << j << " : " << name_ROI[j] << " dim : "
				<< dim_ROI[j] << endl;
		short unsigned int tab[4];
		int i = 0;
		char buffer[128];
		char *pointeur;
		strcpy(buffer, dim_ROI[j].c_str());
		pointeur = strtok(buffer, ",");
		while ((pointeur != NULL) && (i < 4)) {
			// Cherche les autres separateur
			tab[i] = atoi(pointeur);
			i++;
			pointeur = strtok(NULL, ",");
		}

		set_ROI(name_ROI[j], tab[0], tab[1], tab[2], tab[3], false);
	}

	return nb_ROI;
}

int Cam_PERSEE_Listener::set_ROI(string name_ROI, unsigned short x,
		unsigned short y, unsigned short sizex, unsigned short sizey,
		bool sendCmd) {
	unsigned short int tab[4] = { x, y, sizex, sizey };
	ROI_def *zROI = new ROI_def(name_ROI, tab);
	ROI_tab.push_back(zROI);

	if (sendCmd) {
		/* Extraction de sous-ROIs carr�es */
		int tmpInt = zROI->get_ROI()->get_nb_ROIs();
		short unsigned int **tmpTabInt = zROI->get_ROI()->get_ROIs();
		//cout << tmpInt << " sous-ROIs\n";

		/** envoi des commandes � l'electronique **/

		for (int i = 0; i < tmpInt; i++) {
			cerr << tmpTabInt[i][0] << " " << tmpTabInt[i][1] << " "
					<< tmpTabInt[i][2] << "\n";
			pCom_Pautron->Create_window(tmpTabInt[i][0], tmpTabInt[i][1],
					tmpTabInt[i][2], TIMEOUT_ANSWER);
		}
		/**                                      **/
	}
	return ROI_tab.size();
}

/**
 \fn Cam_PERSEE_Listener::start_listen()
 */
void Cam_PERSEE_Listener::start_listen() {
	start_listen_HK();
	start_listen_Data();
}

void Cam_PERSEE_Listener::stop_listen() {
	stop_listen_HK();
	stop_listen_Data();
}

/**
 \fn Cam_PERSEE_Listener::start_trigger(unsigned int triggerTime)
 */
void Cam_PERSEE_Listener::start_trigger(unsigned int triggerTime) {
	if (pCom_Zigtron->get_status()<0) {
		cerr << "Port de communication Zigtron ferme\n";
		return;
	}
	triggering = true;
	this->triggerTime = triggerTime;
	pthread_t triggering_loop;
	pthread_create(&triggering_loop, NULL,
			Cam_PERSEE_Listener::start_triggering, this);
}

/**
 \fn Cam_PERSEE_Listener::start_triggering(void *camera_arg)
 */
void *Cam_PERSEE_Listener::start_triggering(void *camera_arg) {
	Cam_PERSEE_Listener *camera = (Cam_PERSEE_Listener*) camera_arg;
	camera->trigg();
	return (void*) 0L;
}

void Cam_PERSEE_Listener::stop_trigger() {
	triggering = false;
    usleep(500000);
}

void Cam_PERSEE_Listener::trigg() {
	pCom_Zigtron->Set_trigg(1, TIMEOUT_ANSWER);
	pCom_Zigtron->Set_pause(2, TIMEOUT_ANSWER);
	while (triggering) {
		pCom_Zigtron->trigg(TIMEOUT_ANSWER);
		usleep(triggerTime);
	}
	pCom_Zigtron->Set_trigg(0, TIMEOUT_ANSWER);
}

/**
 \fn Cam_PERSEE_Listener::start_listen_HK()
 */
void Cam_PERSEE_Listener::start_listen_HK() {
	if (pCom_HK->get_status()<0) {
		cerr << "Port de communication HK ferme\n";
		return;
	}
        if(listening_HK) return;
	listening_HK = true;
	pthread_t listening_loop;
	pthread_create(&listening_loop, NULL,
			Cam_PERSEE_Listener::start_listening_HK, this);
}

/**
 \fn Cam_PERSEE_Listener::start_listening_HK(void *camera_arg)
 */
void *Cam_PERSEE_Listener::start_listening_HK(void *camera_arg) {
	Cam_PERSEE_Listener *camera = (Cam_PERSEE_Listener*) camera_arg;
	camera->receiveHK();
	return (void*) 0L;
}

void Cam_PERSEE_Listener::stop_listen_HK() {
	listening_HK = false;
}

void Cam_PERSEE_Listener::receiveHK() {

	struct timeval TimeRef, Time;
	unsigned int Dif = 0, fps = 0;
	time_t rawtime;
	struct tm * timeinfo;
	char buffer[80];
	char filename[128];

	gettimeofday(&TimeRef, NULL);

	int bytesRecv = 0;
	cout << "Start listen HK" << endl;

	time(&rawtime);
	timeinfo = localtime(&rawtime);
	strftime(buffer, 80, "%Y%m%d_%H%M%S_HK.txt", timeinfo);
	sprintf(filename, "%s/%s", storageDIR.c_str(), buffer);
	ofstream fichier(filename, ios::out | ios::trunc);  // ouverture en ecriture avec effacement du fichier ouvert
	if(!fichier) cerr << "" <<endl;

	while (listening_HK) {

		bytesRecv = pCom_HK->RecevoirHK(pHK, TIMEOUT_ANSWER);

		if (bytesRecv >= 0) {
			if (debug) {
				gettimeofday(&Time, NULL);
				Dif = (Time.tv_sec - TimeRef.tv_sec) * 1000000 + (Time.tv_usec
						- TimeRef.tv_usec);
				fps++;

				if (Dif >= 1000000) { //1sec
					cout << "Ecoute HK : " << fps << " frames in " << Dif
							/ 1000000. << "s (" << fps * 1000000 / (double) Dif
							<< " fps )" << endl;
					fps = 0;
					gettimeofday(&TimeRef, NULL);
				}
			}
		} else if (debug && (bytesRecv == TIMEOUT)) {
			cerr << "WARNING : HK acquisition time out" << endl;
		} else if (bytesRecv == ERREUR) {
			cerr << "ERROR : bad news..." << endl;
			listening_HK = false;
		}
		
		if (fichier) {
			time(&rawtime);
			timeinfo = localtime(&rawtime);
			strftime(buffer, 80, "%d%m%Y %H:%M:%S", timeinfo);
			fichier << buffer;
			for (int i = 0; i < HK_SIZE; i++)
				fichier << "\t" << pHK->HK[i];
			fichier << endl;
		}
		usleep(updatetimeHK);
	}
	if (fichier)
		fichier.close(); // on ferme le fichier
}

/**
 \fn Cam_PERSEE_Listener::start_listen_Data()
 */
void Cam_PERSEE_Listener::start_listen_Data() {
	if ((useNIacq == 0) && pCom_Data->get_status()<0) {
		cerr << "Port de communication Data ferme\n";
		return;
	}
	if(listening_Data) return;
	listening_Data = true;
	pthread_t listening_loop;
	pthread_create(&listening_loop, NULL,
			Cam_PERSEE_Listener::start_listening_Data, this);
}

/**
 \fn Cam_PERSEE_Listener::start_listening_Data(void *camera_arg)
 */
void *Cam_PERSEE_Listener::start_listening_Data(void *camera_arg) {
	Cam_PERSEE_Listener *camera = (Cam_PERSEE_Listener*) camera_arg;
	camera->receiveFrames();
	return (void*) 0L;
}

void Cam_PERSEE_Listener::stop_listen_Data() {
	listening_Data = false;
}

void Cam_PERSEE_Listener::stop_listen_Data_safe() {
	listening_Data = false;
}

#if USE_NI==1
//cf NIacquire.cpp
TaskHandle NIinitCard(uInt64 sampleToAcquire);
int NIRecevoirTrame(TaskHandle taskHandle, Frame_PERSEE* frame);
int NIstopCard(TaskHandle taskHandle);
#endif

int Cam_PERSEE_Listener::getFrames(Frame_PERSEE **tabFrames, int nbFrames) {
	Moment m;
	struct timeval TimeRef, Time;
	unsigned int Dif = 0, fps = 0;
	int num_timeouts = 0;
#if USE_NI==1
	TaskHandle taskHandle = 0L;

	start_listen_HK();

	//Definition du nombre de pixels a acquerir
	if (useNIacq)
		taskHandle = NIinitCard(tabFrames[0]->nb_pix);
	else {
#endif
		pCom_Zigtron->Set_nbpix(tabFrames[0]->nb_pix, TIMEOUT_ANSWER);
#if USE_NI==1
	}
#endif
	gettimeofday(&TimeRef, NULL);

	int bytesRecv = 0;
	int index = 0;

	
	//cerr << "triggering = " << triggering << endl;
	if(!triggering) {
	  pCom_Zigtron->trigg(TIMEOUT_ANSWER);
	}

	listening_Data=1;
	while (listening_Data) {
		memcpy(tabFrames[index]->HK, pHK->HK, HK_SIZE * sizeof(int));
#if USE_NI==1
		if (useNIacq)
			bytesRecv = NIRecevoirTrame(taskHandle, tabFrames[index]);
		else
#endif
			bytesRecv = pCom_Data->RecevoirTrame(tabFrames[index],
					TIMEOUT_READ);
		cerr << bytesRecv << endl;

		if (bytesRecv > 1) {
			gettimeofday(&Time, NULL);
			Dif = (Time.tv_sec - TimeRef.tv_sec) * 1000000 + (Time.tv_usec
					- TimeRef.tv_usec);
			fps++;

			if (Dif >= 1000000) { //1sec
				if (debug)
					cout << "getFrames: Acquiring Data: " << fps << " frames in " << Dif
							/ 1000000. << "s (" << fps * 1000000 / (double) Dif
							<< " fps )" << endl;
				fps = 0;
				gettimeofday(&TimeRef, NULL);
			}

			if (save_all || tabFrames[index]->acqui) {
				//if(debug) cout << "Acqui " << index << " " << tabFrames[index]->donnees[nb_pix-1] << endl;
				index++;
				if (index == nbFrames) {
				  listening_Data = false;
				  return index;
				}
			} else if(debug) 
			  cout << "Waiting Data..." << endl;

		} else if (debug && (bytesRecv == RIEN_RECU)) {
			// ben en fait, on fait rien...
			cerr << "GetDATA: Warning : Data acquisition error, frame skiped" << endl;
		} else if (debug && (bytesRecv == TIMEOUT)) {
			cerr << "GetDATA: Warning : Data acquisition time out" << endl;
			++num_timeouts;
			if (num_timeouts > 2) {
				cerr << "Too many time outs." << endl;
				return -1;
			}
		} else if (bytesRecv == ERREUR) {
			cerr << "ERROR : bad news..." << endl;
			listening_Data = false;
		}
	}
	if (index != 0) {
	  cerr << "We did not get all the frames we wanted.... returning anyway after " << index << " frames." << endl;
	  listening_Data = false;
	  return index;
	}
#if USE_NI==1
	if (useNIacq)
		NIstopCard(taskHandle);
#endif
	if(!triggering) {
	  pCom_Zigtron->triggOff(TIMEOUT_ANSWER);
	}

	cerr << "We should not be getting here at all." << endl;
	return index;
}

/*
 * THEO
 * Here's our local version of getFrames.
 * Tries to get up to nbFrames of data until it times out. 
 * Returns number of frames it gets
 * Timeout is in milliseconds.
 * Since we don't use "NI" and "useNIacq" all that stuff has been removed.
 */

int Cam_PERSEE_Listener::jouflu_getFrames(Frame_PERSEE **tabFrames,int nbFrames,
		unsigned int time_out)
{
	Moment m;
	int bytesRecv = 0;
	int index = 0;

	// Definition du nombre de pixels a acquerir

	pCom_Zigtron->Set_nbpix(tabFrames[0]->nb_pix, TIMEOUT_ANSWER);
	
	// Not sure what we are triggering, I think nothing.

	if(!triggering) pCom_Zigtron->trigg(TIMEOUT_ANSWER);

	listening_Data=1;
	while (listening_Data)
	{
		/* Try to get a frame */

		memcpy(tabFrames[index]->HK, pHK->HK, HK_SIZE * sizeof(int));

		//bytesRecv = pCom_Data->RecevoirTrame(tabFrames[index], time_out);
		bytesRecv=pCom_Data->jouflu_RecevoirTrame(tabFrames[index], time_out);

		// cerr << bytesRecv << endl;

		/* What happened? */

		switch (bytesRecv)
		{
			case TIMEOUT: /* We timed out.... */
				listening_Data = false;
				return TIMEOUT;
				break;

			case ERREUR: /* Something bad happened.... */
				listening_Data = false;
				return -1;
				break;
			
			case RIEN_RECU: /* This happens a lot */
			        break;

			default: /* We got a frame */

				if (save_all || tabFrames[index]->acqui)
				{
					index++;
		
					/* Is that all the frames we asked for? */

					if (index == nbFrames)
					{
						listening_Data = false;
						return index;
					}
				}
				break;
		}
	}

	// Not sure what we are triggering, I think nothing.

	if(!triggering) pCom_Zigtron->triggOff(TIMEOUT_ANSWER);

	/* We should never reach here */

	return index;

} /* jouflu_getFrames(Frame_PERSEE **tabFrames,int nbFrames,unsigned int time_out) */

int Cam_PERSEE_Listener::jouflu_call_clearftdi(void)
{
	return pCom_Data->jouflu_clearftdi();
}

void Cam_PERSEE_Listener::receiveFrames() {
	Moment m;
	char filename[128];
	char localfilename[128];
	unsigned short nb_pix = get_nb_pix();
	//cerr << nb_pix << " " << nb_read << " " << nb_loop << endl;
	cerr << (int) stopafter1stfits << endl;
	pCircularBuffer = new Frame_PERSEE*[nbframes];
	for (unsigned int i = 0; i < nbframes; i++)
		pCircularBuffer[i] = new Frame_PERSEE(nb_pix, nb_read, nb_loop,
				checkData);
	//cerr << pCircularBuffer[0]->nb_pix << endl;

	struct timeval TimeRef, Time;
	unsigned int Dif = 0, fps = 0;
#if USE_NI==1
	TaskHandle taskHandle = 0L;

	//Definition du nombre de pixels a acquerir
	if (useNIacq)
		taskHandle = NIinitCard(pCircularBuffer[0]->nb_pix);
	else {
#endif
		pCom_Zigtron->Set_nbpix(pCircularBuffer[0]->nb_pix, TIMEOUT_ANSWER);
#if USE_NI==1
	}
#endif
	gettimeofday(&TimeRef, NULL);

	int bytesRecv = 0;
	index = 0;

	
	//cerr << "triggering = " << triggering << endl;
	if(!triggering) {
	  pCom_Zigtron->trigg(TIMEOUT_ANSWER);
	}

	while (listening_Data) {
		memcpy(pCircularBuffer[index]->HK, pHK->HK, HK_SIZE * sizeof(int));
#if USE_NI==1
		if (useNIacq)
			bytesRecv = NIRecevoirTrame(taskHandle, pCircularBuffer[index]);
		else
#endif
			bytesRecv = pCom_Data->RecevoirTrame(pCircularBuffer[index],
					TIMEOUT_READ);

		if (bytesRecv > 1) {
			gettimeofday(&Time, NULL);
			Dif = (Time.tv_sec - TimeRef.tv_sec) * 1000000 + (Time.tv_usec
					- TimeRef.tv_usec);
			fps++;

			if (Dif >= 1000000) { //1sec
				if (debug)
					cout << "Ecoute Data: " << fps << " frames in " << Dif
							/ 1000000. << "s (" << fps * 1000000 / (double) Dif
							<< " fps )" << endl;
				fps = 0;
				gettimeofday(&TimeRef, NULL);
				sprintf(filename, "%s/monitoring.fits", storageDIR.c_str());
				writeFits(filename, &pCircularBuffer[index], 1);

			}

			if (save_all || pCircularBuffer[index]->acqui) {
				//if(debug) cout << "Acqui " << index << " " << pCircularBuffer[index]->donnees[nb_pix-1] << endl;
				index++;
				if (index == nbframes) {
					m.update();
					sprintf(localfilename,
							"%s/%4d%02d%02d_%02d%02d%02d_%03d_%s.fits",
							storageDIR.c_str(), m.year(), m.month(), m.day(),
							m.hour(), m.min(), m.sec(), m.milli(),
							storageSuffix.c_str());
					if (debug)
						cout << "Create " << localfilename << endl;
					writeFits(localfilename, pCircularBuffer, nbframes);
					if(FTP_host.length()) {
					  sprintf(filename,
						  "%s/%4d%02d%02d_%02d%02d%02d_%03d_%s.fits",
						  FTP_dir.c_str(), m.year(), m.month(), m.day(),
						  m.hour(), m.min(), m.sec(), m.milli(),
						  storageSuffix.c_str());
					  if (debug)
					    cout << "Sending into " << filename << endl;
					  FTP_client->send_file(localfilename, filename);
					}
					index = 0;
					if (stopafter1stfits)
						listening_Data = false;
				}
			} else if (index != 0) {
				m.update();
				sprintf(localfilename, "%s/%4d%02d%02d_%02d%02d%02d_%03d_%s.fits",
						storageDIR.c_str(), m.year(), m.month(), m.day(),
						m.hour(), m.min(), m.sec(), m.milli(),
						storageSuffix.c_str());
				if (debug)
					cout << "Create " << localfilename << endl;
				writeFits(localfilename, pCircularBuffer, index);
				if(FTP_host.length()) {
				  sprintf(filename,
					  "%s/%4d%02d%02d_%02d%02d%02d_%03d_%s.fits",
					  FTP_dir.c_str(), m.year(), m.month(), m.day(),
					  m.hour(), m.min(), m.sec(), m.milli(),
					  storageSuffix.c_str());
				  if (debug)
				    cout << "Sending into " << filename << endl;
				  FTP_client->send_file(localfilename, filename);
				}

				index = 0;
				if (stopafter1stfits)
					listening_Data = false;
			}// else
			//if(debug) cout << "Monit" << endl;


		} else if (debug && (bytesRecv == RIEN_RECU)) {
			// ben en fait, on fait rien...
			cerr << "Warning : Data acquisition error, frame skiped" << endl;
		} else if (debug && (bytesRecv == TIMEOUT)) {
			cerr << "Warning : Data acquisition time out" << endl;
		} else if (bytesRecv == ERREUR) {
			cerr << "ERROR : bad news..." << endl;
			listening_Data = false;
		}
	}
	if (index != 0) {
		m.update();
		sprintf(localfilename, "%s/%4d%02d%02d_%02d%02d%02d_%03d_%s.fits",
				storageDIR.c_str(), m.year(), m.month(), m.day(), m.hour(),
				m.min(), m.sec(), m.milli(), storageSuffix.c_str());
		if (debug)
		  cout << "Create " << localfilename << endl;
		writeFits(localfilename, pCircularBuffer, index);
		if(FTP_host.length()) {
		  sprintf(filename,
			  "%s/%4d%02d%02d_%02d%02d%02d_%03d_%s.fits",
			  FTP_dir.c_str(), m.year(), m.month(), m.day(),
			  m.hour(), m.min(), m.sec(), m.milli(),
			  storageSuffix.c_str());
		  if (debug)
		    cout << "Sending into " << filename << endl;
		  FTP_client->send_file(localfilename, filename);
		}
		index = 0;
	}
#if USE_NI==1
	if (useNIacq)
		NIstopCard(taskHandle);
#endif
	if(!triggering) {
	  pCom_Zigtron->triggOff(TIMEOUT_ANSWER);
	}

	delete[] pCircularBuffer;
}

int Cam_PERSEE_Listener::writeFits(char *filename, Frame_PERSEE **Frames,
		unsigned int nb_prises) {
	// Create a FITS primary array containing a 2-D image
	// declare axis arrays.
	long naxis = (nb_prises == 1 ? 2 : 3); // si nb_prises=1 on cree une image sinon un cube
	//cerr <<  Frames[0]->nb_pix << endl;
	long naxes[3] = { Frames[0]->nb_pix, 1, nb_prises };
	unsigned long size = naxes[0] * naxes[1];

	int status = 0, bitpix = USHORT_IMG;

	/*
	 #####################
	 #  create FITS file #
	 #####################
	 */
	fitsfile *fptr;

	remove(filename);
	fits_create_file(&fptr, filename, &status);
	if (status)
		fprintf(stderr, "fits_create_file status = %d\n", status);

	/*
	 ############################
	 #  img definition keywords #
	 ############################
	 */
	if (fits_write_imghdr(fptr, bitpix, naxis, naxes, &status) > 0)
		fprintf(stderr, "ffphps status = %d\n", status);

	/*
	 ############################
	 #  write image properties  #
	 ############################
	 */
	char exp_name[12];
	strcpy(exp_name, storageSuffix.c_str());
	if (fits_write_key_str(fptr, "EXP_NAME", exp_name,
			"Name of the experience", &status) > 0)
		fprintf(stderr, "EXP_NAME keywords status = %d\n", status);
	if (fits_write_key_lng(fptr, "NBREAD", nb_read,
			"How many times the pixel has been read", &status) > 0)
		fprintf(stderr, "NBREAD keywords status = %d\n", status);
	if (fits_write_key_lng(fptr, "NBLOOP", nb_loop,
			"How many times the captor has been read", &status) > 0)
		fprintf(stderr, "NBLOOP keywords status = %d\n", status);
	if (fits_write_key_lng(fptr, "EXPO", exposure, "exposure in ms", &status)
			> 0)
		fprintf(stderr, "EXPO keywords status = %d\n", status);
	if (fits_write_key_lng(fptr, "SUM", sommation,
			"How many images we have summed", &status) > 0)
		fprintf(stderr, "SUM keywords status = %d\n", status);
	if (fits_write_key_lng(fptr, "CLOCKDIV", division, "clock division",
			&status) > 0)
		fprintf(stderr, "CLOCKDIV keywords status = %d\n", status);

	/*
	 ############################
	 #  write HKs               #
	 ############################
	 */
	char HKname[64];
	char HKindex[6];
	for (int i = 0; i < HK_SIZE; i++) {
		strcpy(HKname, name_HK[i].c_str());
		sprintf(HKindex, "HK%02d", i + 1);
		if (fits_write_key_lng(fptr, HKindex, Frames[0]->HK[i], HKname, &status)
				> 0)
			fprintf(stderr, "HK keywords status = %d\n", status);
	}

	/*
	 ############################
	 #  write ROIs              #
	 ############################
	 */
	char ROIname[64];
	char ROIindex[6];
	char ROIdim[16];
	if (fits_write_key_lng(fptr, "NB_ROI", ROI_tab.size(), "Number of ROIs",
			&status) > 0)
		fprintf(stderr, "NB_ROI keyword status = %d\n", status);
	for (unsigned int i = 0; i < ROI_tab.size(); i++) {
		strcpy(ROIname, ROI_tab[i]->get_name().c_str());
		sprintf(ROIindex, "ROI%02d", i + 1);
		strcpy(ROIdim, ROI_tab[i]->get_dim().c_str());
		if (fits_write_key_str(fptr, ROIindex, ROIdim, ROIname, &status) > 0)
			fprintf(stderr, "ROI keywords status = %d\n", status);
	}

	/*
	 ############################
	 #  write data              #
	 ############################
	 */
	/* define the null value (must do this before writing any data)*/
	if (fits_write_key_lng(fptr, "BLANK", -99,
			"value to use for undefined pixels", &status) > 0)
		fprintf(stderr, "BLANK keyword status = %d\n", status);

	long firstpix[3];

	for (unsigned int image = 0; image < nb_prises; image++) {
		firstpix[0] = 1;
		firstpix[1] = 1;
		firstpix[2] = image + 1;

		fits_write_pix(fptr, TUSHORT, firstpix, size, Frames[image]->donnees,
				&status);
		//delete (int *)Frames[image].ImageBuffer;
		//Frames[image].ImageBuffer = 0L;

	}
	//delete Frames;
	//Frames = 0L;

	fits_flush_file(fptr, &status); /* flush all data to the disk file */

	if (fits_close_file(fptr, &status) > 0)
		fprintf(stderr, "ERROR in ftclos (1) = %d", status);

	if (status > 0) {
		char tmp[256];
		fits_get_errstatus(status, tmp);
		fprintf(stderr, "%s\n", tmp);
		exit(1);
	}
	return 0;
}

void Cam_PERSEE_Listener::set_suffix(string suffix) {
	stop_listen_Data();
	this->storageSuffix = suffix;
	//start_listen_Data();
}

void Cam_PERSEE_Listener::set_directory(string directory) {
	stop_listen_Data();
	this->storageDIR = directory;
	//start_listen_Data();
}

void Cam_PERSEE_Listener::set_nbframes(unsigned int nbframes) {
	stop_listen_Data();
	this->nbframes = nbframes;
	//start_listen_Data();
}

int Cam_PERSEE_Listener::set_exposure(unsigned int exposure) {
	stop_listen_Data();
	this->exposure = exposure;
	//start_listen_Data();
	return pCom_Pautron->Set_exposure(exposure, TIMEOUT_ANSWER);
}

int Cam_PERSEE_Listener::set_bank(unsigned char bank) {
	if (pCom_Pautron->get_status()) {
		cerr << "Port de communication Data ferme\n";
		return 1;
	}
	return pCom_Pautron->Select_bank(bank, TIMEOUT_ANSWER);
}

int Cam_PERSEE_Listener::reset_cam() {
	/************************
	 * Effacement des ROIs  *
	 ************************/
	//while(!ROI_tab.empty()) 
	//ROI_tab.erase(ROI_tab.end());
	//ROI_tab.pop_back();
	ROI_tab.clear();
	return pCom_Pautron->Reset(TIMEOUT_ANSWER);
}

int Cam_PERSEE_Listener::set_sommation(unsigned char sommation) {
	stop_listen_Data();
	this->sommation = sommation;
	return pCom_Zigtron->Set_sommation(sommation, TIMEOUT_ANSWER);
}

void Cam_PERSEE_Listener::set_stop1fit(unsigned char stopafter1stfits) {
	this->stopafter1stfits = stopafter1stfits;
}

int Cam_PERSEE_Listener::set_division(unsigned char division) {
	stop_listen_Data();
	this->division = division;
	return pCom_Zigtron->Set_division(division, TIMEOUT_ANSWER);
}

int Cam_PERSEE_Listener::set_nbloop(unsigned char nb_loop) {
	stop_listen_Data();
	this->nb_loop = nb_loop;
	return pCom_Pautron->Set_nbloop(nb_loop, TIMEOUT_ANSWER);
}

int Cam_PERSEE_Listener::set_bias(unsigned char bias, unsigned voltage) {
	stop_listen_Data();
	switch (bias) {
	case 0:
		this->vbias = voltage;
		break;
	case 4:
		this->vreset = voltage;
		break;
	case 8:
		this->vref = voltage;
		break;
	default:
		return 1;
	}

	return pCom_Pautron->Set_bias(bias, voltage, TIMEOUT_ANSWER);
}

int Cam_PERSEE_Listener::set_nbread(unsigned char nb_read) {

	stop_listen_Data();
	this->nb_read = nb_read;
	return pCom_Pautron->Set_nbread(nb_read, TIMEOUT_ANSWER);
}

int Cam_PERSEE_Listener::init_cam() {
	int res = pCom_Pautron->Reset(TIMEOUT_ANSWER);
	res = pCom_Zigtron->Set_division(division, TIMEOUT_ANSWER);
	res = pCom_Zigtron->Set_sommation(sommation, TIMEOUT_ANSWER);
	res = pCom_Pautron->Set_exposure(exposure, TIMEOUT_ANSWER);
	res = pCom_Pautron->Set_latency(latency, TIMEOUT_ANSWER);
	res = pCom_Pautron->Set_nbread(nb_read, TIMEOUT_ANSWER);
	res = pCom_Pautron->Set_nbloop(nb_loop, TIMEOUT_ANSWER);
	res = pCom_Pautron->Set_bias(0, vbias, TIMEOUT_ANSWER);
	res = pCom_Pautron->Set_bias(4, vreset, TIMEOUT_ANSWER);
	res = pCom_Pautron->Set_bias(8, vref, TIMEOUT_ANSWER);

	for (unsigned int k = 0; k < ROI_tab.size(); k++) {
		/* Extraction de sous-ROIs carr�es */
		int tmpInt = ROI_tab[k]->get_ROI()->get_nb_ROIs();
		short unsigned int **tmpTabInt = ROI_tab[k]->get_ROI()->get_ROIs();
		//cout << tmpInt << " sous-ROIs\n";

		/** envoi des commandes � l'electronique **/

		for (int i = 0; i < tmpInt; i++) {
			pCom_Pautron->Create_window(tmpTabInt[i][0], tmpTabInt[i][1],
					tmpTabInt[i][2], TIMEOUT_ANSWER);
		}
		/**                                      **/
	}

	return res;
}

unsigned int Cam_PERSEE_Listener::get_nb_pix() {
	bool trouve = false;
	unsigned nb_pix = 0;

	if (ROI_tab.empty()) {
		cout << "Pas de ROI definie" << endl;
	} else {
		for (unsigned short int i = 0; i < MAX_SIZE_ROI; i++)
			for (unsigned short int j = 0; j < MAX_SIZE_ROI; j++) {
				trouve = false;
				for (unsigned short int k = 0; k < ROI_tab.size() && !trouve; k++) {
					trouve = (ROI_tab[k]->get_ROI())->contient(i, j);
				}
				if (trouve)
					nb_pix++;
			}
	}
	return nb_pix;
}

int Cam_PERSEE_Listener::optimize_read() {
	top = 0;
	left = 0;

	if (ROI_tab.empty()) {
		cout << "Pas de ROI definie" << endl;
		return 1;
	}
	for (unsigned short int i = 0; i < MAX_SIZE_ROI; i++)
		for (unsigned short int j = 0; j < MAX_SIZE_ROI; j++) {
			bool trouve = false;
			for (unsigned short int k = 0; k < ROI_tab.size() && !trouve; k++) {
				trouve = (ROI_tab[k]->get_ROI())->contient(i, j);
			}
			if (trouve && (i > left))
				left = i;
			if (trouve && (j > top))
				top = j;
		};
		pCom_Pautron->Set_topleft(top+2, left+2, TIMEOUT_ANSWER);
	return 0;
}

int Cam_PERSEE_Listener::save_all_frames(unsigned char save_all) {
	this->save_all = save_all;
	return 0;
}

int Cam_PERSEE_Listener::set_filter(unsigned short position) {
    this->current_filter = position;
	return this->pCom_HK->Set_filter(position, TIMEOUT_ANSWER*10);
}

int Cam_PERSEE_Listener::set_filter_home() {
	return this->pCom_HK->Set_filterhome(TIMEOUT_ANSWER);
}

int Cam_PERSEE_Listener::save_filter(unsigned short position) {
	return this->pCom_HK->Save_filter(position, TIMEOUT_ANSWER);
}

int Cam_PERSEE_Listener::move_filter(unsigned short position) {
	return this->pCom_HK->Move_filter(position, TIMEOUT_ANSWER*10);
}

int Cam_PERSEE_Listener::get_position() {
	return this->pCom_HK->Get_position(TIMEOUT_ANSWER);
}

int Cam_PERSEE_Listener::read_ADC(unsigned short position) {
	return this->pCom_HK->Read_ADC(position, TIMEOUT_ANSWER);
}

int Cam_PERSEE_Listener::get_version() {
	return this->pCom_HK->Get_version(TIMEOUT_ANSWER);
}

int Cam_PERSEE_Listener::set_cumul(unsigned short cumul) {
	return this->pCom_HK->Set_cumul(cumul, TIMEOUT_ANSWER);
}

int Cam_PERSEE_Listener::nb_HK(unsigned short nb_HK) {
	return this->pCom_HK->Nb_HK(nb_HK, TIMEOUT_ANSWER);
}

int Cam_PERSEE_Listener::set_motors(unsigned short enable) {
	return this->pCom_HK->Set_motors(enable, TIMEOUT_ANSWER);
}
