/***************************************************************************
 *   Copyright (C) 2007 by Arnaud Sevin
 *   Arnaud.Sevin@obspm.fr
 * 
 * Copyright (c) 1998, Regents of the University of California
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/


#include "cam_persee_simulator.h"

/***************************************************
 *                                                 *
 *          Classe Cam_PERSEE_Simulator            *
 *                                                 *
 ***************************************************/

Cam_PERSEE_Simulator::Cam_PERSEE_Simulator(string pDevice):Com_PERSEE(pDevice, BPS115200, false) 
{
  c1=0x0A;
  c2=0x00;
  c3=0x00;
}

void Cam_PERSEE_Simulator::write_hk( char *buffer ){
  buffer[0]=SYNCHRO;
  buffer[1]=SYNCHRO2;
  for (int i=2;i<HK_SIZE*2+2; i++)
    buffer[i]=10; //random()%254+2;
}

void Cam_PERSEE_Simulator::write_data( bool acqui, int nb_pix, char *buffer ) {
  buffer[0]=SYNCHRO;
  buffer[1]=SYNCHRODATA;
  buffer[2]=acqui;

  //initialisation du checksum
  buffer[3*(nb_pix+1)] = buffer[3*(nb_pix+1)+1] = buffer[3*(nb_pix+1)+2] = 0;

  for (int i=3;i<3*nb_pix; i+=3) {
    buffer[i]   =   0; //random()%254+2;
    buffer[i+1] =   0; //random()%254+2;
    buffer[i+2] =   i; //random()%254+2;
  }
  buffer[3*nb_pix]   = c3;
  buffer[3*nb_pix+1] = c2;
  buffer[3*nb_pix+2] = c1;
 
  
  unsigned int pData[nb_pix+1];
  div_t divresult, divresult2;
  char3int(pData, nb_pix, &buffer[3]);
  unsigned int chksum = 0;
  for(int i=0; i<nb_pix; i++){
    chksum += pData[i];
  }
  chksum = chksum%16777216;
  divresult = div (chksum,256);
  divresult2 = div (divresult.quot,256);

  buffer[3*(nb_pix+1)  ] = divresult2.quot%256;
  buffer[3*(nb_pix+1)+1] = divresult2.rem;
  buffer[3*(nb_pix+1)+2] = divresult.rem;

  c1++;
  if(c1==0x00){
    c2++;
  }
  if((c2==0x00)&&(c1==0x00)){
    c3++;
  }

}

/*!
  \fn Cam_PERSEE_Simulator::start_send()
*/
void Cam_PERSEE_Simulator::start_send()
{
  pthread_t sendin_loop;
  pthread_create (&sendin_loop, NULL, Cam_PERSEE_Simulator::start_sending, this);

  working=true;
}

/*!
  \fn Cam_PERSEE_Simulator::start_sending(void *PERSEE_Simu)
*/
void *Cam_PERSEE_Simulator::start_sending(void *PERSEE_Simu)
{
  Cam_PERSEE_Simulator *camera = (Cam_PERSEE_Simulator*)PERSEE_Simu;
  camera->sendFrame();
  return (void*)0L;
}

void Cam_PERSEE_Simulator::stop_send()
{
  working=false;
}

void Cam_PERSEE_Simulator::sendFrame(){
  struct timeval TimeRef, Time;
  unsigned int Dif = 0, fps=0, bytesRecv=0;
  unsigned short int nb_pix=16;

  gettimeofday(&TimeRef, NULL);

  //unsigned long int pHK[HK_SIZE];
  //unsigned long int pData[nb_pix];

  //2bit pour chaque HK + 2bits de synchro
  char buffer[HK_SIZE*2+2];
  
  //3bit pour chaque donnee + 2bits de synchro + topacq
  char buffer_data[(nb_pix+1)*3+2+1];

  working=true;
  bool acqui=true;
  while(working) {
    write_hk(buffer);
 
    bytesRecv = HK_SIZE*2+2; //write (fd, buffer, HK_SIZE*2+2 );
    /*
          if(DEBUG) cerr << (int)(unsigned char)buffer[0] << " " 
	   << (int)(unsigned char)buffer[1] << " " 
	   << (int)(unsigned char)buffer[2] << endl;
    */

    if(bytesRecv<0){ 
      cerr << "ERROR : Envoi impossible" << endl;
      working = false;
    }else if (bytesRecv!=HK_SIZE*2+2) {
      cerr << "Hoston... j'ai pas pu tout envoyer ! send:" << bytesRecv << " instead of " << HK_SIZE*2+3 << endl;
      working=false;
    }else{

      gettimeofday(&Time, NULL);
      Dif = (Time.tv_sec - TimeRef.tv_sec) * 1000000 + (Time.tv_usec - TimeRef.tv_usec);
      fps++;
      if(DEBUG) {
	  if (Dif >= 1000000){ //1sec
	      cerr << "Envoi : " << fps << " frames in " << Dif/1000000. << "s ("<< fps*1000000/(double)Dif << " fps )" << endl; 
	      fps=0;
	      gettimeofday(&TimeRef, NULL);
	  }
      }

      write_data(acqui, nb_pix, buffer_data);
      //acqui =!acqui;
      
      bytesRecv = write (fd, buffer_data, (nb_pix+1)*3+2+1);
      if(bytesRecv<0){ 
	cerr << "ERROR : Envoi impossible" << endl;
	working = false;
      }else if (bytesRecv!=(unsigned)((nb_pix+1)*3+2+1)) {
	cerr << "Hoston... j'ai pas pu tout envoyer ! send:" << bytesRecv << " instead of " << (nb_pix+1)*3+2 << endl;
	working=false;
      }
      
    }
  }
  cerr << "last frame : " << (unsigned char)c3*256*256+(unsigned char)c2*256+(unsigned char)c1-1 << endl;
}
