/***************************************************************************
 *   Copyright (C) 2007 by Arnaud Sevin
 *   Arnaud.Sevin@obspm.fr
 * 
 * Copyright (c) 1998, Regents of the University of California
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#define DEBUG 0
//#define SIMU  1

#include "com_persee.h"

/***************************************************
 *                                                 *
 *          Classe Com_PERSEE                      *
 *                                                 *
 ***************************************************/

Com_PERSEE::Com_PERSEE(string pDevice, int Vitesse, bool NONBLOCK) :
  tFTDI(pDevice.c_str()) {
}

Com_PERSEE::~Com_PERSEE() {
  if (DEBUG)
    cerr << "\nBye!\n";
}

char *Com_PERSEE::int2char(unsigned int tmpInt) {
  char *tmpStr = new char[23];
  div_t divresult;
  divresult = div(tmpInt, 256);
  sprintf(tmpStr, "%c%c", divresult.quot % 256, divresult.rem);

  return tmpStr;
}

char *Com_PERSEE::int3char(unsigned int tmpInt) {
  char *tmpStr = new char[23];
  div_t divresult, divresult2;
  divresult = div(tmpInt, 256);
  divresult2 = div(divresult.quot, 256);
  sprintf(tmpStr, "%c%c%c", divresult2.quot % 256, divresult2.rem,
      divresult.rem);

  return tmpStr;
}

void Com_PERSEE::char2int(unsigned int tmpInt[], unsigned int nb_elems,
    char *tmpStr) {
  for (unsigned int i = 0; i < nb_elems; i++) {
    tmpInt[i] = (unsigned char) tmpStr[i * 2] * 256 + (unsigned char) tmpStr[i
        * 2 + 1];
  }
}

void Com_PERSEE::char3int(unsigned int tmpInt[], unsigned int nb_elems,
    char *tmpStr) {
  for (unsigned int i = 0; i < nb_elems; i++) {
    tmpInt[i] = (unsigned char) tmpStr[i * 3] * 256 * 256
        + (unsigned char) tmpStr[i * 3 + 1] * 256 + (unsigned char) tmpStr[i
        * 3 + 2];
  }
}

//----------------------------------------------------------------------------
// THEO - This is the function we care about. The timeout is in mS I think.
// THEO - We need our own version of this function. See next function
//
//

int Com_PERSEE::RecevoirTrame(Frame_PERSEE *frame, unsigned int Attente)
{
  const char synchro = SYNCHRO;
  const char synchrodata = SYNCHRODATA;

  //time_t TimeRef;
  //time_t Time;
  struct timeval TimeRef;
  struct timeval Time;
  unsigned int Dif = 0;

  char buffer[4] = "zzz";
  unsigned char Char;
  int Res_buffer;

  gettimeofday(&TimeRef, NULL);
  while ((buffer[0] != synchro) || (buffer[1] != synchrodata) || (buffer[2]
      < 0x10)) {
    //FT_Read(ftHandle, &Char, 1, &Res_buffer);
    //    Res_buffer = read(fd, &Char, 1);
    if ((Res_buffer = ftdi_read_data(&ftdic, &Char, 1)) < 0) {
      sprintf(MessageConfigurer, "ftdi_read_data failed: %d (%s)\n",
          Res_buffer, ftdi_get_error_string(&ftdic));
      return ERREUR;
    }
    if (Res_buffer > 0) {
      buffer[0] = buffer[1];
      buffer[1] = buffer[2];
      buffer[2] = Char;

      gettimeofday(&TimeRef, NULL);

      if (DEBUG > 1)
        cerr << (int) (unsigned char) buffer[0] << " ("
            << (int) (unsigned char) synchro << ") "
            << (int) (unsigned char) buffer[1] << " ("
            << (int) (unsigned char) synchrodata << ") "
            << (int) (unsigned char) buffer[2] << endl;

    }//else usleep(SLEEPINGTIME);
    gettimeofday(&Time, NULL);
    Dif = (Time.tv_sec - TimeRef.tv_sec) * 1000000 + (Time.tv_usec
        - TimeRef.tv_usec);
    if (Dif >= Attente * 1000) {
      return TIMEOUT;
    }
  }

  frame->acqui = buffer[2] & 0x01;
  unsigned int size = 3 * (frame->nb_pix + 1);
  unsigned char pData[size];

  gettimeofday(&TimeRef, NULL);

  unsigned int index = 0;
  while (index < size) {
    //Res_buffer = read(fd, &pData[index], size-index);
    //FT_Read(ftHandle, &pData[index], size, &Res_buffer);
    if ((Res_buffer = ftdi_read_data(&ftdic, &pData[index], size - index)) < 0) {
      sprintf(MessageConfigurer, "ftdi_read_data failed: %d (%s)\n",
          Res_buffer, ftdi_get_error_string(&ftdic));
      return ERREUR;
    }
    if (Res_buffer > 0) {
      //if(DEBUG) cerr << "read : " << Res_buffer << endl;
      index += Res_buffer;
      gettimeofday(&TimeRef, NULL);
    }

    gettimeofday(&Time, NULL);
    Dif = (Time.tv_sec - TimeRef.tv_sec) * 1000000 + (Time.tv_usec
        - TimeRef.tv_usec);

    if (Dif >= Attente * 1000) {
      return TIMEOUT;
    }
  }

  unsigned int chksum = 0;
  for (unsigned int i = 0; i < frame->nb_pix; i++) {
    //frame->donnees[i] = pData[i*3+2]<<16 + pData[i*3+1]<<8 + pData[i*3];
    frame->donnees[i] = pData[i * 3 + 1] * 256 + pData[i * 3];
    chksum += frame->donnees[i];
    if (DEBUG > 1)
      cerr << frame->donnees[i] << " " << (int) pData[i * 3 + 2] << " "
          << (int) pData[i * 3 + 1] << " " << (int) pData[i * 3] << endl;

// THEO - WHAT DOES THIS MEAN..... WHY DOES IT TRIGGER SO OFTEN

    if (pData[i * 3 + 2]) {
      bzero(frame->donnees, frame->nb_pix * sizeof(short int));
      return RIEN_RECU;
    }
  }
  if (frame->checkData) {
    chksum -= frame->donnees[frame->nb_pix - 1];
    //if(DEBUG) cerr << endl;

    chksum = chksum % 16777216;
    unsigned int chksum2 = frame->donnees[frame->nb_pix - 1];
    if (chksum != chksum2) {
      if (DEBUG)
        cerr << "ERROR Chksum (attendu " << chksum << ") trouve " << chksum2
            << endl;
      //return ERREUR;
    } else if (DEBUG)
      cerr << frame->donnees[frame->nb_pix - 2] << " Chksum OK !!!" << endl;
  }
  return frame->nb_pix;
}

/*
 * THEO
 * This is our Jouflu specific version of the function that gets data
 * from the camera. See comments in the code.
 * Time out is in mS.
 */

int Com_PERSEE::jouflu_RecevoirTrame(Frame_PERSEE *frame, unsigned int Attente)
{
  const char synchro = SYNCHRO;
  const char synchrodata = SYNCHRODATA;

  struct timeval TimeRef;
  struct timeval Time;
  unsigned int Dif = 0;
  char buffer[4] = "zzz";
  unsigned char Char;
  int Res_buffer;

  /* What time did we start? */

  gettimeofday(&TimeRef, NULL);

  /* Loop until we get the first byte of data */

  while ((buffer[0] != synchro) || (buffer[1] != synchrodata) ||
		(buffer[2] < 0x10))
  {
    /* Get the data and check for an error */

    if ((Res_buffer = ftdi_read_data(&ftdic, &Char, 1)) < 0)
    {
      sprintf(MessageConfigurer, "ftdi_read_data failed: %d (%s)\n",
          Res_buffer, ftdi_get_error_string(&ftdic));
      return ERREUR;
    }

    /* Did we get any data? */

    if (Res_buffer > 0)
    {
      /* Yes we did. */

      buffer[0] = buffer[1];
      buffer[1] = buffer[2];
      buffer[2] = Char;
    }

    /* Have we timed out? */

    gettimeofday(&Time, NULL);

    Dif = (Time.tv_sec - TimeRef.tv_sec) * 1000000 + (Time.tv_usec
        - TimeRef.tv_usec);
    if (Dif >= Attente * 1000) return TIMEOUT;
  }

  /* Let it be known we have a frame */

  frame->acqui = buffer[2] & 0x01;

  unsigned int size = 3 * (frame->nb_pix + 1);
  unsigned char pData[size];

  gettimeofday(&TimeRef, NULL);

  unsigned int index = 0;

  /* Now try and get the rest of the data */

  while (index < size)
  {
    if ((Res_buffer = ftdi_read_data(&ftdic, &pData[index], size - index)) < 0)
    {
      return ERREUR;
    }

    if (Res_buffer > 0)
    {
      index += Res_buffer;
      gettimeofday(&TimeRef, NULL);
    }

    gettimeofday(&Time, NULL);
    Dif = (Time.tv_sec - TimeRef.tv_sec) * 1000000 + (Time.tv_usec
        - TimeRef.tv_usec);

    /* Have we timed out? */
    if (Dif >= Attente * 1000) return TIMEOUT;
  }

  /*
   * OK, extract the data.
   * NOTE we only want the lower 16 bits.
   * NOTE There used to be a checksum in here, which we now ignore.
   * NOTE This used to check the third byte and flag an error if it was !=0
   */

  unsigned char *p1;

  p1 = (unsigned char *)pData;

  for (unsigned int i = 0; i < frame->nb_pix; i++)
  {
    // Used to be : pData[i * 3 + 1] * 256 + pData[i * 3];

    frame->donnees[i] =  ((short int)(*p1++));
    frame->donnees[i] += (((short int)(*p1++))<<8);
    //printf("0x%x\n", frame->donnees[i]);
    if (*p1++)
    {
      bzero(frame->donnees, frame->nb_pix * sizeof(short int));
      return RIEN_RECU;
    }
  }

  /* We are done. Return the number of pixels we have received. */

  return frame->nb_pix;
}

/*
 * THEO: A function to clear any remaining junk in the buffer.
 */

int Com_PERSEE::jouflu_clearftdi(void)
{
  unsigned char Char;
  int	i, Res_buffer;
  time_t start;

    i = 0;
    start = time(NULL);
    while ((Res_buffer = ftdi_read_data(&ftdic, &Char, 1)) > 0 &&
		time(NULL) < start+2)
    {
	i += Res_buffer;
    }

    return i;
}

//----------------------------------------------------------------------------
int Com_PERSEE::RecevoirTrame2(Frame_PERSEE *frame, unsigned int Attente)
{
  const unsigned char synchro = SYNCHRO;
  const unsigned char synchrodata = SYNCHRODATA;

  //time_t TimeRef;
  //time_t Time;
  struct timeval TimeRef;
  struct timeval Time;
  unsigned int Dif = 0;

  unsigned char buffer[4] = "zzz";
  unsigned char Char;
  int Res_buffer;

  //cerr << (frame->nb_pix) << endl;
  unsigned int size = 3 * (frame->nb_pix + 1);
  unsigned char pData[size];

  gettimeofday(&TimeRef, NULL);

  unsigned int index = 0;
  while (index < size) {
    //Res_buffer = read(fd, &pData[index], size-index);
    //FT_Read(ftHandle, &pData[index], size, &Res_buffer);
    if ((Res_buffer = ftdi_read_data(&ftdic, &pData[index], size - index)) < 0) {
      sprintf(MessageConfigurer, "ftdi_read_data failed: %d (%s)\n",
          Res_buffer, ftdi_get_error_string(&ftdic));
      return ERREUR;
    }
    if (Res_buffer > 0) {
      //if(DEBUG) cerr << "read : " << Res_buffer << endl;
      index += Res_buffer;
      gettimeofday(&TimeRef, NULL);
    }

    gettimeofday(&Time, NULL);
    Dif = (Time.tv_sec - TimeRef.tv_sec) * 1000000 + (Time.tv_usec
        - TimeRef.tv_usec);

    if (Dif >= Attente * 1000) {
      return TIMEOUT;
    }
  }

  unsigned int chksum = 0;
  for (unsigned int i = 0; i < frame->nb_pix; i++) {
    //frame->donnees[i] = pData[i*3+2]<<16 + pData[i*3+1]<<8 + pData[i*3];
    frame->donnees[i] = pData[i * 3 + 1] * 256 + pData[i * 3];
    chksum += frame->donnees[i];
    //if(DEBUG) cerr << frame->donnees[i] << " " <<  (int)pData[i*3+2] << " " << (int)pData[i*3+1] << " " << (int)pData[i*3] << endl;
  }

  if (frame->checkData) {
    chksum -= frame->donnees[frame->nb_pix - 1];
    //if(DEBUG) cerr << endl;

    chksum = chksum % 16777216;
    unsigned int chksum2 = frame->donnees[frame->nb_pix - 1];
    if (chksum != chksum2) {
      if (DEBUG)
        cerr << "ERROR Chksum (attendu " << chksum << ") trouve " << chksum2
            << endl;
      //return ERREUR;
    } else if (DEBUG)
      cerr << frame->donnees[frame->nb_pix - 2] << " Chksum OK !!!" << endl;
  }

  //Res_buffer = read(fd, buffer, 3);
  if ((Res_buffer = ftdi_read_data(&ftdic, buffer, 3)) < 0) {
    sprintf(MessageConfigurer, "ftdi_read_data failed: %d (%s)\n", Res_buffer,
        ftdi_get_error_string(&ftdic));
    return ERREUR;
  }
  gettimeofday(&TimeRef, NULL);
  if ((buffer[0] != synchro) || (buffer[1] != synchrodata) || (buffer[2]
      <= 0x0F)) {
    fprintf(stderr, "Synchro perdue\n");
    while ((buffer[0] != synchro) || (buffer[1] != synchrodata) || (buffer[2]
        <= 0x0F)) {
      //FT_Read(ftHandle, &Char, 1, &Res_buffer);
      //Res_buffer = read(fd, &Char, 1);
      if ((Res_buffer = ftdi_read_data(&ftdic, &Char, 1)) < 0) {
        sprintf(MessageConfigurer, "ftdi_read_data failed: %d (%s)\n",
            Res_buffer, ftdi_get_error_string(&ftdic));
        return ERREUR;
      }
      if (Res_buffer > 0) {
        buffer[0] = buffer[1];
        buffer[1] = buffer[2];
        buffer[2] = Char;
        gettimeofday(&TimeRef, NULL);
        /*
         if(DEBUG) cerr << (int)(unsigned char)buffer[0] << " (" << (int)(unsigned char)synchro << ") "
         << (int)(unsigned char)buffer[1] << " (" << (int)(unsigned char)synchrodata << ") "
         << (int)(unsigned char)buffer[2] << endl;
         */
      }
      //usleep(SLEEPINGTIME);
      gettimeofday(&Time, NULL);
      Dif = (Time.tv_sec - TimeRef.tv_sec) * 1000000 + (Time.tv_usec
          - TimeRef.tv_usec);
      if (Dif >= Attente * 1000) {
        return TIMEOUT;
      }
    }
    //cerr << frame->donnees[i] << " " <<  (int)pData[i*3+2] << " " << (int)pData[i*3+1] << " " << (int)pData[i*3] << endl;
    bzero(frame->donnees, frame->nb_pix * sizeof(short int));
    return RIEN_RECU;
  }

  frame->acqui = buffer[2];

  return frame->nb_pix;
}

//----------------------------------------------------------------------------
int Com_PERSEE::RecevoirHK(Frame_PERSEE *frame, unsigned int Attente) {
  const unsigned char synchro = SYNCHRO;
  const unsigned char synchro2 = SYNCHRO2;

  struct timeval TimeRef;
  struct timeval Time;
  unsigned int Dif = 0;

  unsigned char buffer[3] = "zz";
  unsigned char Char;
  int Res_buffer;

  // Envoi de la demande d'info.
  sprintf((char*)buffer, "?\n");
  //write(fd, buffer, strlen(buffer));
  //FT_Write (ftHandle, buffer, strlen(buffer), &Res_buffer);
  if ((Res_buffer = ftdi_write_data(&ftdic, buffer, strlen( (char*)buffer))) < 0) {
    sprintf(MessageConfigurer, "ftdi_write_data failed: %d (%s)\n", Res_buffer,
        ftdi_get_error_string(&ftdic));
    return ERREUR;
  }

  gettimeofday(&TimeRef, NULL);

  while ((buffer[0] != synchro) || (buffer[1] != synchro2)) {
    //Res_buffer = read(fd, &Char, 1);
    //FT_Read (ftHandle, &Char, 1, &Res_buffer);
    if ((Res_buffer = ftdi_read_data(&ftdic, &Char, 1)) < 0) {
      sprintf(MessageConfigurer, "ftdi_read_data failed: %d (%s)\n",
          Res_buffer, ftdi_get_error_string(&ftdic));
      return ERREUR;
    }
    if (Res_buffer > 0) {
      //cerr << "Lu : " <<(unsigned short)Char << "\t" << Char<<endl;
      buffer[0] = buffer[1];
      buffer[1] = Char;
      gettimeofday(&TimeRef, NULL);
      /*
       if(DEBUG) cerr << (int)(unsigned char)buffer[0] << " "
       << (int)(unsigned char)buffer[1] << " "
       << (int)(unsigned char)buffer[2] << endl;
       */
    }

    usleep(SLEEPINGTIME);
    gettimeofday(&Time, NULL);
    Dif = (Time.tv_sec - TimeRef.tv_sec) * 1000000 + (Time.tv_usec
        - TimeRef.tv_usec);
    if (Dif >= Attente * 1000) {
      return TIMEOUT;
    } else if (Char == '!') {
      //read(fd, &Char, 1);
      if ((Res_buffer = ftdi_read_data(&ftdic, &Char, 1))
          < 0) {
        sprintf(MessageConfigurer, "ftdi_read_data failed: %d (%s)\n", Res_buffer,
            ftdi_get_error_string(&ftdic));
        return ERREUR;
      }
      //FT_Write (ftHandle, &Char, 1, &Res_buffer);
      return ERREUR;
    }
  }

  unsigned int size = HK_SIZE * 2;
  unsigned char pHK[size];

  gettimeofday(&TimeRef, NULL);

  unsigned int index = 0;
  //lecture de 0700
  while (index < 2) {
    //Res_buffer = read(fd, buffer, 2-index);
    if ((Res_buffer = ftdi_read_data(&ftdic, buffer, 2 - index)) < 0) {
      sprintf(MessageConfigurer, "ftdi_read_data failed: %d (%s)\n", Res_buffer,
          ftdi_get_error_string(&ftdic));
      return ERREUR;
    }
    if (Res_buffer > 0) {
      index += Res_buffer;
      gettimeofday(&TimeRef, NULL);
    }
  }

  index = 0;
  while (index < size) {
    //Res_buffer = read(fd, &pHK[index], size-index);
    //FT_Read(ftHandle, &pHK[index], size-index, &Res_buffer);
    if ((Res_buffer = ftdi_read_data(&ftdic, &pHK[index], size
        - index)) < 0) {
      sprintf(MessageConfigurer, "ftdi_read_data failed: %d (%s)\n", Res_buffer,
          ftdi_get_error_string(&ftdic));
      return ERREUR;
    }
    if (Res_buffer > 0) {
      index += Res_buffer;
      gettimeofday(&TimeRef, NULL);
    }

    gettimeofday(&Time, NULL);
    Dif = (Time.tv_sec - TimeRef.tv_sec) * 1000000 + (Time.tv_usec
        - TimeRef.tv_usec);

    if (Dif >= Attente * 1000) {
      return TIMEOUT;
    }
  }

  //  if(DEBUG) cerr << "HK  : ";
  for (int i = 0; i < HK_SIZE; i++) {
    frame->HK[i] = (pHK[i * 2] + pHK[i * 2 + 1] * 256) / 4;
    //    if(DEBUG) cerr << frame->HK[i] << " (" << (short)pHK[i*2] << " " << (short)pHK[i*2+1] << ") ";
  }
  //  if(DEBUG) cerr << endl;

  /*
   for(int i=0; i<2*HK_SIZE; i++)
   fprintf(stderr, "%02X", pHK[i]);
   fprintf(stderr, "\n");
   */

  //lecture du '>'
  while (Char != '>') {
    //read(fd, &Char, 1);
    //FT_Write (ftHandle, &Char, 1, &Res_buffer);
    if ((Res_buffer = ftdi_read_data(&ftdic, &Char, 1)) < 0) {
      sprintf(MessageConfigurer, "ftdi_read_data failed: %d (%s)\n", Res_buffer,
          ftdi_get_error_string(&ftdic));
      return ERREUR;
    }
  }
  return HK_SIZE;
}

//----------------------------------------------------------------------------

int Com_PERSEE::Reset(unsigned int Attente) {
  char buffer[4];
  //sprintf(buffer, "%c", 27); //27 = ESC
  //Envoyer(buffer, Attente);
  sprintf(buffer, "z\n");
  Envoyer(buffer, Attente);
  sprintf(buffer, "b0\n");
  return Envoyer(buffer, Attente);
}

//----------------------------------------------------------------------------

int Com_PERSEE::Acqui(unsigned int Attente) {
  char buffer[3];
  sprintf(buffer, " %c", 13);
  return Envoyer(buffer, Attente);
}

//----------------------------------------------------------------------------

int Com_PERSEE::Set_division(unsigned short division, unsigned int Attente) {
  char buffer[4];
  sprintf(buffer, "d%2.2X\n", division);
  return Envoyer(buffer, Attente);
}

//----------------------------------------------------------------------------

int Com_PERSEE::Set_sommation(unsigned short sommation, unsigned int Attente) {
  char buffer[4];
  sprintf(buffer, "s%2.2X\n", sommation);
  return Envoyer(buffer, Attente);
}

//----------------------------------------------------------------------------

/*
      Dcimal   Octal   Hex  Binaire   Caractre
      -------   -----   ---  --------    ------
        000      000    00   00000000      NUL    (Null char.)
        010      012    0A   00001010       LF    (Line Feed)
        013      015    0D   00001101       CR    (Carriage Return)
        027      033    1B   00011011      ESC    (Escape)
*/
int Com_PERSEE::Set_filter(unsigned short position, unsigned int Attente) {
  char buffer[4];
//  int res;
//  sprintf(buffer, "%c\n", 27); // 27 == ESC
//  if ((res = ftdi_write_data(&ftdic, (unsigned char *)buffer, strlen(buffer))) < 0) {
//    fprintf(stderr, "ftdi_write_data failed: %d (%s)\n", res,
//        ftdi_get_error_string(&ftdic));
//    return ERREUR;
//  }
  sprintf(buffer, "F%d\n", position); // 27 == ESC
  return Envoyer(buffer, Attente);
}

//----------------------------------------------------------------------------

int Com_PERSEE::Set_filterhome(unsigned int Attente) {
  char buffer[3];
  sprintf(buffer, "H\n");
  return Envoyer(buffer, Attente);
}

//----------------------------------------------------------------------------

int Com_PERSEE::Save_filter(unsigned short position, unsigned int Attente) {
  char buffer[4];
  sprintf(buffer, "S%d\n", position);
  return Envoyer(buffer, Attente);
}

//----------------------------------------------------------------------------

int Com_PERSEE::Move_filter(unsigned short position, int Attente) {
  char buffer[16];
  sprintf(buffer, "N%d\n", position);
  return Envoyer(buffer, Attente);
}

//----------------------------------------------------------------------------

int Com_PERSEE::Get_position(int Attente) {
  char buffer[4];
  sprintf(buffer, "I\n");
  return Envoyer(buffer, Attente);
}

//----------------------------------------------------------------------------

int Com_PERSEE::Read_ADC(unsigned short position, int Attente) {
  char buffer[4];
  sprintf(buffer, "R%d\n", position);
  return Envoyer(buffer, Attente);
}

//----------------------------------------------------------------------------

int Com_PERSEE::Get_version(int Attente) {
  char buffer[4];
  sprintf(buffer, "L\n");
  return Envoyer(buffer, Attente);
}

//----------------------------------------------------------------------------

int Com_PERSEE::Set_cumul(unsigned short cumul, int Attente) {
  char buffer[4];
  sprintf(buffer, "C%d\n", cumul);
  return Envoyer(buffer, Attente);
}

//----------------------------------------------------------------------------

int Com_PERSEE::Nb_HK(unsigned short nb_HK, int Attente) {
  char buffer[4];
  sprintf(buffer, "V%d\n", nb_HK);
  return Envoyer(buffer, Attente);
}

//----------------------------------------------------------------------------

int Com_PERSEE::Set_motors(unsigned short enable, int Attente) {
  char buffer[4];
  sprintf(buffer, "E%d\n", enable);
  return Envoyer(buffer, Attente);
}

//----------------------------------------------------------------------------

int Com_PERSEE::Set_exposure(unsigned int exposure, unsigned int Attente) {
  char buffer[4];
  sprintf(buffer, "t%d\n", exposure);
  return Envoyer(buffer, Attente);
}

//----------------------------------------------------------------------------

int Com_PERSEE::Set_latency(unsigned int latency, unsigned int Attente) {
  char buffer[4];
  sprintf(buffer, "l%d\n", latency);
  return Envoyer(buffer, Attente);
}

//----------------------------------------------------------------------------

int Com_PERSEE::Create_window(unsigned short x, unsigned short y,
    unsigned short size, unsigned int Attente) {
  char buffer[14];
  sprintf(buffer, "f %d %d %d\n", x, y, size);
  return Envoyer(buffer, Attente);
}

//----------------------------------------------------------------------------

int Com_PERSEE::Set_bias(unsigned short bias, unsigned short value,
    unsigned int Attente) {
  char buffer[8];
  sprintf(buffer, "s%d%3.3X\n", bias, value << 2);
  return Envoyer(buffer, Attente);
}

//----------------------------------------------------------------------------

int Com_PERSEE::Select_bank(unsigned short bank, unsigned int Attente) {
  char buffer[4];
  sprintf(buffer, "b%i\n", bank);
  return Envoyer(buffer, Attente);
}

//----------------------------------------------------------------------------

int Com_PERSEE::Set_nbread(unsigned short nb_read, unsigned int Attente) {
  char buffer[4];
  sprintf(buffer, "n%i\n", nb_read - 1);
  return Envoyer(buffer, Attente);
}

//----------------------------------------------------------------------------

int Com_PERSEE::Set_nbloop(unsigned short nb_loop, unsigned int Attente) {
  char buffer[4];
  sprintf(buffer, "r%i\n", nb_loop);
  return Envoyer(buffer, Attente);
}

//----------------------------------------------------------------------------
// Commandes pour la ZigTron

int Com_PERSEE::Set_nbpix(unsigned int nb_pix, unsigned int Attente) {
  char buffer[4];
  sprintf(buffer, "t%d\n", nb_pix);
  return Envoyer(buffer, Attente);
}

int Com_PERSEE::triggOff(unsigned int Attente) {
  char buffer[3];
  sprintf(buffer, "a\n");
  return Envoyer(buffer, Attente);
}

int Com_PERSEE::trigg(unsigned int Attente) {
  char buffer[3];
  sprintf(buffer, "z\n");
  return Envoyer(buffer, Attente);
}

int Com_PERSEE::Set_pause(unsigned char pause, unsigned int Attente) {
  char buffer[4];
  sprintf(buffer, "p%d\n", pause);
  return Envoyer(buffer, Attente);
}

int Com_PERSEE::Set_trigg(unsigned char trigg, unsigned int Attente) {
  char buffer[4];
  sprintf(buffer, "i%d\n", trigg);
  return Envoyer(buffer, Attente);
}

//----------------------------------------------------------------------------

int Com_PERSEE::Set_topleft(unsigned char top, unsigned char left,
    unsigned int Attente) {
  char buffer[14];
  sprintf(buffer, "x %d 0 127\n", left);
  Envoyer(buffer, Attente);
  sprintf(buffer, "y %d\n", top);
  return Envoyer(buffer, Attente);
}

//----------------------------------------------------------------------------

int Com_PERSEE::Envoyer(char *message, unsigned int Attente) {
#ifdef SIMU
  cerr << "[MODE SIMU] Envoie du message : "<< message;
  return 0;
#else
  if (DEBUG)
    cerr << "[DEBUG] Envoie du message : " << message;
  int res = 0; //write(fd, message, strlen(message)); //fd descripteur de fichier (socket)
  //DWORD res = 0;
  //FT_Write (ftHandle, message, strlen(message), &res);
  if ((res = ftdi_write_data(&ftdic, (unsigned char*) message, strlen(message))) < 0) {
    sprintf(MessageConfigurer, "ftdi_write_data failed: %d (%s)\n", res,
        ftdi_get_error_string(&ftdic));
    return ERREUR;
  }

  struct timeval TimeRef;
  struct timeval Time;
  unsigned int Dif = 0;

  const unsigned int max_size=1024;
  unsigned char buffer[max_size] = "a";
  //char lastChar = 'a';
  unsigned int index = 0;

  gettimeofday(&TimeRef, NULL);
  do {
    //res = read(fd, &buffer[index], 1);
    if ((res = ftdi_read_data(&ftdic, &buffer[index], 1)) < 0) {
      sprintf(MessageConfigurer, "ftdi_read_data failed: %d (%s)\n", res,
          ftdi_get_error_string(&ftdic));
      return ERREUR;
    }
    //FT_Read(ftHandle, &buffer[index], 1, &res);
    if (res > 0) {
      //lastChar = buffer[index];
      if (DEBUG)
        cerr << buffer[index];
      if (buffer[index] != '\n')
        index++;
      gettimeofday(&TimeRef, NULL);
    }
    //usleep(SLEEPINGTIME);
    gettimeofday(&Time, NULL);
    Dif = (Time.tv_sec - TimeRef.tv_sec) * 1000000 + (Time.tv_usec
        - TimeRef.tv_usec);
    if (Dif >= Attente * 1000) {
      cout << "envoi de " << message << " : TIMEOUT !\n";
      return TIMEOUT;
    }
  } while ((buffer[index - 1] != '>') && (index < max_size));
  //flushIn();

  if (index == max_size) {
    if (DEBUG)
      cerr << "ERROR : bizarre !!!\n";
    return ERREUR;
  } else if (index == 1)
    return OK;
  else if (buffer[index - 2] == '!') {
    if (DEBUG)
      cerr << "ERROR : commande inconnue\n";
    return ERREUR;
  } else {
    buffer[index - 1] = '\0';
    return atoi((char*)buffer);
    //return OK;
  }
#endif
}

