/***************************************************************************
 *   Copyright (C) 2007 by Arnaud Sevin
 *   Arnaud.Sevin@obspm.fr
 * 
 * Copyright (c) 1998, Regents of the University of California
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/


#ifndef ROI_H
#define ROI_H

#define MAX_SIZE_ROI 128

#include <iostream>

using namespace std;

/**
 * @brief Classe permettant de créer des ROIs carrées à pertir de ROIs rectangulaires
 * @author   Arnaud Sevin <Arnaud.Sevin@obspm.fr>
 * @version  1.0
 * @date     02/06/2008
 */
class ROI
{
  unsigned short int x;
  unsigned short int y;
  unsigned short int dx;
  unsigned short int dy;
  
  ROI *first;
  ROI *next;

  unsigned short int nb_ROI;
  
 public:

  /**
   * Permet de créer une ROI à l'aide d'un tableau d'entiers
   * @param tab Tableau de 4 entiers
   */
  ROI(unsigned short int tab[4]);

  /**
   * Permet de créer une ROI à l'aide de 4 entiers
   * @param x abscisse
   * @param y ordonnée
   * @param dx taille suivant x
   * @param dy taille suivant y
   */
  ROI(unsigned short int x, unsigned short int y, unsigned short int dx, unsigned short int dy);
  virtual ~ROI();
  
  /**
   * Permet de connaitre le nombre de ROIs carrées utilisées
   * @return nombre de ROIs carrées utilisées
   */
  unsigned short int  get_nb_ROIs() { return first->nb_ROI;       };

  /**
   * Permet de connaitre le nombre de pixels de la ROI
   * @return nombre de pixels de la ROI
   */
  unsigned int get_size()           { return first->dy*first->dx; };

  /**
   * Permet d'afficher les ROIs utilisées
   */
  void display();

  /**
   * Permet d'obtenir le tableau d'entiers represantant les ROIs utilisées
   * @return tableau d'entiers represantant les ROIs utilisées
   */
  unsigned short int **get_ROIs();

  /**
   * Permet de savoir si un point appartient à la ROI
   * @param x abscisse
   * @param y ordonnée
   * @return vrai ou faux
   */
  bool contient(unsigned short int x, unsigned short int y);

  /**
   * Permet de connaitre le nombre de pixels contenus dand l'union des ROIs
   * @param nb_ROIs nombre de ROIs
   * @param tabROIs tableau des ROIs
   * @return nombre de pixels contenus dand l'union des ROIs
   */
  friend unsigned int interROI(unsigned short int nb_ROIs, ROI **tabROIs);

 protected:
  ROI(ROI *first, unsigned short int x, unsigned short int y, unsigned short int dx, unsigned short int dy);
  void subdivise();
};

#endif // ROI_H
