/***************************************************************************
 *   Copyright (C) 2007 by Arnaud Sevin
 *   Arnaud.Sevin@obspm.fr
 * 
 * Copyright (c) 1998, Regents of the University of California
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/


#include "roi.h"

using namespace std;

ROI::ROI(unsigned short int x, unsigned short int y, unsigned short int dx, unsigned short int dy)
{
  this->x = x;
  this->y = y;
  this->dx = dx;
  this->dy = dy;

  this->nb_ROI = 1;
  this->first = this;
  this->next = new ROI(this, x, y, dx, dy);
}

ROI::ROI( unsigned short int tab[4] ){
  this->x = tab[0];
  this->y = tab[1];
  this->dx = tab[2];
  this->dy = tab[3];

  this->nb_ROI = 1;
  this->first = this;
  this->next = new ROI(this, x, y, dx, dy);  
}

ROI::ROI(ROI *first, unsigned short int x, unsigned short int y, unsigned short int dx, unsigned short int dy)
{
  this->x = x;
  this->y = y;
  this->dx = dx;
  this->dy = dy;

  this->first = first;
  this->next = 0L;

  if(dx!=dy) this->subdivise();
}

ROI::~ROI()
{
  ROI *myROI = first;
  if(myROI->next != 0L){
    delete myROI->next;
    myROI->next=0L;
  }    
}

void ROI::subdivise()
{
  first->nb_ROI++;
  if(dx<dy){
    this->next = new ROI(first, x, y+dx, dx, dy-dx) ;
    this->dy = dx;
  }else{
    this->next = new ROI(first, x+dy, y, dx-dy, dy) ;
    this->dx = dy;
  }
}

void ROI::display()
{
  ROI *myROI = this->first;
  cout << "--- Main ROI ---" << endl;
  cout << "x : " << myROI->x << " y : " << myROI->y
       << " dx : " << myROI->dx << " dy : " << myROI->dy << endl;
  cout << endl << "--- Sub ROIs ---" << endl;
  while(myROI->next != 0L){
    myROI = myROI->next;
    cout << "x : " << myROI->x << " y : " << myROI->y
	 << " dx : " << myROI->dx << " dy : " << myROI->dy << endl;
  }
}

unsigned short int **ROI::get_ROIs(){

  unsigned short int **tab_roi;
  tab_roi = new unsigned short int*[first->nb_ROI];
  for(unsigned short int i=0; i<first->nb_ROI; i++)
    tab_roi[i]=new unsigned short int[3];

  unsigned short int index=0;
  ROI *myROI = this->first;
  while(myROI->next != 0L){
    myROI = myROI->next;
    tab_roi[index][0] = myROI->x;
    tab_roi[index][1] = myROI->y;
    tab_roi[index][2] = myROI->dx;
    index++;
  }
  
  return tab_roi;
}

bool ROI::contient(unsigned short int x, unsigned short int y){
  return ( (x>=first->x) && (x<first->x+first->dx) && (y>=first->y) && (y<first->y+first->dy) );
}

unsigned int interROI(unsigned short int nb_ROIs, ROI **tabROIs){
  bool trouve = false;
  unsigned nb_pix = 0;

  for(unsigned short int i=0; i<MAX_SIZE_ROI; i++)
    for(unsigned short int j=0; j<MAX_SIZE_ROI; j++) {
      trouve = false;
      for(unsigned short int k=0; k<nb_ROIs && !trouve; k++){
	trouve = tabROIs[k]->contient( i, j );
      }
      if (trouve) nb_pix++;
    }
  return nb_pix;
}
