/***************************************************************************
 *   Copyright (C) 2007 by Arnaud Sevin
 *   Arnaud.Sevin@obspm.fr
 * 
 * Copyright (c) 1998, Regents of the University of California
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

//----------------------------------------------------------------------------
#ifndef	FTDI_H
#define	FTDI_H

//----------------------------------------------------------------------------
#include <iostream>
#include <cstring>

#include <ftdi.h>

//----------------------------------------------------------------------------
#define OK		    0
#define ERREUR 	   -1
#define TIMEOUT    -2
#define RIEN_RECU	1

//----------------------------------------------------------------------------

using namespace std;

/**
 * @brief Classe permettant de communiquer facilement avec un port serie
 * @author   Arnaud Sevin <Arnaud.Sevin@obspm.fr>
 * @version  1.0
 * @date     02/06/2008
 */
class tFTDI
{
private:
  ///interfaces permettant de controler les ports de communication asynchrone
protected:
  struct ftdi_context ftdic;
  ///Message de sortie de la function Configurer
  char MessageConfigurer[64];
  ///Status du port
  int status;
  //------------------------------

public:
  /**
   * Ouvre le port pPort avec les parametres par defaut
   * Vitesse BPS9600, BIT8, PAS_DE_PARITE, STOP1, trueport non bloquant
   * @param idCardVendor
   * @param idCardProduct
   * @param idProduct
   */
  tFTDI(const char *idProduct, int speed=9600, int idCardVendor=0x0403, int idCardProduct=0x6001);

  ~tFTDI();

  /**
   * Affiche Affiche le resultat de la configuration.
   * @return Le message
   */
  char *AfficherMessageConfigurer();

  void flushOut();
  void flushIn();

  /**
   * renvoi le status du port.
   * @return Le status
   */
  int get_status() {return status;};

  /**
   * envoi un caractere sur le port.
   * @param pChar Caractere a envoyer
   * @return Le nombre d'octets envoyes
   */
  int Envoyer(unsigned char pChar);

  /**
   * envoi une chaine de caracteres sur le port.
   * @param pChaine Chaine a envoyer
   * @return Le nombre d'octets envoyes
   */
  int Envoyer(char* pChaine);

  /**
   * envoi une chaine de caracteres sur le port.
   * @param pChaine Caractere a envoyer
   * @param length Longueur de la chaine
   * @return Le nombre d'octets envoyes
   */
  int Envoyer(char* pChaine, int length);

  /**
   * recoit une chaine de caracteres sur le port.
   * @param pChaine Caractere a recevoir
   * @param length Longueur de la chaine
   * @return Le nombre d'octets lus
   */
  int Recevoir(char* pChaine, int length);
};

int findAllUSB(int idCardVendor=0x0403, int idCardProduct=0x6001);

#endif
