/***************************************************************************
 *   Copyright (C) 2007 by Arnaud Sevin
 *   Arnaud.Sevin@obspm.fr
 * 
 * Copyright (c) 1998, Regents of the University of California
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include <cstdio>

#include "tFTDI.h"

#define IDLE_TIME 10000

using namespace std;

//----------------------------------------------------------------------------
char *tFTDI::AfficherMessageConfigurer() {
  return MessageConfigurer;
}

//----------------------------------------------------------------------------
tFTDI::tFTDI(const char *idProduct, int speed, int idCardVendor, int idCardProduct) {
  status = 0;
  if (ftdi_init(&ftdic) < 0) {
    sprintf(MessageConfigurer, "ftdi_init of %s failed\n", idProduct);
    status = -1;
    return;
  }
//int idCardVendor=0x0403, int idCardProduct=0x6001
  int ret;
  //fprintf(stderr, "%d%4.4X %d%4.4X %s\n", idCardVendor, idCardVendor, idProduct);
  if ((ret = ftdi_usb_open_desc(&ftdic, 0x0403, 0x6001, NULL, idProduct)) < 0) {
    sprintf(MessageConfigurer, "ftdi_usb_open_desc of %s failed: %d (%s)\n",
        idProduct, ret, ftdi_get_error_string(&ftdic));
    status = -2;
    return;
  }

  if ((ret = ftdi_usb_reset(&ftdic)) < 0) {
    sprintf(MessageConfigurer, "ftdi_usb_reset of %s failed: %d (%s)\n",
        idProduct, ret, ftdi_get_error_string(&ftdic));
    status = -3;
    return;
  }

  if ((ret = ftdi_set_line_property(&ftdic, BITS_8, STOP_BIT_1, NONE)) < 0) {
    sprintf(MessageConfigurer,
        "ftdi_set_line_property of %s failed: %d (%s)\n", idProduct, ret,
        ftdi_get_error_string(&ftdic));
    status = -4;
    return;
  }
  if ((ret = ftdi_set_baudrate(&ftdic, speed)) < 0) {
    sprintf(MessageConfigurer, "ftdi_set_baudrate of %s failed: %d (%s)\n",
        idProduct, ret, ftdi_get_error_string(&ftdic));
    status = -5;
    return;
  }
  if ((ret = ftdi_usb_purge_rx_buffer(&ftdic)) < 0) {
    sprintf(MessageConfigurer, "ftdi_usb_purge_rx of %s failed: %d (%s)\n",
        idProduct, ret, ftdi_get_error_string(&ftdic));
    status = -6;
    return;
  }
  if ((ret = ftdi_usb_purge_tx_buffer(&ftdic)) < 0) {
    sprintf(MessageConfigurer, "ftdi_usb_purge_tx of %s failed: %d (%s)\n",
        idProduct, ret, ftdi_get_error_string(&ftdic));
    status = -7;
    return;
  }
  sprintf(MessageConfigurer, " #%s open\n",idProduct);
  status = 1;
}

//----------------------------------------------------------------------------
tFTDI::~tFTDI() {
  if (!status) {
    ftdi_usb_close(&ftdic);
    ftdi_deinit(&ftdic);
  }
}

//----------------------------------------------------------------------------
void tFTDI::flushOut() {
  int ret;
  if ((ret = ftdi_usb_purge_tx_buffer(&ftdic)) < 0) {
    sprintf(MessageConfigurer, "ftdi_usb_purge_tx failed: %d (%s)\n", ret,
        ftdi_get_error_string(&ftdic));
    status = -7;
  }
}

//----------------------------------------------------------------------------
void tFTDI::flushIn() {
  int ret;
  if ((ret = ftdi_usb_purge_rx_buffer(&ftdic)) < 0) {
    sprintf(MessageConfigurer, "ftdi_usb_purge_rx failed: %d (%s)\n", ret,
        ftdi_get_error_string(&ftdic));
    status = -6;
  }
}

//----------------------------------------------------------------------------
int tFTDI::Envoyer(char* pChaine) {
  if (status < 0)
    return status;
  int ret = 0;
  if ((ret = ftdi_write_data(&ftdic, (unsigned char*) pChaine, strlen(pChaine)))
      < 0) {
    sprintf(MessageConfigurer, "ftdi_write_data failed: %d (%s)\n", ret,
        ftdi_get_error_string(&ftdic));
    return ERREUR;
  }
  return OK;
}

int tFTDI::Envoyer(unsigned char pChaine) {
  if (status < 0)
    return status;
  int ret = 0;
  if ((ret = ftdi_write_data(&ftdic, &pChaine, 1)) < 0) {
    sprintf(MessageConfigurer, "ftdi_write_data failed: %d (%s)\n", ret,
        ftdi_get_error_string(&ftdic));
    return ERREUR;
  }
  return OK;
}

int tFTDI::Envoyer(char* pChaine, int length) {
  if (status < 0)
    return status;
  int ret = 0;
  if ((ret = ftdi_write_data(&ftdic, (unsigned char*) pChaine, length)) < 0) {
    sprintf(MessageConfigurer, "ftdi_write_data failed: %d (%s)\n", ret,
        ftdi_get_error_string(&ftdic));
    return ERREUR;
  }
  return OK;
}

int tFTDI::Recevoir(char* pChaine, const int length) {
  if (status < 0)
    return status;

  char tmpChaine[length];
  int i = 0;
  int ret = 0;
  while (i < length) {
    if ((ret = ftdi_read_data(&ftdic, (unsigned char*) tmpChaine, length - i))
        < 0) {
      sprintf(MessageConfigurer, "ftdi_read_data failed: %d (%s)\n", ret,
          ftdi_get_error_string(&ftdic));
      return ERREUR;
    }
    if (ret > 0) {
      strncpy(&pChaine[i], tmpChaine, ret);
      i += ret;
    }
  }
  return i;
}

//----------------------------------------------------------------------------
int findAllUSB(int idCardVendor, int idCardProduct) {
  struct ftdi_context tmpftdi;
  if (ftdi_init(&tmpftdi) < 0)
  {
      fprintf(stderr, "ftdi_init failed\n");
      return EXIT_FAILURE;
  }
  struct ftdi_device_list *devlist=0L, *curdev=0L;
  char manufacturer[128], description[128],serial[128];

  int ret=0;
  if ((ret = ftdi_usb_find_all(&tmpftdi, &devlist, idCardVendor, idCardProduct)) < 0) {
    fprintf(stderr, "ftdi_usb_find_all failed: %d (%s)\n", ret,
        ftdi_get_error_string(&tmpftdi));
    return EXIT_FAILURE;
  }

  printf("Number of FTDI devices found: %d\n", ret);

  char* serial_list[6]={"FTDZVAS2","FTDW2MMW", "FTDW2UAT","FTCVHJ29", "FTCVHIHN","FTCVMXUA"};
  char* board_list[6]={"PAUTRON","ZIG_CTR","ZIG_CTR","ZIG_DATA","ZIG_DATA","HK_BOARD"};
  int i = 0;
  curdev = devlist;
  while(curdev!=NULL) {
    printf("Checking device: %d\n", i);
    if ((ret = ftdi_usb_get_strings(&tmpftdi, curdev->dev, manufacturer, 128,
        description, 128, serial, 128)) < 0) {
      fprintf(stderr, "ftdi_usb_get_strings failed: %d (%s)\n", ret,
          ftdi_get_error_string(&tmpftdi));
      //return EXIT_FAILURE;
    } else {
	printf("Manufacturer: %s, Description: %s, Serial: %s\n", manufacturer,
	       description, serial);
	for (int j = 0; j < 6; j++) {
	    if (strncmp(serial, serial_list[j], 8) == 0) {
		printf("identification_carte: %s   %d\n", board_list[j], tmpftdi.type);
	    }
	}
    }
    curdev = curdev->next;
    i++;
  }
  ftdi_deinit(&tmpftdi);
  return EXIT_SUCCESS;
}

