/************************************************************************/
/* @(#)times.c	1.6 1/13/94						*/
/*									*/
/* Time and Date Functions						*/
/*									*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : W. J. Tango 						*/
/* Date   : June 1991							*/
/*									*/
/************************************************************************/


/*
 *	INCLUDE FILES
 */

#include "chara.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>


/************************************************************************/
/*	GetTime()							*/
/*									*/
/*	GetTime() returns the current time (determined from the 	*/
/*	clock of the computer that you are using) as a (double)		*/
/*	decimal number equal to 					*/
/*	modified_julian_date.fraction_of_day_since_midnight		*/
/*	GetTime() also changes the value of its argument. The argument	*/
/*	is a TIME structure and the current modified Julian day number	*/
/*	(in days) and the UT (in seconds) are returned. Note that 	*/
/*	although these are both integer numbers they are stored as	*/
/*	doubles.							*/
/*									*/
/*	GetTime() assumes that the system clock returns time in		*/
/*	seconds elapsed from the epoch 1900.000:  it is portable 	*/
/*	to UNIX and Microsoft C compilers.				*/
/************************************************************************/

double GetTime(TIME *time_ptr)

{
	/*	INTERNALS	*/
	double ret_val;
	time_t raw_time;

	/*	Read the system clock	*/
	raw_time = (time_t)( time( (time_t *)0) );

	time_ptr->mjd = raw_time/(time_t)(SECS_IN_DAY) + (double)(MJD_UNIX);
	time_ptr->ut  = raw_time % (time_t)(SECS_IN_DAY);

	ret_val = ((double)raw_time)/(double)(SECS_IN_DAY)+(double)(MJD_UNIX);
	return ret_val;

}/*GetTime()*/

/************************************************************************/
/* get_std_time()							*/
/*									*/
/* Uses the system clock to get a standard tm structure (see ctime(3))	*/
/* whoes contents depend on the arguement.  Thus one can get UT (aka 	*/
/* GMT), or PST (with no daylight saving).  Useful for humans.		*/
/************************************************************************/

struct tm *get_std_time(int type)
{
	time_t systime;

	time(&systime);
	
	switch (type)
	{
	case UT:	break;
	case PST:	systime -= 28800; /* 8 hrs */
			break;
	default:	return NULL;
			break;
	}

	return gmtime((time_t *) &systime);
} /* get_std_time() */

/************************************************************************/
/* ntp_time_error()							*/
/*									*/
/* Tries to find out how far off the local clock is from an external	*/
/* time server. Argument is the name of the external time server.	*/
/* Returns offset in seconds.						*/
/* On error it returns a huge offset.					*/
/************************************************************************/

float ntp_time_error(char *time_server)
{
	char command[256],s[256];
	FILE	*in;
	char	*p;
	float	offset;

	sprintf(command,"/usr/sbin/ntpdate -q %s >& /tmp/ntp_time_error.tmp", 
		time_server);
	system(command);
	if ((in = fopen("/tmp/ntp_time_error.tmp", "r")) == NULL)
	{
		return 1e32;
	}
	if (fgets(s,256,in) == (char *)NULL)
	{
		fclose(in);
		unlink("/tmp/ntp_time_error.tmp");
		return 1e32;
	}
	unlink("/tmp/ntp_time_error.tmp");
	fclose(in);

	/* Read in the result */

	for(p=s; *p!='o' && *p != 0; p++);
	if (sscanf(p,"offset %f", &offset) != 1)
	{
		return 1e32;
	}

	/* That should be it */

	return offset;

} /* ntp_time_error() */
