/************************************************************************/
/* background.c								*/
/*                                                                      */
/* This file controls background job Control.				*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Tony Johnson & Theo ten Brummelaar                          */
/* Date   : Original Version 1990 - ported to Linux 1998                */
/************************************************************************/

/*
 * Include files
 */

#include "charaui.h"
#include <errno.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>

/*
 * Static global - TRUE if not looking at keyboard
 */

static bool key_bypass = FALSE;

/*
 * Public global - TRUE if 
 *	status window is to be refreshed every second.
 */

bool refresh_status_window = TRUE;

/*
 * Local globals for controlling icon testing 
 */

#define SLEEP_TIME 2 /* Number of seconds to sleep between tests */

static bool am_asleep = FALSE;

/*
 * Local variable for keeping the last character typed
 */

static int character_waiting = -1;

/*
 * Global variable to let other functions, like the error function
 * know that a background job is running.
 */

bool background_function = FALSE;

/* DO we process all clients in background*/

static bool process_all_clients_in_background = FALSE;

/* DO we process all messages of all clients in background? */

static bool process_all_messages_in_background = FALSE;

/*
 * Statics to control background process.
 */

#define TIME_REFRESH	0

static bool Background_on = FALSE;/* Should we process background task ? */
static bool Background_started = FALSE; /* Have we started the background ? */
static int (*(background_jobs[MAX_JOBS]))() = 
		{ NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,
		  NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,
		  NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL};
static	int	current_job = 0;
static	time_t 	refresh_time = 1;
static	char	*job_names[MAX_JOBS] =
		{ NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,
		  NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,
		  NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL};
static int (*top_job)(void) = NULL;
static bool top_job_turn = TRUE;

/************************************************************************/
/* get_command()							*/
/*									*/
/* Gets a key from user ( any key ).					*/
/* If the key is KEY_REFRESH it will repaint the screen and wait	*/
/* for another key.							*/
/* Will call function background while waiting for key stroke.		*/
/* ^F is translated to KEY_NPAGE.					*/
/* ^B is translated to KEY_PPAGE.					*/
/* ^U is translated to KEY_UP.						*/
/* ^D is translated to KEY_DOWN.					*/
/* ^K is translated to KEY_LEFT.					*/
/* ^L is translated to KEY_RIGHT.					*/
/* Uses kbhit function to poll keyboard.				*/
/*									*/
/************************************************************************/

int	get_command(void)
{
	int	command;

	while(key_bypass) 
	{
		process_command_socket();
		process_message_socket();
		data_connected();
		background();
	}

	while(TRUE)
	{
		while (!kbhit()) 
		{
		    if (!inside_error_function)
		    {
			process_command_socket();
		        if (process_all_clients_in_background)
				process_message_socket_all_clients(
					process_all_messages_in_background);
			else
				process_message_socket();
			data_connected();
			background();
		    }
		}

		if (character_waiting == -1)
			command = wgetch(system_window);
		else
		{
			command = character_waiting;
			character_waiting = -1;
		}
		if (command != KEY_REFRESH_L && command != KEY_REFRESH_R) break;
		wrefresh(curscr);
	}

	switch (command)
	{
	case 0x0b: /* ^K */
		command = KEY_LEFT;
		break;
	case 0x0c: /* ^L */
		command = KEY_RIGHT;
		break;
	case 0x04: /* ^D */
		command = KEY_DOWN;
		break;
	case 0x15: /* ^U */
		command = KEY_UP;
		break;
	case 0x06: /* ^F */
		command = KEY_NPAGE;
		break;
	case 0x02: /* ^B */
		command = KEY_PPAGE;
		break;
	default:
		break;
	}
	return command;

} /* get_command() */

/************************************************************************/
/* kbhit()								*/
/*									*/
/* Keyboard polling function, returns TRUE if a character is ready	*/
/* and FALSE if not. Note that this used to use the select() function   */
/* but that seems to cause problems... I suspect the linux code is	*/
/* buggy. Anyway, it is now done by using a NON_BLOCKING stdin and so   */
/* it is VERY IMPORTANT that you use get_command() after this to get	*/
/* any key typed, otherwise you may loose the key altogether.		*/
/************************************************************************/

bool kbhit(void)
{
	if (character_waiting != -1) return TRUE;

	if ((character_waiting = wgetch(system_window)) == -1)
		return FALSE;
	else
		return TRUE;

} /* kbhit() */

/************************************************************************/
/* char_waiting()							*/
/*									*/
/* Unix version of kbhit like function. Returns TRUE if a character 	*/
/* waiting at the port, FALSE otherwise.				*/
/* Note that this function can cause problems, so for real keyboard	*/
/* use I go back to a non-block keyboard I/O.				*/
/************************************************************************/

int	char_waiting(int fd)
{
	/* Locals */

	fd_set rfds;		/* File discriptor set */
        struct timeval tv;	/* Time to wait */

	if (fd <0) return FALSE;

	/* Set tings up */

	FD_ZERO(&rfds);		 /* Zero the file discriptor set */
        FD_SET(fd, &rfds);	 /* Add stdin to the set */
	tv.tv_sec = 0;		 /* Wait no more than 1 usec */
	tv.tv_usec = 1;

	if (select(FD_SETSIZE, &rfds, NULL, NULL, &tv) < 0) 
	{
		/* Something is wrong. */

		if (errno != EINTR)
		error(WARNING,
			"char_waiting(): Select() Function returned errno %d.",
			errno);
		return FALSE;
	}

	if (FD_ISSET(fd, &rfds)) /* Was there something? */
	{
		return TRUE;
	}

	/* Must have been nothing there */

	return FALSE;

} /* char_waiting() */

/************************************************************************/
/* write_ready()							*/
/*									*/
/* Returns TRUE if a port is ready for writing,	FALSE otherwise.	*/
/* Uses the select(2) routine to do it's work. 				*/
/************************************************************************/

int	write_ready(int fd)
{
	/* Locals */

	fd_set wfds;		/* File discriptor set */
        struct timeval tv;	/* Time to wait */

	if (fd <0) return FALSE;

	/* Set tings up */

	FD_ZERO(&wfds);		 /* Zero the file discriptor set */
        FD_SET(fd, &wfds);	 /* Add fd to the set */
	tv.tv_sec = 0;		 /* Wait no more than 1 usec */
	tv.tv_usec = 1;

	if (select(FD_SETSIZE, NULL, &wfds, NULL, &tv) < 0) 
	{
		/* Something is wrong. */

		if (errno != EINTR)
		error(WARNING,
			"write_ready(): Select() Function returned errno %d.",
			errno);
		return FALSE;
	}

	if (FD_ISSET(fd, &wfds)) /* Was there something? */
	{
		return TRUE;
	}

	/* Must have been nothing there */

	return FALSE;

} /* write_ready() */

/************************************************************************/
/* exception()								*/
/*									*/
/* Returns TRUE if a port has an exception, FALSE otherwise.		*/
/* Uses the select(2) routine to do it's work. 				*/
/* From charaui!!  							*/
/************************************************************************/

int	exception(int fd)
{
	/* Locals */

	fd_set efds;		/* File discriptor set */
        struct timeval tv;	/* Time to wait */

	if (fd <0) return FALSE;

	/* Set tings up */

	FD_ZERO(&efds);		 /* Zero the file discriptor set */
        FD_SET(fd, &efds);	 /* Add fd to the set */
	tv.tv_sec = 0;		 /* Wait no more than 1 usec */
	tv.tv_usec = 1;

	if (select(FD_SETSIZE, NULL, NULL, &efds, &tv) < 0) 
	{
		/* Something is wrong. */

		if (errno != EINTR)
		error(WARNING,
			"exception(): Select() Function returned errno %d.",
			errno);
		return FALSE;
	}

	if (FD_ISSET(fd, &efds)) /* Was there something? */
	{
		return TRUE;
	}

	/* Must have been nothing there */

	return FALSE;

} /* exception() */

/************************************************************************/
/* background()								*/
/*									*/
/* Responsible for looking after the background jobs. All jobs in the	*/
/* array background_jobs[] are processed, one each time this function	*/
/* is called. Approx every second the window status_window is		*/
/* refreshed (If this function is called in time that is). This refresh */
/* can be stopped by setting the global variable 			*/
/*  refresh_status_window = FALSE;					*/
/* This is called whenever the programme waits for a keystroke via 	*/
/* get_command(). 							*/
/* Returns error level.							*/
/************************************************************************/

int background(void)
{
	time_t current_time;
	int  last_job;
	int  error_level;

	if (!Background_on) return NOERROR;

	/* Get the current time */

	time(&current_time);

	/* See if it's time for a refresh of the status_window */

	if (current_time > refresh_time)
	{
		if(refresh_status_window)
		{ 
			touchwin(status_window);
			wrefresh(status_window);
		}
		refresh_time = current_time;
	}

	/* Is it time to do the top job? */

	if (top_job != NULL && top_job_turn)
	{
		background_function = TRUE;
		error_level = (*top_job)();
		background_function = FALSE;
		if (error_level != NOERROR)
		{
			background_stop(0,NULL);
				return error(WARNING,
				   "Error in top job - Background stopped.");
		}
		top_job_turn = FALSE;

		if (time(NULL) > current_time + 2)
			message(system_window,
			"The top_job() is taking a long time to run.");

		return NOERROR;
	}
	else
	{
		top_job_turn = TRUE;
	}

	/* Otherwise find next non NULL job */

	last_job = current_job;
	do
	{
		if (++current_job >= MAX_JOBS) current_job = 0;
	} while (current_job != last_job && 
		 background_jobs[current_job] == NULL);

	if (background_jobs[current_job] != NULL) 
	{
		
		background_function = TRUE;
		error_level = (*(background_jobs[current_job]))();
		background_function = FALSE;
		if (error_level != NOERROR)
		{
			background_stop(0,NULL);
			if (job_names[current_job] == NULL)
			{
				return error(WARNING,
			   "Error in background %d - Background stopped.",
					current_job);
			}
			else
			{
				return error(WARNING,
			"Error in background job %s - Background stopped.",
					job_names[current_job]);
			}
		}

		if (time(NULL) > current_time + 2)
		{
			if (job_names[current_job] == NULL)
			{
				message(system_window,
			"Background job %d is taking a long time to run.",
					current_job);
			}
			else
			{
				message(system_window,
			"Background job %s is taking a long time to run.",
					job_names[current_job]);
			}
		}
	}

	/* Service the status mirrors */

#ifdef MIRROR_AS_BACKGROUND
	return NOERROR;
#else
	return mirror_background();
#endif

} /* background() */

/************************************************************************/
/* reset_status_window_refresh()					*/
/*									*/
/* Set's refresh_time = 1						*/
/************************************************************************/

int reset_status_window_refresh(int argc, char **argv)
{
	refresh_time = 1;
	return NOERROR;

} /* reset_status_window_refresh() */

/************************************************************************/
/* background_on()							*/
/*									*/
/* Turns background processing on. Always returns NOERROR.		*/
/************************************************************************/

int background_on(int argc, char **argv)
{
	if (Background_started) Background_on = TRUE;
	werase(status_window);
	touchwin(status_window);
	wrefresh(status_window);
	reset_status_window_refresh(0, NULL);

	return NOERROR;

} /* background_on() */

/************************************************************************/
/* background_off()							*/
/*									*/
/* Turns background processing off. Always returns NOERROR.		*/
/************************************************************************/

int background_off(int argc, char **argv)
{
	Background_on = FALSE;
	werase(status_window);
	touchwin(status_window);
	wrefresh(status_window);

	return NOERROR;

} /* background_off() */

/************************************************************************/
/* background_start()							*/
/*									*/
/* Turns background processing on. Always returns NOERROR.		*/
/* Should only be called once.						*/
/************************************************************************/

int background_start(int argc, char **argv)
{
	Background_started = TRUE;
	Background_on = TRUE;
	reset_status_window_refresh(0, NULL);
	return NOERROR;

} /* background_start() */

/************************************************************************/
/* background_stop()							*/
/*									*/
/* Turns background processing off. Always returns NOERROR.		*/
/* Need to call background_start() again to get it going.		*/
/************************************************************************/

int background_stop(int argc, char **argv)
{
	Background_started = FALSE;
	Background_on = FALSE;
	werase(status_window);
	touchwin(status_window);
	wrefresh(status_window);
	return NOERROR;

} /* background_stop() */

/************************************************************************/
/* background_add_name()						*/
/*									*/
/* Adds a function to the background processing. Returns errorlevel.	*/
/* The name argument may be NULL 					*/
/************************************************************************/

int	background_add_name(int (*job)(void), char *name)
{
	int name_length;
	int i;	/* For looping */

	name_length = 0;
	if (name != NULL)
	{
		name_length = strlen(name);
	}

	/* Loop through job table and put job in first empty slot */

	for(i=0;TRUE ;i++)
	{
		if (i >= MAX_JOBS) return error(WARNING,
			"No more space in background for new jobs.");

		if (background_jobs[i] == NULL)
		{
			background_jobs[i] = job;
			if(name_length > 0)
			{
				job_names[i] = malloc(name_length + 1);
				if(job_names[i] == NULL)
				{
					error(FATAL, "Unable to allocate memory for background job name %s", name);
				}
				strcpy(job_names[i], name);
			}
			else
			{
				job_names[i] = NULL;
			}
			return NOERROR;
		}
	}

} /* background_add_name() */

/************************************************************************/
/* background_add()							*/
/*									*/
/* Adds a function to the background processing. Returns errorlevel.	*/
/* The function has no name */
/************************************************************************/

int	background_add(int (*job)(void))
{
	return background_add_name(job, NULL);

} /* background_add() */

/************************************************************************/
/* background_del()							*/
/*									*/
/* Deletes a function from background processing. Returns errorlevel.	*/
/************************************************************************/

int	background_del(int (*job)(void))
{
	int i;	/* For looping */

	for(i=0; i<MAX_JOBS; i++)
	{
		if (background_jobs[i] == job) 
		{
			background_jobs[i] = NULL;
			if(job_names[i] != NULL) free(job_names[i]);
			werase(status_window);
		}
	}

	return NOERROR;

} /* background_del() */

static	char	password[81];

/************************************************************************/
/* bypass_key_poll()							*/
/*									*/
/* Sets key_bypass to true and signal catching to bypas_key_poll_off().	*/
/* The programme will no longer look at the keyboard until a ^C		*/
/* is typed.								*/
/************************************************************************/

int	bypass_key_poll(int argc,char **argv)
{
	no_socket();	/* We can not do this from a socket */

	key_bypass = TRUE;

	signal( SIGINT, bypass_key_poll_off);	/* Catch ^C */

	if (argc < 2)
	{
		*password = 0;
		message(system_window,
			"KEYBOARD POLLING BYPASSED ... TYPE ^C TO RESTART.");
	}
	else
	{
		strcpy(password,argv[1]);
		message(system_window,
			"KEYBOARD LOCKED WITH PASSWORD ... TYPE ^C TO UNLOCK.");
	}

	return NOERROR;

} /* bypass_key_poll() */

/************************************************************************/
/* bypass_key_poll_off()						*/
/*									*/
/* Sets key_bypass to FALSE and resets signal catching to ui_end().	*/
/************************************************************************/

void	bypass_key_poll_off(int sig)
{
	char	att_password[81];

	key_bypass = FALSE;

	if (*password != 0)
	{
		wrefresh(curscr);
		*att_password = 0;
		message(system_window,"Please enter password.");
		signal(SIGINT, call_wrefresh_curscr);
		
		if ((line_edit(command_window,0,2,att_password,
				81,STRING,return_true,TRUE) == KEY_ESC) || 
				(strcmp(password,att_password) != 0))
		{
			key_bypass = TRUE;
			message(system_window,
			"KEYBOARD LOCKED WITH PASSWORD ... TYPE ^C TO UNLOCK.");
			signal(SIGINT, bypass_key_poll_off);
			clean_command_line();
			return;
		}

		clean_command_line();
		*password = 0;
	}

	signal(SIGINT, sig_end);
	werase(system_window);
	wrefresh(system_window);
	wrefresh(curscr);

} /* bypass_key_poll_off() */

void	call_wrefresh_curscr(int sig) 
{ 
	wrefresh(curscr);
	signal(SIGINT, call_wrefresh_curscr);
}

/************************************************************************/
/* go_to_sleep()							*/
/*									*/
/* This function puts programme to sleep. 				*/
/* Basically it sets the local global am_asleep to TRUE. Returns NOERROR*/
/************************************************************************/

int	go_to_sleep(int argc, char **argv)
{
	am_asleep = TRUE;

	no_socket();	/* We can not do this from a socket */

	signal( SIGINT, wake_up);	/* Catch ^C */

	werase(status_window);
	mvwprintw(status_window,4,15,
		"THIS CONTROLLER SLEEPING - TYPE ^C TO WAKE ME UP.");
	wrefresh(status_window);

	all_socket_print("Controller sleeping, commands will be ignored.\n");

	while(am_asleep)
	{
		sleep(SLEEP_TIME);
	}

	return NOERROR;

} /* go_to_sleep() */

/************************************************************************/
/* wake_up()								*/
/*									*/
/* Sets am_asleep to FLASE and resets interrupt. Returns nothing.	*/
/************************************************************************/

void	wake_up(int sig)
{
	am_asleep = FALSE;

	flush_sockets();

	werase(status_window);
	wrefresh(status_window);
	signal(SIGINT, sig_end);
	wrefresh(curscr);

} /* wake_up() */

/************************************************************************/
/* set_top_job()							*/
/*									*/
/* Set's a function to be the top background job.			*/
/* Setting the top job to NULL will turn the thing off.			*/
/************************************************************************/

void set_top_job( int (*new_top_job)())
{
	top_job = new_top_job;

} /* set_top_job() */

/************************************************************************/
/* sleep_with_background()						*/
/*									*/
/* WIll run background for a set nunmber of seconds and return. It 	*/
/* starts at a random time with the first second so will almost always	*/
/* be a little shorter than you ask for.				*/
/************************************************************************/

void sleep_with_background(int seconds)
{
	time_t start;

	start = time(0);

	while(time(0) <= start+seconds) background();

} /* sleep_with_background() */

/************************************************************************/
/* set_process_all_clients_in_background()				*/
/*									*/
/* Dis/Enables processing all clients, or all clients AND all messages 	*/
/* in the background.							*/
/************************************************************************/

void set_process_all_clients_in_background(bool clients, bool messages)
{
	process_all_clients_in_background = clients;
	process_all_messages_in_background = messages;

} /* set_process_all_clients_in_background() */
