/************************************************************************/
/* menuhelp.c								*/
/*                                                                      */
/* 	This file contains one function :				*/
/*									*/
/*		get_menu_help()						*/
/*									*/
/*	This function allows access to the help system via the menu	*/
/*	system.								*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Tony Johnson & Theo ten Brummelaar                          */
/* Date   : Original Version 1990 - ported to Linux 1998                */
/************************************************************************/

/*
 * Include files
 */

#include "charaui.h"

/************************************************************************/
/* get_menu_help()							*/
/*									*/
/* This function allows access to the help system via the menu		*/
/* system.								*/
/*									*/
/* The help() function is called given one of the available functions 	*/
/* of the current menu as the argument.					*/
/*									*/
/************************************************************************/

void	get_menu_help(struct smenu *thismenu)
{
	/* Internal Variables */

	int	keystroke;		/* Key typed by user */
	struct smenu	*menu_pt;	/* Menu pointed at by menu entry */
	int	function_pt;		/* Function pointed at by menu entry */

	/* Put help message into system window */

	werase(system_window);
	mvwprintw(system_window,0,0,"Help for menu: %s",thismenu->name);
	mvwaddstr(system_window,1,0,
	      "Type the key you would like help for or <ESC> to exit.");
	wrefresh(system_window);

	/* See which menu entry user wants help on */

	while(TRUE)
	{
		switch(keystroke = get_user_input(system_window,&menu_pt))
		{
		case KEY_F(1) :
		case KEY_F(2) :
		case KEY_F(3) :
		case KEY_F(4) :
		case KEY_F(5) :
		case KEY_F(6) :
		case KEY_F(7) :
		case KEY_F(8) :
		case KEY_F(9) :
		case KEY_F(10) : keystroke = keystroke - KEY_F(1);

				/* Set menu and function index pointer */

				menu_pt = thismenu->entry[keystroke].pt_menu;
				function_pt = 
					thismenu->entry[keystroke].pt_funct;

				/* If it is a function request call ui_help() */

				if (function_pt != 0) 
				{
					call_function(find_function(F_HELP),
						functions[function_pt].name);
					werase(system_window);
					wrefresh(system_window);
					return;
				} 

				/* If it is a menu say so */
				else if (menu_pt != (struct smenu *)NULL)
				{
					werase(system_window);
					mvwprintw(system_window,0,0,
		"Typing that key will place you in menu %s",menu_pt->name);	
					wrefresh(system_window);
					return;
				}
				
				/* Otherwise ignore the key stroke */

				break;

		case KEY_ESC : 	werase(system_window); /* Leave menu help */
				wrefresh(system_window);
				return;

		default	    : 	break;  /* Non menu help key ignored */
		}
	}

} /* get_menu_help() */
