/***********************************************************************eck_/
/* scred.c								*/
/*                                                                      */
/* Discription                                   	 		*/
/*	This is the main code for the screen editor.			*/
/* 	Functions contained in here are					*/
/*		edit_page()	page editor				*/
/*		scred()		screen editor				*/
/*		return_true()	always returns TRUE			*/
/*		quick_edit()	For entering single values easily	*/
/*				with prompts etc.			*/
/*                                                                      */
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Tony Johnson & Theo ten Brummelaar                          */
/* Date   : Original Version 1990 - ported to Linux 1998                */
/************************************************************************/

/*
 * Include files
 */

#include "charaui.h"

/************************************************************************/
/* edit_page()								*/
/*									*/
/* Given a pointer to an edit page structure as defined in scred.h	*/
/* this function allows screen editing based on the line_edit() and	*/
/* the pick_choice() functions. Normal movement keys for both MS-DOG 	*/
/* and UNIX apply. Returns the value of the key used to exit, for 	*/
/* example KEY_PPAGE.							*/
/************************************************************************/

int	edit_page(struct scred_page *page)
{
	/* Internal variables */

	char	*default_string;/* Default value for string <-- weird bug */
	int	edit_value;	/* For enumerated types */
	char	edit_string[81];/* For all other types */
	int	i;		/* For loops */
	int	current_field;	/* That is being edited */
	bool	loop;		/* For main loop */
	int	exit_key = 0;	/* Key used to leave field */
	bool	return_value;	/* Of setup() or check_value() function */

	/* put up the page title */

	heading(heading_window,page->title,"");

	active_window = sub_main_window;
	werase(sub_main_window);
	touchwin(sub_main_window);

	/* put up all the prompts and defaults */

	for (i = 0; i < page->number_fields; i++)
	{
		mvwaddstr(sub_main_window,page->fields[i].y_pos_prompt,
				page->fields[i].x_pos_prompt,
				page->fields[i].prompt);

		if(page->fields[i].field_type == ENUMERATED)
		{
			(*page->fields[i].setup)(&edit_value,FALSE);
			default_string = 
				page->fields[i].enum_strings[edit_value];
			mvwaddstr(sub_main_window,page->fields[i].y_pos_field,
				page->fields[i].x_pos_field,default_string);
		}
		else
		{
			(*page->fields[i].setup)(edit_string,FALSE);
			mvwaddstr(sub_main_window,page->fields[i].y_pos_field,
				page->fields[i].x_pos_field,edit_string);
		}
	}

	/* Refresh the window */

	wrefresh(sub_main_window);

	/* Initialise */

	current_field = 0;
	loop = TRUE;

	/* Start main loop */

	while(loop)
	{
		/* Highlight the current prompt */

		wstandout(sub_main_window);

		mvwaddstr(sub_main_window,
				page->fields[current_field].y_pos_prompt,
				page->fields[current_field].x_pos_prompt,
				page->fields[current_field].prompt);
		wrefresh(sub_main_window);
		wstandend(sub_main_window);

		/* Set up appropriate editor value */

		if(page->fields[current_field].field_type == ENUMERATED)
		{
			return_value = 
		      (*page->fields[current_field].setup)(&edit_value,TRUE);
		}
		else
		{
			return_value = 
		      (*page->fields[current_field].setup)(edit_string,TRUE);
		}

		/* See if we are allowed to edit this field yet */

		if (!return_value)
		{
			/* If not go back to previous field */

			if (current_field == 0)
			{
				werase(system_ol_window);
				mvwaddstr(system_ol_window,0,0,
				"Sorry --- not ready to edit this page yet !");
				wrefresh(system_ol_window);
				return KEY_PPAGE;
			}

			/* Unighlight the current prompt */

			mvwaddstr(sub_main_window,
				page->fields[current_field].y_pos_prompt,
				page->fields[current_field].x_pos_prompt,
				page->fields[current_field].prompt);
			wrefresh(sub_main_window);

			/* Force your way back to last field */

			current_field--;

			/* Go back to top of loop */

			continue;
		}

		/* invoke appropriate editor */

		if(page->fields[current_field].field_type == ENUMERATED)
		{
			exit_key = pick_choice(sub_main_window,
				page->fields[current_field].y_pos_field,
				page->fields[current_field].x_pos_field,
				&edit_value,
				page->fields[current_field].enum_strings,
				page->fields[current_field].check_value);
		}
		else
		{
			exit_key = line_edit(sub_main_window,
				page->fields[current_field].y_pos_field,
				page->fields[current_field].x_pos_field,
				edit_string,
				page->fields[current_field].field_length,
				page->fields[current_field].field_type,
				page->fields[current_field].check_value,
				FALSE);
		}

		/* Unighlight the current prompt */

		mvwaddstr(sub_main_window,
				page->fields[current_field].y_pos_prompt,
				page->fields[current_field].x_pos_prompt,
				page->fields[current_field].prompt);
		wrefresh(sub_main_window);

		/* Erase system_window */

		werase(system_window);
		wrefresh(system_window);
			
		/* Check to see what exit key was */

		switch(exit_key)
		{
			case KEY_PPAGE :
			case KEY_NPAGE : 
			case KEY_ESC   :
					loop = FALSE;
					break;

			case KEY_DOWN :
			case KEY_RIGHT :
			case '\r' :
					if (++current_field >= 
						page->number_fields)
					{
						exit_key = KEY_NPAGE;
						loop = FALSE;
					}
					break;

			case KEY_UP :
			case KEY_LEFT :
					if (--current_field < 0) 
					{
						exit_key = KEY_PPAGE;
						loop = FALSE;
					}
					break;

			case KEY_HOME:
					current_field = 0;
					break;

			case KEY_LL:
					current_field = page->number_fields - 1;
					break;

		}

	} /* End main loop */

	/* Test all fields and set page_ok value */

	page->page_ok = TRUE;

	for(i=0; i< page->number_fields; i++)
	{
		if (page->fields[i].field_type == ENUMERATED)
		{
			(*page->fields[i].setup)(&edit_value,FALSE);
			page->fields[i].field_ok = 
				(*page->fields[i].check_value)(&edit_value);
		}
		else
		{
			(*page->fields[i].setup)(edit_string,FALSE);
			page->fields[i].field_ok = 
				(*page->fields[i].check_value)(edit_string);
		}

		page->page_ok = page->fields[i].field_ok && page->page_ok;
	}

	/* Clear the system_window */

	werase(system_window);
	wrefresh(system_window);

	/* Exit routine */

	return exit_key;

} /* edit_page() */

/************************************************************************/
/* scred()								*/
/* Process 6.5								*/
/*									*/
/* This function uses edit_page() and a structure containing edit pages	*/
/* to create a multipage editor. A complete set of check() and setup()	*/
/* funtions are required for this to work. Return value is either TRUE  */
/* FALSE reflecting whether all the pages have been visited and if all  */
/* values pass their checks.						*/
/************************************************************************/

bool	scred(struct scred *page_list)
{
	/* internal variables */

	int	current_page;	/* that we are editing */
	bool	return_value;	/* to send back to calling function */
	bool	loop;		/* for main loop control */
	int	i;		/* looping variable */

	/* Initialise */

	current_page = 0;
	return_value = TRUE;
	loop = TRUE;

	/* Start main loop */

	while(loop)
	{
		switch(edit_page(page_list->pages[current_page]))
		{
		case KEY_NPAGE : if (++current_page > 
					page_list->number_pages-1) loop = FALSE;
				 break;

		case KEY_PPAGE : if (--current_page < 0) loop = FALSE;
				 break;

		case KEY_ESC   : loop = FALSE;
				 break;
		}
	}

	/* test all page_ok's and set return value */

	for(i=0; i< page_list->number_pages; i++)
		return_value = return_value && page_list->pages[i]->page_ok;

	/*  return it */

	return return_value;

} /* scred() [6.5] */

/************************************************************************/
/* copy_scred_page()							*/
/*									*/
/* Function that completely copies one scred page into another.	*/
/* The function returns void.						*/
/*									*/
/************************************************************************/

void	copy_scred_page(
		struct scred_page *page_target,
		struct scred_page *page_source)
{
	/* Internal Variables */

	int	field;	/* Field number */

	/* OK here we go */

	page_target->title = page_source->title;
	page_target->number_fields = page_source->number_fields;
	page_target->page_ok = page_source->page_ok;

	for(field = 0; field < MAXFIELDS; field++)
		copy_scred_field(page_target,field,page_source,field);

} /* copy_scred_page() */

/************************************************************************/
/* copy_scred_field()							*/
/*									*/
/* Copies a given field in one page into a given field in another.	*/
/* The function returns void.						*/
/*									*/
/************************************************************************/

void	copy_scred_field(
struct scred_page	*page_target,	/* Target page */
int			field_target,	/* Target field in target page */	
struct scred_page	*page_source,	/* Source page */
int			field_source)	/* Source field in source page */	
{
	page_target->fields[field_target].field_length =
		page_source->fields[field_source].field_length;

	page_target->fields[field_target].field_type =
		page_source->fields[field_source].field_type;

	page_target->fields[field_target].enum_strings =
		page_source->fields[field_source].enum_strings;

	page_target->fields[field_target].prompt =
		page_source->fields[field_source].prompt;

	page_target->fields[field_target].y_pos_prompt =
		page_source->fields[field_source].y_pos_prompt;

	page_target->fields[field_target].x_pos_prompt =
		page_source->fields[field_source].x_pos_prompt;

	page_target->fields[field_target].y_pos_field =
		page_source->fields[field_source].y_pos_field;

	page_target->fields[field_target].x_pos_field =
		page_source->fields[field_source].x_pos_field;

	page_target->fields[field_target].setup =
		page_source->fields[field_source].setup;

	page_target->fields[field_target].check_value =
		page_source->fields[field_source].check_value;

	page_target->fields[field_target].field_ok =
		page_source->fields[field_source].field_ok;

} /* copy_scred_field() */

/************************************************************************/
/* return_true()							*/
/*									*/
/* A function that will simply return TRUE to the calling function.	*/
/* Use as the checking function for the enumerated type editor.		*/
/*									*/
/************************************************************************/

bool	return_true(void *ptr)
{
	return TRUE;
}

/************************************************************************/
/* quick_edit()								*/
/*									*/
/* This function prompts the user for input via the system window and	*/
/* allows editing on the command line. See synopsys for a description	*/
/* of the arguments. 							*/
/* If the value_type is not ENUMERATED, the strings[] array need not be */
/* supplied and should be NULL. If the value_type is STRING any 	*/
/* can be typed in, if it is FLOAT any number, decimal point, E and G	*/
/* will be allowed, and if it is integer, only numbers will be allowed.	*/
/*									*/
/* It is up to the application programme to scan the value out of the	*/
/* edited string.							*/
/*									*/
/* The return value is the key used to exit the editor. If this was 	*/
/* KEY_ESC, the value array will not be changed.			*/
/*									*/
/************************************************************************/

int	quick_edit(
char	*item,		/* string describing what is being changed */
char	*status,	/* Default value */
void	*value,		/* pointer to string to be played with  */
char	*strings[],	/* string array for enumerated types */
int	value_type)	/* STRING, FLOAT, INTEGER or ENUMERATED */
{
	/* Internal variables */

	int	c;		/* exit key from editor */

	/* lets do it.... */

	if (active_socket == -1)
	{
	    if(value_type == ENUMERATED)
	    {
		message(system_window,
			"%s is currently: %s\n%s",item,status,ENUM_ED_TEXT);
		c = pick_choice(command_window,0,2,value,strings,return_true);
	    }
	    else
	    {
		message(system_window,
			"%s is currently: %s\nPlease enter new %s",
			item,status,item);
		c = line_edit(command_window,0,2,value,strlen(value)+1,
			value_type,return_true,TRUE);
	    }

	    clean_command_line();
	    werase(system_window);
	    wrefresh(system_window);
	}
	else
	{
	    if(value_type == ENUMERATED)
	    {
		socket_print(active_socket,
			"%s is currently: %s\n%s\n",item,status,ENUM_ED_TEXT);
		c = pick_choice(command_window,0,2,value,strings,return_true);
	    }
	    else
	    {
		socket_print(active_socket,
			"%s is currently: %s\nPlease enter new %s\n",
			item,status,item);
		c = line_edit(command_window,0,2,value,strlen(value)+1,
			value_type,return_true,TRUE);
	    }
	}

	return c;

} /* quick_edit() */
