/************************************************************************/
/* uiauto.c 								*/
/*                                                                      */
/*	Performs the automatic function calling using the autolist[]	*/
/*	array. The functions are :					*/
/*									*/
/*		init_auto() - Initialises system			*/
/*		ui_auto() -  Does the work				*/
/*		ui_auto_socket() - Version for sockets.			*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Tony Johnson & Theo ten Brummelaar                          */
/* Date   : Original Version 1990 - ported to Linux 1998                */
/************************************************************************/
/*
 * Include files
 */

#include "charaui.h"

/* Globals */

int autolist[MAXAUTO]; /*Array of function indexes for auto proceedure */
char autoargs[MAXAUTO][81]; /* Arguments for auto list functions */

/*
 * Some special keys used
 */

#define	KEY_SKIP	's'		/* Go back on in the list */
#define	KEY_BACKONE	KEY_BACKSPACE	/* Go back on in the list */
#define KEY_STOP	KEY_ESC		/* ESCAPE to leave auto system */
#undef  KEY_NEXT			/* Used in ncurses.h */
#define KEY_NEXT	' '		/* Call next function */
#define SKIP_TEXT	"<s>	       Skip this function call"
#define BACKONE_TEXT	"<BACKSPACE>   Go back to the previous function"
#define STOP_TEXT	"<ESC>         Leave the Auto system"
#define NEXT_TEXT_1	"<SPACEBAR>    Call the next function"
#define NEXT_TEXT_2	"<SPACEBAR>    Move on to the next function"
#define HELP_TEXT	"<?>           Get help on the next function"
#define GOTO_TEXT	"<g>           Go back to this function"

/* Local global */

int auto_position;

/************************************************************************/
/* init_auto()								*/
/* Process 3.2								*/
/*									*/
/* Initialise the auto system.						*/
/* Look at the source for more information on what it does.		*/
/*									*/
/************************************************************************/

void	init_auto(void)
{
	auto_position = 1;

} /* init_auto() [3.2] */

/************************************************************************/
/* ui_auto()								*/
/*									*/
/* Follows the autolist array and calls the functions in order.		*/
/* Returns an error level (always NOERROR at the moment).		*/
/* It will allow the user to call the help system for any		*/
/* of the functions.							*/
/* 									*/
/* Consult analysis, design and source for more information.		*/
/*									*/
/************************************************************************/

int	ui_auto(int argc, char **argv)
{
	/* Internal variables */

	int	answer;		/* Key hit by user as answer to question */
	char	args[81];	/* Argument list sent to function caller */
#ifdef __NCURSES_H
        MEVENT  mouse;          /* A mouse event. */
#endif

	if (active_socket != -1) return ui_auto_socket();

	/* Clean up the command line */

	clean_command_line();

	/* Check if user wants to start from where they left auto system */

	if (auto_position != 1 && 
		!ask_yes_no("Start from where auto system was up to before?",
			""))
	{
		init_auto();
	}

	/* OK start moving down the list */

	while(TRUE)
	{
		/* See if we have reached the end of the auto list */

		if (autolist[auto_position] == 0)
		{
			init_auto();
			werase(system_window);
			mvwaddstr(system_window,0,0,
				"Completed automatic function list");
			wrefresh(system_window);
			return	NOERROR;
		}

		/* First set up the display page */

		heading(heading_window,"Automated function caller","");

		active_window = sub_main_window;
		werase(sub_main_window);
		if(autolist[auto_position] >= 0)
		{
			mvwprintw(sub_main_window,0,0,
				"The next function in the list is:");
			wstandout(sub_main_window);
			mvwprintw(sub_main_window,1,0,"%s %s",
				functions[autolist[auto_position]].name,
				autoargs[auto_position]);
			wstandend(sub_main_window);
			mvwaddstr(sub_main_window,4,10,NEXT_TEXT_1);
			mvwaddstr(sub_main_window,7,10,HELP_TEXT);
		}
		else
		{
			mvwprintw(sub_main_window,0,0,
			"There is an optional move back to the function:"); 
			wstandout(sub_main_window);
			mvwprintw(sub_main_window,1,0,"%s %s",
			   functions[autolist[-autolist[auto_position]]].name,
			   autoargs[-autolist[auto_position]]);
			wstandend(sub_main_window);
			mvwaddstr(sub_main_window,4,10,NEXT_TEXT_2);
			mvwaddstr(sub_main_window,7,10,GOTO_TEXT);
		}

		mvwaddstr(sub_main_window,2,0,
				"Choose from the following operations :");
		mvwaddstr(sub_main_window,3,10,SKIP_TEXT);
		mvwaddstr(sub_main_window,5,10,BACKONE_TEXT);
		mvwaddstr(sub_main_window,6,10,STOP_TEXT);

		touchwin(sub_main_window);
		wrefresh(sub_main_window);

		/* Get a keystroke or mouse click */

		answer = get_command();

#ifdef __NCURSES_H
                if (answer == KEY_MOUSE)
                {
                        getmouse(&mouse);

                        if (mouse.bstate == BUTTON1_CLICKED)
                        {
                                /* OK, what key did the mouse represent. */

                                if (mouse.y == (SUB_MAIN_BEGIN_Y + 3) &&
					mouse.x > 11 &&
                                        mouse.x <=  11 + strlen(SKIP_TEXT))
                                {
                                        answer = KEY_NEXT;
                                }
                                else if (mouse.y == (SUB_MAIN_BEGIN_Y + 4) &&
					mouse.x > 11 &&
                                        mouse.x <=  11 + strlen(NEXT_TEXT_2))
                                {
                                        answer = KEY_NEXT;
                                }
				else if (mouse.y == (SUB_MAIN_BEGIN_Y + 5) &&
					mouse.x > 11 &&
                                        mouse.x <=  11 + strlen(BACKONE_TEXT))
                                {
                                        answer = KEY_BACKONE;
                                }
				else if (mouse.y == (SUB_MAIN_BEGIN_Y + 6) &&
					mouse.x > 11 &&
                                        mouse.x <=  11 + strlen(STOP_TEXT))
                                {
                                        answer = KEY_STOP;
                                }
				else if (mouse.y == (SUB_MAIN_BEGIN_Y + 7) &&
					mouse.x > 11 &&
                                        mouse.x <=  11 + strlen(HELP_TEXT))
                                {
					if (autolist[auto_position]<0)
					{
						answer = 'g';
					}
					else
					{
						answer = KEY_HELP;
					}
                                }
                        }
                }
#endif

		/* See if this is a goto */

		if (autolist[auto_position] < 0)
		{
			/* If user wants to go to function do it */

			if (answer == 'g' || answer == 'G')
			{
				auto_position = -autolist[auto_position];
				continue;
			}

			/* Otherwise just move on */

			auto_position++;
			continue;
		}

		/* If we get here it is a normal entry on the auto list */

		switch(answer)
		{
			case KEY_SKIP : auto_position++;
					break;

			case KEY_STOP : return NOERROR;

			case KEY_HELP :	call_function(find_function(F_HELP),
				       functions[autolist[auto_position]].name);
				       	break;

			case KEY_BACKONE : if (--auto_position < 0)
						auto_position++;
					   if (autolist[auto_position] < 0)
						auto_position--;
					   break;

			case KEY_NEXT : /* 
					 * We make a copy of the argument
					 * list because call_function tends
					 * to mess it up.
					 */

					strcpy(args,autoargs[auto_position]);
	if ( call_function(autolist[auto_position],args) != NOERROR)
					{
	if (ask_yes_no("An error occurred performing the last function.",
"Do you wish to ignore this error and go on to the next function?")) 
						auto_position++;
					}
					else
					{
						auto_position++;
					}
					break;

			default : break;
		}
	}

} /* ui_auto() */

/************************************************************************/
/* ui_auto_socket()							*/
/*									*/
/* Follows the autolist array and calls the functions in order.		*/
/* Returns an error level, of last function called.			*/
/* This version is for running from sockets and it will ignore		*/
/* gotos and will not ask the user what to do.				*/
/* 									*/
/* Consult analysis, design and source for more information.		*/
/*									*/
/************************************************************************/

int	ui_auto_socket(void)
{
	/* Internal variables */

	char	args[81];	/* Argument list sent to function caller */
	int	ret;		/* Return value */

	/* We should be with an active socket */

	if (active_socket == -1) return NOERROR;

	/* always start from the begining */

	init_auto();

	/* OK start moving down the list */

	while(TRUE)
	{
		/* See if we have reached the end of the auto list */

		if (autolist[auto_position] == 0)
		{
			init_auto();
			socket_print(active_socket,
				"Completed automatic function list.\n");
			return	NOERROR;
		}

		/* See if this is a goto */

		if (autolist[auto_position] < 0)
		{
			/* Just move on */

			auto_position++;
			continue;
		}

		/* 
		 * Now run the next command.
		 * We make a copy of the argument
		 * list because call_function tends
		 * to mess it up.
		 */

		strcpy(args,autoargs[auto_position]);
		if ((ret=call_function(autolist[auto_position],args))!=NOERROR)
		{
		    socket_print(active_socket,"ERROR IN AUTO LIST AT CALL:\n");
		    socket_print(active_socket,"%s %s\n",
			functions[autolist[auto_position]].name,
			autoargs[auto_position]); 

		    return ret;
		    
		}

		auto_position++;
	}

} /* ui_auto_socket() */
