/************************************************************************/
/* bunker_clock_window.c 	                                        */
/*                                                                      */
/* Lets you play with the bunker clock controls.			*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE                        */
/*                 Based on the SUSI User Interface                     */
/*              In turn based on the CHIP User interface                */
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA                 */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu                                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/* Author : Theo ten Brummelaar                                         */
/* Date   : Jan 2007                                                    */
/************************************************************************/

#include "../../cliserv/include/cliserv.h"
#include "bunker_clockgtk.h"
#include "../../include/chara_messages.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include "../../charalib/chara.h"
#include "../../slalib/slalib.h"

#define WINDOW_WIDTH 150
#define SYNC_NONE	0
#define SYNC_ONE	1
#define SYNC_ALL	2
#define NUM_MACHINES 12

/* Globals for this file only */

static char *machines[] = {"gps", "gdt", "irimage", "flimage",
			   "ople", "tiptilt", "wobble",
			"S1", "S2", "E1", "E2", "W1", "W2", NULL};
static GtkWidget *msync_window;
static char msync_window_up = FALSE;
static GtkWidget *machine_select_menu_menu;
static GtkWidget *machine_menuitems[NUM_MACHINES];
static int sync_all = SYNC_ALL;
static int sync_one = SYNC_ONE;
static int *bunker_clock_server;

/************************************************************************/
/* create_msync_window()						*/
/*									*/
/* Allows you to do things with teh BUNKER CLOCK. Data is the server.	*/
/************************************************************************/

GtkWidget* create_msync_window (GtkButton *thebutton, gpointer *data)
{
	GtkWidget *button;
	GtkWidget *hbox;
	GtkWidget *vbox;
        GtkWidget *machine_select_menu;
	int	i;

	/* Do nothing if we exist already */

	if (msync_window_up) return msync_window;

	/* What server are we using */

	bunker_clock_server = (int *)data;

	/* Create a new window */

	msync_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (msync_window), "BUNKR_CLOCK");
	gtk_container_set_border_width (GTK_CONTAINER (msync_window), 1);

        /* We create a vbox to pack widgets into. */

        vbox = gtk_vbox_new(FALSE, 0);
        gtk_container_add (GTK_CONTAINER (msync_window), vbox);
        gtk_widget_show(vbox);

        /* Make an hbox put list in */

        hbox = gtk_hbox_new(FALSE, 0);
        gtk_container_add (GTK_CONTAINER (vbox), hbox);
        gtk_widget_show(hbox);

        button = gtk_button_new_with_label ("SYNC");
        gtk_signal_connect (GTK_OBJECT (button), "clicked",
                GTK_SIGNAL_FUNC (msync_callback), (gpointer)&sync_one);
        gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
        gtk_container_set_border_width (GTK_CONTAINER(button), 1);
        gtk_widget_set_usize (button, WINDOW_WIDTH/2, 24);
        gtk_widget_show(button);

	machine_select_menu = gtk_option_menu_new ();
	gtk_widget_show (machine_select_menu);
	machine_select_menu_menu = gtk_menu_new ();
	gtk_box_pack_start(GTK_BOX(hbox), machine_select_menu, TRUE, TRUE, 0);
	for(i=0; i<NUM_MACHINES; i++)
	{
		machine_menuitems[i] = 
			gtk_menu_item_new_with_label(machines[i]);
		gtk_widget_show (machine_menuitems[i]);
		gtk_menu_append (GTK_MENU (machine_select_menu_menu), 
		machine_menuitems[i]);
	}
	gtk_option_menu_set_menu (GTK_OPTION_MENU (machine_select_menu),
	machine_select_menu_menu);

	/* SYNC ALL button */

        button = gtk_button_new_with_label ("SYNC ALL MACHINES");
        gtk_signal_connect (GTK_OBJECT (button), "clicked",
		GTK_SIGNAL_FUNC (msync_callback), (gpointer)&sync_all);
        gtk_box_pack_start(GTK_BOX(vbox), button, TRUE, TRUE, 0);
        gtk_container_set_border_width (GTK_CONTAINER(button), 1);
        gtk_widget_set_usize (button, WINDOW_WIDTH/2, 24);
        gtk_widget_show(button);

        /* Make an hbox put list in */

        hbox = gtk_hbox_new(FALSE, 0);
        gtk_container_add (GTK_CONTAINER (vbox), hbox);
        gtk_widget_show(hbox);

	/* Creates a start button */

	button = gtk_button_new_with_label ("START");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
		GTK_SIGNAL_FUNC (bunker_clock_message_callback), 
		(gpointer)(message_array+BUNKER_CLOCK_START));
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
        gtk_container_set_border_width (GTK_CONTAINER(button),1);
	gtk_widget_set_usize (button, WINDOW_WIDTH/2, 24);
	gtk_widget_show(button);

	/* Creates a stop button */

	button = gtk_button_new_with_label ("STOP");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
		GTK_SIGNAL_FUNC (bunker_clock_message_callback), 
		(gpointer)(message_array+BUNKER_CLOCK_STOP));
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
        gtk_container_set_border_width (GTK_CONTAINER(button),1);
	gtk_widget_set_usize (button, WINDOW_WIDTH/2, 24);
	gtk_widget_show(button);

        /* Make an hbox put list in */

        hbox = gtk_hbox_new(FALSE, 0);
        gtk_container_add (GTK_CONTAINER (vbox), hbox);
        gtk_widget_show(hbox);

	/* Creates a reset button */

	button = gtk_button_new_with_label ("RESET");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
		GTK_SIGNAL_FUNC (bunker_clock_message_callback), 
		(gpointer)(message_array+BUNKER_CLOCK_RESET_GPS));
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
        gtk_container_set_border_width (GTK_CONTAINER(button),1);
	gtk_widget_set_usize (button, WINDOW_WIDTH/2, 24);
	gtk_widget_show(button);

	/* OK button */

        button = gtk_button_new_with_label ("OK");
        gtk_signal_connect (GTK_OBJECT (button), "clicked",
                GTK_SIGNAL_FUNC (bunker_ok_callback), (gpointer)NULL);
        gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
        gtk_container_set_border_width (GTK_CONTAINER(button), 1);
        gtk_widget_set_usize (button, WINDOW_WIDTH/2, 24);
        gtk_widget_show(button);

	/* Now, we can show this window if we like */

	gtk_widget_show(msync_window);

	/* Make sure we only get one version of this window */

	msync_window_up = TRUE;

	/* That should be all */

	return msync_window;

} /* create_msync_window() */

/************************************************************************/
/* msync_callback()	   						*/
/*									*/
/* Callback function for msync button. 					*/
/************************************************************************/

void msync_callback(GtkButton *button, gpointer data)
{
	int	machine;
	GtkWidget *current_machine_menuitem;
        struct smessage mess;
	char	s[256];

	if (!msync_window_up) return;

	/* Set up the message */

	mess.type = BUNKER_CLOCK_MSYNC;

	if (*((int *)data) == SYNC_ONE)
	{
		/* OK which machine are we syncing? */

		current_machine_menuitem = gtk_menu_get_active(
                                (GtkMenu *)machine_select_menu_menu);

		for(machine=0; machine<NUM_MACHINES &&
			machine_menuitems[machine] != 
			current_machine_menuitem; machine++);

	        if (machine >= NUM_MACHINES)
		{
#ifdef GTK2 
			if (status_buffer == NULL)
#else
			if (status_text == NULL)
#endif
				create_message_window(
					"Invalid machine number!");
			else
				print_status(ERROR,
					"Invalid machine number!\n");
			gtk_widget_destroy(msync_window);
			msync_window_up = FALSE;
			return;
		}
		mess.data = (unsigned char *)machines[machine];
		mess.length = strlen(machines[machine])+1;

		/* Send the message */

		if (!send_message(*bunker_clock_server, &mess))
		{
		    sprintf(s,"Failed to send BUNKER_CLOCK_MSYNC to %s.",
			machines[machine]);
#ifdef GTK2 
                    if (status_buffer == NULL)
#else
                    if (status_text == NULL)
#endif
			    create_message_window(s);
		    else
			    print_status(ERROR,"%s\n",s);
		}

	}
	else if (*((int *)data) == SYNC_ALL)
	{
		/* GO through list of all machines */

		for(machine=0; machine<NUM_MACHINES; machine++)
		{
			mess.data = (unsigned char *)machines[machine];
			mess.length = strlen(machines[machine])+1;

			if (!send_message(*bunker_clock_server, &mess))
			{
			    sprintf(s,
				"Failed to send BUNKER_CLOCK_MSYNC to %s.",
					machines[machine]);
#ifdef GTK2 
			    if (status_buffer == NULL)
#else
			    if (status_text == NULL)
#endif
			        create_message_window(s);
			    else
			        print_status(ERROR,"%s\n",s);
			}
		}

	}

} /* msync_callback() */

/************************************************************************/
/* bunker_ok_callback()	   						*/
/*									*/
/************************************************************************/

void bunker_ok_callback(GtkButton *button, gpointer data)
{
	gtk_widget_destroy(msync_window);
	msync_window_up = FALSE;

} /* bunker_ok_callback() */

/************************************************************************/
/* bunker_clock_message_callback()                                      */
/*                                                                      */
/* A callback that sends a signal with no data. 			*/
/************************************************************************/

void bunker_clock_message_callback(GtkButton *widget, gpointer type)
{
        struct smessage mess;

	mess.type = *((short int *)type);
	mess.length = 0;
	mess.data = NULL;
	if (!send_message(*bunker_clock_server, &mess))
	{
#ifdef GTK2 
                if (status_buffer == NULL)
#else
                if (status_text == NULL)
#endif
			create_message_window("Failed to send message.");
		else
			print_status(ERROR,"Failed to send message.\n");
	}

} /* bunker_clock_message_callback() */
