/************************************************************************/
/* clock_lpt_window.c 	                                                */
/*                                                                      */
/* Routine for controlling the basic CHARA LPT functions.               */
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE                        */
/*                 Based on the SUSI User Interface                     */
/*              In turn based on the CHIP User interface                */
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA                 */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu                                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/* Author : Theo ten Brummelaar                                         */
/* Date   : April 2002                                                  */
/************************************************************************/

#include "../../../cliserv/include/cliserv.h"
#include "../../../include/chara_messages.h"
#include "../include/cliserv_clock.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
//#include "../../../charalib/chara.h"
#include "../../../slalib/slalib.h"

/* Globals */

GtkWidget *lpt_control_window;
GtkWidget *lpt_label;
GtkWidget *lpt_entry;
static char lpt_control_window_up = FALSE;

/************************************************************************/
/* create_lpt_control_window()						*/
/*									*/
/* ROutine for creating the lpt control window. This should be self	*/
/* contained and give you all you need to control the basic lpt      	*/
/* functionality. 							*/
/************************************************************************/

GtkWidget* create_lpt_control_window(GtkButton *button, gpointer *data)
{
  GtkWidget *lpt_control_box;
  GtkWidget *ok_button;
  GtkWidget *sync_handler_button;
  GtkWidget *set_lpt_button;
  GtkWidget *set_bit_button;
  GtkWidget *unset_bit_button;
  GtkWidget *get_lpt_button;
  struct control_window_data *the_data;

  /* Do nothing if we exist already */

  if (lpt_control_window_up) return lpt_control_window;

  /* Get the data */

  the_data = (struct control_window_data *)data;

  /* Create the window itself */

  lpt_control_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (lpt_control_window), 
	"lpt_control_window", lpt_control_window);
  gtk_window_set_title (GTK_WINDOW (lpt_control_window), the_data->name);

  /* Box to put things in */

  lpt_control_box = gtk_fixed_new ();
  gtk_widget_ref (lpt_control_box);
  gtk_object_set_data_full (GTK_OBJECT (lpt_control_window), 
	"lpt_control_box", lpt_control_box,
        (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lpt_control_box);
  gtk_container_add (GTK_CONTAINER (lpt_control_window), lpt_control_box);

  /* OK button */

  ok_button = gtk_button_new_with_label ("OK");
  gtk_widget_ref (ok_button);
  gtk_object_set_data_full (GTK_OBJECT (lpt_control_window), 
	"ok_button", ok_button,
        (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect(GTK_OBJECT (ok_button), "clicked",
                      GTK_SIGNAL_FUNC (lpt_control_ok_callback),
                      NULL);
  gtk_widget_show (ok_button);
  gtk_fixed_put (GTK_FIXED (lpt_control_box), ok_button, 80, 72);
  gtk_widget_set_uposition (ok_button, 80, 72);
  gtk_widget_set_usize (ok_button, 80, 24);

  /* Sync handler button */

  sync_handler_button = gtk_button_new_with_label ("CALL SYNC");
  gtk_widget_ref (sync_handler_button);
  gtk_object_set_data_full (GTK_OBJECT (lpt_control_window), 
	"sync_handler_button", sync_handler_button,
        (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect(GTK_OBJECT (sync_handler_button), "clicked",
                GTK_SIGNAL_FUNC (lpt_control_sync_handler_callback),
                (gpointer) &(the_data->server));
  gtk_widget_show (sync_handler_button);
  gtk_fixed_put (GTK_FIXED (lpt_control_box), sync_handler_button, 80, 48);
  gtk_widget_set_uposition (sync_handler_button, 80, 48);
  gtk_widget_set_usize (sync_handler_button, 80, 24);

  /* Text entry box */

  lpt_entry = gtk_entry_new ();
  gtk_widget_ref (lpt_entry);
  gtk_object_set_data_full (GTK_OBJECT (lpt_control_window), 
		"lpt_entry", lpt_entry,
                (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lpt_entry);
  gtk_fixed_put (GTK_FIXED (lpt_control_box), lpt_entry, 0, 0);
  gtk_widget_set_uposition (lpt_entry, 0, 0);
  gtk_widget_set_usize (lpt_entry, 80, 24);
  gtk_entry_set_text (GTK_ENTRY (lpt_entry), "0xFF");

  /* Set LPT button */

  set_lpt_button = gtk_button_new_with_label ("SET LPT");
  gtk_widget_ref (set_lpt_button);
  gtk_object_set_data_full (GTK_OBJECT (lpt_control_window), 
	"set_lpt_button", set_lpt_button,
        (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect(GTK_OBJECT (set_lpt_button), "clicked",
                      GTK_SIGNAL_FUNC (lpt_control_set_lpt_callback),
                      (gpointer) &(the_data->server));
  gtk_widget_show (set_lpt_button);
  gtk_fixed_put (GTK_FIXED (lpt_control_box), set_lpt_button, 0, 72);
  gtk_widget_set_uposition (set_lpt_button, 0, 72);
  gtk_widget_set_usize (set_lpt_button, 80, 24);

 /* Set bit button */

  set_bit_button = gtk_button_new_with_label ("SET BIT");
  gtk_widget_ref (set_bit_button);
  gtk_object_set_data_full (GTK_OBJECT (lpt_control_window), 
	"set_bit_button", set_bit_button,
        (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect(GTK_OBJECT (set_bit_button), "clicked",
                      GTK_SIGNAL_FUNC (lpt_control_set_bit_callback),
                      (gpointer) &(the_data->server));
  gtk_widget_show (set_bit_button);
  gtk_fixed_put (GTK_FIXED (lpt_control_box), set_bit_button, 0, 48);
  gtk_widget_set_uposition (set_bit_button, 0, 48);
  gtk_widget_set_usize (set_bit_button, 80, 24);

  /* Unset BIT button */

  unset_bit_button = gtk_button_new_with_label ("UNSET BIT");
  gtk_widget_ref (unset_bit_button);
  gtk_object_set_data_full (GTK_OBJECT (lpt_control_window), 
	"unset_bit_button", unset_bit_button,
        (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect(GTK_OBJECT (unset_bit_button), "clicked",
                      GTK_SIGNAL_FUNC (lpt_control_unset_bit_callback),
                      (gpointer) &(the_data->server));
  gtk_widget_show (unset_bit_button);
  gtk_fixed_put (GTK_FIXED (lpt_control_box), unset_bit_button, 0, 24);
  gtk_widget_set_uposition (unset_bit_button, 0, 24);
  gtk_widget_set_usize (unset_bit_button, 80, 24);

  /* Get LPT button */

  get_lpt_button = gtk_button_new_with_label ("GET LPT");
  gtk_widget_ref (get_lpt_button);
  gtk_object_set_data_full (GTK_OBJECT (lpt_control_window), 
	"get_lpt_button", get_lpt_button,
        (GtkDestroyNotify) gtk_widget_unref);
  gtk_signal_connect(GTK_OBJECT (get_lpt_button), "clicked",
                      GTK_SIGNAL_FUNC (lpt_control_get_lpt_callback),
                      (gpointer) &(the_data->server));
  gtk_widget_show (get_lpt_button);
  gtk_fixed_put (GTK_FIXED (lpt_control_box), get_lpt_button, 80, 24);
  gtk_widget_set_uposition (get_lpt_button, 80, 24);
  gtk_widget_set_usize (get_lpt_button, 80, 24);

  /* LPT label */

  lpt_label = gtk_label_new ("0xFF");
  gtk_widget_ref (lpt_label);
  gtk_object_set_data_full (GTK_OBJECT (lpt_control_window), 
	"lpt_label", lpt_label,
        (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lpt_label);
  gtk_fixed_put (GTK_FIXED (lpt_control_box), lpt_label, 80, 0);
  gtk_widget_set_uposition (lpt_label, 80, 0);
  gtk_widget_set_usize (lpt_label, 80, 24);

  /* Now, we can show this window if we like */

  gtk_widget_show(lpt_control_window);

  /* Make sure we only get one version of this window */

  lpt_control_window_up = TRUE;
 
  /* That is all */

  return lpt_control_window;

} /* create_lpt_control_window() */

/************************************************************************/
/* lpt_control_ok_callback()                                          	*/
/*                                                                      */
/* Callback function for lpt control OK button. 			*/
/************************************************************************/

void lpt_control_ok_callback(GtkButton *button, gpointer user_data)
{
        if (!lpt_control_window_up) return;

        gtk_widget_destroy(lpt_control_window);
        lpt_control_window_up = FALSE;

} /* lpt_control_ok_callback() */

/************************************************************************/
/* lpt_control_get_lpt_callback()					*/
/*									*/
/* Sends MESS_GET_LPT function and waits for response.			*/
/************************************************************************/

void lpt_control_get_lpt_callback(GtkButton *button, gpointer user_data)
{
	time_t	start;
        struct smessage mess;
        int     *server;
	char	s[256];

	/* Send message */

        server = (int *)user_data;
        mess.type = MESS_GET_LPT;
        mess.length = 0;
        mess.data = NULL;
        if (!send_message(*server, &mess))
	{
#ifdef GTK2
                if (status_buffer == NULL)
#else
                if (status_text == NULL)
#endif
			create_message_window("Failed to send MESS_GET_LPT.");
		else
			print_status(ERROR,"Failed to send MESS_GET_LPT.\n");
	}

	/* Wait for response */

	start = time(0);
	while(lpt[*server] == -1)
	{
		if (time(0) > start+3)
		{
#ifdef GTK2
			if (status_buffer == NULL)
#else
			if (status_text == NULL)
#endif
			    create_message_window(
				"Timed out waiting for lpt value.");
			else
			    print_status(ERROR,
				"Timed out waiting for lpt value.\n");
		    return;
		}
	}

	/* Put value into label */

	sprintf(s,"0x%02x",lpt[*server]);
	gtk_label_set((GtkLabel *)lpt_label, s);

} /* lpt_control_get_lpt_callback() */

/************************************************************************/
/* lpt_control_set_lpt_callback()					*/
/*									*/
/* Sends MESS_SET_LPT function and waits for response.			*/
/************************************************************************/

void lpt_control_set_lpt_callback(GtkButton *button, gpointer user_data)
{
        struct smessage mess;
        int     *server;
	char	*s;
	unsigned char value;
	int	i;

	/* Extract the value from the text entry feild */

	s = gtk_entry_get_text(GTK_ENTRY(lpt_entry));
	sscanf(s,"%x",&i);
	if (i<0 || i>255)
	{
#ifdef GTK2
                if (status_buffer == NULL)
#else
                if (status_text == NULL)
#endif
			create_message_window(
				"LPT value must be between 0 and 255.");
		else
			print_status(ERROR,
				"LPT value must be between 0 and 255.\n");
		return;
	}
	value = i;

	/* Send message */

        server = (int *)user_data;
        mess.type = MESS_SET_LPT;
        mess.length = sizeof(value);
        mess.data = &value;
        if (!send_message(*server, &mess))
	{
#ifdef GTK2
                if (status_buffer == NULL)
#else
                if (status_text == NULL)
#endif
			create_message_window("Failed to send MESS_SET_LPT.");
		else
			print_status(ERROR,"Failed to send MESS_SET_LPT.\n");
	}

	/* See if it takes */

	lpt_control_get_lpt_callback(button, user_data);

} /* lpt_control_set_lpt_callback() */

/************************************************************************/
/* lpt_control_set_bit_callback()					*/
/*									*/
/* Sends MESS_SET_LPT function and waits for response.			*/
/************************************************************************/

void lpt_control_set_bit_callback(GtkButton *button, gpointer user_data)
{
        struct smessage mess;
        int     *server;
	char	*s;
	unsigned char bit;
	int	i;

	/* Extract the value from the text entry feild */

	s = gtk_entry_get_text(GTK_ENTRY(lpt_entry));
	sscanf(s,"%x",&i);
	if (i < 0 || i>7)
	{
#ifdef GTK2
                if (status_buffer == NULL)
#else
                if (status_text == NULL)
#endif
			create_message_window(
				"Bit value must be between 0 and 7.");
		else
			print_status(ERROR,
				"Bit value must be between 0 and 7.\n");
		return;
	}
	bit = i;

	/* Send message */

        server = (int *)user_data;
        mess.type = SET_LPT_BIT;
        mess.length = sizeof(bit);
        mess.data = &bit;
        if (!send_message(*server, &mess))
	{
#ifdef GTK2
                if (status_buffer == NULL)
#else
                if (status_text == NULL)
#endif
			create_message_window("Failed to send SET_LPT_BIT.");
		else
			print_status(ERROR,"Failed to send SET_LPT_BIT.\n");
	}

	/* See if it takes */

	lpt_control_get_lpt_callback(button, user_data);

} /* lpt_control_set_bit_callback() */

/************************************************************************/
/* lpt_control_unset_bit_callback()					*/
/*									*/
/* Sends MESS_UNSET_LPT function and waits for response.		*/
/************************************************************************/

void lpt_control_unset_bit_callback(GtkButton *button, gpointer user_data)
{
        struct smessage mess;
        int     *server;
	char	*s;
	unsigned char bit;
	int	i;

	/* Extract the value from the text entry feild */

	s = gtk_entry_get_text(GTK_ENTRY(lpt_entry));
	sscanf(s,"%x",&i);
	if (i < 0 || i>7)
	{
#ifdef GTK2
                if (status_buffer == NULL)
#else
                if (status_text == NULL)
#endif
			create_message_window(
				"Bit value must be between 0 and 7.");
		else
			print_status(ERROR,
				"Bit value must be between 0 and 7.\n");
		return;
	}
	bit = i;

	/* Send message */

        server = (int *)user_data;
        mess.type = UNSET_LPT_BIT;
        mess.length = sizeof(bit);
        mess.data = &bit;
        if (!send_message(*server, &mess))
	{
#ifdef GTK2
                if (status_buffer == NULL)
#else
                if (status_text == NULL)
#endif
			create_message_window("Failed to send UNSET_LPT_BIT.");
		else
			print_status(ERROR,"Failed to send UNSET_LPT_BIT.\n");
	}

	/* See if it takes */

	lpt_control_get_lpt_callback(button, user_data);

} /* lpt_control_unset_bit_callback() */

/************************************************************************/
/* lpt_control_sync_handler_callback()					*/
/*									*/
/* Sends CALL_SYNC_HANDLER function.					*/
/************************************************************************/

void lpt_control_sync_handler_callback(GtkButton *button, gpointer user_data)
{
        struct smessage mess;
	int	*server;

	/* Send message */

        server = (int *)user_data;
        mess.type = CALL_SYNC_HANDLER;
        mess.length = 0;
        mess.data = NULL;
        if (!send_message(*server, &mess))
	{
#ifdef GTK2
                if (status_buffer == NULL)
#else
                if (status_text == NULL)
#endif
			create_message_window(
				"Failed to send CALL_SYNC_HANDLER.");
		else
			print_status(ERROR,"Failed to send CALL_SYNC_HANDLER.\n");
	}

} /* lpt_control_sync_handler_callback() */
