/************************************************************************/
/* clock_messages.c							*/
/*                                                                      */
/* Message handling routines.						*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY CLIENT INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar			                        */
/* Date   : Wed Oct 10 03:42:20 PDT 2001				*/
/************************************************************************/

#include "../../../cliserv/include/cliserv.h"
#include "../include/cliserv_clock.h"
#include "../../../include/chara_messages.h"

/* Globals */

rt_time_struct rt_time[MAX_SERVERS];	/* Most recent rt_timefrom server */
CHARA_TIME time_next_second[MAX_SERVERS];  /* Most recent time_next second */
int lpt[MAX_SERVERS];		/* Most recent LPT value */
int lpt_data[MAX_SERVERS];	/* Most recent LPT value */

/************************************************************************/
/* setup_clock_messages()						*/
/*									*/
/* Guess what this does.						*/
/************************************************************************/

int setup_clock_messages(int server)
{
	int	i;

	/* Setup globals */
	
	for(i=0; i<MAX_SERVERS; i++)
	{
		rt_time[i].time_now = -1;
		rt_time[i].lost_ticks = 0;
		rt_time[i].lost_seconds = 0;
		time_next_second[i] = -1;
		lpt[MAX_SERVERS] = 0;
	}

        /* Here are the standard GTK messages */

        if (!add_message_job(server, NOOP, message_noop)) return FALSE;
        if (!add_message_job(server, MESSAGE, message_message)) return FALSE;
        if (!add_message_job(server, PING, message_ping)) return FALSE;
        if (!add_message_job(server, UI_STATUS, message_status)) return FALSE;
        if (!add_message_job(server, SERVER_STOPPING, message_server_stopping)) 		return FALSE;
        if (!add_message_job(server, MAKE_PING_NOISE, message_make_ping_noise)) 		return FALSE;
        if (!add_message_job(server, MAKE_RING_NOISE, message_make_ring_noise)) 		return FALSE;


	/* Here we set up the local messages */

	if (!add_message_job(server, RT_TIME, message_rt_time)) return FALSE;
	if (!add_message_job(server,TIME_NEXT_SECOND,message_time_next_second))
	if (!add_message_job(server,MESS_SET_LPT,message_set_lpt))
		return FALSE;

	return TRUE;

} /* setup_messages() */

/************************************************************************/
/* message_rt_time()							*/
/*									*/
/* Responds to RT_TIME message from server.				*/
/************************************************************************/

int message_rt_time(int server, struct smessage *mess)
{
	rt_time_struct *new_rt_time;

	/* Make sure data is right length */

	if (mess->length != sizeof(*new_rt_time))
	{
#ifdef GTK2
                if (status_buffer == NULL)
#else
                if (status_text == NULL)
#endif
			create_message_window(
			"Recieved RT_TIME message with wrong amount of data.");
		else
			print_status(ERROR,
			"Recieved RT_TIME message with wrong amount of data.\n");
	}
	else
	{
		new_rt_time = (rt_time_struct *)mess->data;
		rt_time[server] .time_now = new_rt_time->time_now;
		rt_time[server] .lost_ticks = new_rt_time->lost_ticks;
		rt_time[server] .lost_seconds = new_rt_time->lost_seconds;
	}

	return TRUE;

} /* message_rt_time() */

/************************************************************************/
/* send_rt_time()							*/
/*									*/
/* Send RT_TIME message to server. Set's current time_now to -1 to show */
/* current global is not valid.						*/
/************************************************************************/

void send_rt_time(int server)
{
	struct smessage mess;

	/* Show outside the current time is not valid */

	rt_time[server] .time_now= -1;

	/* Send message to go and get the time */

	mess.type = RT_TIME;
	mess.length = 0;
	mess.data = NULL;
	if (!send_message(server, &mess))
	{
#ifdef GTK2
                if (status_buffer == NULL)
#else
                if (status_text == NULL)
#endif
			create_message_window(
				"Failed to send RT_TIME message.");
		else
			print_status(ERROR,
				"Failed to send RT_TIME message.\n");
	}

} /* send_rt_time() */

/************************************************************************/
/* message_time_next_second()						*/
/*									*/
/* Responds to TIME_NEXT_SECOND message from server.			*/
/************************************************************************/

int message_time_next_second(int server, struct smessage *mess)
{
	CHARA_TIME *new_time_next_second;

	/* Make sure data is right length */

	if (mess->length != sizeof(*new_time_next_second))
	{
#ifdef GTK2
                if (status_buffer == NULL)
#else
                if (status_text == NULL)
#endif
			create_message_window(
		"Recieved TIME_NEXT_SECOND message with wrong amount of data.");
		else
			print_status(ERROR,
		"Recieved TIME_NEXT_SECOND message with wrong amount of data.\n");
	}
	else
	{
		new_time_next_second = (CHARA_TIME *)mess->data;
		time_next_second[server] = *new_time_next_second;
	}

	return TRUE;

} /* message_time_next_second() */

/************************************************************************/
/* send_time_next_second()						*/
/*									*/
/* Send TIME_NEXT_SECOND message to server. 				*/
/* Set's current time_now to -1 to show current global is not valid.	*/
/************************************************************************/

void send_time_next_second(int server)
{
	struct smessage mess;

	/* Show outside the current time is not valid */

	time_next_second[server] = -1;

	/* Send message to go and get the time */

	mess.type = TIME_NEXT_SECOND;
	mess.length = 0;
	mess.data = NULL;
	if (!send_message(server, &mess))
	{
#ifdef GTK2
                if (status_buffer == NULL)
#else
                if (status_text == NULL)
#endif
			create_message_window(
			"Failed to send TIME_NEXT_SECOND message.");
		else
			print_status(ERROR,
			"Failed to send TIME_NEXT_SECOND message.\n");
	}

} /* send_time_next_second() */

/************************************************************************/
/* message_set_lpt()							*/
/*									*/
/* Responds to MESS_SET_LPT message from server.			*/
/************************************************************************/

int message_set_lpt(int server, struct smessage *mess)
{
	unsigned char *new_lpt;

	/* Make sure data is right length */

	if (mess->length != sizeof(*new_lpt))
	{
#ifdef GTK2
                if (status_buffer == NULL)
#else
                if (status_text == NULL)
#endif
			create_message_window(
		"Recieved MESS_SET_LPT message with wrong amount of data.");
		else
			print_status(ERROR,
		"Recieved MESS_SET_LPT message with wrong amount of data.\n");
	}
	else
	{
		new_lpt = (unsigned char *)mess->data;
		lpt[server] = *new_lpt;
	}

	return TRUE;

} /* message_set_lpt() */

/************************************************************************/
/* send_get_lpt()							*/
/*									*/
/* Send MESS_GET_LPT message to server. 				*/
/* Set's current time_now to -1 to show current global is not valid.	*/
/************************************************************************/

void send_get_lpt(int server)
{
	struct smessage mess;

	/* Show outside the current time is not valid */

	lpt[server] = -1;

	/* Send message to go and get the time */

	mess.type = MESS_GET_LPT;
	mess.length = 0;
	mess.data = NULL;
	if (!send_message(server, &mess))
	{
#ifdef GTK2
                if (status_buffer == NULL)
#else
                if (status_text == NULL)
#endif
			create_message_window(
			"Failed to send MESS_GET_LPT message.");
		else
			print_status(ERROR,
			"Failed to send MESS_GET_LPT message.\n");
	}

} /* send_get_lpt() */

/************************************************************************/
/* message_set_lpt_data()						*/
/*									*/
/* Responds to MESS_SET_LPT_DATA message from server.			*/
/************************************************************************/

int message_set_lpt_data(int server, struct smessage *mess)
{
	unsigned char *new_lpt_data;

	/* Make sure data is right length */

	if (mess->length != sizeof(*new_lpt_data))
	{
#ifdef GTK2
                if (status_buffer == NULL)
#else
                if (status_text == NULL)
#endif
			create_message_window(
	"Recieved MESS_SET_LPT_DATA message with wrong amount of data.");
		else
			print_status(ERROR,
	"Recieved MESS_SET_LPT_DATA message with wrong amount of data.\n");
	}
	else
	{
		new_lpt_data = (unsigned char *)mess->data;
		lpt_data[server] = *new_lpt_data;
	}

	return TRUE;

} /* message_set_lpt_data() */

/************************************************************************/
/* send_get_lpt_data()							*/
/*									*/
/* Send MESS_GET_LPT_DATA message to server. 				*/
/* Set's current time_now to -1 to show current global is not valid.	*/
/************************************************************************/

void send_get_lpt_data(int server)
{
	struct smessage mess;

	/* Show outside the current time is not valid */

	lpt_data[server] = -1;

	/* Send message to go and get the time */

	mess.type = MESS_GET_LPT_DATA;
	mess.length = 0;
	mess.data = NULL;
	if (!send_message(server, &mess))
	{
#ifdef GTK2
                if (status_buffer == NULL)
#else
                if (status_text == NULL)
#endif
			create_message_window(
			"Failed to send MESS_GET_LPT_DATA message.");
		else
			print_status(ERROR,
			"Failed to send MESS_GET_LPT_DATA message.\n");
	}

} /* send_get_lpt_data() */
