/************************************************************************/
/* clock_cliserv.h							*/
/*                                                                      */
/* Header file for the clock module/client/server controller.		*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : MARCH 2002                                               	*/      
/************************************************************************/

#ifndef __CLOCK_CLISERV__
#define __CLOCK_CLISERV__

/************************************************************************/
/* This stuff for servers and clients.					*/
/************************************************************************/

/* Macros */

#ifndef TICKS_PER_SECOND
#define TICKS_PER_SECOND	1000L	/* How fast is the clock? */
#define TICKS_PER_MINUTE	(TICKS_PER_SECOND*60L)
#define TICKS_PER_HOUR		(TICKS_PER_MINUTE*60L)
#define TICKS_PER_DAY		(TICKS_PER_HOUR*24L)
#endif

#define CHARA_TIME	int	/* Variable used for all times */
#define CHARA_TIME_END  0x7FFFFFFF

#define TTL_IN		0
#define TTL_OUT		1

/*
 * Global Variables
 */

typedef struct	RT_COMMAND 
	{
		unsigned int comm;
		unsigned char data;

	} rt_command_struct;	/* Structure used to send commands */

typedef struct	RT_TIME_STRUCT 
	{
		CHARA_TIME time_now;
		CHARA_TIME lost_ticks;
		CHARA_TIME lost_seconds;

	} rt_time_struct;	/* Structure used to send times */

typedef struct  AP_TIME_STRUCT 
        {
                char            sign; /* -1 or +1 */
                CHARA_TIME      hrs;
                CHARA_TIME      min;
                CHARA_TIME      sec;
                CHARA_TIME      tick;

        } ap_time_struct;       /* Structure used to send times */

#ifdef SERVER
/************************************************************************/
/* This stuff only for servers						*/
/************************************************************************/

/* Include files */

#include <sys/types.h>

/* Macros for printer port and hardware */

#ifdef USE_REAL_TIME_VERSION
#define CLOCK_CARD_IRQ  11      /* Clock card interrupt */
#define CLOCK_CARD_ADD  0x300   /* Address for clock card */
#define LPT_BASE      0x378	/* Parallel Port Base Address. */
#define LPT_STATUS (LPT_BASE+1)	/* Status register */
#define LPT_CONTROL (LPT_BASE+2)/* Control register */

#define LPT_STATUS_IRQ 0x04	/* IRQ Status */
#define LPT_STATUS_ERR 0x08	/* !(ERROR) */
#define LPT_STATUS_SL  0x10	/* Select  */
#define LPT_STATUS_PE  0x20	/* Paper End */
#define LPT_STATUS_ACK 0x40	/* !(Acknowledge) */
#define LPT_STATUS_BSY 0x80	/* !(Busy) */

#define LPT_CONTROL_ST 0x01	/* Strobe Line */
#define LPT_CONTROL_AF 0x02	/* Auto Line Feed line */
#define LPT_CONTROL_IN 0x04	/* !(Initialize Printer) */
#define LPT_CONTROL_SI 0x08	/* Select In line */
#define LPT_CONTROL_IQ 0x10	/* Enable IRQ */
#define LPT_CONTROL_D  0x20	/* Port Direction */

#define COMM_FIFO_DEV	"/dev/rtf0" /* Command channel device */
#define DATA_OUT_FIFO_DEV	"/dev/rtf1" /* Data out channel device */
#define DATA_IN_FIFO_DEV	"/dev/rtf2" /* Data in channel device */
#define ERR_FIFO_DEV	"/dev/rtf3" /* Error channel device */

/* These are the RT commands we have as part of the clock module */

#define REPORT_LPT_REGISTER	255	/* Command to read LPT status reg */
#define CONTROL_REG		0 	/* Argument for above function */
#define STATUS_REG		1 	/* Argument for above function */

#define WRITE_LPT_CONTROL	254	/* Command write to LPT command reg */
#define REPORT_TIME		253	/* Command to report current time */
#define SET_TIME		252	/* Command to report current time */
#define GET_TIME_NEXT_SECOND    251	/* Report the time of the next second */
#define FLUSH_DATA		250	/* Command to report current time */
#define CLEAR_LOST_COUNTS	249	/* Command to clear lost tick count */
#define SYNC_HANDLER            248     /* Command to call sync handler */

#define TTL_CONTROL		247 	/* Command that plays with bits */
#define IN_1			0	/* Argument for above function */
#define IN_2			1	/* Argument for above function */
#define IN_3			3	/* Argument for above function */
#define OUT_1			4	/* Argument for above function */
#define OUT_2			5	/* Argument for above function */
#define SET_OUT_1		6	/* Argument for above function */
#define CLEAR_OUT_1		7	/* Argument for above function */
#define SET_OUT_2		8	/* Argument for above function */
#define CLEAR_OUT_2		9	/* Argument for above function */

#define SET_LPT                 246     /* Command: set value in printer port */
#define GET_LPT                 245     /* Command: get value in printer port */
#define SET_BIT_LPT             244     /* Command: set bit in printer port */
#define UNSET_BIT_LPT           243     /* Command: unset bit in printer port */

#define INITIALIZE_CLOCK        242     /* Command to call initialize clock */

#define MAX_CHARA_COMMAND       242

/* Gobals */

extern int	fd_comm;		/* Command channel file handle */
extern int	fd_data_out;		/* Data out channel file handle */
extern int	fd_data_in;		/* Data in channel file handle */
extern int	fd_err;			/* Error channel file handle */
#else
extern FILE	*cbfp;			/* File pointer to clock device */
#define DEVICE_NAME "/dev/cbdriver"
#endif

/* Prototypes */

/* clock_comms.c */

#ifdef USE_REAL_TIME_VERSION
char open_rt_fifos(void);
void close_rt_fifos(void);
char send_rt_command(unsigned char command, unsigned char argument);
int get_rt_error_message(char *s);
int read_rt_data(void *buf, size_t count);
int write_rt_data(void *buf, size_t count);
int flush_rt_data_in(void);
int flush_rt_data_out(void);
int flush_data(void);
int in_set(int in_or_out, int port);
int error_status(void);
unsigned char get_lpt_register(int reg);
int call_sync_handler(void);
int set_lpt(unsigned char value);
int get_lpt(void);
int set_bit_lpt(unsigned char bit);
int unset_bit_lpt(unsigned char bit);
unsigned char write_lpt_control(unsigned char reg);
unsigned char write_lpt_control(unsigned char reg);
int initialize_clock(void);
#else
int open_clock_device(void);
void close_clock_device(void);
#endif
int clear_lost_counts(void);
int get_rt_time(rt_time_struct *the_time);
CHARA_TIME chara_time_now(void);
CHARA_TIME get_time_next_second(int wait);
int set_rt_time(CHARA_TIME time_next_second);
int sync_times(void);

/* clock_messages.c */

void setup_standard_clock_messages(void);
#ifdef USE_REAL_TIME_VERSION
int clock_flush_fifos(struct smessage *message);
int clock_in_set(struct smessage *message);
int clock_call_sync_handler(struct smessage *message);
int clock_set_lpt(struct smessage *message);
int clock_get_lpt(struct smessage *message);
int clock_set_lpt_bit(struct smessage *message);
int clock_unset_lpt_bit(struct smessage *message);
int clock_reinit_clock(struct smessage *message);
#endif
int clock_clear_lost_counts(struct smessage *message);
int clock_rt_time(struct smessage *message);
int clock_time_next_second(struct smessage *message);
int clock_set_rt_time(struct smessage *message);
int clock_sync_times(struct smessage *message);

/* clock_time_convert.c */

struct AP_TIME_STRUCT rt_to_ap_time(CHARA_TIME rt_time);
CHARA_TIME ap_to_rt_time(struct AP_TIME_STRUCT t);
struct AP_TIME_STRUCT str_to_ap_time(char *s);
char *ap_to_str_time(char *s, struct AP_TIME_STRUCT t);

#else
/************************************************************************/
/* This stuff only for gtk clients					*/
/************************************************************************/

#ifdef USE_GTK

/* Globals */

/* Most recent rt_timefrom server */

extern rt_time_struct rt_time[MAX_SERVERS];  

/* Most recent time_next second */

extern CHARA_TIME time_next_second[MAX_SERVERS];

/* Most recent LPT value */

extern int lpt[MAX_SERVERS];         
extern int lpt_data[MAX_SERVERS];         

struct control_window_data { char *name; int server; };

/* Prototypes */

/* clock_control_wondow.c */

GtkWidget* create_clock_control_window (GtkButton *button, gpointer *data);
void clock_control_ok_callback(GtkButton *button, gpointer user_data);
gint clock_control_background(gpointer data);
void clock_control_clear_callback(GtkButton *button, gpointer user_data);
void clock_control_reinit_callback(GtkButton *button, gpointer user_data);
void clock_control_flush_callback(GtkButton *button, gpointer user_data);
void clock_control_sync_callback(GtkButton *button, gpointer user_data);
void clock_control_time_callback(GtkWidget *widget, gpointer user_data);

/* clock_lpt_wondow.c */

GtkWidget* create_lpt_control_window(GtkButton *button, gpointer *data);
void lpt_control_ok_callback(GtkButton *button, gpointer user_data);
void lpt_control_get_lpt_callback(GtkButton *button, gpointer user_data);
void lpt_control_set_lpt_callback(GtkButton *button, gpointer user_data);
void lpt_control_set_bit_callback(GtkButton *button, gpointer user_data);
void lpt_control_unset_bit_callback(GtkButton *button, gpointer user_data);
void lpt_control_sync_handler_callback(GtkButton *button, gpointer user_data);

/* clock_time_convert.c */

struct AP_TIME_STRUCT rt_to_ap_time(CHARA_TIME rt_time);
CHARA_TIME ap_to_rt_time(struct AP_TIME_STRUCT t);
struct AP_TIME_STRUCT str_to_ap_time(char *s);
char *ap_to_str_time(char *s, struct AP_TIME_STRUCT t);

/* clock_messages.c */

int setup_clock_messages(int server);
int message_rt_time(int server, struct smessage *mess);
void send_rt_time(int server);
int message_time_next_second(int server, struct smessage *mess);
void send_time_next_second(int server);
int message_set_lpt(int server, struct smessage *mess);
void send_get_lpt(int server);
int message_set_lpt_data(int server, struct smessage *mess);
void send_get_lpt_data(int server);

#endif
#endif
#endif
