/************************************************************************/
/* fast_clock_messages.c						*/
/*                                                                      */
/* Message handling routines.						*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY CLIENT INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar			                        */
/* Date   : Wed Oct 10 03:42:20 PDT 2001				*/
/************************************************************************/

#include "../../slalib/slalib.h"
#include "../../cliserv/include/cliserv.h"
#include "../../include/chara_messages.h"
#include "cliserv_fast_clock.h"

/* Globals */

fast_clock_time_struct fast_time[MAX_SERVERS];
int fast_clock_rate[MAX_SERVERS];
int fast_clock_cal_time[MAX_SERVERS];

/************************************************************************/
/* setup_fast_clock_messages()						*/
/*									*/
/* Guess what this does.						*/
/************************************************************************/

int setup_fast_clock_messages(int server)
{
	int	i;

	/* Setup globals */
	
	for(i=0; i<MAX_SERVERS; i++)
	{
		fast_time[i].time_now = -1;
		fast_time[i].lost_ticks = 0;
		fast_clock_rate[i] = 0;
		fast_clock_cal_time[i] = 0;
	}

	/* Here we set up the local messages */

	if (!add_message_job(server, FAST_CLOCK_TIME, message_fast_clock_time))
		return FALSE;

	if (!add_message_job(server, FAST_CLOCK_SET_RATE,
		message_fast_clock_set_rate)) return FALSE;

	if (!add_message_job(server, FAST_CLOCK_CALIBRATE_INFO,
		message_fast_clock_calibration_info)) return FALSE;

	return TRUE;

} /* setup_messages() */

/************************************************************************/
/* message_fast_clock_time()						*/
/*									*/
/* Responds to FAST_CLOCK_TIME message.					*/
/************************************************************************/

int message_fast_clock_time(int server, struct smessage *mess)
{
	fast_clock_time_struct *now;

	if (mess->length != sizeof(*now))
	{
		create_message_window(
		"Recieved FAST_CLOCK_TIME message with wrong amount of data.");
	}
	else
	{
		now = (fast_clock_time_struct *)mess->data;
		fast_time[server].time_now = now->time_now;
		fast_time[server].lost_ticks = now->lost_ticks;
	}

	return TRUE;

} /* message_fast_clock_time() */

/************************************************************************/
/* message_fast_clock_set_rate()					*/
/*									*/
/* Responds to FAST_CLOCK_SET_RATE message.				*/
/************************************************************************/

int message_fast_clock_set_rate(int server, struct smessage *mess)
{
	int *rate;

	if (mess->length != sizeof(*rate))
	{
		create_message_window(
	"Recieved FAST_CLOCK_SET_RATE message with wrong amount of data.");
	}
	else
	{
		rate = (int *)mess->data;
		fast_clock_rate[server] = *rate;
	}

	return TRUE;

} /* message_fast_clock_set_rate() */


/************************************************************************/
/* message_fast_clock_calibration_info()				*/
/*									*/
/* Responds to FAST_CLOCK_CALIBRATE_INFO message.			*/
/************************************************************************/

int message_fast_clock_calibration_info(int server, struct smessage *mess)
{
	int *cal_time;

	if (mess->length != sizeof(*cal_time))
	{
		create_message_window(
"Recieved FAST_CLOCK_CALIBRATE_INFO message with wrong amount of data.");
	}
	else
	{
		cal_time = (int *)mess->data;
		fast_clock_cal_time[server] = *cal_time;
	}

	return TRUE;

} /* message_fast_clock_calibration_info() */
