/************************************************************************/
/* uipico_messages.c							*/
/*                                                                      */
/* Message handling routines.						*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY CLIENT INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar			                        */
/* Date   : Wed Oct 10 03:42:20 PDT 2001				*/
/************************************************************************/

#include "../../charaui/src/charaui.h"
#include "../../include/chara_messages.h"
#include "../../sockman/sockman.h"
#include "../../cliserv/include/cliserv.h"
#include "../../cliserv/uiclient/uiclient.h"
#include "uipico.h"

/************************************************************************/
/* open_pico_server()							*/
/*									*/
/* Returns server number on exit or					*/
/* -1 if fails to open server.						*/
/************************************************************************/

int open_pico_server(char *name)
{
	int server;
	struct smessage mess;
        time_t   start;

        /* Attempt to open the socket */

        if ((server = open_server_by_name(name)) == -1)
        {
		return -1;
        }

        /* Setup up local message servers */

        if (!set_up_standard_jobs(server)) 
	{
		close_server_socket(server);
		return -2;
	}

	/* Local jobs */

        if (!add_message_job(server, PICO_MIRROR_LIST, message_mirror_list)) 
	{
		close_server_socket(server);
		return -3;
	}

        /* Send out requests for machine name and port names */

        mess.type = PICO_GET_MIRROR_LIST;
        mess.length = 0;
        mess.data = NULL;

        if (!send_message(server, &mess))
        {
		close_server_socket(server);
		return -4;
        }

        /* Now let's wait a while and see if we get an answer. */

        start = time(NULL);
        mirror_list[server].n = 0;
        while(time(NULL) < start+TIME_OUT && mirror_list[server].n == 0)
        {
                process_server_socket(server);
        }

        if (mirror_list[server].n == 0)
        {
		close_server_socket(server);
		return -5;
        }

	return server;

} /* open_pico_server() */

/************************************************************************/
/* message_mirror_list()						*/
/*									*/
/************************************************************************/

int message_mirror_list(int server, struct smessage *mess)
{
	unsigned char *p1,*p2;
	int i;

	/* Is the length right? */

	if (mess->length != sizeof(mirror_list[server]))
	{
		error(ERROR,"Mirror list incorrect size.");
		return FALSE;
	}

	/* OK, copy the data over to the real list */

	p1 = mess->data;
	p2 = (unsigned char *)&(mirror_list[server]);
	for(i=0; i<sizeof(mirror_list[server]); i++) *p2++ = *p1++;

	return TRUE;

} /* message_mirror_list() */
