/************************************************************************/
/* shutter.h								*/
/*                                                                      */
/* Header file for the shutter module/client/server controller.		*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar, Laszlo Sturmann                        */
/* Date   : NOVEMBER 2000                                               */      
/************************************************************************/

#ifndef __SHUTTER__
#define __SHUTTER__

/************************************************************************/
/* The stuff for MODULES, CLIENTS and SERVERS				*/
/************************************************************************/

/* 
 * The commands 
 */

#define SHUTTER_OPEN		1
#define SHUTTER_CLOSE   	2
#define SHUTTER_STATUS 		3
#define SHUTTER_MAX_COMMAND 	SHUTTER_STATUS

/* The states */

#define NOTHING -1
#define CLOSED 0
#define OPEN  1
#define MOVING 2
#define INSANE 3

/* The number of shutters */

#define NUMBER_SHUTTERS	16

#ifdef MODULE
/************************************************************************/
/* The stuff for only MODULES                                           */
/************************************************************************/

/* Macros */

#define BASE     0x280
#define CONF1    BASE+3 
#define CONF2    BASE+7
#define CONF3    BASE+11
#define CONF4    BASE+15
#define COMMAND_FIFO	6
#define DATA_FIFO	7

/* structure */

struct shutter_struct {
                int out_offset;
                int out_bit;
                int status_offset;
                int close_bit;
                int open_bit;
               };

typedef struct shutter_struct shutter;

/* prototypes */

int init_module(void);
int shutter_open(unsigned char arg);
int shutter_close(unsigned char arg);
int shutter_status(unsigned char arg);
shutter shutter_map(int id);
int shutter_command_handler(unsigned int fifo_num);
void cleanup_module(void);

#else
/************************************************************************/
/* This stuff for servers and clients.					*/
/************************************************************************/

#define SHUTTER_PROCESS_NAME	"shutters"

#ifdef SERVER
/************************************************************************/
/* This stuff only for servers						*/
/************************************************************************/

#define COMMAND_FIFO_DEV	"/dev/rtf6"
#define DATA_FIFO_DEV		"/dev/rtf7"
#define SHUTTER_DELAY		100000L

/* Some globals */

extern int fd_command;
extern int fd_data;
extern char name[];

/* shutter_server.c */

int main(int argc, char **argv);
void close_function(void);
void scan_init_file(char *filename);

/* shutter_messages.c */

int shutter_configuration(struct smessage *message);
int shutter_open(struct smessage *message);
int shutter_close(struct smessage *message);
int shutter_status(struct smessage *message);

#else
/************************************************************************/
/* This stuff only for clients						*/
/************************************************************************/

#include <errno.h>

extern int server;

#ifdef USE_GTK

extern int shutter_index[NUMBER_SHUTTERS];

/* shutgtk.c */

int main(int   argc, char *argv[]);
void reopen_socket_callback(GtkButton *button, gpointer user_data);
void update_callback( GtkWidget *widget, gpointer   data );
void close_all_callback( GtkWidget *widget, gpointer   data );
void open_all_callback( GtkWidget *widget, gpointer   data );
void shutter_callback( GtkWidget *widget, gpointer   data );
gint delete_event( GtkWidget *widget, GdkEvent  *event, gpointer   data );
gint background_code(gpointer data);
void conf_callback(GtkButton *button, gpointer user_data);
void update_shutter_status(struct sshutter_status *status);

/* shutter_messages.c */

int setup_messages(void);
int message_shutter_status(int server, struct smessage *mess);
int message_shutter_conf(int server, struct smessage *mess);
int send_shutter_open(unsigned char shutter);
int send_shutter_close(unsigned char shutter);
int send_shutter_status(unsigned char shutter);

#endif
#endif
#endif
#endif
