/************************************************************************/
/* weather_messages.c							*/
/*                                                                      */
/* Message handling routines.						*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY CLIENT INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar			                        */
/* Date   : Wed Oct 10 03:42:20 PDT 2001				*/
/************************************************************************/

#include "../../charaui/src/charaui.h"
#include "../../include/chara_messages.h"
#include "../../cliserv/include/cliserv.h"
#include "../../cliserv/uiclient/uiclient.h"
#include "../../sockman/sockman.h"
#include "ui_weather_client.h"

char *weather_file_name = NULL;
int  weather_server = -1;
struct s_weather_data weather;

/************************************************************************/
/* open_weather_server_socket()						*/
/*									*/
/* Opens a socket so we can start sending server data to an weather    	*/
/* server. Returns server number or 					*/
/* -1 on error opening server.						*/
/* -2 on error adding standard jobs					*/
/* -3 on error adding local jobs					*/
/* -4 on error to send name and status messages				*/
/* -5 got no response from server					*/
/************************************************************************/

int open_weather_server_socket(char *name)
{
	int	server;

	/* Try and open the socket */


	if ((server = open_server_by_name(name)) == -1)
	{
		return -1;
	}

	if (!set_up_standard_jobs(server)) 
	{
		close_server_socket(server);
		return -2;
	}
	
	/* Add all the local messages */

        if (!add_message_job(server, WEATHER_NOW, message_weather_now) ||
            !add_message_job(server, WEATHER_FILE_NAME,
		message_weather_file_name)) 
	{
		close_server_socket(server);
		return -3;
	}

	/* That is all */

	return server;

} /* open_weather_server_socket() */

/************************************************************************/
/* message_weather_now()						*/
/*									*/
/************************************************************************/

int message_weather_now(int server, struct smessage *mess)
{
	int	i;
	char	*p1,*p2;

        if (mess->length != sizeof(struct s_weather_data))
        {
		return error(ERROR, 
		"Receiced WEATHER_NOW with wrong amount of data.");
        }

	/* Copy it into the global structure */

	for (p1 = (char *) &weather, p2 = (char *)mess->data, i=0;
		i < sizeof(struct s_weather_data); i++) *p1++ = *p2++;

        return NOERROR;

} /* message_weather_status() */

/************************************************************************/
/* message_weather_file_name()						*/
/*									*/
/************************************************************************/

int message_weather_file_name(int server, struct smessage *mess)
{
	int	i;
	char	*p1, *p2;

	if (weather_file_name != NULL) free(weather_file_name);
	weather_file_name = malloc(mess->length+2);

	for(p1 = (char *)mess->data, p2 = weather_file_name, i=0; 
		i<mess->length; i++) *p2++ = *p1++;
	*p2 = 0;

	return TRUE;

} /* message_weather_names() */

/************************************************************************/
/* get_weather_file_name()						*/
/*									*/
/* Go ask for teh name of the weather data file.			*/
/* Value returned or NULL on error.					*/
/************************************************************************/

char *get_weather_file_name(void)
{
	struct smessage mess;
	time_t  start;

	if (weather_server == -1)
	{
		error(ERROR,"Weather server is not open.");
		return NULL;
	}

	/* prepare */

	if (weather_file_name != NULL) free(weather_file_name);
	weather_file_name = NULL;

	/* Send request */

	mess.type = WEATHER_FILE_NAME;
	mess.length = 0;
	mess.data = NULL;
	if (!send_message(weather_server, &mess))
	{
		error(ERROR, "Failed to send WEATHER_FILE_NAME.\n");
		return NULL;
	}

	/* wait for a response */

	start = time(NULL);
	while(time(NULL) < start+TIME_OUT)
	{
		process_server_socket(weather_server);
		if (weather_file_name != NULL) break;
	}
	if (time(NULL) >= start+TIME_OUT)
	{
		error(ERROR,"Failed to get weahter log file name.\n");
		return NULL;
	}

	return weather_file_name;

} /* get_weather_file_name() */
