/************************************************************************/
/* char_waiting.c							*/
/*                                                                      */
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY CLIENT LIB				*/
/*                 Based on the CHARA User Interface			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                          		*/
/* Date   : Oct 2001							*/
/************************************************************************/

/*
 * Include files
 */

#include "../include/cliserv.h"
#include <errno.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>

/************************************************************************/
/* client_char_waiting()						*/
/*									*/
/* Unix version of kbhit like function. Returns TRUE if a character 	*/
/* waiting at the port, FALSE otherwise.				*/
/* Note that this function can cause problems, so for real keyboard	*/
/* use I go back to a non-block keyboard I/O.				*/
/************************************************************************/

int	client_char_waiting(int fd)
{
	/* Locals */

	fd_set rfds;		/* File discriptor set */
        struct timeval tv;	/* Time to wait */

	if (fd <0) return FALSE;

	/* Set tings up */

	FD_ZERO(&rfds);		 /* Zero the file discriptor set */
        FD_SET(fd, &rfds);	 /* Add stdin to the set */
	tv.tv_sec = 0;		 /* Wait no more than 1 usec */
	tv.tv_usec = 1;

	if (select(FD_SETSIZE, &rfds, NULL, NULL, &tv) < 0) 
	{
		/* Something is wrong. */

		if (errno != EINTR)
		return FALSE;
	}

	if (FD_ISSET(fd, &rfds)) /* Was there something? */
	{
		return TRUE;
	}

	/* Must have been nothing there */

	return FALSE;

} /* client_char_waiting() */

/************************************************************************/
/* client_write_ready()							*/
/*									*/
/* Returns TRUE if a port is ready for writing,	FALSE otherwise.	*/
/* Uses the select(2) routine to do it's work. 				*/
/************************************************************************/

int	client_write_ready(int fd)
{
	/* Locals */

	fd_set wfds;		/* File discriptor set */
        struct timeval tv;	/* Time to wait */

	if (fd <0) return FALSE;

	/* Set tings up */

	FD_ZERO(&wfds);		 /* Zero the file discriptor set */
        FD_SET(fd, &wfds);	 /* Add fd to the set */
	tv.tv_sec = 0;		 /* Wait no more than 1 usec */
	tv.tv_usec = 1;

	if (select(FD_SETSIZE, NULL, &wfds, NULL, &tv) < 0) 
	{
		/* Something is wrong. */

		if (errno != EINTR)
		return FALSE;
	}

	if (FD_ISSET(fd, &wfds)) /* Was there something? */
	{
		return TRUE;
	}

	/* Must have been nothing there */

	return FALSE;

} /* client_write_ready() */

/************************************************************************/
/* client_exception()							*/
/*									*/
/* Returns TRUE if a port has an exception, FALSE otherwise.		*/
/* Uses the select(2) routine to do it's work. 				*/
/* From charaui!!  							*/
/************************************************************************/

int	client_exception(int fd)
{
	/* Locals */

	fd_set efds;		/* File discriptor set */
        struct timeval tv;	/* Time to wait */

	if (fd <0) return FALSE;

	/* Set tings up */

	FD_ZERO(&efds);		 /* Zero the file discriptor set */
        FD_SET(fd, &efds);	 /* Add fd to the set */
	tv.tv_sec = 0;		 /* Wait no more than 1 usec */
	tv.tv_usec = 1;

	if (select(FD_SETSIZE, NULL, NULL, &efds, &tv) < 0) 
	{
		/* Something is wrong. */

		if (errno != EINTR)
		return FALSE;
	}

	if (FD_ISSET(fd, &efds)) /* Was there something? */
	{
		return TRUE;
	}

	/* Must have been nothing there */

	return FALSE;

} /* client_exception() */
