/************************************************************************/
/* gtk_stand_mess.c							*/
/*                                                                      */
/* Standard message handling routines.					*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY CLIENT INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar			                        */
/* Date   : Wed Oct 10 03:42:20 PDT 2001				*/
/************************************************************************/

#define	USE_GTK

#include "../include/cliserv.h"
#include "../../include/chara_messages.h"
#include <stdio.h>
#include <errno.h>

/* Local Globals */

static char gtkclient_name[256];
#ifdef GTK2
GtkTextBuffer *status_buffer = NULL;
static GtkWidget *status_view = NULL;
static GtkTextTag *error_tag, *message_tag, *warning_tag, *noerror_tag;
#else
GtkWidget *status_text = NULL;
#endif

/* Global that keeps track of the number of messages up */

int     number_of_messages_displayed = 0;

/* GLobals for use in pointing at things */

int true_pointer = TRUE;
int false_pointer = FALSE;

/* File pointer for logging status messages. */

FILE *status_logfile = NULL;

/* Page to bring forward when print_status is used. GTK2 only. */

static int status_page = -1;
static GtkWidget *status_notebook = NULL;

/************************************************************************/
/* message_noop()							*/
/*									*/
/* A message to do nothing.						*/
/************************************************************************/

int message_noop(int server, struct smessage *mess)
{
	return TRUE;
}

/************************************************************************/
/* message_message()							*/
/*									*/
/* A message to display a message.					*/
/* If a scroll window exisits it will go there, otherwsie it will	*/
/* make a new window for display.					*/
/************************************************************************/

int message_message(int server, struct smessage *mess)
{
        char *s,*p1,*p2;
        int     i;

        if (mess->length == 0)
        {
		return TRUE;
        }
        else
        {
                if ((s = malloc(mess->length+1)) == NULL)
                {
#ifdef GTK2
			if (status_buffer == NULL)
#else
			if (status_text == NULL)
#endif
				create_message_window(
				"Ran out of memory geting message string.");
			else
				print_status(ERROR,
				"Ran out of memory geting message string.\n");
			exit(-7);
                }

                for(i=0, p1=s, p2=(char *)mess->data;
                    i < mess->length; i++, p1++, p2++) *p1 = *p2;
                *p1 = 0;

#ifdef GTK2
		if (status_buffer == NULL)
#else
		if (status_text == NULL)
#endif
			create_message_window("MESSAGE FROM SERVER: %s\n\n%s",
				servers[server].name,s);
		else
		{
			if (*(p1-1) == '\n')
				print_status(MESSAGE, "%s:%s",
					servers[server].name,s);
			else
				print_status(MESSAGE, "%s:%s\n",
					servers[server].name,s);
		}

                free(s);
        }

	return TRUE;

} /* message_message() */

/************************************************************************/
/* set_client_name()							*/
/************************************************************************/

void set_client_name(char *name)
{
	strcpy(gtkclient_name, name);

} /* set_client_name() */

/************************************************************************/
/* message_ping()							*/
/*									*/
/* If we get pinged by server, although why I do not know.		*/
/************************************************************************/

int message_ping(int server, struct smessage *mess)
{
        print_message(server, "%s is alive.",gtkclient_name);
	return TRUE;

} /* message_ping() */

/************************************************************************/
/* message_server_stopping()						*/
/*									*/
/* What do we do if we get told the server has died?			*/
/************************************************************************/

int message_server_stopping(int server, struct smessage *mess)
{

	close_server_socket(server);
	server = -1;
#ifdef GTK2
        if (status_buffer == NULL)
#else
        if (status_text == NULL)
#endif
	create_message_window(
"Server is stopping.\nYou will have to REOPEN the connection\nonce it is running again.");
	else
	{
		print_status(WARNING,"Server is stopping.\n");
		print_status(WARNING,
			"You will have to REOPEN the connection\n");
		print_status(WARNING,"once it is running again.\n");
	}
	return TRUE;

} /* message_server_stopping() */

/************************************************************************/
/* create_message_window()                                              */
/*                                                                      */
/* Displays a text message in a window for the user to see.             */
/* Code mostly generated by GLADE.                                      */
/************************************************************************/

void create_message_window (char *fmt, ...)
{
  va_list args;
  char    mess[2048];
  GtkWidget *message_window;
  GtkWidget *vbox1;
  GtkWidget *label1;
  GtkWidget *ok_button;

  /* Assemble the message */

  va_start(args, fmt);
  vsprintf(mess, fmt, args);

  /* Display the message */

  message_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (message_window), "message_window",
                message_window);
#warning KLUDGE!!!!!
/* This cause trouble I do not understand 
  gtk_window_set_title (GTK_WINDOW (message_window), 
		(const gchar *)gtkclient_name);
*/

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox1);
  gtk_object_set_data_full (GTK_OBJECT (message_window), "vbox1", vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (message_window), vbox1);

  label1 = gtk_label_new (mess);
  gtk_widget_ref (label1);
  gtk_object_set_data_full (GTK_OBJECT (message_window), "label1", label1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (vbox1), label1, TRUE, TRUE, 0);

  ok_button = gtk_button_new_with_label ("OK");
  gtk_widget_ref (ok_button);
  gtk_object_set_data_full (GTK_OBJECT (message_window), "ok_button", ok_button,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (ok_button);
  gtk_box_pack_start (GTK_BOX (vbox1), ok_button, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (ok_button), "clicked",
                      GTK_SIGNAL_FUNC (message_ok_button_clicked),
                      NULL);

  number_of_messages_displayed++;

  gtk_widget_show(message_window);

}

/************************************************************************/
/* message_ok_button_clicked()                                          */
/*                                                                      */
/* Callback function for OK button in create_message_window().          */
/************************************************************************/

void message_ok_button_clicked(GtkButton *button, gpointer user_data)
{
        gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET (button)));
	if (--number_of_messages_displayed < 0)
		number_of_messages_displayed = 0;

} /* message_ok_button_clicked() */

/************************************************************************/
/* noop_callback()							*/
/*									*/
/* Callback for NOOP, data is the server to use.			*/
/************************************************************************/

void noop_callback( GtkWidget *widget, gpointer   data )
{
	struct smessage mess;
	int	*server;

	server = (int *) data;

        /* Make sure everything is OK */

        if (!server_socket_open(*server)) return;

        /* Construct command */

        mess.type = NOOP;
        mess.length = 0;
        mess.data = NULL; 

        if (!send_message(*server, &mess))
	{
#ifdef GTK2
			if (status_buffer == NULL)
#else
			if (status_text == NULL)
#endif
			create_message_window("Failed to send NOOP message.");
		else
			print_status(ERROR, "Failed to send NOOP message.\n");
	}

} /* noop_callback() */

/************************************************************************/
/* ping_callback()							*/
/*									*/
/* Callback for PING, data is the server to use.			*/
/************************************************************************/

void ping_callback( GtkWidget *widget, gpointer   data )
{
	struct smessage mess;
	int	*server;

	server = (int *) data;

        /* Make sure everything is OK */

        if (!server_socket_open(*server)) return;

        /* Construct command */

        mess.type = PING;
        mess.length = 0;
        mess.data = NULL; 

        if (!send_message(*server, &mess))
	{
#ifdef GTK2
                if (status_buffer == NULL)
#else
                if (status_text == NULL)
#endif
			create_message_window("Failed to send PING.");
		else
			print_status(ERROR,"Failed to send PING.\n");
	}

} /* ping_callback() */

/************************************************************************/
/* all_noop_callback()							*/
/*									*/
/* Callback for ALL_NOOP, data is the server to use.			*/
/************************************************************************/

void all_noop_callback( GtkWidget *widget, gpointer   data )
{
	struct smessage mess;
	int	*server;

	server = (int *) data;

        /* Make sure everything is OK */

        if (!server_socket_open(*server)) return;

        /* Construct command */

        mess.type = ALL_NOOP;
        mess.length = 0;
        mess.data = NULL; 

        if (!send_message(*server, &mess))
	{
#ifdef GTK2
                if (status_buffer == NULL)
#else
                if (status_text == NULL)
#endif
			create_message_window(
				"Failed to send ALL_NOOP message.");
		else
			print_status(ERROR,
				"Failed to send ALL_NOOP message.\n");
	}

} /* all_noop_callback() */

/************************************************************************/
/* all_ping_callback()							*/
/*									*/
/* Callback for ALL_PING, data is the server to use.			*/
/************************************************************************/

void all_ping_callback( GtkWidget *widget, gpointer   data )
{
	struct smessage mess;
	int	*server;

	server = (int *) data;

        /* Make sure everything is OK */

        if (!server_socket_open(*server)) return;

        /* Construct command */

        mess.type = ALL_PING;
        mess.length = 0;
        mess.data = NULL; 

        if (!send_message(*server, &mess))
	{
#ifdef GTK2
                if (status_buffer == NULL)
#else
                if (status_text == NULL)
#endif
			create_message_window(
				"Failed to send ALL_PING message.");
		else
			print_status(ERROR,
				"Failed to send ALL_PING message.\n");
	}

} /* all_ping_callback() */

/************************************************************************/
/* send_ui_signal()							*/
/*									*/
/* Sends a signal to the server.					*/
/************************************************************************/

void send_ui_signal(int server, unsigned char signal)
{
	struct smessage mess;

        /* Make sure everything is OK */

        if (!server_socket_open(server)) return;

        /* Construct command */

        mess.type = UI_SIGNAL;
        mess.length = sizeof(signal);
        mess.data = &signal; 

        if (!send_message(server, &mess))
	{
#ifdef GTK2
                if (status_buffer == NULL)
#else
                if (status_text == NULL)
#endif
			create_message_window(
				"Failed to send UI_SIGNAL message.");
		else
			print_status(ERROR,
				"Failed to send UI_SIGNAL message.\n");
	}

} /* send_ui_signal() */

/************************************************************************/
/* fill_status_view()							*/
/*									*/
/* Given a container, places the stuff required to create a status text	*/
/* area that can be used by print_status.				*/
/************************************************************************/

void fill_status_view(GtkWidget *vbox, int width, int length)
{
        GtkWidget *scroll_window;

        /* Put in a scroll window so we can receive messages */

        scroll_window = gtk_scrolled_window_new (NULL, NULL);
        gtk_widget_ref (scroll_window);
        gtk_container_add (GTK_CONTAINER (vbox), scroll_window);
        gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll_window),
                GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
        gtk_widget_set_usize (scroll_window, width, length);
        gtk_widget_show (scroll_window);

#ifdef GTK2
        status_view = gtk_text_view_new();
        status_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW (status_view));
        gtk_text_view_set_wrap_mode((GtkTextView *)status_view,GTK_WRAP_WORD);
        gtk_widget_ref (status_view);
        gtk_container_add (GTK_CONTAINER (scroll_window), status_view);
        gtk_widget_show (status_view);

        /* Create tags for colors */

        error_tag = gtk_text_buffer_create_tag(status_buffer, 
                        "red_foreground", "foreground", "red", NULL);
        warning_tag = gtk_text_buffer_create_tag(status_buffer, 
                        "orange_foreground", "foreground", "orange", NULL);
        message_tag = gtk_text_buffer_create_tag(status_buffer, 
                        "blue_foreground", "foreground", "blue", NULL);
        noerror_tag = gtk_text_buffer_create_tag(status_buffer, 
                        "black_foreground", "foreground", "black", NULL);
#else
        status_text = gtk_text_new (NULL, NULL);
        gtk_widget_ref (status_text);
        gtk_container_add (GTK_CONTAINER (scroll_window), status_text);
        gtk_widget_show (status_text);
#endif

} /* fill_status_view() */

/************************************************************************/
/* print_status()							*/
/*									*/
/* Like a printf,will print stuff to the status text window.		*/
/************************************************************************/

int	print_status(int level, char *fmt, ...)
{
	time_t current_time;
        struct tm *gmt_now;
	va_list	args;
	char	s[10000];
	int	retval;
#ifdef GTK2
        GtkTextIter start, end;
	GtkTextMark *mark;
#else
	GdkColor color_error = {4L,65535,0,0};
	GdkColor color_message = {4L,0,0,65535};
	GdkColor color_warning = {4L,65535,32000,0};
	GdkColor color_noerror = {4L,0,0,0};
	GdkColor *color;
#endif

	va_start(args, fmt);
	retval = vsprintf(s, fmt, args);

#ifdef GTK1
	switch(level)
	{
		case NOERROR: color = &color_noerror;
			      break;

		case MESSAGE: color = &color_message;
			      break;

		case WARNING: color = &color_warning;
			      break;

		default: color = &color_error;
			      break;
	}
#endif
	if (status_logfile != NULL) 
	{

		time(&current_time);
                gmt_now = gmtime(&current_time);
			
		fprintf(status_logfile,"%02d:%02d:%02d: %s",
			gmt_now->tm_hour,gmt_now->tm_min,gmt_now->tm_sec,s);
		fflush(status_logfile);
	}

#ifdef GTK2
        if (status_buffer != NULL)
        {
		/* Mark current cursor position */

		gtk_text_buffer_get_end_iter(status_buffer, &end);
		mark = gtk_text_buffer_create_mark(status_buffer, 
			NULL, &end, TRUE);

		/* Put in the text */

                gtk_text_buffer_insert_at_cursor(status_buffer, s, strlen(s));

		/* Where did that text go? */

		gtk_text_buffer_get_iter_at_mark(status_buffer, &start, mark);
		gtk_text_buffer_get_end_iter(status_buffer, &end);

		/* Force color on this bit of text */

		switch(level)
		{
			case NOERROR: gtk_text_buffer_apply_tag(status_buffer, 
					noerror_tag, &start, &end);
				      break;

			case MESSAGE: gtk_text_buffer_apply_tag(status_buffer,
					message_tag, &start, &end);

				      break;

			case WARNING: gtk_text_buffer_apply_tag(status_buffer,
					warning_tag, &start, &end);
				      break;

			default: gtk_text_buffer_apply_tag(status_buffer,
					error_tag, &start, &end);
				      break;
		}
		gtk_text_buffer_delete_mark(status_buffer, mark);

		/* Now ensure user sees this. */

		gtk_text_buffer_get_end_iter(status_buffer, &end);
		mark = gtk_text_buffer_create_mark(status_buffer, 
			NULL, &end, TRUE);
		gtk_text_view_scroll_mark_onscreen(GTK_TEXT_VIEW (status_view),
			mark);
		gtk_text_buffer_delete_mark(status_buffer, mark);
			
		if (status_page != -1 && status_notebook != NULL)
			gtk_notebook_set_current_page(
				(GtkNotebook *)status_notebook, status_page);
	}
#else
        if (status_text != NULL)
        {
                gtk_text_insert(GTK_TEXT(status_text),
                        NULL, color, NULL, s, strlen(s));
	}
#endif
	else
	{
		fprintf(stderr, s);
		fflush(stderr);
	}

	return retval;

} /* print_status() */

/************************************************************************/
/* set_status_notebook_and_page()					*/
/*									*/
/* Set global status_page. This page will be made the current page if	*/
/* not -1, and after anything is printed in the status area.		*/
/************************************************************************/

void set_status_page(GtkWidget *notebook, int page)
{
	status_notebook = notebook;
	status_page = page;

} /* set_status_page() */

/************************************************************************/
/* message_make_ping_noise()						*/
/*									*/
/* The machien that goes ping!.						*/
/************************************************************************/

int message_make_ping_noise(int server, struct smessage *mess)
{
	system("play /ctrscrut/chara/etc/ping.wav &");
	return TRUE;

} /* message_make_ping_noise() */

/************************************************************************/
/* message_make_ring_noise()						*/
/*									*/
/* The machien that goes ring!.						*/
/************************************************************************/

int message_make_ring_noise(int server, struct smessage *mess)
{
	system("play /ctrscrut/chara/etc/ring.wav &");
	return TRUE;

} /* message_make_ring_noise() */

/************************************************************************/
/* error()								*/
/*									*/
/* Like a printf,will print stuff to the status text window.		*/
/* Mostly herer for sake of Macs and to fake out the error() function.	*/
/************************************************************************/

int error(int level, char *fmt, ...)
{
	va_list	args;
	char	s[10000];
	

	va_start(args, fmt);
	vsprintf(s, fmt, args);

	return print_status(level, s);

} /* error() */
