/************************************************************************/
/* cliserv.h								*/
/*                                                                      */
/* General Header File.							*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY CLIENT/SERVER				*/
/*                 Based on the CHARA User Interface			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@@chara.gsu.edu                                       */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                          		*/
/* Date   : Oct 2001							*/
/************************************************************************/

#ifndef __CLISERV__
#define __CLISERV__

/*
 * General include files 
 */

#include <stdio.h>
#include <time.h>
#include <signal.h>
#include <netinet/tcp.h>
#include <string.h>
#include <stdlib.h>
#include <termios.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdarg.h>
#include <ctype.h>

/************************************************************************/
/* Stuff for both Clients and Servers.					*/
/************************************************************************/

#ifndef YES
#define YES		1
#define NO		0
#endif
#ifndef TRUE
#define TRUE		1
#define FALSE		0
#endif
#ifndef OFF
#define OFF		0
#define ON		1
#endif
#ifndef EVEN
#define EVEN		2
#define ODD		3
#endif
#ifndef bool
#define bool char
#endif

#ifndef TIME_OUT
#define TIME_OUT	5
#endif

/* Error messages */

#ifndef ERROR
#ifndef MESSAGE
#define MESSAGE 1 /* A message tot he user */
#endif
#define NOERROR	0 /* No error has occured */
#define WARNING (-1) /* A warning, nothing too dangerous has happened */
#define ERROR	(-2) /* Some kind of major error.Prog does not have to reboot */
#define FATAL	(-3) /* A fatal error has occured. The programme must exit(1)*/ 
#endif 

#ifndef SERVER_SOCKET_BUFFER
#define SERVER_SOCKET_BUFFER   65535		/* For socket read/writes */
#endif

#ifndef MAX_JOBS
#define MAX_JOBS	5000
#endif

/* Here we define the message structure */

#ifndef __CHARAUI__
#ifndef __SMESSAGE__
struct smessage {
			unsigned short int type;
                        unsigned int       length;
                        unsigned char *data;
		};

typedef struct smessage message_struct;
#define __SMESSAGE__
#endif
#endif

/************************************************************************/
/* Stuff for Servers only.						*/
/************************************************************************/

#ifdef SERVER

/* Macros */


#define DEFAULT_SERVER_PORT 4000

#define ALLOWED_DOMAIN "chara-array.org"/* allowed domain for socket commands */

#define LOCK_DIR	"/var/lock"
#define LOCK_START	"LCK.."

#define MAX_CONNECTIONS 32		/* Maximum number of socket connects */
#define BUFFER_SIZE     1024		/* For serial read/writes */

#define serial_char_waiting(port) char_waiting(port.fd)

#define serial_putstr(port,s)	write(port.fd,s,strlen(s))

#define ui_clear_screen()	system("clear")

#define serial_char_waiting(port) char_waiting(port.fd)

#define serial_putstr(port,s)	write(port.fd,s,strlen(s))

/* 
 * These macros let you play with buffers
 */

#define	buffer_not_full(x)	(x->in != x->out)
#define buffer_full(x)		(x->in == x->out)
#define buffer_not_empty(x)	((x->out != x->in - 1) && \
				((x->out != x->top) || (x->in != x->bottom)))
#define buffer_empty(x)		((x->out == x->in - 1) || \
				((x->out == x->top) && (x->in == x->bottom)))

#define buffer_in(x,y)		{if (buffer_not_full(x)) \
					 {\
						*(x->in) = (y);\
						if (++(x->in) > x->top) \
							x->in = x->bottom;\
					 }\
				}
	
#define buffer_out(x,y)		{if (buffer_not_empty(x)) \
					 {\
						if (++(x->out) > x->top) \
							x->out = x->bottom;\
						(y) = *(x->out);\
					 }\
				}

/* Structures */

struct sserial 
{
	int    fd;		     /* Port file descriptor */
	char   name[81];	     /* Port name */
	char   lock_file[81];	     /* lock_file name */
        struct termios orig_termios; /* original termio data */
        struct termios curr_termios; /* current termio data */
};

typedef	struct sbuffer{
			unsigned char	*bottom;
			unsigned char	*top;
			unsigned char 	*in;
			unsigned char	*out;

		} BUFFER;

/* Globals */

extern int active_socket;		/* 
				 	 * The current active socket. 
					 * Set to -1 if no active socket.
					 */

extern char *active_machine;		/* 
				 	 * The current active machine. 
					 * Set to 0 if no active machine.
					 */

extern char *server_name;		/* The name of our server */

extern bool use_type_backlog;		/* Mostly we don't, 
					 * only in socket manager
					 */

extern bool use_non_socket_manager;	/* What kind of socket manager? */

/* socket.c */

int auto_open_message_socket(char *name, char *restart_command);
int open_message_socket(int port);
void set_process_name(char *name);
void close_message_socket(void);
int	process_message_socket(void);
char *get_server_name(char *name);
void flush_socket(int sock);
void flush_sockets(void);
int server_send_message(int sock, struct smessage *message);
int server_send_message_all(struct smessage *message);
int read_socket(int fd, char *buf, size_t count);
void    sig_pipe(int sig);
void set_fatal_error_on_identify_error(bool which);
void set_standalone(bool value);

/* char_waiting.c */

int char_waiting(int fd);
int write_ready(int fd);
int exception(int fd);
char    *getline_cs(char *string, int n, FILE *stream);

/* error.c */

int error(int level, char *fmt, ...);
bool open_server_logfile(char *logfilename);
void close_server_logfile(void);

/* serial.c */

extern struct sserial open_serial_port(char *port_name, bool blocking);
extern char 	*lock_file_name(char *port, char *s);
extern int	close_serial_port(struct sserial port);
extern int set_serial_baud_rate(struct sserial *port, speed_t speed);
extern int set_serial_parity(struct sserial *port, int parity);
extern int set_serial_xonxoff(struct sserial *port, int xonxoff);
extern int set_serial_hard_handshake(struct sserial *port, int hard_handshake);
extern int set_serial_bitlength(struct sserial *port, int bits);
extern int set_serial_stopbits(struct sserial *port, int bits);
extern unsigned char serial_getchar(struct sserial port);
extern unsigned char serial_putchar(struct sserial port, unsigned char c);
extern int serial_print(struct sserial port, char *fmt, ...);
extern int serial_scan(struct sserial port, char *fmt, ...);
extern char *serial_gets(struct sserial port, char *s, int n);

/* job_control.c */

void server_init_jobs(void);
void server_add_message_job(unsigned int type, 
	int (*job)(struct smessage *message));
void add_top_job(int (*job)(void));
void add_background_job(int (*job)(void));
void process_job(void);
void process_jobs(void);
void setup_close_down(void (*close_down_routine)(void));
void close_server(int sig);
void server_run_message_job(struct smessage *message);
void set_server_name(char *name);

/* standard_messages.c */

void setup_signals(void);
int message_server_noop(struct smessage *message);
int message_server_message(struct smessage *message);
int message_server_ping(struct smessage *message);
int message_server_all_noop(struct smessage *message);
int message_server_all_ping(struct smessage *message);
int message_server_uistatus(struct smessage *message);
int message_server_signal(struct smessage *message);
bool any_client_signal_set(void);
bool client_signal_set(unsigned char signal);
void clear_client_signal(unsigned char signal);
int server_print_message(int sock, char *fmt, ...);
int server_print_message_all(char *fmt, ...);
int print_log(char *fmt, ...);
int send_server_stopping(void);

/************************************************************************/
/* Stuff for Clients only.						*/
/************************************************************************/
#else

/* Macros */

#define MAX_SERVERS	256	/* How many servers can we talk to? */

/* Globals */

extern struct sservers {
        int fd;
	char name[256];
	int port;
	char machine[256];
        int (*(message_jobs[MAX_JOBS]))(int server, struct smessage *message);
              } servers[MAX_SERVERS];

extern short int message_array[MAX_JOBS];
extern unsigned char signal_array[256];

/* Prototypes */

/* char_waiting.c */

int client_char_waiting(int fd);
int client_write_ready(int fd);
int client_exception(int fd);

/* job_control.c */

void init_jobs(void);
int add_message_job(int server, unsigned int type, 
		int (*job)(int server, struct smessage *message));
void remove_message_job(int server, unsigned int type);
int run_message_job(int server, struct smessage *message);
int message_do_nothing(int server, struct smessage *mess);

/* socket.c */

int server_socket_open(int server);
int open_server_by_name(char *name);
int open_server_socket(char *machine, int port, char *name);
void close_server_socket(int server);
int process_server_socket_all_messages(int server);
int process_server_socket(int server);
int client_read_socket(int fd, char *buf, size_t count);
void flush_server_socket(int server);
int send_message(int server, struct smessage *message);
int print_message(int server, char *fmt, ...);

/************************************************************************/
/* Stuff for GTK Clients only.						*/
/************************************************************************/
#ifdef USE_GTK

#include <gtk/gtk.h>

/*
 * This structure is for passing the name and a server ID to
 * the create_status_window() callback function.
 */

struct s_status_window  {
				char	name[256];
				int	server;
			};

#ifdef GTK2
extern GtkTextBuffer *status_buffer;
#else
extern GtkWidget *status_text;
#endif

/* Global that keeps track of the number of messages up */

extern int	number_of_messages_displayed;

/* 
 * Note that we assume there is a function called delete_event
 * whose job it is to close everything down.
 * This must be part of the client's code.
 */

extern gint delete_event(GtkWidget *widget, GdkEvent  *event, gpointer   data );

/* Logfile for error messages and such */

extern FILE *status_logfile;

/* Global for a status page */


/*
 * Some globals for use in pointing at things.
 */

extern int true_pointer;
extern int false_pointer;
#define P_TRUE (&true_pointer)
#define P_FALSE (&false_pointer)

/* gtk_standard_messages.c */

int message_noop(int server, struct smessage *mess);
int message_message(int server, struct smessage *mess);
void set_client_name(char *name);
int message_ping(int server, struct smessage *mess);
int message_server_stopping(int server, struct smessage *mess);
void create_message_window (char *fmt, ...);
void message_ok_button_clicked(GtkButton *button, gpointer user_data);
void noop_callback( GtkWidget *widget, gpointer   data );
void ping_callback( GtkWidget *widget, gpointer   data );
void all_noop_callback( GtkWidget *widget, gpointer   data );
void all_ping_callback( GtkWidget *widget, gpointer   data );
void send_ui_signal(int server, unsigned char signal);
void fill_status_view(GtkWidget *vbox, int width, int length);
int	print_status(int level, char *fmt, ...);
int message_make_ping_noise(int server, struct smessage *mess);
int message_make_ring_noise(int server, struct smessage *mess);
void set_status_page(GtkWidget *notebook, int page);
int error(int level, char *fmt, ...);

/* gtk_status_window.c */

GtkWidget* create_status_window (GtkButton *button, gpointer *data);
void status_ok_callback(GtkButton *button, gpointer user_data);
gint status_background(gpointer data);
int message_status(int server, struct smessage *mess);
void fill_ui_status_page(GtkWidget *container);
void start_ui_status(GtkButton *button, gpointer *data);
void stop_ui_status(GtkButton *button, gpointer *data);
void set_ui_status_callback(void (*callback)(void));

#endif
#endif
#endif
