/************************************************************************/
/* clock.h								*/
/*                                                                      */
/* Header file for RT CHARA Clock.					*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : May 1998                                                    */
/************************************************************************/

#ifndef __CLOCK__
#define __CLOCK__

#include <string.h>
#include <sys/types.h>
#include <unistd.h>

/* Macros */

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

#ifndef YES
#define YES 1
#define NO  0
#endif

#define TICKS_PER_SECOND	1000	/* How fast is the clock? */
#define TICKS_PER_MINUTE	(TICKS_PER_SECOND*60)
#define TICKS_PER_HOUR		(TICKS_PER_MINUTE*60)
#define TICKS_PER_DAY		(TICKS_PER_HOUR*24)

#define CHARA_TIME	int	/* Variable used for all times */
#define CHARA_TIME_END  0x7FFFFFFF

#define DEFAULT_BYPASS_CLOCK_DELAY	350

/*
 * Global Variables
 */

typedef struct	RT_TIME_STRUCT 
	{
		CHARA_TIME time_now;
		CHARA_TIME lost_ticks;
		CHARA_TIME lost_seconds;

	} rt_time_struct;	/* Structure used to send times */

/* Macros */

#ifndef bool
#define bool char
#endif

/* Structures */

typedef struct	AP_TIME_STRUCT 
	{
		char		sign; /* -1 or +1 */
		CHARA_TIME	hrs;
		CHARA_TIME	min;
		CHARA_TIME	sec;
		CHARA_TIME	tick;

	} ap_time_struct;	/* Structure used to send times */

#ifndef __SMESSAGE__
struct smessage {
                        unsigned short int      type;
                        unsigned int            length;
                        unsigned char           *data;
                };

typedef struct smessage message_struct;
#define __SMESSAGE__
#endif

/* chara_clock.c */

int set_bypass_clock_delay(int delay);
int set_bypass_clock_driver(bool bypass);
int open_clock_device(void);
void close_clock_device(void);
int clear_lost_counts(int argc, char **argv);
int get_rt_time(rt_time_struct *the_time);
CHARA_TIME chara_time_now(void);
int set_rt_time(CHARA_TIME time_next_second);
CHARA_TIME get_time_next_second(int wait);
struct AP_TIME_STRUCT rt_to_ap_time(int rt_time);
CHARA_TIME ap_to_rt_time(struct AP_TIME_STRUCT t);
struct AP_TIME_STRUCT str_to_ap_time(char *s);
char *ap_to_str_time(char *s, struct AP_TIME_STRUCT t);
#ifdef __CHARAUI__
int set_time(int argc, char **argv);
int sync_times(int argc, char **argv);
int show_time(int argc, char **argv);
#else
int sync_times(void);
#endif
double sidereal_time(void);
double modified_julian_date(void);
bool clock_running(void);
int check_clock_offset(void);
void set_max_clock_offset(int max_offset);
void set_clock_offset_callback(void (*offset_callback)(void));

/* clock_messages.c */

void setup_standard_clock_messages(void);
int clock_clear_lost_counts(struct smessage *message);
int clock_rt_time(struct smessage *message);
int clock_time_next_second(struct smessage *message);
int clock_set_rt_time(struct smessage *message);
int clock_sync_times(struct smessage *message);

#endif
