/************************************************************************/
/* test_clock.c								*/
/*                                                                      */
/* Test program for 2.6 kernel clock driver code.			*/
/* Not built automatically. You need to type:				*/
/*									*/
/* make -f Makefile.2.6.18-128.el5 test_clock				*/
/*									*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar			                        */
/* Date   : Oct 2009							*/
/************************************************************************/

#include "../charaui/src/charaui.h"
#include "./clock.h"
#include "../charalib/chara.h"

/* Prototypes */

int main(int argc, char **argv);
int report_time_next_second(int argc, char **argv);
int linux_time_status(void);
int rt_time_status(void);
void chara_close(void);

void print_usage_message(char *name)
{
        fprintf(stderr,"usage: %s [-flags]\n",name);
        fprintf(stderr,"Flags:\n");
        fprintf(stderr,"-c\tBypass clock driver and use local clock (FALSE)\n");
        fprintf(stderr,"-h\tPrint this message\n");
        fprintf(stderr,"-s\tToggle use of socketmanager (ON)\n");
}

void test_offset_callback(void)
{
	message(system_window,"OFFSET TOO LARGE!");
	usleep(20000);
	werase(system_window);
	wrefresh(system_window);
}

int main(int argc, char **argv)
{
	char	*p, *progname;

	/* Check out command line */

	progname = argv[0];
        while(--argc > 0 && (*++argv)[0] == '-')
        {
             for(p = argv[0]+1; *p != '\0'; p++)
             {
                switch(*p)
                {
			case 'c': set_bypass_clock_driver(TRUE);
                                  break;

			case 'h': print_usage_message(progname);
                                  exit(0);

                        case 's': use_socket_manager = !use_socket_manager;
                                  break;

			default: fprintf(stderr,"Unknown flag %c.\n",*p);
                                 print_usage_message(progname);
                                 exit(-1);
                                 break;
		}
	    }
	}

	/* Title page. */

	ui_clear_screen();
	put_line("");
	center_line("CHARA 2.6 KERNEL CLOCK TEST PROGRAM");
	put_line("");
	center_line("The CHARA Array");
	center_line("Center for High Angular Resolution Astronomy");
	center_line("Mount Wilson Observatory, CA 91001, USA");
	put_line("");
	center_line("Telephone: 1-626-796-5405");
	center_line("Fax: 1-626-796-6717");
	center_line("email: theo@chara-array.edu");
	center_line("WWW: http://www.chara.gsu.edu");
	put_line("");
	center_line("(C) This executable program is copyright.");
	wait_for_title();

	/* Initialize the user interface */

	TITLE = "CLOCK TEST VERSION 1.0";
	initialise_ui("menus.ini", "help.ini", "test_clock");
        set_user_close_function(chara_close);

	/* Try and open up the FIFOs */

        if (!open_clock_device()) error(FATAL,"Failed to open FIFOs");

	/* Setup background job(s) */

	background_add(linux_time_status);
	background_add(rt_time_status);
	//background_add(check_clock_offset);
	//set_clock_offset_callback(test_offset_callback);
	//set_max_clock_offset(1000);

        /* Set up the client port */

	setup_standard_clock_messages();

	/* Make sure the background runs */

	background_start(0, NULL);

	/* Let's go! */

	start_ui(); /* Should never return from here. */

	exit(0);

} /* main() */

/************************************************************************/
/* linux_time_status()							*/
/* 									*/
/* Displays the linux system time in the status window.			*/
/************************************************************************/

int linux_time_status(void)
{
	time_t current_time;
	struct tm *now;

	time(&current_time);
	now = localtime(&current_time);
	wstandout(status_window);
	mvwaddstr(status_window,0,0,"Linux Time: ");
	wstandend(status_window);
	wprintw(status_window," %2d:%02d:%02d",
		now->tm_hour,now->tm_min,now->tm_sec);

	return NOERROR;

} /* linux_time_status() */

/************************************************************************/
/* rt_time_status()							*/
/* 									*/
/* Displays the rt system time in the status window.			*/
/************************************************************************/

int rt_time_status(void)
{
	rt_time_struct now;
	ap_time_struct t;
	char	s[20],*p;

	if (get_rt_time(&now) != 0) 
		return error(ERROR,"Failed to get RT time.");

	wstandout(status_window);
	mvwaddstr(status_window,1,0,"CHARA Time: ");
	wstandend(status_window);
	t = rt_to_ap_time(now.time_now);
	wprintw(status_window,"%s",ap_to_str_time(s,t));

	wstandout(status_window);
        mvwaddstr(status_window,2,0,"Sid Time  : ");
        wstandend(status_window);
        time_to_string(sidereal_time(),s);
        for(p=s; *p!= 0; p++)
        {
                if (p != s && *p == ' ') *p = ':';
                if (*p == '.') *p = 0;
        }
        wprintw(status_window,"%s",s);

	wstandout(status_window); mvwaddstr(status_window,3,0,"Lost Ticks: ");
	wstandend(status_window); wprintw(status_window,"%9d",now.lost_ticks);

	wstandout(status_window);
	mvwaddstr(status_window,4,0,"Lost Secs : ");
	wstandend(status_window);
	wprintw(status_window,"%9d",now.lost_seconds);

	wstandout(status_window);
	mvwaddstr(status_window,5,0,"Tick time : ");
	wstandend(status_window);
	wprintw(status_window,"%9d",now.time_now);

	return NOERROR;

} /* rt_time_status() */

/************************************************************************/
/* report_time_next_second() 						*/
/* 									*/
/* User callable routine to test get_time_next_second().		*/
/************************************************************************/

int report_time_next_second(int argc, char **argv)
{
	message(system_window,
		"Time of next second = %d.",get_time_next_second(TRUE));

	return NOERROR;

} /* report_time_next_second() */

/************************************************************************/
/* report_time() 							*/
/* 									*/
/************************************************************************/

int report_time(int argc, char **argv)
{
	rt_time_struct the_time;

	if (get_rt_time(&the_time) != 0)
		return error(ERROR,"Failed ot get current time.");

	
	message(system_window,
		"Time = %d Lost ticks = %d Lost seconds = %d", 
		the_time.time_now, the_time.lost_ticks, the_time.lost_seconds);

	return NOERROR;

} /* report_time() */

/************************************************************************/
/* chara_close()                                                        */
/*                                                                      */
/* Function called at close time.                                       */
/************************************************************************/
 
void chara_close(void)
{
	close_clock_device();

} /* chara_close() */
