#include "cpk.h"

void cpkC2s ( double x, double y, double z, double *a, double *b )

/*
**  - - - - - - -
**   c p k C 2 s
**  - - - - - - -
**
**  Cartesian coordinates to spherical coordinates.
**
**  Given:
**     x       d     x coordinate
**     y       d     y coordinate
**     z       d     z coordinate
**
**  Returned:
**     a       d*    longitude angle (e.g. RA) in radians
**     b       d*    latitude angle (e.g. Dec) in radians
**
**  Notes:
**
**    1  No validation of the arguments is performed.
**
**    2  The Cartesian coordinates need not be for a unit vector.
**
**    3  Right-handed coordinate systems are assumed:  RA=a, HA=-a,
**       Az=pi-a.  For the (HA,Dec) case, the alternative is to flip y
**       first.  For the (Az,El) case, the alternative is to flip x
**       first.
**
**  Last revision:   9 January 2007
**
**  Copyright P.T.Wallace.  All rights reserved.
*/

{
   double r2;


   r2 = x*x + y*y;
   *a = x != 0.0 || y != 0.0 ? atan2 ( y, x ) : 0.0;
   *b = r2 != 0.0 || z != 0.0 ? atan2 ( z, sqrt ( r2 ) ) : 0.0;

}
