#include "cpk.h"

void cpkGuide ( double dc, double db, TRANSP *pars )

/*
**  - - - - - - - - -
**   c p k G u i d e
**  - - - - - - - - -
**
**  Make temporary adjustments to the pointing model in order to move
**  the target image to the right place in the focal plane.
**
**  Given:
**     dc       d        guiding correction in collimation (radians)
**     db       d        guiding correction in pitch (radians)
**
**  Returned:
**     pars   TRANSP*    updated
**
**  Notes:
**
**    1  No validation of the arguments is performed.  For example,
**       omitting a scale factor from arcseconds to radians would simply
**       produce extremely large guiding corrections.
**
**    2  The guiding correction in collimation is added to the pointing
**       coefficient CH for an equatorial or CA for an altazimuth.  The
**       guiding correction in pitch is added to the pointing
**       coefficient ID for an equatorial or IE for an altazimuth.
**
**    3  The guiding correction is an offset in position that persists
**       until explicitly canceled, by a call to the present function
**       with both arguments zero.  Except for the initial transient, it
**       does not produce a tracking rate adjustment.
**
**    4  This form of guiding moves the telescope without changing
**       the celestial coordinates of the target.  The TCS application
**       will normally also support a mode where the guiding comes
**       instead from adjusting the celestial coordinates of the target,
**       which does not involve the present function.  The third option,
**       namely controlling which place in the focal plane is to receive
**       the target's image, does involve the present function, but
**       using guiding corrections that depend on the instrument mount
**       position angle.  It is the responsibility of the telescope user
**       to decide which of these modes is appropriate to the task in
**       hand, and it is the responsibility of the TCS application
**       developer to implement suitable code.
**
**    5  The character of the guiding corrections (i.e. as collimation
**       adjustments) makes them suitable for start-of-night "trimming"
**       corrections, matching such effects as optical alignment and
**       temperature differentials.  If the guiding corrections required
**       to center the first star (or their average over a few stars)
**       are saved by the TCS application and used as a fixed basis for
**       subsequent operations, the all-sky pointing will probably be
**       improved.  However, the character of the corrections does not
**       match loss of encoder zero points, at least in the "roll"
**       coordinate (i.e. HA or Az).  A start-of-night alignment
**       procedure for a mount lacking absolute encoders would require
**       stars at different pitch angles to be observed and the results
**       analyzed to distinguish between (i) index errors in HA or Az
**       and (ii) nonperpendicularity between the boresight and the
**       Dec or El axis.
**
**  Last revision:   11 January 2007
**
**  Copyright P.T.Wallace.  All rights reserved.
*/

{

   pars->gc = dc;
   pars->gb = db;

}
