#include "cpk.h"

double cpkMjd ( int iy, int im, int id )

/*
**  - - - - - - -
**   c p k M j d
**  - - - - - - -
**
**  Gregorian calendar date to Modified Julian Date.
**
**  Given:
**     iy       i    year
**     im       i    month
**     id       i    day
**
**  Returned:
**              d    Modified Julian Date: JD-2400000.5
**
**  Notes:
**
**    1  The algorithm used is valid from -4800 March 1.
**
**    2  No validation of the arguments is performed.
**
**    3  In early eras the conversion is from the "Proleptic Gregorian
**       Calendar";  no account is taken of the date(s) of adoption of
**       the Gregorian Calendar, nor is the AD/BC numbering convention
**       observed.
**
**  Reference:
**
**     Explanatory Supplement to the Astronomical Almanac,
**     P.Kenneth Seidelmann (ed), University Science Books (1992),
**     Section 12.92 (p604).
**
**  Last revision:   9 January 2007
**
**  Copyright P.T.Wallace.  All rights reserved.
*/

{
   int my, iypmy;


   my = ((im-14)/12);
   iypmy = iy+my;

   return (double) ( ( 1461L * (long) ( iypmy + 4800 ) ) / 4L
                     + (long) ( ( 367 * ( im - 2 - 12*my ) ) / 12
                     - ( 3 * ( ( iypmy + 4900 ) / 100 ) ) / 4
                     + id ) - 2432076L );
}
