#include "cpk.h"

void cpkS2c ( double a, double b, double *x, double *y, double *z )

/*
**  - - - - - - -
**   c p k S 2 c
**  - - - - - - -
**
**  Spherical coordinates to Cartesian coordinates.
**
**  Given:
**     a       d     longitude angle (e.g. RA) in radians
**     b       d     latitude angle (e.g. Dec) in radians
**
**  Returned:
**     x       d*    x coordinate
**     y       d*    y coordinate
**     z       d*    z coordinate
**
**  Notes:
**
**    1  No validation of the arguments is performed.
**
**    2  The Cartesian coordinates returned are for a unit vector.
**
**    3  Right-handed coordinate systems are assumed:  call using
**       (RA,Dec), (-HA,Dec) or (pi-Az,El).  For the (HA,Dec) case, the
**       alternative is to call using (HA,Dec) and then flip y.  For the
**       (Az,El) case, the alternative is to call using (Az,El) and then
**       flip x.
**
**  Last revision:   9 January 2007
**
**  Copyright P.T.Wallace.  All rights reserved.
*/

{
   double c;


   c = cos(b);
   *x = cos(a) * c;
   *y = sin(a) * c;
   *z = sin(b);

}
