#include "cpk.h"

void cpkUpdt ( double ut1, TRANSP *pars )

/*
**  - - - - - - - -
**   c p k U p d t
**  - - - - - - - -
**
**  Update the rapidly-changing Earth rotation portion of the
**  transformation parameters.
**
**  Given:
**     ut1      d        UT1 as Modified Julian Date (JD-2400000.5)
**     pars   TRANSP*    transformation parameters
**
**  Returned:
**     pars   TRANSP*    updated
**
**  Notes:
**
**    1  No validation of the arguments is performed.
**
**    2  It is the responsibility of the application to distinguish
**       between UT1 and UTC.
**
**  Last revision:   9 January 2007
**
**  Copyright P.T.Wallace.  All rights reserved.
*/

{
   double t, eral;



/* Days since J2000. */
   t = ut1 - 51544.5;

/* Local Earth rotation angle. */
   eral = pars->elon + D360 * fmod ( 0.7790572732640
                                   + 1.00273781191135448 * t, 1.0 );
   pars->st = sin(eral);
   pars->ct = cos(eral);

}
