/* EPIX camerlink card test program */

#include <stdio.h>              // c library
#include <signal.h>             // c library
#include <stdlib.h>             // c library
#include <stdarg.h>             // c library
#include <unistd.h>             // c library
#include <xcliball.h>           // function prototypes

#define DRIVERPARMS ""
#define FORMATFILE    "wfs.fmt"         // using format file saved by XCAP

/*  This example expects only one unit.  */

#if !defined(UNITS)
    #define UNITS       1
#endif
#define UNITSMAP    ((1<<UNITS)-1)  // shorthand - bitmap of all units

static int fieldirqcount = 0;

/* This gets called for each video frame */

void videoirqfunc(int sig)
{
    fieldirqcount++;
}

int main(int argc, char **argv)
{
	int	size, x, y, npix;
	int	i = 0, j, value, err;
	ushort  *monoimage_buffer;
	ushort  *p;

	/* Open the device */

	if ((i = pxd_PIXCIopen(DRIVERPARMS, "", FORMATFILE)) == 0)
	{
		/* Setup a callback routine */

		signal(SIGUSR1, videoirqfunc);
		pxd_eventCapturedFieldCreate(1, SIGUSR1, NULL);
		printf("Open OK\n");
	}
	else
	{
		fprintf(stderr, "Open Error %d\a\a\n", i);

		/* Display a message concerning the fault. */

		pxd_mesgFault(UNITSMAP);

		/* Could use pxd_mesgFaultText(UNITSMAP, s, 256) */
	}

	/* How many buffers etc do we have? */

	printf("Image frame buffer memory size: %.3f Kbytes\n",
		(double)pxd_infoMemsize(UNITSMAP)/1024);
	printf("Image frame buffers           : %d\n", pxd_imageZdim());
	printf("Number of boards              : %d\n", pxd_infoUnits());
	printf("Image resolution:\n");
	printf("xdim           = %d\n", x = pxd_imageXdim());
	printf("ydim           = %d\n", y = pxd_imageYdim());
	printf("colors         = %d\n", pxd_imageCdim());
	printf("bits per pixel = %d\n", pxd_imageCdim()*pxd_imageBdim());
	npix = x*y;

	/*
	 * How many video fields have come by?
	 * These are counted regardless of whether or not the
	 * video field is captured.
	 */

	printf("Field IRQ count before = %d\n", fieldirqcount);
	printf("Field count before wait= %d\n", pxd_videoFieldCount(1));
	sleep(1);
	printf("Field count after wait = %d\n", pxd_videoFieldCount(1));
	sleep(1);
	printf("Field count after 1Sec = %d\n", pxd_videoFieldCount(1));

	/* Initiate capturing a single image into the buffer. */

	printf("Field IRQ before Snap  = %d\n", fieldirqcount);
	err = pxd_doSnap(UNITSMAP, 1, 0);
	printf("pxd_doSnap: %s\n", err>=0? "Ok": pxerrnomesg(err));
	printf("Field count after  snap= %d\n", pxd_videoFieldCount(1));
	printf("Field IRQ count   after= %d\n", fieldirqcount);

	/*
 	 * Check for faults, such as erratic sync
	 * or insufficient PCI bus bandwidth
    	 */

	pxd_mesgFault(UNITSMAP);

	/* Is this a color camera? */

	if (pxd_imageCdim() !=1)
	{
		fprintf(stderr,"Seems to be a color camera!!!\n");
	}

	/* What is the bit depth? */

	printf("Image bit depth: %d\n", pxd_imageBdim());

	/* Allocate memory for a frame */

	monoimage_buffer = malloc(size = sizeof(ushort) * npix);

	if (monoimage_buffer == NULL)
	{
		fprintf(stderr,"Out of memory.\n");
		exit(-2);
	}

	/* We should read a frame */

	if ((i = pxd_readushort(UNITSMAP, /* Which unit? */
		1,			 /* Which frame buffer? */
		0, 0, 			 /* Upper left corner */
		-1, -1, 		 /* Lower right corner or whole*/
		monoimage_buffer, 	 /* Where to put it */
		size,			 /* How big is the buffer */
		"Grey")) != npix)
	{
		if (i < 0)
		{
			fprintf(stderr, "pxd_readushort: %s\n",
				pxd_mesgErrorCode(i));
		}
		else
		{
			fprintf(stderr,"pxd_readushort error: %d != %d\n",
				i, size);
		}
	}
	else
	{
		printf("Image buffer read.\n");

		/* Save this as a TIFF file (Simmilar callto above ) */

		err = pxd_saveTiff(UNITSMAP, "image.tiff", 
			1, 0, 0, -1, -1, 0, 0);
		if (err < 0)
		{
			fprintf(stderr, "pxd_saveTiff: %s\n", 
				pxd_mesgErrorCode(err));
		}
		printf("Image buffer saved\n");

/*
		for (p = monoimage_buffer, j=0; j<y; j++)
		for(i=0; i<x ;i++)
		{
			value = (int)(*p++);
			printf("%d %d %d\n",i,j,value);
		}
*/
	}

	free(monoimage_buffer);

	/* Close connection */

	pxd_PIXCIclose();
	printf("PIXCI(R) imaging board closed.\n");
	
	exit(0);
}
