/************************************************************************/
/* espclient.h								*/
/*                                                                      */
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar (but mostly taken from espgtk)          */
/* Date   : May 2007                                                    */
/************************************************************************/

#ifndef __ESP_CLIENT__
#define __ESP_CLIENT__

#include <stdio.h>
#include <errno.h>

/* Macros */

#define ESP_ERROR_BIT  0x40

#define OFF     0
#define ON      1

#define ESP_MAX_STAGE 6

/* Config file names */

#define CONFIG_BASEFILE   "/ctrscrut/chara/etc/"
#define LDC1_CONFIG_FILE  CONFIG_BASEFILE"ldc1.cfg"
#define LDC2_CONFIG_FILE  CONFIG_BASEFILE"ldc2.cfg"
#define BS_CONFIG_FILE    CONFIG_BASEFILE"bs.cfg"
#define PERI1_CONFIG_FILE CONFIG_BASEFILE"peri1.cfg"
#define PERI2_CONFIG_FILE CONFIG_BASEFILE"peri2.cfg"
#define RETRO_CONFIG_FILE CONFIG_BASEFILE"retro.cfg"
#define BC1_CONFIG_FILE CONFIG_BASEFILE"bc1.cfg"
#define BC2_CONFIG_FILE CONFIG_BASEFILE"bc2.cfg"
#define BS_POSITION_FILE  CONFIG_BASEFILE"bs_positions.cfg"
#define BC1_POSITION_FILE  CONFIG_BASEFILE"visbeam_positions.cfg"


/* Globals */

extern int esp_num_axes[MAX_SERVERS];
extern char *esp_port_names[MAX_SERVERS][ESP_MAX_STAGE];
extern float esp_port_min[MAX_SERVERS][ESP_MAX_STAGE];
extern float esp_port_max[MAX_SERVERS][ESP_MAX_STAGE];
extern char *esp_unit_name[MAX_SERVERS];
extern unsigned char esp_status[MAX_SERVERS];
extern struct AXIS esp_stage[MAX_SERVERS][ESP_MAX_STAGE];
extern char esp_power_status_bit[MAX_SERVERS];
extern char esp_motion_status_bit[MAX_SERVERS];
extern char esp_update_power_status[MAX_SERVERS];
extern char esp_update_motion_status[MAX_SERVERS];
extern char esp_update_position[MAX_SERVERS][ESP_MAX_STAGE];
extern char esp_update_velocity[MAX_SERVERS][ESP_MAX_STAGE];
extern char esp_update_acceleration[MAX_SERVERS][ESP_MAX_STAGE];
extern char esp_update_minmax[MAX_SERVERS][ESP_MAX_STAGE];

/* esp_messages.c */

int open_esp_client(char *server_name);
void close_esp_client(int esp_server);
int message_esp_server_stopping(int server, struct smessage *mess);
int message_esp_status(int server, struct smessage *mess);
int message_esp_names(int server, struct smessage *mess);
int message_esp_minmax(int server, struct smessage *mess);
int message_esp_power(int server, struct smessage *mess);
int message_esp_motion_done(int server, struct smessage *mess);
int message_esp_position(int server, struct smessage *mess);
int message_esp_velocity(int server, struct smessage *mess);
int message_esp_acceleration(int server, struct smessage *mess);
int send_esp_on(int esp_server, int axis);
int send_esp_off(int esp_server, int axis);
int send_esp_move_absolute(int esp_server, int axis, double position);
int send_esp_set_beam(int esp_server, int axis, int beam);
int send_esp_set_velocity(int esp_server, int axis, double velocity);
int send_esp_set_acceleration(int esp_server, int axis, double acceleration);
int send_esp_move_relative(int esp_server, int axis, double relative_step_size);
int send_esp_move_home(int esp_server, int axis);
int send_esp_text(int esp_server, char *entry_text);
int esp_get_position(int esp_server, int axis);
int esp_wait_position(int esp_server, int axis);
int esp_get_motion_status(int esp_server);
int esp_wait_motion_status(int esp_server);
int esp_get_power_status(int esp_server);
int esp_wait_power_status(int esp_server);
int esp_get_minmax(int esp_server, int axis);
int esp_wait_minmax(int esp_server, int axis);
int esp_get_velocity(int esp_server, int axis);
int esp_wait_velocity(int esp_server, int axis);
int esp_get_acceleration(int esp_server, int axis);
int esp_wait_acceleration(int esp_server, int axis);
int esp_update(int esp_server);

#endif
