/*#define DIF*/
/************************************************************************/
/* testfft.c VERSION 1.0						*/
/*									*/
/* Takes a set of numbers from the input file is given or else stdout	*/
/* Picks the closest power of 2 it can to it and does an fft and	*/
/* sends it to output file if given ir stdout.				*/
/* Forces things to integers first.  					*/
/* All data as ascii.							*/
/* usage: fft dt <input> <output>					*/
/************************************************************************/
/*                                        	                      	*/
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : May 1998							*/
/************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "fft.h"

#define	MAX_DATA	4096

int main(int argc, char **argv)

{
	FILE 	*input,*output;	/* Data streams */
	short int real_data[MAX_DATA];
	short int image_data[MAX_DATA];
	short int real_fft[MAX_DATA];
	short int image_fft[MAX_DATA];
	int	pow_2;
	int	real_n;
	int	n;
	register int i;
	char	s[81];
	double	dt,f;
#ifdef DIF
	short   divide_stages; 
#endif

	if (argc < 2)
	{
		fprintf(stderr,"usage: fft dt <input> <output>\n");
		exit(-01);
	}
	sscanf(argv[1],"%lg",&dt);

	/* Check out the command line */

	if (argc > 2)
	{
		if ((input = fopen(argv[2],"r")) == NULL)
		{
			fprintf(stderr,"Could not open file %s for reading.\n",
					argv[2]);
			exit(-1);
		}
	}
	else
	{
		input = stdin;
	}

	if (argc > 3)
	{
		if ((output = fopen(argv[3],"r")) != NULL)
		{
			fprintf(stderr,"File %s already exists.\n", argv[3]);
			fclose(output);
			exit(-1);
		}

		if ((output = fopen(argv[3],"w")) == NULL)
		{
			fprintf(stderr,"Could not open file %s for writing.\n",
					argv[3]);
			exit(-1);
		}
	}
	else
	{
		output = stdout;
	}

	/* Read in the data */

	n = 0;
	while(fgets(s,81,input) != NULL)
	{
		sscanf(s,"%hd %hd",real_data+n,image_data+n);

		if (++n >= MAX_DATA)
		{
			fprintf(stderr,
				"Data truncated to maximum of %d points\n",
				MAX_DATA);
			break;
		}
	}
	fclose(input);
	
	/* Find the power of 2 closest but <= to n */

	pow_2 = log((double)n) / log(2.0);
	if (pow(2.0,(double)pow_2+1.0) == n)
	{
		pow_2 += 1.0;
	}
	real_n = pow(2.0,(double)pow_2);
	if (real_n != n)
	{
		fprintf(stderr,"Data truncated to 2^%d = %d\n",pow_2,real_n);
	}
	fprintf(output,"#pow_2 = %d\n",pow_2);
	fprintf(output,"#n =  = %d\n",n);
	fprintf(output,"#real_n =  = %d\n",real_n);

	/* Do the transform */

	setup_fft(pow_2);
#ifdef DIF
	divide_stages = 0;
	cfft_dif(pow_2, real_data, image_data, &divide_stages);
	printf("#divide_stages =  = %d\n",divide_stages);
	bitrev_int16(real_n,real_data,real_fft);
	bitrev_int16(real_n,image_data,image_fft);
	cifft_dif(pow_2, real_data, image_data, &divide_stages);
	printf("#divide_stages =  = %d\n",divide_stages);
#else
	bitrev_int16(real_n,real_data,real_fft);
	bitrev_int16(real_n,image_data,image_fft);
	cfft_dit(pow_2, real_fft, image_fft);
#endif

	/* Print out the results */
	
	fprintf(output,"#         FFT                 DATA          F        Power\n");
	fprintf(output,"#      R         I         R         I      Hz\n");

	for(i=0; i < real_n; i += 1)
	{
		f = (double)(i)/((double)real_n * dt);
		fprintf(output,"%8d  %8d  %8d  %8d  %8.1f  %10.0f\n",
				real_fft[i],image_fft[i],
				real_data[i],image_data[i],
				f,(float)real_fft[i]*(float)real_fft[i]+
				 (float)image_fft[i]*(float)image_fft[i]);
	}
	fclose(output);

	exit(0);

} /* main() */
