/************************************************************************/
/* filter.h VERSION 1.0							*/
/*									*/
/* Header file for Digital filter.					*/
/************************************************************************/
/*                                        	                      	*/
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.						*/
/*                                                              	*/
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : December 1994						*/
/************************************************************************/

#ifndef _FILTER_H_ /* Let's make sure of only one pass through */
#define _FILTER_H_

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

#ifdef DISPLAY
#include <nsimpleX.h>
#endif

typedef struct  DIGITAL_FILTER {
		float	dt;	  /* Time step for each sample */
		int	n_params; /* Number of paramters in filter */
		float	*params;  /* 1..n_params, the actual parameters */
		float (*process)(float *p,float i);/*Function that does work. */

	} digital_filter;

/*
 * Function declarations
 */


/* filter.c */

void	clean_up_filter(digital_filter filter);

/* simple_lag.c */

float simple_lag(digital_filter *filter, float dt, float tau);
float process_simple_lag(float *params, float input);
fcomplex simple_lag_response(float freq, float tau);

/* pid.c */

float pid(digital_filter *filter, float dt, float p, float i, float d);
float process_pid(float *params, float input);
fcomplex pid_response(float freq, float p, float i, float d);

/* bandpass.c */

float bandpass(digital_filter *filter, float dt, float a, float b);
float process_bandpass(float *params, float input);
fcomplex bandpass_response(float freq, float f0, float e, float dt);

/* quadratic_lag.c */

float quadratic_lag(digital_filter *filter, float dt, float res, float damp);
float process_quadratic_lag(float *params, float input);
fcomplex quadratic_lag_response(float freq, float resonance, float damp);

/* impulse_response.c */

fcomplex *impulse_response(digital_filter filter, int *length, 
		float **freqs, char display);

/*
 * Macro functions
 */

#define process_filter(filter, input) \
	((*(filter).process)((filter).params,(input)))
#define mag_db(x) (x.i+x.r==0.0?0.0:20*log10(sqrt(x.r*x.r+x.i*x.i)))
#define phase(x)  (x.i==0.0?0.0:atan2(x.i,x.r)/M_PI*180.0)

#endif /* Ensures only one pass through header file */
