/************************************************************************/
/* chebft.c VERSION 1.0							*/
/*									*/
/* Fit Chebyshev polyinomials to strehl to visibility function.  	*/
/* 									*/
/* usage: chebft n							*/
/*									*/
/* Where n is the degree of polynomials to use.				*/
/************************************************************************/
/*                                        	                      	*/
/* Author : Theo ten Brummelaar                                         */
/* Date   : March 1995 							*/
/************************************************************************/

#include "../zernike.h"

main(int argc, char **argv)
{
	float	*c;
	int	i;
	int	n;
	float	logstrehl,strehl;

	/* Check command line */

	if (argc < 2)
	{	
		fprintf(stderr,"usage: chebft n\n");
		exit(-1);
	}
	sscanf(argv[1],"%d",&n);

	/* Do some setup work */

	set_up_zernike_calcs(21);

	c = vector(0,n-1);

	/* Print out the header stuff */

	printf("#include <zernike.h>\n\n");
	printf("/* Globals */\n\n");
	printf("#define CHEB_N %d\n",n);
	printf("static float *c_tilt;\n");
	printf("static float *c_no_tilt;\n\n");
	printf("int set_up_cheb_strehl_to_vis(void)\n");
	printf("{\n");
	printf("\t/* Allocate memory */\n\n");
	printf("\tc_tilt = vector(0,CHEB_N-1);\n");
	printf("\tc_no_tilt = vector(0,CHEB_N-1);\n\n");
	printf("\t/* Fill in the numbers... */\n\n");

	/* Do the fit for tilt case */

	chebft(0.0, 1.0, c, n, zbrent_strehl_to_vis_tilt);

	for(i=0;i<n;i++) printf("\tc_tilt[%i] = %e;\n",i,c[i]);
	printf("\n\n");

	/* Do the fit for non-tilt case */

	chebft(0.0, 1.0, c, n, zbrent_strehl_to_vis_no_tilt);

	for(i=0;i<n;i++) printf("\tc_no_tilt[%i] = %e;\n",i,c[i]);
	printf("\n\n");

	/* That is all */

	printf("\treturn CHEB_N;\n\n");
	printf("} /* set_up_cheb_strehl_to_vis() */\n\n");

	/* Other usefull functions */

	printf("void clean_up_cheb_strehl_to_vis(void)\n");
	printf("{\n");
	printf("\t/* De-allocate memory */\n\n");
	printf("\tfree_vector(c_tilt,0,CHEB_N-1);\n");
	printf("\tfree_vector(c_no_tilt,0,CHEB_N-1);\n\n");
	printf("} /* clean_up_cheb_strehl_to_vis() */\n\n");

	printf("float cheb_strehl_to_vis_tilt(float strehl, int m)\n");
	printf("{\n");
	printf("\tif (strehl > 0.8) return strehl;\n\n");
	printf("\tif (m > CHEB_N) m = CHEB_N;\n\n");
	printf("\treturn chebev(0.0,1.0,c_tilt, m, strehl);\n\n");
	printf("} /* cheb_strehl_to_vis_tilt() */\n\n");
	
	printf("float cheb_strehl_to_vis_no_tilt(float strehl, int m)\n");
	printf("{\n");
	printf("\tif (strehl > 0.8) return strehl;\n\n");
	printf("\tif (m > CHEB_N) m = CHEB_N;\n\n");
	printf("\treturn chebev(0.0,1.0,c_no_tilt, m, strehl);\n\n");
	printf("} /* cheb_strehl_to_vis_no_tilt() */\n\n");
	
	/* Clean up */

	free_vector(c,0,n-1);
	clean_up_zernike_calcs();

	exit(0);
}
