/************************************************************************/
/* integrate_fileter.c 							*/
/*									*/
/* Reads in filter data file, multiplies atmosphere and filter response */
/* And then integrates the thing to try and get the mean wvaelength and */
/* width of an equivalent square pass filter.				*/
/************************************************************************/
/*                                        	                      	*/
/* Center for High Angular Resolution Astronomy				*/
/*									*/
/* INTERFEROMETRY CODE LIBRARY.						*/
/*									*/
/* Telephone: 404-651-2932						*/
/* Fax      : 404-651-1384						*/
/* Internet : theo@@chara.gsu.edu					*/
/*									*/
/* (C) This source code and its associated executable           	*/
/* program(s) are copyright.                                    	*/
/*                                                              	*/
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : January 2001    						*/
/************************************************************************/

#include <math.h>
#include <nrc.h>
#include <chara.h>

#define MAX_DATA	100

int main(int argc, char **argv)
{
	FILE	*input;
	char	s[81];
	float	wl[MAX_DATA];
	float	filt[MAX_DATA];
	float	atm[MAX_DATA];
	float	tot[MAX_DATA];
	float	dwl;
	float	filt_int;
	float	tot_int;
	float	mean_wl;
	float	n_wl;
	float	max_tot;
	float	equiv_width;
	int	i,n;
	/* Check command line */

	if (argc != 2)
	{
		fprintf(stderr,"usage: %s filename\n",argv[0]);
		exit(-1);
	}

	/* Open input file */

	if ((input = fopen(argv[1],"r")) == NULL)
	{
		fprintf(stderr,"Could not open file %s\n",argv[1]);
		exit(2);
	}

	/* Read in the data */

        n = 0;
	dwl = 0.0;
	mean_wl = 0.0;
	n_wl = 0.0;
	max_tot = 0.0;
        while(fgets(s,256,input) != NULL)
        {
                if (*s == '#') continue;

                sscanf(s,"%f %f %f",wl+n,filt+n,atm+n);

		tot[n] = filt[n] * atm[n];

		if (n > 0) dwl += (wl[n] - wl[n-1]);

		mean_wl += (wl[n] * tot[n]);
		n_wl += tot[n];
	
		if (tot[n] > max_tot) max_tot = tot[n];

                if (++n>= MAX_DATA)
                {
                      fprintf(stderr,"Warning: Data truncated at %d points.\n",
                                n);
                      break;
                }

        }
	fclose(input);

	/* What is the wave length step size ? */

	dwl /= (n-1);

	/* What is the weighted mean wavelength ? */

	mean_wl /= n_wl;

	/* OK, integrate the filter response */

	filt_int = simpson(n, filt, dwl);

	/* OK, integrate the total response */

	tot_int = simpson(n, tot, dwl);

	/* And the equivalent width */

	equiv_width = tot_int/max_tot;

	/* Spit out the results */

	printf("# Data file = %s\n",argv[1]);
	printf("# Maximum response = %5.3f\n",max_tot);
	printf("# mean wl = %5.3f\n",mean_wl);
	printf("# dwl = %5.3f\n",dwl);
	printf("# int(filt) = %5.3f\n",filt_int);
	printf("# Equiv width = %5.3f\n",filt_int/max_tot);
	printf("# int(tot) = %5.3f\n",tot_int);
	printf("# Equiv width = %5.3f\n",equiv_width);
	printf("# um    Filt    Atm    Total    Equiv\n");
	for(i=0; i<n; i++) 
	{
		printf("%4.2f\t%4.2f\t%4.2f\t%4.2f\t",
			         wl[i],filt[i],atm[i],tot[i]);
		if (wl[i] < (mean_wl-equiv_width/2.0) ||
		    wl[i] > (mean_wl+equiv_width/2.0))
			printf("%5.3f\n",0.0);
		else
			printf("%5.3f\n",1.0);
	}

	exit(0);

}
