/************************************************************************/
/* matrox_meteor.h                                                      */
/*                                                                      */
/* Header file for matrox meteor routines.				*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : April 1999                                                  */
/************************************************************************/

#ifndef _MATROX_METEOR_H_ /* Ensure we haven't been here before */
#define _MATROX_METEOR_H_

/************************************************************************/
/* The stuff for both MODULES and User Interfaces                       */
/************************************************************************/

/************************************************************************/
/* The stuff for only MODULES                                           */
/************************************************************************/

#ifdef MODULE

/* Here is where the data on the frame grabber is kept */

extern void *meteor_frame_buffer_location;

/* 
 * These routines are here so that the meteor can call a realtime function 
 * when frame is ready.
 */

int set_meteor_frame_handler(void (*new_frame_handler)(void *buffer_location));
void unset_meteor_frame_handler(void);

#else

/************************************************************************/
/* This stuff only for user interfaces                                  */
/************************************************************************/

/* Globals */

extern int errno;
extern int meteor_fd;
extern int last_brightness;
extern int last_contrast;

/* Defines */

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

#define DEFAULT_DEVICE	"/dev/meteor0"

#define DEFAULT_COLS 256
#define DEFAULT_ROWS 192

#define MAX_COLS 640
#define MAX_ROWS 480

#define MIN_BRIGHTNESS		0
#define MAX_BRIGHTNESS		255
#define DEFAULT_BRIGHTNESS	160 
#define MIN_CONTRAST		0
#define MAX_CONTRAST		127
#define DEFAULT_CONTRAST	100
#define FPS			30	/* Frames per second */
#define NOFRAMES		2	/* capacity of frane buffer */

/* matrox_general.c */

int set_meteor_chan(int chan);
int set_meteor_contrast(int contrast);
int set_meteor_brightness(int brightness);

/* matrox_signal.c */

int open_meteor_signal(char *devname, 
		       int col, int row, 
		       void (*handler)(unsigned char *, int cols, int rows));
void close_meteor_signal(void);
void signal_handler(int sig);
int get_signal_number_of_frames(void);

/* matrox_continuous.c */

int open_meteor_continuous(char *devname, int col, int row);
void close_meteor_continuous(void);
unsigned char *meteor_buffer_location(void);
int meteor_continuous_get_frame(unsigned char *buffer, int *xpos, int *ypos);

#endif
#endif
