/************************************************************************/
/* mkman.c 								*/
/* 4 system								*/
/*									*/
/* Assuming that the source code for a program has been written exactly */
/* as defined in the SUSI coding practices standard (Document 		*/
/* SUSI/COM/030-1/TEX), this program will				*/
/* (to as large a degree as possible) write a manual entry.		*/
/* Input is from the command line file name given. Output is to a number*/
/* of file names generated internally.					*/
/* Output is in the form of a series of files ready for input to 	*/
/* nroff -man.								*/
/*									*/
/* If the MAKE_HELP is defined it will build a help file instead.	*/
/************************************************************************/
/*                                        	                      	*/
/* Documentation Utilities                                             	*/
/*                                                              	*/
/* Butchered from: 							*/
/*                                                              	*/
/* Sydney University Stellar Interferometer.                    	*/
/*                                                              	*/
/* Chatterton Astronomy Department, School of Physics,          	*/
/* University of Sydney, N.S.W. 2006, Australia.                	*/
/*                                                              	*/
/* Telephone: +61 2 692 2544                                    	*/
/* Telex: AA 26169 UNISYD                                       	*/
/* Facsimile: +61 2 660 2903                                    	*/
/* ACSnet: root@astron.physics.su.oz                            	*/
/*                                                              	*/
/* (C) This source code and its associated executable           	*/
/* program(s) are copyright.                                    	*/
/*                                                              	*/
/************************************************************************/
/*									*/
/* Author : Theo ten Brummelaar						*/
/* Date   : 6 November 1989						*/
/*									*/
/* Changes:								*/
/*		6/25/96 Added code to test for man directory and make 	*/
/*		it if it isn't there.					*/
/*									*/
/*		2/09/04 Modified the date output function in the nroff	*/
/*		man page to be more ``Y2K'' compliant.  Also cleaned up	*/
/*		the code so that ``-pedantic'' doesn't find anything to	*/
/*		complain about.						*/
/************************************************************************/

/*
 * Include files
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/timeb.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>

/*
 * Defines
 */

#define	DIVIDER		"/**********"
#define LEN_DIVIDER 	11
#define MAX_INCLUDES	25
#define MAX_DESCRIPTION	50
#define	TRUE	1
#define FALSE	0
#ifdef MAKE_HELP
#define DATADIR	"help"
#define DATAEXT "hlp"
#else
#define DATADIR	"man"
#define DATAEXT "man"
#endif

char	includes[MAX_INCLUDES][81];		/* Include information */
char	description[MAX_DESCRIPTION][81];  	/* Description information */

/*
 * Prototypes
*/

void mkman(int argc,char **argv);
void clean_line(char *outstr, char *instr);
char 	*get_next_line(char *string, int n, FILE *stream);

int main(int argc, char **argv)
{

	mkman(argc,argv);
	exit(0);

} /* main() */

/************************************************************************/
/* mkman()								*/
/*									*/
/* Assuming that the source code for a program has been written exactly */
/* as defined in the SUSI coding practices standard (Document 		*/
/* SUSI/COM/030-1/TEX), this program will				*/
/* (to as large a degree as possible) write a manual entry. 		*/
/* Input is from the command line file name given. Output is to a number*/
/* of file names generated internally.					*/
/* Output is in the form of a series of files ready for input to 	*/
/* nroff -man.								*/
/************************************************************************/

void mkman(int argc,char **argv)
{
	/* Internal Variables */

	char	line_buffer[81];		/* line buffer */
	char	filename[81];			/* filename buffer */
	char	author[81];			/* author name buffer */
	char	function[81];			/* func. name buffer */
#ifdef MAKE_HELP
	char	title[81];			/* Function name in capitals */
	char	*p1,*p2;			/* For string play */
	int	ini_file_exists;		/* Does it exist? */
#endif
	char	*end;				/* tidy up strings */
	int	number_includes;		/*...include files */
	int	number_description;		/* ho hum */
	int	i;			/* Looping variable */
	FILE	*input;				/* File pointer for input */
	FILE	*output;			/* File pointer for output */
#ifdef MAKE_HELP
	FILE	*index;				/* File pointer to index */
#endif
	int	loop;				/* Flag for main loop */
	time_t  clock;				/* Current clock setting */
#ifndef MAKE_HELP
	char	date_buffer[20];		/* Human readable buffer */
#endif

	/* initialise */

	*line_buffer = '\0';
	*filename = '\0';
	*author = '\0';
	*function = '\0';
	clock = time((time_t *)0);

	/* Check out that an argument has been placed in command line */

	if (argc != 3)
	{
#ifdef MAKE_HELP
		fprintf(stderr,"Usage: mkhelp index_name filename.c\n");
#else
		fprintf(stderr,"Usage: mkman volume_name filename.c\n");
#endif
		exit(-1);
	}


	/* See if there is a directory */

	if (chdir(DATADIR) != 0)
	{
#ifdef MAKE_HELP
		fprintf(stderr,"mkhelp: Creating directory %s.\n",DATADIR);
#else
		fprintf(stderr,"mkman: Creating directory %s.\n",DATADIR);
#endif
		
		if (mkdir(DATADIR, (mode_t) 0777) != 0)
		{
			fprintf(stderr,
#ifdef MAKE_HELP
			    "mkhelp: Could not create directory %s.\n",DATADIR);
#else
			    "mkman: Could not create directory %s.\n",DATADIR);
#endif
			exit(-2);
		}
	}
	else
	{
		chdir("..");
	}

	/* Open the input file */

	if ((input = fopen(argv[2],"r")) == NULL)
	{
#ifdef MAKE_HELP
		fprintf(stderr,"mkhelp: Can not open input file %s\n",argv[2]);
#else
		fprintf(stderr,"mkman: Can not open input file %s\n",argv[2]);
#endif
		exit(-2);
	}

	/* Skip the first three dividers */

	for (i=0; i<3; i++)
	{
		if (get_next_line(line_buffer,81,input) == (char *) EOF) 
			exit(0);
		while(strncmp(DIVIDER,line_buffer,LEN_DIVIDER) != 0)
			if (get_next_line(line_buffer,81,input) == (char *) EOF)
				exit(0);
	}

	/* Get the Author's name (assumes it is second line from now) */

	if (get_next_line(line_buffer,81,input) == (char *) EOF) exit(0);
	if (get_next_line(line_buffer,81,input) == (char *) EOF) exit(0);
	clean_line(function,line_buffer); 	/* Clean it up */
	end = function;
	while(*end != ':') end++;
	end ++;				/* Find start of name */
	strcpy(author,end);		/* Copy into correct space */
	
	/* Skip another divider */

	if (get_next_line(line_buffer,81,input) == (char *) EOF) exit(0);
	while(strncmp(DIVIDER,line_buffer,LEN_DIVIDER) != 0)
		if (get_next_line(line_buffer,81,input) == (char *) EOF) 
			exit(0);

	/* get the include statements */

	number_includes = 0;

	if (get_next_line(line_buffer,81,input) == (char *) EOF) exit(0);

#ifdef MAKE_HELP
	/* Build filename and open index file */

	sprintf(filename,"%s/%s_help.ini",DATADIR,argv[1]);

	/* Does this file exist already ? */

	if ((ini_file_exists = ((index = fopen(filename,"r")) != NULL)))
	{
		/* Yes it does exist! */

		fclose(index);
	}

	if ((index = fopen(filename,"a")) == NULL)
	{
	    fprintf(stderr,"mkhelp: Can not open file %s\n",filename);
	    exit(-2);
	}

	if (!ini_file_exists)
	{
		fprintf(index,
			"# FILE AUTOMATICALLY CREATED BY MKHELP FROM FILE %s\n",
			argv[2]);
		fprintf(index, "# THIS WILL NEED TO BE EDITED.\n");
	}
#endif

	while(strncmp(DIVIDER,line_buffer,LEN_DIVIDER) != 0)
	{
		if(strncmp("#include",line_buffer,8) == 0 
			&& number_includes < MAX_INCLUDES)
		{
			strcpy(&includes[number_includes++][0],line_buffer);
		}	

		if (get_next_line(line_buffer,81,input) == (char *) EOF) 
			exit(0);
	}

	/* 
	 * We should now be pointing directly at the first function name.
	 * We therefore start looping around these function names
	 */

	loop = TRUE;
	while(loop)
	{
		/* Next line should be the function name */

		if (get_next_line(line_buffer,81,input) == (char *)EOF) break;
			
		clean_line(function,line_buffer);

		/* search for DIVIDER while loading in description */

		number_description = 0;

		if (get_next_line(line_buffer,81,input) == (char *)EOF) break;

		while(strncmp(DIVIDER,line_buffer,LEN_DIVIDER) != 0)
		{
			if(number_description < MAX_DESCRIPTION)
			{
			     clean_line(&description[number_description++][0],
					line_buffer);
			}

			if (get_next_line(line_buffer,81,input) == (char *)EOF) 
				break;
		}
	
		/* Create the file for output of stuff */

		end = function;
		while(*end != '(' && *end != 0) end++;
		*end = 0;
		sprintf(filename,"%s/%s.%s",DATADIR,function,DATAEXT);
#ifdef VERBOSE
#ifdef MAKE_HELP
		printf("Making help file %s\n",filename);
#else
		printf("Making manual entry %s\n",filename);
#endif
#endif

		if ((output = fopen(filename,"w")) == NULL)
		{
#ifdef MAKE_HELP
		    fprintf(stderr,"mkhelp: Can not open file %s\n",filename);
#else
		    fprintf(stderr,"mkman: Can not open file %s\n",filename);
#endif
		    exit(-2);
		}

#ifdef MAKE_HELP
		/* Add entry for index file */

		fprintf(index,"%s\t%s\n",filename,function);
#endif

		/* Start entry */

#ifndef MAKE_HELP
		/* Make the date something a human can read */

		strftime(date_buffer,19,"%Y-%b-%d",localtime(&clock));

		fprintf
		(
			output,
			".TH %s S %s CHARA %s CHARA\n",
			function,
			date_buffer,
			argv[1]
		);

		fprintf(output,".SH NAME\n%s\n",function);
		fprintf(output,".SH SYNOPSIS\n.nf\n");

		/* Print include files */

		for(i=0;i<number_includes;i++)
			fprintf(output,"%s\n",&includes[i][0]);

#else
		/* Turn the function name into all capitals */

		for (p1=function,p2=title; *p1 != 0; p1++, p2++)
		{
			if (*p1 >= 'a' && *p1 <= 'z')
				*p2 = *p1 - 'a' + 'A';
			else
				*p2 = *p1;
		}
		*p2 = 0;

		fprintf(output,".center\n%s\n.center\n",title);

		/* Now underline it */

		for(p1 = title; *p1 != 0; p1++) putc('=',output);
		fprintf(output,"\n.paragraph\n");
		fprintf(output,"SYNOPSIS:\n.paragraph\n%s\n.paragraph\n",
			function);
#endif

		/* Look for left curly bracket and print out synopsis */

		if (get_next_line(line_buffer,81,input) == (char *)EOF) 
			loop = FALSE;

		while(*line_buffer != '{')
		{
#ifndef MAKE_HELP
			fprintf(output,"%s\n",line_buffer);
#endif
			if (get_next_line(line_buffer,81,input) == (char *)EOF) 
			{
				loop = FALSE;
				break;
			}
		}

		/* Skip to the next divider - start of next function */

		if (get_next_line(line_buffer,81,input) == (char *)EOF) 
			loop = FALSE;
		while(strncmp(DIVIDER,line_buffer,LEN_DIVIDER) != 0)
			if (get_next_line(line_buffer,81,input) == (char *)EOF) 
			{
				loop = FALSE;
				break;
			}

		/* Print description */

#ifdef MAKE_HELP
		fprintf(output,"DESCRIPTION:\n.paragraph\n");
#else
		fprintf(output,".fi\n.SH DESCRIPTION\n");
#endif

		for(i=0; i < number_description; i++)
			fprintf(output,"%s\n",&description[i][0]);

		/* Spit out file and author */

#ifdef MAKE_HELP
		fprintf(output,".paragraph\nAUTHOR\n.paragragh\n");
		fprintf(output,"%s\n",author);
#else
		fprintf(output,".SH SOURCE FILE\n%s\n",argv[2]);
		fprintf(output,".SH AUTHOR\n%s\n",author);
#endif

		/* Close output file */

		fclose(output);
		
	} /* End main loop */

#ifdef MAKE_HELP
	fclose(index);
#endif

} /* mkman() */

/************************************************************************/
/* clean_line()								*/
/*									*/
/* Places second string into first string without a leading comment	*/
/* designator (a "/" followed by a "*") or trailing comment ender (you 	*/
/* know ... a * followed by a / ).					*/
/* Will also get rid of trailing white space.				*/
/************************************************************************/

void clean_line(char *outstr, char *instr)
{
	char		*end;	/* Points to end of string */
	char		*start;	/* Where to start coppying string */

	/* initialise */

	/* First look for first character that is not '/' or '*' or ' ' or tab*/

	for (start = instr; 
		*start == '/' || 
		*start == '*' || 
		*start == ' ' || 
		*start == '\t'; start++ );

	/* Find end of string */

	for (end = instr; *end != 0 ; end++);

	--end;

	/* Find the first character that is not a '/' a '*' a tab or space */

	while (
		(*end == '/' || *end == '*'|| *end == ' '|| *end == '\t' ) && 
		end > start
	      ) end--;
	
	/* Place new NULL at new end of string */

	*(++end) = 0;

	/* Copy string accross */

	strcpy(outstr,start);

	return;

} /* clean_line() */

/************************************************************************/
/* get_next_line()							*/
/* 									*/
/* Reads a line from a given stream as text. 				*/
/* The new line character at the end of the				*/
/* string (if present) is replaced by a NULL. Returns a pointer 	*/
/* to the string or EOF if the end of the file has been 		*/
/* reached. Will read up to n-1 characters and then terminate.		*/
/************************************************************************/

char 	*get_next_line(char *string, int n, FILE *stream)
{
	/* Internal variables */

	int	c;		/* Character */
	char	*retval;	/* Return value of function */

	/* Set return value to string begining */

	retval = string;

	c = 'A';

	/* Read in the string */

	while ( --n > 0 && (c = getc(stream)) != EOF && c != '\n')
		if (c != '\r') *string++ = c;

	/* Strip any tabs or spaces from the end of the string */

	while ( *--string == '\t' || *string == ' ') 
		;

	/* Put terminating NULL on end of string */

	*++string = 0;

	/* Return  pointer to the string */

	if (c == EOF)
		return (char *) EOF;
	else
		return retval;

} /* get_next_line() */
