/************************************************************************/
/* bunker_clock.h							*/
/*                                                                      */
/* Header file for RT Bunker Clock.					*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : Jan 2007                                                    */
/************************************************************************/

#ifndef __BUNKER_CLOCK__
#define __BUNKER_CLOCK__

/************************************************************************/
/* The stuff for both MODULES and User Interfaces			*/
/************************************************************************/

typedef struct	BUNKER_TIME_STRUCT 
	{
		int month;
		int day;
		int year;
		int hour;
		int min;
		int sec;
		int msec;
		int chara_now;
		int num_sat;
		int num_tracked;
		unsigned char pico_t_status;
		unsigned char card_status;
		unsigned char second_tick;
		unsigned char control_reg;
		int module_installed;

	} bunker_time_struct;	/* Structure used to send times */

/* Bits used in pico_t_status */

#define BUNKER_POSITION_PROPAGATE_MODE	(1<<0x7)
#define BUNKER_POOR_GEOMETRY		(1<<0x6)
#define BUNKER_3D_FIX			(1<<0x5)
#define BUNKER_2D_FIX			(1<<0x4)
#define BUNKER_ACQUIRING_SATELLITES	(1<<0x3)
#define BUNKER_DIFFERENTIAL_FIX		(1<<0x2)
#define BUNKER_INSUFFICIENT_SATELLITES	(1<<0x1)
#define BUNKER_BAD_ALMANAC		(1<<0x0)

/* Bits used in command register */

#define BUNKER_CLOCK_RESET		(1<<0x0)
#define BUNKER_CLOCK_RUN		(1<<0x1)
#define BUNKER_CLOCK_TX_PIC		(1<<0x2)
#define BUNKER_CLOCK_RX_PIC		(1<<0x3)

/* Bits used in card_status */

#define BUNKER_PIC_TIME			(1<<0x1)
#define BUNKER_GPS_TIME			(1<<0x0)

/* The commands */

#define BUNKER_CLOCK_SET_REG		(FIRST_BUNKER_CLOCK_COMMAND+0)

/************************************************************************/
/* The stuff for only MODULES						*/
/************************************************************************/

#ifdef MODULE

/* Include Files */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/version.h>
#include <asm/irq.h>
#include <asm/rt_irq.h>
#include <asm/io.h>
#include <linux/errno.h>
#include <rtl_fifo.h>
#include <linux/mm.h>
#include <linux/malloc.h>
#include <asm/dma.h>

/* Macros */

#define BUNKER_CLOCK_IRQ	  11
#define BUNKER_CLOCK_BASE_ADDRESS 0x300
#define BUNKER_CLOCK_FIFO_ADDRESS (BUNKER_CLOCK_BASE_ADDRESS+1)

/* Prototypes */

int init_module(void);
unsigned int bunker_clock_intr_handler(unsigned int irq, struct pt_regs *regs);
int bunker_clock_set_fast_handler(void (*handler)(void));
void bunker_clock_unset_fast_handler(void);
int     bunker_clock_is_second_tick(void);
int bunker_clock_set_reg(unsigned char arg);
void cleanup_module(void);

#else

/************************************************************************/
/* This stuff only for user interfaces					*/
/************************************************************************/

/* Include files */

#include <sys/types.h>
#include <unistd.h>

/* Global for the sharred memory */

extern struct BUNKER_TIME_STRUCT *bunker_clock;

/* Prototypes */

bool bunker_clock_open(void);
void bunker_clock_close(void);
int bunker_clock_reset(void);
int bunker_clock_stop(void);
int bunker_clock_start(void);
int bunker_clock_set(unsigned char arg);
int call_bunker_clock_reset(int argc, char **argv);
int call_bunker_clock_stop(int argc, char **argv);
int call_bunker_clock_start(int argc, char **argv);
int call_bunker_clock_set(int argc, char **argv);
int show_bunker_clockstruct(int argc, char **argv);
int sync_machine(int argc, char **argv);
int bunker_clock_start_message(struct smessage *mess);
int bunker_clock_stop_message(struct smessage *mess);
int bunker_clock_reset_gps_message(struct smessage *mess);
int bunker_clock_msync_message(struct smessage *mess);
void setup_standard_bunker_clock_messages(void);
void bunker_clock_date(int *year, int *month, int *day);

#endif
#endif
