/************************************************************************/
/* clock.h								*/
/*                                                                      */
/* Header file for RT CHARA Clock.					*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : May 1998                                                    */
/************************************************************************/

#ifndef __CLOCK__
#define __CLOCK__

/************************************************************************/
/* The stuff for both MODULES and User Interfaces			*/
/************************************************************************/

/* Here's where we define the ranges of commands for each module */

#define	FIRST_SYSTEM_COMMAND		  0
#define LAST_SYSTEM_COMMAND		 49

#define	FIRST_ARCCCD_COMMAND		 50
#define LAST_ARCCCD_COMMAND		 99

#define	FIRST_DALSA_COMMAND		100
#define LAST_DALSA_COMMAND		149

#define	FIRST_DITHER_COMMAND		150
#define LAST_DITHER_COMMAND		199

#define	FIRST_RTCCD_COMMAND		200
#define LAST_RTCCD_COMMAND		249

#define	FIRST_NIRO_COMMAND		250
#define LAST_NIRO_COMMAND		299

#define	FIRST_VISDITHER_COMMAND		300
#define LAST_VISDITHER_COMMAND		349

#define	FIRST_FAST_CLOCK_COMMAND	350
#define LAST_FAST_CLOCK_COMMAND		399

#define	FIRST_GPS_COMMAND		400
#define LAST_GPS_COMMAND		449

#define	FIRST_IRIMAGE_COMMAND		450
#define LAST_IRIMAGE_COMMAND		499

#define	FIRST_TELESCOPE_COMMAND		500
#define LAST_TELESCOPE_COMMAND		549

#define	FIRST_TELETRAC_COMMAND		550
#define LAST_TELETRAC_COMMAND		599

#define	FIRST_WOBBLE_COMMAND		600
#define LAST_WOBBLE_COMMAND		649

#define	FIRST_IRTRACK_COMMAND		650
#define LAST_IRTRACK_COMMAND		699

#define FIRST_TIPTILT_COMMAND           700
#define LAST_TIPTILT_COMMAND            749
                                                                                
#define FIRST_CLASSIC_COMMAND           750
#define LAST_CLASSIC_COMMAND            799
                                                                                
#define FIRST_BUNKER_CLOCK_COMMAND      800
#define LAST_BUNKER_CLOCK_COMMAND       849

#define MAX_COMMAND			1024

/* Macros */

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

#ifndef YES
#define YES 1
#define NO  0
#endif

#define CLOCK_CARD_IRQ  11      /* Clock card interrupt */
#define CLOCK_CARD_ADD  0x300   /* Address for clock card */
#define LPT_BASE      0x378	/* Parallel Port Base Address. */
#define LPT_STATUS (LPT_BASE+1)	/* Status register */
#define LPT_CONTROL (LPT_BASE+2)/* Control register */
#define LPT_ADDRESS (LPT_BASE+3)/* EPP address port */
#define LPT_DATA    (LPT_BASE+4)/* Data port r*/

#define LPT_STATUS_IRQ 0x04	/* IRQ Status */
#define LPT_STATUS_ERR 0x08	/* !(ERROR) */
#define LPT_STATUS_SL  0x10	/* Select  */
#define LPT_STATUS_PE  0x20	/* Paper End */
#define LPT_STATUS_ACK 0x40	/* !(Acknowledge) */
#define LPT_STATUS_BSY 0x80	/* !(Busy) */

#define LPT_CONTROL_ST 0x01	/* Strobe Line */
#define LPT_CONTROL_AF 0x02	/* Auto Line Feed line */
#define LPT_CONTROL_IN 0x04	/* !(Initialize Printer) */
#define LPT_CONTROL_SI 0x08	/* Select In line */
#define LPT_CONTROL_IQ 0x10	/* Enable IRQ */
#define LPT_CONTROL_D  0x20	/* Port Direction */

#define TICKS_PER_SECOND	1000	/* How fast is the clock? */
#define TICKS_PER_MINUTE	(TICKS_PER_SECOND*60)
#define TICKS_PER_HOUR		(TICKS_PER_MINUTE*60)
#define TICKS_PER_DAY		(TICKS_PER_HOUR*24)

#define CHARA_TIME	int	/* Variable used for all times */
#define CHARA_TIME_END  0x7FFFFFFF

#define DEFAULT_BYPASS_CLOCK_DELAY      350

/* Command to read LPT status reg */

#define REPORT_LPT_REGISTER	(FIRST_SYSTEM_COMMAND+0)
#define CONTROL_REG		0 	/* Argument for above function */
#define STATUS_REG		1 	/* Argument for above function */

/* Command write to LPT command reg */

#define WRITE_LPT_CONTROL	(FIRST_SYSTEM_COMMAND+1)

/* Command to report current time */

#define REPORT_TIME		(FIRST_SYSTEM_COMMAND+2)

/* Command to report current time */

#define SET_TIME		(FIRST_SYSTEM_COMMAND+3)

/* Report the time of the next second */

#define GET_TIME_NEXT_SECOND    (FIRST_SYSTEM_COMMAND+4)

/* Command to flush data fifos */

#define FLUSH_DATA		(FIRST_SYSTEM_COMMAND+5)

/* Command to clear lost tick count */

#define CLEAR_LOST_COUNTS	(FIRST_SYSTEM_COMMAND+6)

/* Command to call sync handler */

#define SYNC_HANDLER            (FIRST_SYSTEM_COMMAND+7)

/* Command that plays with bits */

#define TTL_CONTROL		(FIRST_SYSTEM_COMMAND+8) 
#define IN_1			0	/* Argument for above function */
#define IN_2			1	/* Argument for above function */
#define IN_3			3	/* Argument for above function */
#define OUT_1			4	/* Argument for above function */
#define OUT_2			5	/* Argument for above function */
#define SET_OUT_1		6	/* Argument for above function */
#define CLEAR_OUT_1		7	/* Argument for above function */
#define SET_OUT_2		8	/* Argument for above function */
#define CLEAR_OUT_2		9	/* Argument for above function */

/* Command: set value in printer port */

#define SET_LPT                 (FIRST_SYSTEM_COMMAND+9)

/* Command: get value in printer port */

#define GET_LPT                 (FIRST_SYSTEM_COMMAND+10)

/* Command: set bit in printer port */

#define SET_BIT_LPT             (FIRST_SYSTEM_COMMAND+11)

/* Command: unset bit in printer port */

#define UNSET_BIT_LPT           (FIRST_SYSTEM_COMMAND+12)

/* Command to call initialize clock */

#define INITIALIZE_CLOCK        (FIRST_SYSTEM_COMMAND+13)

/* Command to list all command numbers */

#define LIST_COMMANDS		(FIRST_SYSTEM_COMMAND+14)

/* Command: get value in printer data port */

#define GET_LPT_DATA            (FIRST_SYSTEM_COMMAND+15)

#define MAX_CHARA_COMMAND       GET_LPT_DATA


/*
 * Global Variables
 */

typedef struct	RT_COMMAND 
	{
		unsigned int comm;
		unsigned char data;

	} rt_command_struct;	/* Structure used to send commands */

typedef struct	RT_TIME_STRUCT 
	{
		CHARA_TIME time_now;
		CHARA_TIME lost_ticks;
		CHARA_TIME lost_seconds;

	} rt_time_struct;	/* Structure used to send times */

/************************************************************************/
/* The stuff for only MODULES						*/
/************************************************************************/

#ifdef MODULE

/* Include Files */

#ifdef KERNEL_2_4_4_RTL
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/version.h>
#include <asm/irq.h>
#include <asm/rt_irq.h>
#include <asm/io.h>
#include <linux/errno.h>
#include <rtl_fifo.h>
#include <linux/mm.h>
#include <linux/malloc.h>
#include <asm/dma.h>
#include <rtl_core.h>
#endif

#ifdef KERNEL_2_2_18_RTL
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/version.h>
#include <asm/irq.h>
#include <asm/rt_irq.h>
#include <asm/io.h>
#include <linux/errno.h>
#include <rtl_fifo.h>
#include <linux/mm.h>
#include <linux/malloc.h>
#include <asm/dma.h>
#endif

/* Macros */

#define FIFO_SIZE	310000		/* Size in bytes of FIFO channels */
#define COMM_FIFO	0		/* FIFO used to input commands/data */
#define DATA_OUT_FIFO	1		/* FIFO used to send out data */
#define DATA_IN_FIFO	2		/* FIFO used to suck in data */
#define ERR_FIFO	3		/* FIFO used to report errors */

#define LPT_IRQ		7	/* IRQ level of LPT */

/* Macro Functions */

#define chara_in_1_set()	(inb(LPT_STATUS) & LPT_STATUS_ERR)
#define chara_in_2_set()	(!(inb(LPT_STATUS) & LPT_STATUS_BSY))
#define chara_in_3_set()	(inb(LPT_STATUS) & LPT_STATUS_PE)
#define chara_out_1_set()	(chara_current_lpt_control & LPT_CONTROL_IN)
#define chara_out_2_set()	(!(chara_current_lpt_control & LPT_CONTROL_AF))

#ifdef BOGUS
/* There's something very ODD about this */
#define set_out_1()	outb((chara_current_lpt_control|= LPT_CONTROL_IN),LPT_CONTROL)
#define clear_out_1()	outb((chara_current_lpt_control&= (~LPT_CONTROL_IN)),LPT_CONTROL)
#define set_out_2()	outb((chara_current_lpt_control&=(~LPT_CONTROL_AF)),LPT_CONTROL)
#define clear_out_2()	outb((chara_current_lpt_control|=LPT_CONTROL_AF),LPT_CONTROL)
#endif

/* 
 * The following is straight out of charaui.h except that it
 * no longers users pointers to structures, but the structures themselves.
 */
                                                                                
typedef struct sbuffer{
                        unsigned char   *bottom;
                        unsigned char   *top;
                        unsigned char   *in;
                        unsigned char   *out;
                                                                                
                } BUFFER;
                                                                                
/*
 * These macros let you play with buffers
 */
                                                                                
#define buffer_not_full(x)      (x.in != x.out)
#define buffer_full(x)          (x.in == xout)
#define buffer_not_empty(x)     ((x.out != x.in - 1) && \
                                ((x.out != x.top) || (x.in != x.bottom)))
#define buffer_empty(x)         ((x.out == x.in - 1) || \
                                ((x.out == x.top) && (x.in == x.bottom)))
                                                                                
#define buffer_in(x,y)          {if (buffer_not_full(x)) \
                                         {\
                                                *(x.in) = (y);\
                                                if (++(x.in) > x.top) \
                                                        x.in = x.bottom;\
                                         }\
                                }
                                                                                
#define buffer_out(x,y)         {if (buffer_not_empty(x)) \
                                         {\
                                                if (++(x.out) > x.top) \
                                                        x.out = x.bottom;\
                                                (y) = *(x.out);\
                                         }\
                                }

#define setup_buffer(bufstruct, buffer, size) {\
					bufstruct.bottom = buffer;\
					bufstruct.top = buffer + size -1;\
					bufstruct.in = bufstruct.bottom;\
					bufstruct.out = bufstruct.top;\
				}

/* Functions */

int chara_open_rt_fifo(int fifo_num, int size);
void chara_called_clock(int chara_now);
int chara_add_command(unsigned int command_number, int (*handler)(unsigned char arguement));
int chara_remove_command(unsigned int command_number);
void chara_remove_command_range(unsigned int start, unsigned int stop);
int chara_printerr(char *fmt, ...);
int chara_set_fast_handler(void (*handler)(void));
void chara_unset_fast_handler(void);
CHARA_TIME chara_time(void);
void chara_set_out_1(void);
void chara_clear_out_1(void);
void chara_set_out_2(void);
void chara_clear_out_2(void);
int chara_set_called_handler(void (*handler)(void));
void chara_set_timer(CHARA_TIME when);
void chara_unset_called_handler(void);
int chara_set_sync_handler(void (*handler)(void));
void chara_unset_sync_handler(void);
int chara_call_sync_handler(unsigned char arg);
int chara_open_rt_fifo(int fifo_num, int size);
void chara_set_lpt(unsigned char value);
void chara_set_time_next_second(CHARA_TIME new_time);
unsigned char chara_get_lpt(void);
unsigned char chara_get_lpt_data(void);
void chara_set_bit_lpt(unsigned char bit);
void chara_unset_bit_lpt(unsigned char bit);
int chara_call_initialize_clock(unsigned char data);
int chara_initialize_clock(void);
int chara_list_commands(unsigned char data);

#else

/************************************************************************/
/* This stuff only for user interfaces					*/
/************************************************************************/

/* Include files */

#include <sys/types.h>
#include <unistd.h>

/* Macros */

#define COMM_FIFO_DEV	"/dev/rtf0" /* Command channel device */
#define DATA_OUT_FIFO_DEV	"/dev/rtf1" /* Data out channel device */
#define DATA_IN_FIFO_DEV	"/dev/rtf2" /* Data in channel device */
#define ERR_FIFO_DEV	"/dev/rtf3" /* Error channel device */

#define TTL_IN		0
#define TTL_OUT		1

#ifndef bool
#define bool char
#endif

/* Gobals */

extern int	fd_comm;		/* Command channel file handle */
extern int	fd_data_out;		/* Data out channel file handle */
extern int	fd_data_in;		/* Data in channel file handle */
extern int	fd_err;			/* Error channel file handle */
extern CHARA_TIME *chara_time_now;	/* Shared memory clock time */
extern CHARA_TIME *chara_time_next_second;	/* Shared memory clock time */

/* Structures */

typedef struct	AP_TIME_STRUCT 
	{
		char		sign; /* -1 or +1 */
		CHARA_TIME	hrs;
		CHARA_TIME	min;
		CHARA_TIME	sec;
		CHARA_TIME	tick;

	} ap_time_struct;	/* Structure used to send times */

/* chara_clock.c */

int set_bypass_clock_delay(int delay);
int set_bypass_clock_driver(bool bypass);
char open_rt_fifos(void);
void close_rt_fifos(void);
char send_rt_command(unsigned int command, unsigned char argument);
int get_rt_error_message(char *s);
int read_rt_data(void *buf, size_t count);
int write_rt_data(void *buf, size_t count);
int flush_rt_data_in(void);
int flush_rt_data_out(void);
int flush_data(int argc, char **argv);
int clear_lost_counts(int argc, char **argv);
int in_set(int in_or_out, int port);
int out_set(int argc, char **argv);
int send_command(int argc, char **argv);
int error_status(void);
unsigned char get_lpt_register(int reg);
int get_rt_time(rt_time_struct *the_time);
int set_rt_time(CHARA_TIME time_next_second);
CHARA_TIME get_time_next_second(int wait);
struct AP_TIME_STRUCT rt_to_ap_time(int rt_time);
CHARA_TIME ap_to_rt_time(struct AP_TIME_STRUCT t);
struct AP_TIME_STRUCT str_to_ap_time(char *s);
char *ap_to_str_time(char *s, struct AP_TIME_STRUCT t);
int set_time(int argc, char **argv);
int set_control_reg(int argc, char **argv);
int sync_times(int argc, char **argv);
int show_time(int argc, char **argv);
int call_sync_handler(int argc, char **argv);
int set_lpt(int argc, char **argv);
int get_lpt(void);
int call_get_lpt(int argc, char **argv);
int get_lpt_data(void);
int call_get_lpt_data(int argc, char **argv);
int set_bit_lpt(int argc, char **argv);
int unset_bit_lpt(int argc, char **argv);
int initialize_clock(int argc, char **argv);
int list_commands(int argc, char **argv);
double sidereal_time(void);
double modified_julian_date(void);
bool clock_running(void);
bool alloc_chara_time_now(void);
void free_chara_time_now(void);
int check_clock_offset(void);
void set_max_clock_offset(int max_offset);
void set_clock_offset_callback(void (*offset_callback)(void));

/* clock_messages.c */

void setup_standard_clock_messages(void);
int clock_flush_fifos(struct smessage *message);
int clock_clear_lost_counts(struct smessage *message);
int clock_in_set(struct smessage *message);
int clock_rt_time(struct smessage *message);
int clock_time_next_second(struct smessage *message);
int clock_set_rt_time(struct smessage *message);
int clock_sync_times(struct smessage *message);
int clock_call_sync_handler(struct smessage *message);
int clock_set_lpt(struct smessage *message);
int clock_get_lpt(struct smessage *message);
int clock_set_lpt_bit(struct smessage *message);
int clock_unset_lpt_bit(struct smessage *message);
int clock_reinit_clock(struct smessage *message);
int clock_get_lpt_data(struct smessage *message);

#endif
#endif
