/************************************************************************/
/* fast_clock.h								*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : Dec 2004                                                   */      
/************************************************************************/

#ifndef __FAST_CLOCK__
#define __FAST_CLOCK__

/************************************************************************/
/* The stuff for both MODULES and User Interfaces 			*/
/************************************************************************/

/* 
 * The commands 
 */

#define GET_TIME		(FIRST_FAST_CLOCK_COMMAND+0)
#define CLEAR_LOST_TICKS	(FIRST_FAST_CLOCK_COMMAND+1)
#define CALIBRATE		(FIRST_FAST_CLOCK_COMMAND+2)
#define CALIBRATE_GET_STAGE	(FIRST_FAST_CLOCK_COMMAND+3)
#define GET_PERIOD		(FIRST_FAST_CLOCK_COMMAND+4)
#define SET_PERIOD		(FIRST_FAST_CLOCK_COMMAND+5)
#define GET_RATE		(FIRST_FAST_CLOCK_COMMAND+6)
#define SET_RATE		(FIRST_FAST_CLOCK_COMMAND+7)

#define FAST_CLOCK_MAX_COMMAND SET_RATE

/* At what rate should we run (Hz) ? */

#define DEFAULT_FAST_CLOCK_RATE		5000L

/* Define the various stages of the calibration process */

#define CAL_STAGE_NONE  0
#define CAL_STAGE_START 1
#define CAL_STAGE_CALC  2
#define CAL_STAGE_DONE  3

#ifndef FAST_CLOCK_TIME_STRUCT_DEFINED
typedef struct FAST_CLOCK_TIME_STRUCT
	{
		CHARA_TIME time_now;
		CHARA_TIME lost_ticks;

	} fast_clock_time_struct;
#define FAST_CLOCK_TIME_STRUCT_DEFINED
#endif

/************************************************************************/
/* The stuff for only MODULES                                           */
/************************************************************************/

#ifdef MODULE

/* Macros */

/* prototypes */

int init_module(void);
void fast_clock_clock(int arg);
CHARA_TIME chara_fast_clock_time(void);
CHARA_TIME chara_fast_clock_rate(void);
int fast_clock_report_time(unsigned char argument);
int fast_clock_clear_lost_counts(unsigned char argument);
int fast_clock_set_fast_handler(void (*handler)(void));
void fast_clock_unset_fast_handler(void);
int fast_clock_set_called_handler(void (*handler)(void));
void fast_clock_set_timer(CHARA_TIME when);
void fast_clock_unset_called_handler(void);
int fast_clock_calibrate(unsigned char argument);
int fast_clock_calibrate_get_stage(unsigned char argument);
int fast_clock_get_period(unsigned char argument);
int fast_clock_set_period(unsigned char argument);
int fast_clock_get_rate(unsigned char argument);
int fast_clock_set_rate(unsigned char argument);
void cleanup_module(void);

#else

/************************************************************************/
/* This stuff only for user interfaces                                  */
/************************************************************************/

#include <stdio.h>
#include <math.h>
#include <time.h>
#ifndef __APPLE__
#include <malloc.h>
#endif

/* Globals */

CHARA_TIME FAST_CLOCK_RATE;
CHARA_TIME FAST_CLOCK_TICKS_PER_CHARA_TICK;
CHARA_TIME FAST_CLOCK_TICKS_PER_SECOND;
CHARA_TIME FAST_CLOCK_TICKS_PER_MINUTE;
CHARA_TIME FAST_CLOCK_TICKS_PER_HOUR;
CHARA_TIME FAST_CLOCK_TICKS_PER_DAY;

/* Macros */

/* Prototypes */

/* fast_clock_comms.c */

int get_fast_clock_time(fast_clock_time_struct *now);
int fast_clock_calibrate(int seconds);
int fast_clock_calibrate_get_stage(void);
int fast_clock_get_period(void);
int fast_clock_set_period(int period);
int fast_clock_get_rate(void);
int fast_clock_set_rate(int period);

/* fast_clock_control.c */

int fast_clock_clear_lost_counts(int argc, char **argv);
int call_fast_clock_calibrate(int argc, char **argv);
int call_fast_clock_set_period(int argc, char **argv);
int call_fast_clock_set_rate(int argc, char **argv);
int fast_clock_sync(int argc, char **argv);

/* fast_clock_messages.c */

void setup_standard_fast_clock_messages(void);
int message_fast_clock_clear_lost_counts(struct smessage *message);
int message_fast_clock_calibrate(struct smessage *message);
int message_fast_clock_set_period(struct smessage *message);
int message_fast_clock_set_rate(struct smessage *message);
int message_fast_clock_get_rate(struct smessage *message);
int message_fast_clock_time(struct smessage *message);
int message_fast_clock_sync(struct smessage *message);

#endif
#endif
