/************************************************************************/
/* calculate_fwhm.c 							*/
/*                                                                      */
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : June 1998                                                   */
/************************************************************************/

#include "../nrc/nrc.h"
#include "nsimpleX.h"

/************************************************************************/
/* calculate_fwhm()							*/
/*									*/
/* Finds the full-width, half maximum (FWHM) of an input image matrix.	*/
/************************************************************************/

float calculate_fwhm(float **psf, int xpix, int ypix, float scale)
{
	int xm,ym,x1,x2,y1,y2;
	float fwhm,max;

	/* First find the maximum intensity */

	max = max_image_intensity(psf,xpix,ypix,&xm,&ym)/2.0;

	/* Now search for the half intensity position in four directions */

	for(x1 = xm; x1 <= xpix; x1++) if (psf[x1][ym] < max) break;
	for(x2 = xm; x2 >= 1; x2--) if (psf[x2][ym] < max) break;
	for(y1 = ym; y1 <= ypix; y1++) if (psf[xm][y1] < max) break;
	for(y2 = ym; y2 >= 1; y2--) if (psf[xm][y2] < max) break;

	fwhm = (x1 - x2 + y1 - y2)/(2.0*scale);

	return fwhm;

} /* calculate_fwhm() */
