/************************************************************************/
/* calculate_fwhp.c 							*/
/*                                                                      */
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : June 1998                                                   */
/************************************************************************/

#include "../nrc/nrc.h"
#include "nsimpleX.h"

/************************************************************************/
/* calculate_fwhp()							*/
/*									*/
/* Finds the full width of half power of an input image matrix.		*/
/************************************************************************/

float calculate_fwhp(float **psf, int xpix, int ypix, float scale)
{
	float	total2,total;
	int	xpos, ypos;
	float	max;
	int	r2,d;
	int	npix,i,j,k;

	/* First we find the position of the maximum intensity in the image */

	max = max_image_intensity(psf, xpix, ypix, &xpos, &ypos);
	total2 = total_image(psf, xpix, ypix)/2.0;

	/* Work out how many pixels we can move from here to edge */

	npix = xpos-1;
	if (xpix - xpos < npix) npix = xpix - xpos;
	if (ypos-1 < npix) npix = ypos;
	if (ypix - ypos < npix) npix = ypos;

	/* Now do the caluclation */

	for(k = 0; k <= npix; k++)
	{
		total = 0;
		r2 = k*k;

		for(j=ypos-k; j<=ypos+k;j++)
		{
			for(i=xpos-k; i<=xpos+k;i++)
			{
				/* Is this on the circle result pixels away? */

				d = (xpos-i)*(xpos-i)+(ypos-j)*(ypos-j);
				if (d <= r2)
				{
					total += psf[i][j];
				}
			}
		}

		/* Is this the half power point ? */

		if (total >= total2) break;
	}

	return 2.0*k/scale;

} /* caluclate_fwhp() */
