/************************************************************************/
/* display_picture.c 							*/
/*                                                                      */
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : June 1998                                                   */
/************************************************************************/

#include "../nrc/nrc.h"
#include "nsimpleX.h"

/* A local global */

static XImage  *theImage = NULL;

/************************************************************************/
/* display_picture()							*/
/*									*/
/* Display a (square) picture in the previously created "theWindow".	*/
/* This is basically a wrapper for the "make_picture()" subroutine that	*/
/* that figures out the X display color depth automatically.		*/
/************************************************************************/

void display_picture(Window theWindow, 
		     float **values, 
		     int npix, 
		     int pixmult, 
		     char disp_type)
{
        char *picture;

	if (theImage != NULL)
	{
		XDestroyImage(theImage);
	}

	picture = make_picture(npix,npix,pixmult,values,disp_type);

        theImage = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
			picture,npix*pixmult,npix*pixmult, theBitmapPad,0);

	XPutImage(theDisplay,theWindow,theGC,theImage,0,0,0,0,
		npix*pixmult,npix*pixmult);

        XMapWindow(theDisplay, theWindow);

        XFlush(theDisplay);
	return;

} /* display_picture() */
