/************************************************************************/
/* get_mouse.c								*/
/*									*/
/* Gets the location of the mouse within a window.			*/
/* Part of the simpleX library.						*/
/* This version works for 8, 16 or 32 bit depths.                       */
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : May 1997	   						*/
/************************************************************************/

#include <stdio.h>
#include "../nrc/nrc.h"
#include "nsimpleX.h"

//#define DEBUG

/************************************************************************/
/* get_mouse()								*/
/*									*/
/* Waits for a left mouse button click WITHIN "theWindow", and figures	*/
/* out the "x" and "y" values, taking "pixmult" into account.		*/
/* It reverses the Y coordinate so that it gives the bottom left as	*/
/* the origin.								*/
/* Returns TRUE if all is OK and FALSE otherwise.			*/
/************************************************************************/

Bool get_mouse(
	Window	theWindow,
	int	pixmult,
	int	*x,
	int	*y)
{
	Window	root_return, child_return;
	int	root_x, root_y;
	int	win_x, win_y;
	Bool	return_value;
	unsigned int mask_return;
	XWindowAttributes attribs;
	int	x_size, y_size;

	/* First get the attributes of that window */

	XGetWindowAttributes(theDisplay, theWindow, &attribs);

	x_size = attribs.width;
	y_size = attribs.height;

#ifdef DEBUG
	printf("x_size = %d\ny_size = %d\n",x_size,y_size);
#endif
	/* Wait for a click ON */

	do
	{
		return_value =XQueryPointer(theDisplay, theWindow, &root_return,
		&child_return, &root_x, &root_y, &win_x, &win_y, &mask_return);
#ifdef DEBUG
		printf("mask_return = %d win_x = %d win_y = %d\r", 
			mask_return, win_x, win_y);
#endif
	} while (!(mask_return & 1<<8) 
		|| win_x<0 || win_x>=x_size || win_y<0 || win_y>=y_size);

	/* Wait for a click OFF */

	do
	{
		return_value =XQueryPointer(theDisplay, theWindow, &root_return,
		&child_return, &root_x, &root_y, &win_x, &win_y, &mask_return);
#ifdef DEBUG
		printf("mask_return = %d win_x = %d win_y = %d\r", 
			mask_return, win_x, win_y);
#endif
	} while ((mask_return & 1<<8) 
		|| win_x<0 || win_x>=x_size || win_y<0 || win_y>=y_size);

#ifdef DEBUG
	printf("mask_return = %d win_x = %d win_y = %d\n", 
			mask_return, win_x, win_y);
#endif
	*x = win_x/pixmult + 1;
	*y = (y_size - win_y)/pixmult+1;

	return return_value;

} /* get_mouse() */
