/************************************************************************/
/* initX.c 								*/
/*									*/
/* Sets up X windows in a simple way ....				*/
/* Part of the simpleX library.						*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : August 1994   						*/
/************************************************************************/

#include <stdio.h>
#include "../nrc/nrc.h"
#include "nsimpleX.h"

/*
 * Globals
 */

Display	*theDisplay;
int	theScreen;
int	theDepth = 8;
int	theDepthBytes = 1;
int      theBitmapPad = 8;
int      theBitmapBytes = 1;
int	theWidth = 1024;
int	theHeight = 768;
Window   theRootWindow;
Visual	*theVisual;
GC	theGC;
unsigned int	fcol,bcol;
Colormap	theCmap;
unsigned int    	GreyScale[NUM_COLORS];
int display_is_open = FALSE;

/************************************************************************/
/* initX()								*/
/*									*/
/* Initializes the current program for use of "nsimpleX".  It opens up	*/
/* "display_name" as the name of the display to open and sets up a	*/
/* simple grey-scale color map.  If the "display_name" is NULL, the	*/
/* default display is opened. Returns number of colours allocated	*/
/* meaning that <=0 is an error.					*/ 
/* Sometimes this fails. Run "gdmsetup" as root and turn on TCP 	*/
/* connections, then restart X.						*/
/************************************************************************/

unsigned char initX(char *display_name)
{
   	int i,color_step;
        XColor  color;
	int	num_sucess;

	if (display_is_open) return 0;

	/* Open the display */

	if ((theDisplay = XOpenDisplay( display_name )) == NULL) return 0;

	/* Check out what the display is like */

	theScreen     = DefaultScreen(theDisplay);
	theCmap       = DefaultColormap(theDisplay, theScreen);
	theRootWindow = RootWindow(theDisplay, theScreen);
	theGC         = DefaultGC(theDisplay,theScreen);
	theDepth      = DefaultDepth(theDisplay, theScreen);
	theWidth      = DisplayWidth(theDisplay, theScreen);
	theHeight     = DisplayHeight(theDisplay, theScreen);
	theVisual     = DefaultVisual(theDisplay,theScreen);
	fcol          = WhitePixel(theDisplay,theScreen);
	bcol          = BlackPixel(theDisplay,theScreen);

	/* Check that the display depth is OK */

	if (theDepth % 8 != 0)
	{
		fprintf(stderr,"Display depth must be multiple of 8.\n");
		return -1;
	}
	if ((theDepthBytes = theDepth/8) < 1 || theDepthBytes > 4)
	{
		fprintf(stderr,
		"Display depth out of range of 8,16,24 or 32.\n");
		return -2;
	}

	if ((theBitmapPad = theDepth) == 24) theBitmapPad = 32;
	theBitmapBytes = theBitmapPad/8;

	XSetForeground(theDisplay,theGC,fcol);
	XSetBackground(theDisplay,theGC,bcol);

	/* Set up GreyScale color map */

	color_step = 65536 / (NUM_COLORS-1);
	num_sucess = 0;

	for(i=0;i<NUM_COLORS-1;i++)
	{
	        color.red = i*color_step;
                color.green = i*color_step;
                color.blue = i*color_step;
                color.flags = DoRed | DoGreen | DoBlue;
                if (XAllocColor(theDisplay,theCmap,&color)) num_sucess++;

		GreyScale[i] = color.pixel;
	}

	for(i=0; i<NUM_OTHER_COLORS; i++)
	{
		if (set_plot_color(other_colors_rgb[i][0],
				   other_colors_rgb[i][1],
				   other_colors_rgb[i][2])) num_sucess++;
		other_colors[i] = plot_color;
	}
	plot_color = GreyScale[NUM_COLORS-1] = other_colors[COLOR_WHITE];

	display_is_open = TRUE;

	return num_sucess;

} /* initX() */
