/************************************************************************/
/* zero_edge.c	 							*/
/*                                                                      */
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : June 1998                                                   */
/************************************************************************/

#include "../nrc/nrc.h"
#include "nsimpleX.h"

/************************************************************************/
/* zero_edge()								*/
/*									*/
/* Calculates the mean pixel value around the edge of "image" and then	*/
/* subtracts that value from every pixel in "image".			*/
/************************************************************************/

float zero_edge(float **image, int nx, int ny)
{
	float	edge_mean;
	int	count;
	int	i,j;
	
	/* Initialise */

	edge_mean = 0.0;
	count = 0;

	/* Go along the edges in order */

	for(i=1; i<= nx ; i++)
	{
		edge_mean += image[i][1];
		count++;
	}
	for(j=2; j<= ny; j++)
	{
		edge_mean += image[nx][j];
		count++;
	}
	for(i=nx-1; i>= 1; i--)
	{
		edge_mean += image[i][ny];
		count++;
	}
	for(j=ny; j> 1; j--)
	{
		edge_mean += image[1][j];
		count++;
	}

	/* Find mean */

	edge_mean /= (float) count;

	/* KLUDGE KLUDGE KLUDGE */

/*
	for(i=nx-10;i<=nx;i++)
	for(j=ny-10;j<=ny;j++)
	{
		image[i][j] = edge_mean;
	}
*/

	/* Subtract it from the images */

	for (i=1; i<= nx; i++)
	for (j=1; j<= ny; j++)
	{
		image[i][j] -= edge_mean;
	}

	return edge_mean;

} /* zero_edge() */
