/************************************************************************/
/* opleft.c								*/
/*                                                                      */
/* Example use of the ople fringe tracking library.			*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar 		                        */
/* Date   : March 2010							*/
/************************************************************************/

/*
 * Include files
 */

#include "opleft.h"
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

int main(int argc, char **argv)
{
	double	targets[NUM_DELAY_LINES];
	double	set_targets[NUM_DELAY_LINES];
	int	i;

	/* Check the command line */

	if (argc != NUM_DELAY_LINES+1)
	{
		fprintf(stderr,"Usage: %s S1 S2 E1 E2 W1 W2\n", argv[0]);
		exit(-1);
	}

	/* Get the new targets */

	for (i=0; i<NUM_DELAY_LINES; i++)
	{
		if (sscanf(argv[i+1], "%lf", targets+i) != 1)
		{
			fprintf(stderr,"Failed to read target %d.\n", i+1);
			exit(-2);
		}
	}

	/* Open the socket */

	switch(open_ople_fringe_tracking_socket())
	{
		case 0: break;

		case -1: fprintf(stderr,"Failed to resolve VME hostname.\n");
			 exit(-3);

		case -2: fprintf(stderr,"Failed to create a socket.\n");
			 exit(-4);

		case -3: fprintf(stderr,"Failed to turn off nangle.\n");
			 exit(-4);

		case -4: fprintf(stderr,"Failed to connect to the socket.\n");
			 exit(-5);

		default: fprintf(stderr,"Unknown error.\n");
			 exit(-5);
	}

	/* Set these targets */

	if (set_fringe_tracking_targets(targets))
	{
		fprintf(stderr,"Failed to set targets.");
		close_ople_fringe_tracking_socket();
	}

	/* Find out what the targets are now */

	current_fringe_tracking_targets(set_targets);
	printf("Targets:");
	for(i=0; i<NUM_DELAY_LINES; i++) printf(" %.2lf", set_targets[i]);
	putchar('\n');

	/* Wait for a while */

	sleep(5);

	/* Now incriment the targets */

	if (incriment_fringe_tracking_targets(targets))
	{
		fprintf(stderr,"Failed to incriment targets.");
		close_ople_fringe_tracking_socket();
	}

	/* Find out what the targets are now */

	current_fringe_tracking_targets(set_targets);
	printf("Targets:");
	for(i=0; i<NUM_DELAY_LINES; i++) printf(" %.2lf", set_targets[i]);
	putchar('\n');

	/* That should be all */

	close_ople_fringe_tracking_socket();
		
	exit(0);

} /* main() */
