/************************************************************************/
/* picom.h								*/
/*                                                                      */
/* Header file for the picomotor control.				*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : SEPTEMBER 1998                                              */
/************************************************************************/

#ifndef __PICO__
#define __PICO__

#define INITFREQ 10.0
#define FORWARD	1
#define BACKWARD 2
#define UNCOUPLED 1
#define COUPLED 2
#define SINGLE 3

typedef struct PSEUDOCONN {
	int	slot_a;
	int	conn_a;
	int	axis_a;
	int	dirc_a;
	double	count_ratio; /* to send different counts to each motor */
	int	slot_b;
	int	conn_b;
	int	axis_b;
	int	dirc_b;
} PseudoConn;

/* picom_comms.c */

int   pico_port_open(int argc, char **argv);
int   pico_port_close(int argc, char **argv);
char *pico_get_string(char *s);
int   pico_idn(char *s);
int   pico_reset(void);
int   pico_go_to_local(void);
int   pico_set_channel(int channel);
int   pico_get_channel(void);
int   pico_set_frequency(double frequency);
int   pico_init_frequency(double frequency);
int   pico_get_frequency(char *s);
int   pico_op_complete(void);
int   pico_run_one_axis(int counts,PseudoConn *conn);
int   pico_run_two_axis(int counts,PseudoConn *conn);
int   pico_check_pseudo_conn(PseudoConn *conn);
int   pico_init_pseudo_conn(PseudoConn *conn);

/* picom_control.c */

int   call_pico_idn(int argc, char **argv);
int   call_pico_reset(int argc, char **argv);
int   call_pico_go_to_local(int argc, char **argv);
int   call_pico_set_channel(int argc, char **argv);
int   call_pico_get_channel(int argc, char **argv);
int   call_pico_set_frequency(int argc, char **argv);
int   call_pico_init_frequency(int argc, char **argv);
int   call_pico_get_frequency(int argc, char **argv);
int   call_pico_op_complete(int argc, char **argv);
int   call_pico_run_one_axis(int argc, char **argv);
int   call_pico_run_two_axis(int argc, char **argv);
int   pico_control_uncoupled_mirror(char *name, int pulses, int az, int el);
int   call_pico_control_uncoupled_mirror(int argc, char **argv);
int   pico_control_single_mirror(char *name, int pulses, int chan);
int   call_pico_control_single_mirror(int argc, char **argv);
int   pico_control_coupled_mirror(char *name, int pulses, int az, int el);
int   call_pico_control_coupled_mirror(int argc, char **argv);
int   control_mirror(char *name,
                   int  *pulses,
                   char **names,
                   int *coupled,
                   int *axis_one,
                   int *axis_two);

#endif
