/************************************************************************/
/* pop_ui_client.c							*/
/*                                                                      */
/* UI interface for POP library routines.				*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY CLIENT INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar (but mostly taken from popgtk)          */
/* Date   : Oct 2007                                                    */
/************************************************************************/

#include "../charaui/src/charaui.h"
#include "../cliserv/include/cliserv.h"
#include "../include/chara_messages.h"
#include "../charalib/chara.h"
#include "popclient.h"
#include "popuiclient.h"

char *all_pop_types[N_POPS+1];

/************************************************************************/
/* call_open_pop_client()						*/
/*									*/
/* Calles routine to open connection to an POP server.			*/
/************************************************************************/

int call_open_pop_client(int argc, char **argv)
{
	int	i;

	/* Send the command */

	if (!open_pop_client())
	{
		return error(ERROR, "Failed to open POP server");
	}

	/* Construct the all_pop_types list */

	for(i=0; i<N_POPS; i++)
	{
		all_pop_types[i] = pops[i].name;
	}
	all_pop_types[N_POPS] = NULL;

	return NOERROR;

} /* call_open_pop_client() */

/************************************************************************/
/* call_close_pop_client()						*/
/*									*/
/* UI callable version of close_pop_client().				*/
/************************************************************************/

int call_close_pop_client(int argc, char **argv)
{
	int i;

	close_pop_client();
	for(i=0; i<N_POPS; i++) all_pop_types[i] = NULL;

	return NOERROR;

} /* call_close_pop_client() */

/************************************************************************/
/* call_send_pop_in()							*/
/*									*/
/* UI callable version of send_pop_in().				*/
/************************************************************************/

int call_send_pop_in(int argc, char **argv)
{
	int	id;

        if ((id = pop_get_id(argc, argv)) < 0) return NOERROR;

	if (!send_pop_in(id))
		return error(ERROR,
			"Failed to send POP in on %s.",all_pop_types[id]);

	return NOERROR;

} /* call_send_pop_in() */

/************************************************************************/
/* call_send_pop_out()							*/
/*									*/
/* UI callable version of send_pop_out().				*/
/************************************************************************/

int call_send_pop_out(int argc, char **argv)
{
	int	id;

        if ((id = pop_get_id(argc, argv)) < 0) return NOERROR;

	if (!send_pop_out(id))
		return error(ERROR,
			"Failed to send POP out on %s.",all_pop_types[id]);

	return NOERROR;

} /* call_send_pop_out() */

/************************************************************************/
/* call_send_pop_off()							*/
/*									*/
/* UI callable version of send_pop_off().				*/
/************************************************************************/

int call_send_pop_off(int argc, char **argv)
{
	int	id;

        if ((id = pop_get_id(argc, argv)) < 0) return NOERROR;

	if (!send_pop_off(id))
		return error(ERROR,
			"Failed to turn POP off on %s.",all_pop_types[id]);

	return NOERROR;

} /* call_send_pop_off() */

/************************************************************************/
/* call_send_pop_led_red()						*/
/*									*/
/* UI callable version of send_pop_led_red().				*/
/************************************************************************/

int call_send_pop_led_red(int argc, char **argv)
{
	int	id;

        if ((id = pop_get_id(argc, argv)) < 0) return NOERROR;

	if (!send_pop_led_red(id))
		return error(ERROR,
			"Failed to turn RED LED on for %s.",all_pop_types[id]);

	return NOERROR;

} /* call_send_pop_led_red() */

/************************************************************************/
/* call_send_pop_led_green()						*/
/*									*/
/* UI callable version of send_pop_led_green().				*/
/************************************************************************/

int call_send_pop_led_green(int argc, char **argv)
{
	int	id;

        if ((id = pop_get_id(argc, argv)) < 0) return NOERROR;

	if (!send_pop_led_green(id))
		return error(ERROR,
			"Failed to turn GREEN LED on for %s.",
			all_pop_types[id]);

	return NOERROR;

} /* call_send_pop_led_green() */

/************************************************************************/
/* call_send_pop_led_yellow()						*/
/*									*/
/* UI callable version of send_pop_led_yellow().				*/
/************************************************************************/

int call_send_pop_led_yellow(int argc, char **argv)
{
	int	id;

        if ((id = pop_get_id(argc, argv)) < 0) return NOERROR;

	if (!send_pop_led_yellow(id))
		return error(ERROR,
			"Failed to turn YELLOW LED on for %s.",
			all_pop_types[id]);

	return NOERROR;

} /* call_send_pop_led_yellow() */

/************************************************************************/
/* call_send_pop_led_off()						*/
/*									*/
/* UI callable version of send_pop_led_off().				*/
/************************************************************************/

int call_send_pop_led_off(int argc, char **argv)
{
	int	id;

        if ((id = pop_get_id(argc, argv)) < 0) return NOERROR;

	if (!send_pop_led_off(id))
		return error(ERROR,
			"Failed to turn LED off for %s.",all_pop_types[id]);

	return NOERROR;

} /* call_send_pop_led_off() */

/************************************************************************/
/* call_get_all_pop_status()						*/
/*									*/
/* UI callable version of get_all_pop_status().				*/
/************************************************************************/

int call_get_all_pop_status(int argc, char **argv)
{
	message(system_window,"Getting current summary data.");
	if (!get_all_pop_status())
	{
		werase(system_window);
		wrefresh(system_window);
		return error(ERROR, "Failed to get current status.");
	}
	werase(system_window);
	wrefresh(system_window);

	return NOERROR;

} /* call_get_all_pop_status() */

/************************************************************************/
/* show_pop_status()							*/
/*									*/
/* Displays current status of a single POP unit.			*/
/************************************************************************/

int show_pop_status(int argc, char **argv)
{
	int	id;
	int	keystroke;
	char	s[82];
	bool	loop;
#ifdef __NCURSES_H
        MEVENT  mouse;          /* A mouse event. */
#endif

        if ((id = pop_get_id(argc, argv)) < 0) return NOERROR;

	no_socket();

        active_window = sub_main_window;
        loop = TRUE;
	sprintf(s,"POP %s STATUS",all_pop_types[id]);
        while(loop)
        {
                heading(heading_window,s,"");

                /* run the background */

                background();

                /* And service the clients */

                process_message_socket();

		/* Display what we have */

		werase(sub_main_window);
		message(system_window,CONT_TEXT);

		/* Here's where it all lives. */

		switch(pops[id].status_limit)
                {
                        case STATUS_LIMIT_NONE:
                                mvwprintw(sub_main_window,0,30,
					"LIMIT   : NONE");
                                break;
                        case STATUS_LIMIT_FWD:
                                mvwprintw(sub_main_window,0,30,
					"LIMIT   : IN");
                                break;
                        case STATUS_LIMIT_REV:
                                mvwprintw(sub_main_window,0,30,
					"LIMIT   : OUT");
                                break;
                        case STATUS_LIMIT_BOTH:
                                mvwprintw(sub_main_window,0,30,
					"LIMIT   : BOTH");
                                break;
                        default:
                                mvwprintw(sub_main_window,0,30,
					"LIMIT   : ???");
                                break;
                }

		/* Motor Status */

		switch(pops[id].status_motor)
                {
                        case STATUS_MOTOR_OFF:
                                mvwprintw(sub_main_window,1,30,
						"MOTOR   : OFF (%i/256)",
                                                pops[id].position);
                                break;
                        case STATUS_MOTOR_FWD:
                                mvwprintw(sub_main_window,1,30,
						"MOTOR   : FWD (%i/256)",
                                                pops[id].position);
                                break;
                        case STATUS_MOTOR_REV:
                                mvwprintw(sub_main_window,1,30,
						"MOTOR   : REV (%i/256)",
                                                pops[id].position);
                                break;
                        default:
                                 mvwprintw(sub_main_window,1,30, "???");
                                break;
                }

		/* LED */

		switch(pops[id].status_led)
                {
                        case STATUS_LED_OFF:
                                mvwprintw(sub_main_window,2,30,"LED     : OFF");
                                break;
                        case STATUS_LED_GREEN:
                                mvwprintw(sub_main_window,2,30,"LED     : ON");
                                break;
                        default:
                                mvwprintw(sub_main_window,2,30,"LED     : ???");
                                break;
                }

		/* Time out and general status */

		mvwprintw(sub_main_window,3,30,"TIMEOUT : %s",
			pops[id].status_timeout?"YES":"NO");
		mvwprintw(sub_main_window,4,30,"MODE    : %s",
			pops[id].status_timeout?"LOCAL":"REMOTE");

		/* Make sure they see it. */

		wrefresh(sub_main_window);

		/* Was a key hit? */

                if (kbhit())
                {
                        keystroke = get_command();

                        /* Was it a mouse click? */
#ifdef __NCURSES_H
                        if (keystroke == KEY_MOUSE)
                        {
                                getmouse(&mouse);

                                if (mouse.bstate == BUTTON1_CLICKED)
                                {
                                        if (mouse.y == SYSTEM_BEGIN_Y &&
                                        mouse.x <  strlen(CONT_TEXT)+2)
                                        {
                                                keystroke = KEY_ESC;
                                        }
                                }
                        }

#endif
                        /* Was it KEY_ESC? */

                        if (keystroke == KEY_ESC)
                        {
                                loop = FALSE;
                        }
                        else
                        {
                                /* Is it a command ? */

                                command_processor(keystroke);
                        }
                }
        }
        werase(system_window);
        wrefresh(system_window);

        return NOERROR;

} /* show_pop_status() */

/************************************************************************/
/* show_pop_summary()							*/
/*									*/
/* Displays current status of all POPs.					*/
/************************************************************************/

int show_pop_summary(int argc, char **argv)
{
	int	id;
	int	scope,pop;
	int	row,col;
	char	*argl[2];
	char	arg1[80];
	int	keystroke;
	char	s[82];
	bool	loop;
#ifdef __NCURSES_H
        MEVENT  mouse;          /* A mouse event. */
#endif

	no_socket();

	if (call_get_all_pop_status(0, NULL) != NOERROR) return ERROR;
        active_window = sub_main_window;
        loop = TRUE;
	sprintf(s,"POP SUMMARY");
        while(loop)
        {
                heading(heading_window,"POP SUMMARY - TYPE U TO UPDATE","");

                /* run the background */

                background();

                /* And service the clients */

                process_message_socket();

		/* Display what we have */

		werase(sub_main_window);
		message(system_window,CONT_TEXT);

		/* Setup the display */

		wstandout(sub_main_window);
		mvwprintw(sub_main_window,0,16,"POP1 LED");
		mvwprintw(sub_main_window,0,32,"POP2 LED");
		mvwprintw(sub_main_window,0,48,"POP3 LED");
		mvwprintw(sub_main_window,0,64,"POP4 LED");
		for(scope=0; scope<NUM_SCOPES; scope++)
			mvwprintw(sub_main_window,scope+1,0,"%s",
				scope_types[scope]);
		wstandend(sub_main_window);

		/* OK loop through all possibilities. */

		for(scope=0; scope<NUM_SCOPES && loop; scope++)
		for(pop=1; pop<NUM_POPS && loop; pop++)
		{
			/* Which ID is this? */

			sprintf(arg1,"%s-%1d", scope_types[scope], pop);
			argl[0] = "popsum";
			argl[1] = arg1;
			if ((id = pop_get_id(2,argl)) < 0)
			{
				loop = FALSE;
				error(ERROR,"Failed to work out ID for %s",
					arg1);
				break;
			}

			col = pop * 16;
			row = scope+1;

			switch(pops[id].status_limit)
			{
				case STATUS_LIMIT_NONE:
					mvwprintw(sub_main_window,row,col,
						"NONE");
					break;

				case STATUS_LIMIT_FWD:
					mvwprintw(sub_main_window,row,col,
						"IN");
					break;

				case STATUS_LIMIT_REV:
					mvwprintw(sub_main_window,row,col,
						"OUT");
					break;

				case STATUS_LIMIT_BOTH:
					mvwprintw(sub_main_window,row,col,
						"BOTH");
					break;

				default:
					mvwprintw(sub_main_window,row,col,
						"???");
					break;
			}

			switch(pops[id].status_led)
			{
				case STATUS_LED_OFF:
					mvwprintw(sub_main_window,row,col+5,
					"OFF");
					break;
				case STATUS_LED_GREEN:
					mvwprintw(sub_main_window,row,col+5,
					" ON");
					break;
				default:
					mvwprintw(sub_main_window,row,col+5,
					"???");
					break;
			}

		}

		/* Make sure they see it. */

		wrefresh(sub_main_window);

		/* Was a key hit? */

                if (kbhit())
                {
                        keystroke = get_command();

                        /* Was it a mouse click? */
#ifdef __NCURSES_H
                        if (keystroke == KEY_MOUSE)
                        {
                                getmouse(&mouse);

                                if (mouse.bstate == BUTTON1_CLICKED)
                                {
                                        if (mouse.y == SYSTEM_BEGIN_Y &&
                                        mouse.x <  strlen(CONT_TEXT)+2)
                                        {
                                                keystroke = KEY_ESC;
                                        }
                                }
                        }

#endif
                        /* Was it KEY_ESC? */

                        if (keystroke == KEY_ESC)
                        {
                                loop = FALSE;
                        }
                        else if (keystroke == 'U')
			{
				if (call_get_all_pop_status(0, NULL) != NOERROR)
				{
					loop = FALSE;
				}
			}
			else
                        {
                                /* Is it a command ? */

                                command_processor(keystroke);
                        }
                }
        }
        werase(system_window);
        wrefresh(system_window);

        return NOERROR;

} /* show_pop_status() */

/************************************************************************/
/* pop_get_id()								*/
/*									*/
/* Command lines stuff to get which id is required.			*/
/* Returns id number or -1 if user stops or -2 on error.		*/
/* Returns -3 if the server connectio0n is not open.			*/
/* Returns -4 if port not even open.					*/
/************************************************************************/

int pop_get_id(int argc, char ** argv)
{
	int	id;

	if (all_pop_types[0] == 0 || all_pop_types == NULL || pop_server == -1)
		return -4;

	if (argc > 1)
        {
                for (id=0; id<N_POPS; id++)
                {
                        if (strcmpi(argv[1],all_pop_types[id]) == 0)
                        {
                                break;
                        }
                }

                if (id >= N_POPS)
                {
                        error(ERROR,"Unkown pop %s.",argv[1]);
                        return -2;
                }
        }
        else
        {
                id = 0;
		clean_command_line();
                if (quick_edit("Pop",all_pop_types[id],
                        &id,all_pop_types,ENUMERATED) == KEY_ESC)
                                return -1;
        }

	return id;

} /* pop_get_id() */
