/************************************************************************/
/* rwfits.h								*/
/*									*/
/* Header file for							*/
/* Simplified and hacked FITS file reader and writer routines.  	*/
/************************************************************************/
/*                                        	                      	*/
/* Center for High Angular Resolution Astronomy				*/
/*									*/
/* Telephone: 404-651-2932						*/
/* Fax      : 404-651-1384						*/
/* Internet : theo@chara.gsu.edu					*/
/*									*/
/* (C) This source code and its associated executable           	*/
/* program(s) are copyright.                                    	*/
/*                                                              	*/
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : April 1997   						*/
/************************************************************************/

#ifndef _RWFITS_
#define _RWFITS_

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#if !(defined(macintosh) || (defined(__MACH__) && defined(__APPLE__)))
#include <values.h>
#include <malloc.h>
#endif

#define MAXHEADER 100		/* Maximum number of header lines */

#define BYTE_SWAP /* OFF FOR SUNOS, ON FOR LINUX, ON FOR DEC */

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

#define FITS_BLOCK_SIZE 2880
#define FITS_LINE_SIZE 80
#define FITS_KEY_SIZE 8
#define SIMPLE_T_STR "SIMPLE  =                    T  / rwfits C Library by Theo ten Brummelaar 1997"
#define int1	char
#define int2 	short int
#define int4 	int
#define real4	float
#define real8	double

/* Potenital conflict (or inconsistancy) with charaui.h here! */

#define UNKNOWN 0
#define FLOAT	1
#define INT	2
#define TEXT	3

#define mem_test(a) {if (a == 0) {fprintf(stderr,"Out of memory.\n");exit(-2);}}

#define file_to_physical(x)	(x*bscale + bzero)
#define physical_to_file(x)	((x-bzero)/bscale)

typedef struct FITS_DATA {

		int nheader;	/* Number of header entries */
		char **header;	/* The header entries themselves  1..n */
		int nx;		/* Number of X pixels */
		int ny;		/* Number of Y pixels */
		float **data;	/* The data[1..nx][1..ny] */

		} fits_data;

fits_data read_fits(char *filename);

fits_data copy_fits(fits_data data);

void free_fits_data(fits_data data);

void print_fits_header(fits_data data);

void format_key(char *oldkey, char *newkey);

unsigned char read_fits_block(unsigned char *block, FILE *input);

void write_fits(fits_data data, char *filename);

unsigned char write_fits_block(unsigned char *block, FILE *input);

int read_fits_header_int(fits_data, char *keyword, char *comment);

float read_fits_header_float(fits_data data, char *keyword, char *comment);

char *read_fits_header_text(fits_data data,char *keyword,char *s,char *comment);

int read_fits_datasec(fits_data data, int *xmin, int *xmax, int *ymin, int *ymax);

int read_fits_ccdsec(fits_data data, int *xmin, int *xmax, int *ymin, int *ymax);

int read_fits_biassec(fits_data data, int *xmin, int *xmax, int *ymin, int *ymax);

int find_key(fits_data data, char *keyword);

void write_fits_header_int(fits_data *data, char *keyword, 
                                int value, char *comment);

void write_fits_header_float(fits_data *data, char *keyword, 
                                float value, char *comment);

void write_fits_header_text(fits_data *data, char *keyword, 
                                char *value, char *comment);

void reverse_byte_order(void *p, int nbytes);

int fits_header_type(fits_data data, char *keyword);

#endif
