/************************************************************************/
/* get_mouse.c								*/
/*									*/
/* Gets the location of the mouse within a window.			*/
/* Part of the simpleX library.						*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : May 1997	   						*/
/************************************************************************/

#include <stdio.h>
#include "../nrc/nrc.h"
#include "simpleX.h"

/************************************************************************/
/* get_mouse()								*/
/*									*/
/* Tries to get the mouse position within the current window.		*/
/* Waits for a left mouse button click WITHIN the current window.	*/
/* Reverses the Y coordinate so that it gives the bottom left as	*/
/* the origin. Uses pixmult to divide said mouse position.		*/
/* Returns TRUE if all is OK and FLASE otherwise.			*/
/************************************************************************/

Bool get_mouse(
	Window	theWindow,
	int	pixmult,
	int	*x,
	int	*y)
{
	Window	root_return, child_return;
	int	root_x, root_y;
	int	win_x, win_y;
	Bool	return_value;
	unsigned int mask_return;
	XWindowAttributes attribs;
	int	x_size, y_size;

	/* First get the attributes of that window */

	XGetWindowAttributes(theDisplay, theWindow, &attribs);

	x_size = attribs.width;
	y_size = attribs.height;

#ifdef DEBUG
	printf("x_size = %d\ny_size = %d\n",x_size,y_size);
#endif
	do
	{
		return_value = XQueryPointer(theDisplay, theWindow, &root_return,
		&child_return, &root_x, &root_y, &win_x, &win_y, &mask_return);

	} while (mask_return != 256 
		|| win_x<0 || win_x>=x_size || win_y<0 || win_y>=y_size);

#ifdef DEBUG
	printf("win_x = %d\nwin_y = %d\n",win_x,win_y);
#endif
	*x = win_x/pixmult + 1;
	*y = (y_size - win_y)/pixmult+1;

	return return_value;

} /* get_mouse() */
