/************************************************************************/
/* initX.c 								*/
/*									*/
/* Sets up X windows in a simple way ....				*/
/* Part of the simpleX library.						*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : August 1994   						*/
/************************************************************************/

#include <stdio.h>
#include "../nrc/nrc.h"
#include "simpleX.h"

/*
 * Globals
 */

Display	*theDisplay;
int	theScreen;
int	theDepth;
int	theWidth;
int	theHeight;
Window   theRootWindow;
Visual	*theVisual;
GC	theGC;
unsigned int	fcol,bcol;
Colormap	theCmap;
unsigned char    	GreyScale[NUM_COLORS];
int display_is_open = FALSE;

/************************************************************************/
/* initX()								*/
/*									*/
/* Simple X windows use set up. Does nothing fancy, also sets up simple */
/* grey-scale color map.						*/
/* Argument is the name of the display to open. If this name is NULL 	*/
/* the default display is opened. Returns number of colours allocated	*/
/* meaning that a 0 is an error.					*/ 
/************************************************************************/

unsigned char initX(char *display_name)
{
   	int i,color_step;
        XColor  color;
	int	num_sucess;

	if (display_is_open) return 0;

	/* Open the display */

	if ((theDisplay = XOpenDisplay( display_name )) == NULL) return 0;

	/* Check out what the display is like */

	theScreen     = DefaultScreen(theDisplay);
	theCmap       = DefaultColormap(theDisplay, theScreen);
	theRootWindow = RootWindow(theDisplay, theScreen);
	theGC         = DefaultGC(theDisplay,theScreen);
	theDepth      = DefaultDepth(theDisplay, theScreen);
	theWidth      = DisplayWidth(theDisplay, theScreen);
	theHeight     = DisplayHeight(theDisplay, theScreen);
	theVisual     = DefaultVisual(theDisplay,theScreen);
	fcol          = WhitePixel(theDisplay,theScreen);
	bcol          = BlackPixel(theDisplay,theScreen);

	XSetForeground(theDisplay,theGC,fcol);
	XSetBackground(theDisplay,theGC,bcol);

	/* Set up GreyScale color map */

	color_step = 65536 / (NUM_COLORS-1);
	num_sucess = 0;

	for(i=0;i<NUM_COLORS-1;i++)
	{
	        color.red = i*color_step;
                color.green = i*color_step;
                color.blue = i*color_step;
                color.flags = DoRed | DoGreen | DoBlue;
                if (XAllocColor(theDisplay,theCmap,&color)) num_sucess++;

		GreyScale[i] = color.pixel;
	}
	color.red = 65535;
        color.green = 65535;
        color.blue = 65535;
        color.flags = DoRed | DoGreen | DoBlue;
        if (XAllocColor(theDisplay,theCmap,&color)) num_sucess++;

        GreyScale[NUM_COLORS-1] = color.pixel;

	display_is_open = TRUE;

	return num_sucess;

} /* initX() */
