/************************************************************************/
/* openwindow.c								*/
/*									*/
/* Opens a simple X windows window					*/
/* Part of the simpleX library.						*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : August 1994   						*/
/************************************************************************/

#include <stdio.h>
#include "../nrc/nrc.h"
#include "simpleX.h"

/************************************************************************/
/* openWindow()								*/
/*									*/
/* Creates a simple X windows window ready for use by programmer.	*/
/************************************************************************/

Window openWindow(char *name,
		  int x,
		  int y, 
		  int width, 
		  int height) 
{
	XSetWindowAttributes	theWindowAttributes;
	XSizeHints		theSizeHints;
	unsigned int		mask;
	Window			theNewWindow;
	/*XGCValues		GCvalues;*/

	/* Set up `Hints' */

	theSizeHints.flags  = USPosition | USSize | PMaxSize;
	theSizeHints.x	    = x;
	theSizeHints.y	    = y;
	theSizeHints.width  = width;
	theSizeHints.height = height;
	theSizeHints.max_width = theWidth;
	theSizeHints.max_height = theHeight;

	/* Set up a simple window attributes */

	theWindowAttributes.border_pixel = bcol;
	theWindowAttributes.background_pixel = fcol;
	theWindowAttributes.backing_store = Always;
	theWindowAttributes.bit_gravity = StaticGravity;

	mask = (CWBackPixel |
			 CWBorderPixel |
			 CWBackingStore |
			 CWBitGravity);

	/* Create the window */

	theNewWindow = XCreateWindow( theDisplay,
			theRootWindow,
			x,y,
			width, height,
			BORDER_SIZE,
			0,
			CopyFromParent,
			CopyFromParent,
			mask,
			&theWindowAttributes );

	XSetStandardProperties(theDisplay,theNewWindow,name,name,None,
                    (char **)NULL,0,&theSizeHints);

	/* Put the display onto the screen */

	XMapWindow(theDisplay, theNewWindow);

	/* Flush the display */

	XFlush(theDisplay);

	/* That's it for now */

	return theNewWindow;

} /* openWindow() */
