/************************************************************************/
/* testX.c 								*/
/*									*/
/* Test programme for simple X library.  				*/
/* Part of the simpleX library.						*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : August 1994   						*/
/************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "../nrc/nrc.h"
#include "simpleX.h"

int main(int argc, char **argv)
{
	Window	theWindow1;
	Window	theWindow2;
	Window	theWindow3;
	Window	theWindow4;
	XImage	*theImage1;
	XImage	*theImage2;
	XImage	*theImage3;
	XImage	*theImage4;
	int	i,j;
	char *picture1;
	char *picture2;
	char *picture3;
	char *picture4;
	float	**values;
	int	npix,pixmult;

	if (argc < 3)
	{
		fprintf(stderr,"usage: %s npix pixmult <display>\n",argv[0]);
		exit(-1);
	}

	sscanf(argv[1],"%d",&npix);
	sscanf(argv[2],"%d",&pixmult);

	if (argc <4)
		i = initX(NULL);
	else
		i = initX(argv[3]);

	if (!i)
	{
		fprintf(stderr,"Failed to open display.\n");
		exit(-1);
	}
	infoX();
	printf("Allocated %i colors.\n",i);

	theWindow1 = openWindow("LIN",100,100,npix*pixmult,npix*pixmult);
	theWindow2 = openWindow("LOG",120+npix*pixmult,100,
			npix*pixmult,npix*pixmult);
	theWindow3 = openWindow("EXP",100,140+npix*pixmult,
			npix*pixmult,npix*pixmult);
	theWindow4 = openWindow("STD",120+npix*pixmult,140+npix*pixmult,
			npix*pixmult,npix*pixmult);


	/* Build a test image */

	values = matrix(1,npix,1,npix);

	for(i=1;i<=npix;i++)
	{
		for(j=1;j<=npix;j++)
		{
			values[i][j] = sqrt((double)i*j);
		}
	}
/*
        for(i=0;i<NUM_COLORS;i++) printf("Color %d = %d\n",i,GreyScale[i]);
*/

	picture1 = make_picture(npix,npix,pixmult,values,LIN);
	picture2 = make_picture(npix,npix,pixmult,values,LOG);
	picture3 = make_picture(npix,npix,pixmult,values,EXP);
	picture4 = make_picture(npix,npix,pixmult,values,STD);
	

        theImage1 = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
			picture1,npix*pixmult,npix*pixmult,
			8,npix*pixmult);

        theImage2 = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
			picture2,npix*pixmult,npix*pixmult,
			8,npix*pixmult);

        theImage3 = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
			picture3,npix*pixmult,npix*pixmult,
			8,npix*pixmult);

        theImage4 = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
			picture4,npix*pixmult,npix*pixmult,
			8,npix*pixmult);

	if (!theImage1 || !theImage2 || !theImage3 || !theImage4) 
	{
		fprintf(stderr,"Could not create image....\n");
	}

	XPutImage(theDisplay,theWindow1,theGC,theImage1,0,0,0,0,
		npix*pixmult,npix*pixmult);

	XPutImage(theDisplay,theWindow2,theGC,theImage2,0,0,0,0,
		npix*pixmult,npix*pixmult);

	XPutImage(theDisplay,theWindow3,theGC,theImage3,0,0,0,0,
		npix*pixmult,npix*pixmult);

	XPutImage(theDisplay,theWindow4,theGC,theImage4,0,0,0,0,
		npix*pixmult,npix*pixmult);

	/* Try and display it */

        XMapWindow(theDisplay, theWindow1);
        XMapWindow(theDisplay, theWindow2);
        XMapWindow(theDisplay, theWindow3);
        XMapWindow(theDisplay, theWindow4);

        XFlush(theDisplay);

	sleep(2);

	XDestroyImage(theImage1);
	XDestroyImage(theImage2);
	XDestroyImage(theImage3);
	XDestroyImage(theImage4);

	free_matrix(values,1,npix,1,npix);

	XDestroyWindow(theDisplay, theWindow1);
	XDestroyWindow(theDisplay, theWindow2);
	XDestroyWindow(theDisplay, theWindow3);
	XDestroyWindow(theDisplay, theWindow4);

	quitX();

	exit(0);
}
