/************************************************************************/
/* astropos.c                                                           */
/*                                                                      */
/* Reads in a time and a cart and works out where it should have	*/
/* been according to the model at that time. Does a linear interp.	*/
/* Assumes the time is in milliseconds since GMT midnight.		*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : November 99                                                 */
/************************************************************************/


#include <unistd.h>
#include <stdlib.h>
#include "../nrc/nrc.h"
#include "../charalib/chara.h"


int main(int argc, char **argv)
{

	/*
	 * Internals
	 */

	double	pos1,pos2;
	int	secs_since_midnight1,secs_since_midnight2;
	int	millisecs_since_midnight;
	char	ref_cart[10];

	/* Check the command line */

	if (argc < 3 || argc > 4)
	{
		fprintf(stderr,"usage: %s CART time <model file>.\n",argv[0]);
		exit(-1);
	}

	sscanf(argv[2],"%d",&millisecs_since_midnight);

	/*
	 * Round it off ot the previous whole number 
	 * of ASTROMOD_TICK_TIMES.
	 */

	secs_since_midnight1 = (int)millisecs_since_midnight/1000;
	secs_since_midnight1 -= secs_since_midnight1 % ASTROMOD_TICK_TIME;
	secs_since_midnight2 = secs_since_midnight1 + ASTROMOD_TICK_TIME;

	if (argc > 3)
	{
	   if (open_astromod(".",argv[3]))
	   { 
		   fprintf (stderr, "Could not open Astromod %s.\n",argv[3]);
		   exit (-1);
	   }
	}
	else
	{
	   if (open_astromod(NULL,NULL))
	   { 
		   fprintf (stderr, "Could not open Astromod.\n");
		   exit (-1);
	   }
	}

	if (get_fixed_cart(ref_cart) != 0)
        {
                fprintf(stderr,"Failed to get reference cart name.\n");
		exit(-3);
        }
	printf("Reference cart = %s\n",ref_cart);


	get_cart_pos (secs_since_midnight1, argv[1], &pos1);
	get_cart_pos (secs_since_midnight2, argv[1], &pos2);

	printf("Time 1 = %d Pos 1 = %15.9f\n",secs_since_midnight1,pos1);
	printf("Time 2 = %d Pos 2 = %25.9f\n",secs_since_midnight2,pos2);

	printf("Extrapolated position = %15.9f\n",pos1+
		((double)millisecs_since_midnight/1000.0-secs_since_midnight1)/10.0*
		(pos2-pos1));

	close_astromod();

	exit(0);
}
