/************************************************************************/
/* astroui.c								*/
/*                                                                      */
/* Test program for astromod UI interface.				*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Tony Johnson & Theo ten Brummelaar                          */
/* Date   : Original Version 1990 - ported to Linux 1998                */
/************************************************************************/

#define FUNCTS
#include "../charaui/src/charaui.h"
#undef FUNCTS
#include "../include/chara_messages.h"
#include "../charalib/chara.h"
#include "../astromod/astromod.h"
#include "../astromod/astromod_ui.h"
#include <time.h>

/* Proto types */

int test_create_astromod_data_file_name(int argc, char **argv);
int time_status(void);		/* Example background job below */
int astromod_status(void);
int current_config_callback(void);
int star_callback(void);
int init_callback(void);
int active_scopes_callback(void);
int set_n_air(int argc, char **argv);
int make_ping_noise(int argc, char **argv);

/* Function list */

struct {                       
                char    *name;
                int     (*function)(int argc, char **argv);
        } functions[] = 

	{	{NULL,		NULL},
#include "../charaui/src/std_ui_functs.h"
#include "../astromod/std_astro_functs.h"
		{"nair",	set_n_air},
		{"file",	test_create_astromod_data_file_name},
		{"doping",	make_ping_noise},
		{NULL,		NULL}	};

/* Global for refractive index of air */

static double n_air = 1.0;

void open_function(void)
{
	astromod_init(NULL, NULL);
	set_no_star(0, NULL);
	background_start(0,NULL);
	set_current_config_callback(current_config_callback);
	set_star_callback(star_callback);
	set_init_callback(init_callback);
	set_active_scopes_callback(active_scopes_callback);
}

int main(int argc, char **argv)
{
	/* Check command line */

	if (argc !=2)
	{
		fprintf(stderr,"usage: %s name\n", argv[0]);
		exit(-1);
	}

	/* Title page. */

	ui_clear_screen();
	put_line("");
	center_line("CHARA USER INTERFACE ASTROMOD TEST ROUTINE");
	put_line("");
	center_line("The CHARA Array");
	center_line("Center for High Angular Resolution Astronomy");
	center_line("Mount Wilson Observatory, CA 91001, USA");
	put_line("");
	center_line("Telephone: 1-626-796-5405");
	center_line("Fax: 1-626-796-6717");
	center_line("email: theo@chara.gsu.edu");
	center_line("WWW: http://www.chara.gsu.edu");
	put_line("");
	center_line("(C) This executable program is copyright.");
	wait_for_title();

	/* Initialize the user interface */

	TITLE = "ASTROMODUI VERSION 0.0";
	initialise_ui("astro_menus.ini","astro_help.ini",argv[1]);
	set_user_open_function(open_function);

	/* Setup background job(s) */

	background_add(time_status);
	background_add(astromod_status);

	/* Setup the messages */

	setup_astromod_messages();

	/* Let's go! */

	start_ui(); /* Should never return from here. */

	exit(0);
}

/************************************************************************/
/* time_status()							*/
/* 									*/
/* Displays the time in the status window.				*/
/************************************************************************/

int time_status(void)
{
	time_t current_time;
	struct tm *now;

	time(&current_time);
	now = localtime(&current_time);
	wstandout(status_window);
	mvwaddstr(status_window,0,0,"Local Time : ");
	wstandend(status_window);
	wprintw(status_window,"     %2d:%02d:%02d",
		now->tm_hour,now->tm_min,now->tm_sec);

	return NOERROR;

} /* time_status() */

/************************************************************************/
/* astromod_status()							*/
/* 									*/
/* Displays the astromod in the status window.				*/
/************************************************************************/

int astromod_status(void)
{
	int year, month, day;
	struct  tm      *gmt_now;
	time_t	current_time;
	double	ut;
	double  mjd;
	double az,el,ref_el,hour_angle;
	char	s[256];
	int	i;
	double  delta_air_path;
	
	wstandout(status_window);
	mvwaddstr(status_window,1,0,"Star Name  : ");
	wstandend(status_window);
	wprintw(status_window,"%13s",star.name);

	time(&current_time);
	gmt_now = gmtime(&current_time);

	/* Note... not really Y2K cool.... */

        year = gmt_now->tm_year + 1900;
        if (year < 1950) year+=100;
	month = gmt_now->tm_mon+1;
	day = gmt_now->tm_mday;
	ut = ((double)gmt_now->tm_sec/(24.0*60.0*60.0) +
	      (double)gmt_now->tm_min/(24.0*60.0) +
	      (double)gmt_now->tm_hour/(24.0)) * M_PI * 2.0;
	mjd = astromod_mjd(year, month, day, ut);

	wstandout(status_window);
	mvwaddstr(status_window,2,0,"Mod Jul Dat: ");
	wstandend(status_window);
	wprintw(status_window,"%13.6f",mjd);
	
	wstandout(status_window);
	mvwaddstr(status_window,3,0,"Atomic Time: ");
	wstandend(status_window);
	wprintw(status_window,"%13.6f",astromod_atomic_time(mjd));
	
	astromod_azel_mjd(star, mjd, &az, &el, &ref_el, &hour_angle);

	wstandout(status_window);
	mvwaddstr(status_window,4,0,"Azimuth    : ");
	wstandend(status_window);
	wprintw(status_window,"  %11.4f",az/D2R);

	wstandout(status_window);
	mvwaddstr(status_window,5,0,"Elevation  : ");
	wstandend(status_window);
	wprintw(status_window,"  %11.4f",el/D2R);

	wstandout(status_window);
	mvwaddstr(status_window,6,0,"Refract El : ");
	wstandend(status_window);
	wprintw(status_window,"  %11.4f",ref_el/D2R);

	wstandout(status_window);
	mvwaddstr(status_window,7,0,"Hour Angle : ");
	wstandend(status_window);
	wprintw(status_window,"%13s",time_to_string(hour_angle,s));

	wstandout(status_window);
	mvwaddstr(status_window,0,27,"Reference : ");
	wstandend(status_window);
	wprintw(status_window,"%15s",ople_types[reference_cart]);
	for(i=0; i<NUM_DELAY_LINES; i++)
	{
		if (scope_active[i]) wstandout(status_window);
		mvwprintw(status_window,i+1,27,"%2s OPLE   : ", ople_types[i]);
		if (scope_active[i])
		{
		    wstandend(status_window);
		    wprintw(status_window,"%15.10f",
			astromod_cart_position(i, reference_cart, 
			    n_air ,az,el,star.chara_number,&delta_air_path));
		    wstandout(status_window);
		}
		else
		{
		    wprintw(status_window,"%15.10f",0.0);
		}
		mvwprintw(status_window,i+1,55,"dAir : ");
		if (scope_active[i])
		{
		    wstandend(status_window);
		    wprintw(status_window,"%15.10f", delta_air_path);
		}
		else
		{
		    wprintw(status_window,"%15.10f", 0.0);
		}
	}

	return NOERROR;

} /* astromod_status() */

/************************************************************************/
/* set_n_air()								*/
/*									*/
/* Allows user to set refractive index of air.				*/
/************************************************************************/

int set_n_air(int argc, char **argv)
{
	char s[81];
	float n;

        if (argc > 1)
        {
                sscanf(argv[1],"%f",&n);
        }
        else
        {
                sprintf(s,"%15.9f",n_air);
                if (quick_edit("Refractive index of air",
                        s,s,NULL,FLOAT) == KEY_ESC) return NOERROR;
                sscanf(s,"%f",&n);
        }

	n_air = (double)n;

	return NOERROR;

} /* set_n_air() */

int test_create_astromod_data_file_name(int argc, char **argv)
{
	char s[256];

	message(system_window,create_astromod_data_file_name("directory/","test",s));

	return NOERROR;

}

int current_config_callback(void)
{
	message(system_window,"Someone wants the configuration!");
	sleep(1);
	return NOERROR;

}

int star_callback(void)
{
	message(system_window,"The star data has changed!");
	sleep(1);
	return NOERROR;

}

int init_callback(void)
{
	message(system_window,"The configuration data has changed!");
	sleep(1);
	return NOERROR;

}
int active_scopes_callback(void)
{
	message(system_window,"The active scopes have changed!");
	sleep(1);
	return NOERROR;

}

/************************************************************************/
/* make_ping_noise()							*/
/*									*/
/************************************************************************/

int make_ping_noise(int argc, char **argv)
{
	struct smessage mess;

	mess.type = MAKE_PING_NOISE;
	mess.length = 0;
	mess.data = NULL;

	ui_send_message_all(&mess);

	return NOERROR;

} /* make_ping_noise() */
