/************************************************************************/
/* fit1Dg.c VERSION 1.0							*/
/*									*/
/* Fits Gaussian to a data set.						*/
/*									*/
/* fit1D <input <output>>						*/
/*									*/
/* If no input file is given uses stdin.				*/
/* If no output files is given fit results is sent to stdout.		*/
/************************************************************************/
/*                                        	                      	*/
/* Sydney University Stellar Interferometer.                    	*/
/*                                                              	*/
/* Chatterton Astronomy Department, School of Physics,          	*/
/* University of Sydney, N.S.W. 2006, Australia.                	*/
/*                                                              	*/
/* Telephone: +61 2 692 2544                                    	*/
/* Telex: AA 26169 UNISYD                                       	*/
/* Facsimile: +61 2 660 2903                                    	*/
/* ACSnet: root@astron.physics.su.oz                            	*/
/*                                                              	*/
/* (C) This source code and its associated executable           	*/
/* program(s) are copyright.                                    	*/
/*                                                              	*/
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : Jan 2014							*/
/************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include <string.h>
#include "../nsimpleX/nsimpleX.h"
#include "../nrc/nrc.h"

#define MAX_DATA 	2000
#define NUM_CONST	3
#define PERCENT_ERR 	100
#define ERR_INC		0.5


int main(int argc,char **argv)
{
	/* Locals */

	int	i, ndata;
	float	x[MAX_DATA];
	float	y[MAX_DATA];
	FILE	*input;		/* data file */
	FILE	*output;	/* data file */
	char	s[256];		/* Buffer */
	float	floor, amp2, mean2, stddev2;
	float	amp1, mean1, stddev1;
	float   z,t,c2;
	float	res;

	/* Open the input file */

	if (argc > 1)
	{
		if ((input = fopen(argv[1],"r")) == NULL)
		{
			fprintf(stderr,"Could not open file %s.\n",argv[1]);
			exit(-2);
		}
	}
	else
	{
		input = stdin;
	}

	/* Open the output file */

	if (argc > 2)
	{
		if ((output = fopen(argv[2],"r")) != NULL)
		{
			fprintf(stderr,"File %s already exists.\n",argv[2]);
			exit(-3);
		}
		if ((output = fopen(argv[2],"w")) == NULL)
		{
			fprintf(stderr,"Could not open file %s.\n",argv[2]);
			exit(-4);
		}
	}
	else
	{
		output = stdout;
	}

	printf("# fit1DG\n");
	if (input == stdin)
	{
		printf("# Input file : stdin\n");
	}
	else	
	{
		printf("# Input file : %s \n",argv[1]);
	}

	if (output == stdout)
	{
		printf("# Output file : stdin\n");
	}
	else	
	{
		printf("# Output file : %s \n",argv[2]);
	}

	/* Read in the data and doit */

	ndata = 0;
	while(fgets(s,256,input) != NULL)
	{
		sscanf(s,"%f %f",x+(ndata+1),y+(ndata+1));
		ndata++;
	}
	if (input != stdin) fclose(input);

	if (ndata <= 4)
	{
		fprintf(stderr,"Must have more data than unknowns !!\n");
		exit(-6);
	}
	printf("Number of data points : %d\n\n",ndata);

	fit_1D_gaussian(ndata, x, y, &amp1, &mean1, &stddev1, 
				0.01, 20, SHOW_ALL);
	fit_1D_gaussian_with_floor(ndata, x, y, &floor, &amp2, &mean2, &stddev2,
				0.01, 20, SHOW_ALL);

	printf("#Results of fit with no floor: \n");
	printf("#Amplitude   %le\n",amp1);
	printf("#Centre      %le\n",mean1);
	printf("#S.D.        %le\n",stddev1);
	printf("#Results of fit with floor: \n");
	printf("#Floor       %le\n",floor);
	printf("#Amplitude   %le\n",amp2);
	printf("#Centre      %le\n",mean2);
	printf("#S.D.        %le\n",stddev2);

	/* Print the results */

	fprintf(output,"#    X         Y        Fit1     Fit2\n");

	for(i=1; i<=ndata;i++)
	{
		z = x[i] - mean1;
		c2 = stddev1 * stddev1;
		t = z*z;
		res = amp1*exp(t/(-2.0*c2));

		fprintf(output,"%9.2e %9.2e %9.2e ",x[i], y[i], res);

		z = x[i] - mean2;
		c2 = stddev2 * stddev2;
		t = z*z;
		res = floor + amp2*exp(t/(-2.0*c2));

		fprintf(output,"%9.2e\n", res);
	}
	
	exit(0);
}
