/************************************************************************/
/* impulselag.c 							*/
/*									*/
/* Test driver for simple lag digital filter.				*/
/************************************************************************/
/*                                        	                      	*/
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.						*/
/*                                                              	*/
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : December 1994						*/
/* Changes:                                                             */
/*                                                                      */
/* 7/25/96 Removed refferences to fcvector as it is not really a part   */
/*         of NRC.                                                      */
/************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "../nrc/nrc.h"
#include "../filter/filter.h"

int main(int argc, char **argv)
{
	int 	i;
	float	dt;
	float	tau;
	digital_filter filter;
	fcomplex *response;
	fcomplex theory;
	float f6db;
	int	length;
	float	*freqs;

	/* Check command line */

	if (argc < 4)
	{
		fprintf(stderr,"usage: impulselag dt tau length\n");
		exit(-1);
	}

	sscanf(argv[1],"%f",&dt);
	sscanf(argv[2],"%f",&tau);
	sscanf(argv[3],"%d",&length);

#ifdef DISPLAY
	if (!initX(NULL))
	{
		fprintf(stderr,"Failed to open display.\n");
		exit(-1);
	}
#endif
	/* Create the simple lag response */

	f6db = simple_lag(&filter, dt, tau);

	/* Caluclate the response */
	
#ifdef DISPLAY
	response = impulse_response(filter,&length,&freqs,TRUE);
#else
	response = impulse_response(filter,&length,&freqs,FALSE);
#endif 

	/* Print out the results */

	printf("# FINAL RESULTS\n#\n");
	printf("# tau = %f\n",tau);
	printf("# f -6dB = %f\n",f6db);
	printf("#   f         Mag(T)     Phase(T)    Mag(Resp)  Phase(Resp)\n");

	for(i=0; i<=length/2; i++)
	{
		theory = simple_lag_response(freqs[i],tau);

		printf("%10.3e  %10.3e  %10.3e  %10.3e  %10.3e\n",
		freqs[i],
		mag_db(theory),phase(theory),
		mag_db(response[i]),-phase(response[i]));
		fflush(stdout);
	}
		
	/* Clean up */

#ifdef DISPLAY
	quitX();
#endif

	free_vector((float*)response,0,length);
	free_vector(freqs,0,length/2);
	clean_up_filter(filter);
	exit(0);

} /* main() */
