/************************************************************************/
/* lineedit.c								*/
/*                                                                      */
/* Discription                                   	 		*/
/* 	This file contains the functions that make up the		*/
/*	line editor. They are :						*/
/*									*/
/*		line_edit()	The editor itself			*/
/*		show_string()	Places string in window			*/
/*		show_char()	Places character in window		*/
/*		pick_choice()	Pick from a list using space bar	*/
/*									*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Tony Johnson & Theo ten Brummelaar                          */
/* Date   : Original Version 1990 - ported to Linux 1998                */
/************************************************************************/

/*
 * include files
 */

#include "charaui.h"

/*
 * defines
 */

#define EMPTY_SPACE	'_'

#define	string_insert()	    {for (i=length-1; i>cursor-1; i--) \
					string[i] = string[i-1];}

#define string_delete()	    {for (i=cursor; i<length; i++) \
					string[i] = string[i+1];}

#define cursor_right()	    {if (++cursor>=length || \
				string[cursor-1]==(char)NULL ) cursor--;}

#define cursor_left()	    {if (--cursor<0) cursor++;}

#define exit_line_edit(arg)  {  if ((*value_check)(string)) \
			     {  \
				show_string(win,y,x,string,length,-1,' '); \
				werase(mode_window); \
				wrefresh(mode_window); \
				return arg; \
			     }\
			     else continue;}
/*
 * A Local global
 */

static bool	insert = TRUE;		/* Is insert mode on ? */

/************************************************************************/
/* line_edit()								*/
/* Process 4.3								*/
/*									*/
/* This function provides a line editor. It mimicks both normal		*/
/* MS-DOS type editing as well as UNIX ed(1) type editing.		*/
/* That is, it is a mode type editor. It goes straight			*/
/* into insert mode.  To get out					*/
/* of insert mode type ESC. Then the normal movement keys 'h','j','k'	*/
/* and 'l' will work. BACKSPACE is the normal erase key. To return	*/
/* to insert mode type 'i' or the INSERT key (if it exists).		*/
/* When not in insert mode the keys w,b,x,$ and ^ work as in vi.	*/
/* The editor will try and exit if any of the following keys are typed: */
/* ENTER, KEY_DOWN, KEY_UP, KEY_PPAGE, KEY_NPAGE, 			*/
/* or 'j','k','^U','^D','^F',ESC or '^B' when not in insert mode.	*/
/* The function pointed at by (*value_check)() is then called		*/
/* to test the value of the string. This function must be supplied	*/
/* by the calling programme. No exit if this function returns FALSE.	*/
/* Note that (*value_check)() is not called when exiting by KEY_ESC.	*/
/*									*/
/* The return value is the key used to exit.				*/
/*									*/
/* Look at the source for a clear idea of what happens.			*/
/************************************************************************/

int	line_edit(
WINDOW	*win,			/* Window that string lives in */
int	y, int x,		/* Position in window to place string */
char	*string,		/* The string to edit */
int	length,			/* The length of the string including NULL */
int	value_type,		/* Type of value string will hold ie :
				 * INTEGER, FLOAT or STRING */
bool	(*value_check)(),	/* Function to test value of string */
bool	insert_on)		/* TRUE if we are to go straight into insert
			         * mode, FALSE if we are to go back to what ever
				 * mode we were in before.
				 */
{
	/* Variables */

	int	i;		/* Used for looping */
	int	cursor;		/* Position of cursor */
	int	c; 		/* Last character typed */
	char	old_string[81];	/* Origional string returned by ESC */
#ifdef __NCURSES_H
        MEVENT  mouse;                  /* A mouse event. */
	int	line_y,line_start_x,line_end_x; /* Position of line on screen */

	/* Work out where on screen the line is. */

	line_y = win->_begy + y;
	line_start_x = win->_begx + x;
	line_end_x = line_start_x + length-1;
#endif

	/* If this is a socket, just go ahead and scan something in */

	if (active_socket != -1)
	{
		socket_scan(active_socket, "%s", string);
		if (*string == KEY_ESC) return KEY_ESC;
		return KEY_DOWN;
	}

	/* Initialisation */

	if (insert_on) insert = TRUE;

	if (insert)
	{
		werase(mode_window);
		mvwaddstr(mode_window,0,0,"  INSERT");
		touchwin(mode_window);
		wrefresh(mode_window);
	}

	/* Set the initial cursor position at end of string or beginning */

	for( cursor = 0; string[cursor] != (char)NULL && cursor <= length; 
		cursor++);
	if (cursor == length) cursor = 0;

	/* Force a NULL at end of string */

	string[length] = (char)NULL;

	/* Make a copy of the origional string */

	strcpy(old_string,string);

	/* Do it .... have a look, it may be obvious how it works */

	while(TRUE)
	{
		show_string(win,y,x,string,length,cursor,EMPTY_SPACE);

		c = get_command();

#ifdef __NCURSES_H
                if (c == KEY_MOUSE)
                {
                        getmouse(&mouse);
                        if (mouse.bstate == BUTTON1_CLICKED)
                        {
				/* Work out where it was */

				if (mouse.y == line_y &&
				    mouse.x >= line_start_x &&
				    mouse.x <= line_end_x)
				{
					cursor = mouse.x - line_start_x;
		
					if (cursor>=strlen(string))
					{
						while (string[cursor] != 0) 
						{
							cursor--;
						}
					}
				}
			}
		}
#endif

		if (c == KEY_LEFT || (!insert && c == 'h') )
		{
			cursor_left();
			continue;
		}

		if (c == KEY_RIGHT || (!insert && c == 'l') )
		{
			cursor_right();
			continue;
		}

		/* Forward one word */

		if (!insert && c =='w')
		{
			/* Find the next white space */

			while((c = string[cursor]) != ' ' && c != (char)NULL) 
				cursor_right();

			/* Find the start of the next word */

			while((c = string[cursor]) == ' ' )
				cursor_right();

			continue;
		}

		/* Backward one word */

		if (!insert && c =='b')
		{
			if (cursor == 0) continue;

			cursor_left();

			while( cursor > 0 && 
			    !(string[cursor] != ' ' && string[cursor-1] == ' '))
			{
				cursor_left();
			}

			continue;
		}

		/* Leave editor */

		if (c == KEY_UP || c == KEY_DOWN || c == KEY_PPAGE 
			|| c == KEY_NPAGE || c == '\r' 
		)
		{
			exit_line_edit(c);
		}

		/* Go to start of string */

		if ( (insert && c == KEY_HOME) || (!insert && c == '^') )
		{
			cursor = 0;
			continue;
		}

		/* To end of string */

		if ( (insert && c == KEY_LL) || (!insert && c == '$') ) 
		{
			for (cursor = 0; *(string+cursor) != (char)NULL; 
				cursor++);
			continue;
		}

		/* Erase a char by backspace (to left of cursor) */
	
		if ( c == '\b' || c == KEY_BACKSPACE)
		{
			if (cursor == 0) continue;
			cursor_left();
			string_delete();
			continue;
		}

		/* Erase char cursor is on */

		if ( c == KEY_DC || (!insert && c == 'x') )
		{
			if (*(string+cursor) == (char)NULL ) continue;
			string_delete();
			continue;
		}

		if (!insert)
		{
			if ( c == 'j')
			{
				exit_line_edit(KEY_DOWN);
			}

			if (c == 'k')
			{
				exit_line_edit(KEY_UP);
			}

			if (c == KEY_ESC)
			{
				/* Exit without test */

				strcpy(string,old_string);
				show_string(win,y,x,string,length,-1,' ');
				return c;
			}

			if (c == 'i'|| c == KEY_IC)
			{
				insert = TRUE;
				werase(mode_window);
				mvwaddstr(mode_window,0,0,"  INSERT");
				touchwin(mode_window);
				wrefresh(mode_window);
				continue;
			}

			if (c == KEY_HOME)
			{
				exit_line_edit(KEY_HOME);
			}

			if (c == KEY_LL)
			{
				exit_line_edit(KEY_LL);
			}
		}
		else
		{
			if (c == KEY_ESC) 
			{
				insert = FALSE;
				werase(mode_window);
				touchwin(mode_window);
				wrefresh(mode_window);
				continue;
			}

			/* Check the type of character */

			switch(value_type)
			{
				case INTEGER : if (!isdigit(c)
							&& c != '+'
							&& c != '-') continue;
					       break;

			 	case FLOAT   : if (!isdigit(c) 
							&& c != '.' 
							&& c != 'e' 
							&& c != 'E'
							&& c != '+'
							&& c != '-') continue;
					       break;

				default      : if (!isalpha(c)
							&& !isdigit(c)
							&& c != '.'
							&& c != ','
							&& c != '+'
							&& c != '-'
							&& c != '/'
							&& c != '\\'
							&& c != ':'
							&& c != '_'
							&& c != '|'
							&& c != '>'
							&& c != '<'
							&& c != '!'
							&& c != '*'
							&& c != '?'
							&& c != ' ') continue;
					       break;
			}

			string_insert();
			string[cursor] = c;
			cursor_right();
		}
	}

} /* line_edit() [4.3] */

/************************************************************************/
/* show_string()							*/
/*									*/
/* Displays a given string in a given position in a given window.	*/
/* Blank space is made to look like the character blank and the		*/
/* cursor position is marked by the hardware cursor.			*/
/*									*/
/************************************************************************/

void 	show_string(
WINDOW *win,		/* Window to use */
int	y, int x,	/* Position in window to use */
char	*string,	/* The string to show */
int	length,		/* The length of the string */
int	cursor,		/* The cursor position */
char	blank)		/* The blank character to use */
{
	/* Variables */

	int 	position;	/* Position in string */

	/* Loop through the entire string */
	
	wmove(win,y,x);

	for(position = 0; position < length; position++)
	{
		if (position == cursor) wstandout(win);

		switch(*string)
		{
			case 0    : waddch(win,blank);
				    wstandend(win);
				    for (position++; position < length-1; 
								  position++)
					    waddch(win,blank);
				    break;

			case ' '  : waddch(win,'_');
				    break;

			default   : waddch(win,*string);
				    break;
		}

	        wstandend(win);
		string++;
	}

	wrefresh(win);

} /* show_string() */

/************************************************************************/
/* show_char()								*/
/*									*/
/* Place a character on the screen in a given window. If it is at	*/
/* the current cursor position highlight it.				*/
/* Not used at all anymore. We are now using the hardware cursor.	*/
/*									*/
/************************************************************************/

void 	show_char(
WINDOW	*win,		/* Window to use */
int	position,	/* Current position in string */
int	cursor,		/* Current cursor position */
int	c)		/* Character to show */
{
	if (position == cursor) wstandout(win);
	waddch(win,c);
	if (position == cursor) wstandend(win);

} /* show_char() */

/************************************************************************/
/* pick_choice()							*/
/* Process 6.6								*/
/*									*/
/* This function is used to select values for enumerated types. Given	*/
/* an array of string pointers, a window and a position, this will	*/
/* allow the user to keep pressing the space-bar key until the desired 	*/
/* choice is listed and then select this choice with the following	*/
/* keys:								*/
/*									*/
/*	KEY_ESC,							*/
/*	KEY_UP, KEY_DOWN, KEY_PPAGE, KEY_NPAGE, 'j' or 'k'		*/
/*	^F or ^B.							*/
/*									*/
/* The user can list the next choice by pressing l, KEY_RIGHT, ^L	*/
/* or the space bar.							*/
/* The previous choice may be listed by pressing h, KEY_LEFT, 		*/
/*	or ^K.								*/
/* This function does not require all the strings to be blank-padded	*/
/* to the same length.							*/
/* Before this function returns it will call the check_value()		*/
/* function except when exiting via KEY_ESC. 				*/
/* No exit if this function returns FALSE.				*/
/* The value returned is the key used to exit.				*/
/*									*/
/************************************************************************/

int 	pick_choice(

WINDOW 	*win,			/* Window that value lives in */
int	y, int x,		/* Position in window to place value */
int	*value,			/* Variable to be set and default */
char	*strings[],		/* The array of strings to show */
bool	(*value_check)())	/* Function to test value */
{
	/* Internal variables */

	int	current_value;	/* the current value */
	int	old_string_len;	/* number of bytes in old string */
	int	c;		/* character typed by user */
	int	ic;		/* character index */
	int	max_value;		/* the maximum value */
	int	i;
#ifdef __NCURSES_H
        MEVENT  mouse;                  /* A mouse event. */
#endif
	char	s[81];

	/* Set maximum value */

	for
	(	
		max_value = 0; 
		strings[max_value] != NULL; 
		max_value++
	);
	max_value--; /* If the default exists, max_value >= 0 */

	/* If this is a socket do the obvious thing */

	if (active_socket != -1)
	{
		do
		{
			socket_print(active_socket,"Choices are:\n");
			for(i=0; i<= max_value; i++) 
			    socket_print(active_socket,
			    "%2d\t%s\n",i,strings[i]);
			socket_gets(active_socket,s,81);
			if (*s == KEY_ESC) return KEY_ESC;
			sscanf(s,"%d",value);

		} while(*value<0 || *value>max_value);
		socket_print(active_socket,"Choice was %s\n",strings[*value]);
		return KEY_DOWN;
	}

	/* Set default */

	for(current_value = 0; 
		strings[current_value] != NULL && current_value != *value; 
			current_value++);

	/* Check that default exists */

	if(strings[current_value] == NULL)
	{
		error(WARNING,WARN_16);
		current_value = 0;
	}

	old_string_len = 0;

	/* Start main loop */

	while(TRUE)
	{
		/* Erase old string */

		wmove(win, y, x);
		for(ic = 0; ic < old_string_len; ic ++)
		{
			waddch(win, ' ');
		}
		old_string_len = strlen(strings[current_value]);

		/* Put up current string */

		mvwaddstr(win,y,x,strings[current_value]);
		wrefresh(win);
		
		/* Respond to key stroke */

		c = get_command();
#ifdef __NCURSES_H
                if (c == KEY_MOUSE)
                {
                        getmouse(&mouse);

                        if (mouse.bstate == BUTTON1_CLICKED)
                        {
                                /* OK, what key did the mouse represent. */

                                if (mouse.y == SYSTEM_BEGIN_Y+1 &&
                                        mouse.x >= 20 && mouse.x <=29)
                                {
                                        c = ' ';
                                }
                                else if (mouse.y == SYSTEM_BEGIN_Y+1 &&
                                        mouse.x >= 39 && mouse.x <=43)
                                {
                                        c = '\r';
                                }
                        }
                }
#endif
		switch(c)
		{
			/* Next value */

			case KEY_RIGHT :
			case 'l' :
			case ' ' :	if(++current_value > max_value)
						current_value = 0;
					break;

			/* Previous value */

			case KEY_LEFT :
			case 'h' :	if(--current_value < 0)
						current_value = max_value;
					break;

			/* Any of the following will cause exit */

			case KEY_UP :
			case KEY_DOWN :
			case KEY_PPAGE :
			case KEY_NPAGE :
			case KEY_HOME :
			case KEY_LL :
			case '$' :
			case '^' :
			case '\r' :
			case 0x06 :	/* ^F forward page */
			case 0x02 :	/* ^B back page */
			case 'j' :
			case 'k' :	if(c == 'j') c = KEY_DOWN;
					if(c == 'k') c = KEY_UP;
					if(c == 0x06) c = KEY_NPAGE;
					if(c == 0x02) c = KEY_PPAGE;
					if(c == '$') c = KEY_LL;
					if(c == '^') c = KEY_HOME;

					/* Test the value */

					if((*value_check)(current_value))
					{
						*value = current_value;
						return c;
					}
					else 
						break;

			case KEY_ESC : /* Exit without test or change in value*/

					return c;

			default :	break;
		}
	}

} /* pick_choice() */
