/************************************************************************/
/* save_tv.c		                                                */
/*                                                                      */
/* Routine to save a movie file in the same format as the 		*/
/* wobble_detect program. Should be able to do 30 frames/sec.		*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE                        */
/*                 Based on the SUSI User Interface                     */
/*              In turn based on the CHIP User interface                */
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA                 */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu                                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : August 1999                                                 */
/************************************************************************/

#include <stdio.h>
#if !(defined(macintosh) || (defined(__MACH__) && defined(__APPLE__)))
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "../meteor/matrox_meteor.h"
#include "../nsimpleX/nsimpleX.h"
#include <sys/mman.h>
#include "../meteor/ioctl_meteor.h"
#include <unistd.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <time.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/types.h>
#include <asm/errno.h>
#include <errno.h>

int errno;

void local_handler(unsigned char *data, int col, int row);
void stopit(int sig);
static time_t start_time;
static unsigned char *buffer;
static int output;
static	int size;
static int frames;
#endif

int main(int argc, char **argv)
{
#if defined(macintosh) || (defined(__MACH__) && defined(__APPLE__))
	printf("Not meteor driver on MAC\n");
#else
	int  c;
	int	brightness;
	int 	contrast;
	int	chan;
	int	col,row;
	char	*filename;

	if (argc < 7)
	{
		fprintf(stderr,
		"usage: %s chan col row contrast brightness file\n",argv[0]);
		exit(-2);
	}
	sscanf(argv[1],"%d",&chan);
	sscanf(argv[2],"%d",&col);
	sscanf(argv[3],"%d",&row);
	sscanf(argv[4],"%d",&contrast);
	sscanf(argv[5],"%d",&brightness);
	filename = argv[6];
	frames = 0;

	if (chan < 1 || chan > 4)
	{
		fprintf(stderr,"Chan must be 1,2,3 or 4.\n");
		exit(-1);
	}

	/* First make sure the file does not exist */

	if (open(filename,O_RDONLY) != -1)
	{
		close(output);
		fprintf(stderr,"File %s already exists.\n",filename);
		exit(-2);
	}

	/* Open the file */

	if ((output = 
		open(filename,O_WRONLY | O_CREAT,S_IWUSR|S_IRUSR| S_IRGRP))==-1)
	{
		fprintf(stderr,"Failed to open file %s.",filename);
		exit(-3);
	}

	/* Save header */

	if (write(output,&col,sizeof(col))!=sizeof(col))
	{
		fprintf(stderr, "Failed to write col.");
		exit(-3);
	}
	if (write(output,&row,sizeof(row))!=sizeof(row))
	{
		fprintf(stderr, "Failed to write col.");
		exit(-3);
	}

	/* Start setup */

	size = col*row;
	if ((buffer = malloc(size)) == NULL)
	{
		fprintf(stderr,"Ran out of memory\n");
		exit(3);
	}

	if ((c = open_meteor_signal(NULL, col, row, local_handler)) != 0)
	{
		fprintf(stderr,"Failed opening device: %d %d\n",c,errno);
		exit(-1);
	}

	if ((c = set_meteor_chan(chan)) != 0)
	{
		fprintf(stderr,"Failed setting channel: %d %d\n",c,errno);
		exit(-1);
	}
	
	if ((c = set_meteor_brightness(brightness)) != 0)
	{
		fprintf(stderr,"Failed setting brightness: %d %d\n",c,errno);
		exit(-1);
	}
	
	if ((c = set_meteor_contrast(contrast)) != 0)
	{
		fprintf(stderr,"Failed setting contrast: %d %d\n",c,errno);
		exit(-1);
	}
	
	/* Make sure stdin is non-blocking */

/*
	if (fcntl(fileno(stdin), F_SETFL, O_NONBLOCK) == -1)
	{
		fprintf(stderr,"failed to set non-blocking stdin.\n");
		exit(-5);
	}
*/

	signal(SIGINT, stopit);
	time(&start_time);
	while(1)
	{
		if (getchar() == 'q') break;
	}
	stopit(1);

	exit(0);
}

void local_handler(unsigned char *data, int col, int row)
{
	frames++;
	if (write(output, data, size) != size)
	{
		fprintf(stderr,"Failed to write frame %d.",frames);
		stopit(1);
	}
	printf("%d\r",frames);
	fflush(stdout);

	/* That is all */

	return;
}

void stopit(int sig)
{
	time_t stop_time;
	float	time_period;

	time(&stop_time);
	time_period = stop_time - start_time;
	printf("Frames processed per second = %f\n",
		(float)get_signal_number_of_frames()/time_period);
	fflush(stdout);

	close_meteor_signal();

	close(output);
#endif

	exit(0);
}
