#include <stdio.h>
#if !(defined(macintosh) || (defined(__MACH__) && defined(__APPLE__)))
#include "../meteor/matrox_meteor.h"
#include "../nsimpleX/nsimpleX.h"
#include <string.h>
#include <sys/mman.h>
#include "../meteor/ioctl_meteor.h"
#include <unistd.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <time.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/types.h>
#include <asm/errno.h>
#include <errno.h>

int errno;

static int lock = 0;
void local_handler(unsigned char *data, int col, int row);
void stop(int sig);
static time_t start_time;
static int	frames;
static Window theWindow;
static unsigned char *buffer;
static char displayed = 0;
#endif

int main(int argc, char **argv)
{
#if defined(macintosh) || (defined(__MACH__) && defined(__APPLE__))
	printf("No metoer driver on a MAC\n");
#else
	int  c;
	XImage *theImage;
	unsigned char *picture,*p8;
	unsigned short int *p16;
	unsigned int *p32;
	int size;
	unsigned char  *mbuf;
	int 	i,j,color;
	int	brightness;
	int 	contrast;
	int	chan;
	int	col,row;

	if (argc < 6)
	{
		fprintf(stderr,
		"usage: %s chan col row contrast brightness\n",argv[0]);
		exit(-2);
	}
	sscanf(argv[1],"%d",&chan);
	sscanf(argv[2],"%d",&col);
	sscanf(argv[3],"%d",&row);
	sscanf(argv[4],"%d",&contrast);
	sscanf(argv[5],"%d",&brightness);
	size = col*row;
	if ((buffer = malloc(size)) == NULL)
	{
		fprintf(stderr,"Ran out of memory\n");
		exit(3);
	}

	if ((c = open_meteor_signal(NULL, col, row, local_handler)) != 0)
	{
		fprintf(stderr,"Failed opening device: %d %s\n",c,
			strerror(errno));
		exit(-1);
	}

	if ((c = set_meteor_chan(chan)) != 0)
	{
		fprintf(stderr,"Failed setting channel: %d %s\n",c,
			strerror(errno));
		exit(-1);
	}
	
	if ((c = set_meteor_brightness(brightness)) != 0)
	{
		fprintf(stderr,"Failed setting brightness: %d %s\n",c,
			strerror(errno));
		exit(-1);
	}
	
	if ((c = set_meteor_contrast(contrast)) != 0)
	{
		fprintf(stderr,"Failed setting contrast: %d %s\n",c,
			strerror(errno));
		exit(-1);
	}
	
	initX(NULL);
	theWindow = openWindow("CAM", 5, 5, col, row);

	/* Make sure stdin is non-blocking */

	if (fcntl(fileno(stdin), F_SETFL, O_NONBLOCK) == -1)
	{
		fprintf(stderr,"failed to set non-blocking stdin.\n");
		exit(-5);
	}

	signal(SIGINT, stop);
	frames = 0;
	time(&start_time);
	while(1)
	{
		/* Do nothing when locked */

		if (lock) continue;
		if (displayed) continue;

		/* Allocate the memory */

		if ((picture = malloc(size*theBitmapPad)) == NULL)
		{
			fprintf(stderr,"Ran out of memory\n");
			exit(3);
		}

		p8 = picture;
		p16 = (unsigned short int *)picture;
		p32 = (unsigned int *)picture;
		mbuf = buffer;
		lock = 1;
		for(j=0; j< row; j++)
		for(i=0; i< col; i++)
		{
			color = (*mbuf++)/2;

			if (color < 0) 
				color = 0;
			else if (color > NUM_COLORS-1)
				color = NUM_COLORS-1;

			switch(theBitmapPad)
			{
				case 1: *p8++ = GreyScale[color];
					break;
				case 2: *p16++ = GreyScale[color];
					break;
				default: *p32++ = GreyScale[color];
					break;
			}
		}
		lock = 0;
		
		theImage = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
                        (char *)picture,col,row, theDepth ,col*theDepthBytes);

		XPutImage(theDisplay,theWindow,theGC,theImage,0,0,0,0,col,row);

		XMapWindow(theDisplay, theWindow);

		XFlush(theDisplay);

		XDestroyImage(theImage);
		
		frames++;
		displayed = 1;

		if (getchar() == 'q') break;
	}

	exit(0);
}

void local_handler(unsigned char *data, int col, int row)
{
	int	i,j;
	unsigned char *p;

	/* Do nothing if things are locked */

	if (lock) return;

	/* Otherwise we lock it up */

	lock = 1;

	/* And copy things across */

	p = buffer;
	for (j=0;j<row;j++)
	for (i=0;i<col;i++)
	{
		*p++ = *data++;
	}

	/* Unlock */

	lock = 0;
	displayed = 0;

	/* That is all */

	return;
}

void stop(int sig)
{
	time_t stop_time;
	float	time_period;

	time(&stop_time);

	time_period = stop_time - start_time;
	printf("Frames displayed per second = %f\n",(float)frames/time_period);
	printf("Frames processed per second = %f\n",
		(float)get_signal_number_of_frames()/time_period);

	close_meteor_signal();
#endif

	exit(0);
}
