/************************************************************************/
/* pistonerr.c 								*/
/*									*/
/* Spit out example piston error vectors.				*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : April 2005   						*/
/************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include "../nrc/nrc.h"
#include "../interferometry/interferometry.h"

int main(int argc, char **argv)
{
	int	i;
	float	*pos;
	float 	r0, v, dt;
	int	n;

	if (argc != 5)
	{
		fprintf(stderr,"Usage: %s r0(cm) v(m/s) dt(s) n\n", argv[0]);
		exit(-1);
	}

	sscanf(argv[1], "%f", &r0);
	r0 /= 100.0;
	sscanf(argv[2], "%f", &v);
	sscanf(argv[3], "%f", &dt);
	sscanf(argv[4], "%d", &n);

	pos = piston_error(r0, v, dt, n);

	for(i=1; i<=n ;i++) printf("%.4f\n", pos[i]);

	free_vector(pos,1,n);

	exit(0);

}
