/************************************************************************/
/* testuipop.c								*/
/*                                                                      */
/* Test program for POP libraries.					*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar    		                        */
/* Date: May 2007							*/
/************************************************************************/

#include "../charaui/src/charaui.h"
#include "../cliserv/include/cliserv.h"
#include "../include/chara_messages.h"
#include "../charalib/chara.h"
#include "../pop_client/popclient.h"
#include "../pop_client/popuiclient.h"


int time_status(void);		/* Example background job below */
int pop_background(void);		/* Background job tests data socket */
void pop_open(void);
void pop_close(void);

int main(int argc, char **argv)
{
	/* Title page. */

	ui_clear_screen();
	put_line("");
	center_line("CHARA POP INTERFACE TEST ROUTINE");
	put_line("");
	center_line("The CHARA Array");
	center_line("Center for High Angular Resolution Astronomy");
	center_line("Mount Wilson Observatory, CA 91001, USA");
	put_line("");
	center_line("Telephone: 1-626-796-5405");
	center_line("Fax: 1-626-796-6717");
	center_line("email: theo@chara.gsu.edu");
	center_line("WWW: http://www.chara.gsu.edu");
	put_line("");
	center_line("(C) This executable program is copyright.");
	wait_for_title();

	/* Initialize the user interface */

	TITLE = "POP VERSION 0.0";
	initialise_ui("pop_menus.ini","pop_help.ini","testuipop");
	set_user_open_function(pop_open);
	set_user_close_function(pop_close);

	/* Setup background job(s) */

	background_add(time_status);
	background_add(pop_background);

	/* Let's go! */

	start_ui(); /* Should never return from here. */

	exit(0);
}

/************************************************************************/
/* pop_open()								*/
/************************************************************************/

void pop_open(void)
{
	/* Try and open an POP unit */

	message(system_window,"Opening socket to POP unit.");
	if (call_open_pop_client(0, NULL) != NOERROR)
	{
		error(FATAL,"Failed to open POP server.\n");
	}
	werase(system_window);
	wrefresh(system_window);

	/* Start the background */

        background_start(0, NULL);

} /* pop_open() */

/************************************************************************/
/* pop_close()								*/
/************************************************************************/

void pop_close(void)
{
	close_pop_client();

} /* pop_open() */

/************************************************************************/
/* time_status()							*/
/* 									*/
/* Displays the time in the status window.				*/
/************************************************************************/

int time_status(void)
{
	time_t current_time;
	struct tm *now;

	time(&current_time);
	now = localtime(&current_time);
	wstandout(status_window);
	mvwaddstr(status_window,0,0,"Time : ");
	wstandend(status_window);
	wprintw(status_window,"%2d:%02d:%02d",
		now->tm_hour,now->tm_min,now->tm_sec);

	return NOERROR;

} /* time_status() */

/************************************************************************/
/* pop_background()							*/
/*									*/
/* Tries to display status information.					*/
/************************************************************************/

int pop_background(void)
{
	int     id;
        int     scope,pop;
        int     row,col;
        char    *argl[2];
        char    arg1[80];

	process_server_socket(pop_server);

	/* Are we even open yet? */

	if (all_pop_types[0] == 0 || all_pop_types[0] == NULL) return NOERROR;

	/* Setup the display */

        wstandout(status_window);
        mvwprintw(status_window,1,16,"POP1 LED");
        mvwprintw(status_window,1,32,"POP2 LED");
        mvwprintw(status_window,1,48,"POP3 LED");
        mvwprintw(status_window,1,64,"POP4 LED");
        for(scope=0; scope<NUM_SCOPES; scope++)
		mvwprintw(status_window,scope+2,0,"%s",scope_types[scope]);
        wstandend(status_window);

	/* OK loop through all possibilities. */

        for(scope=0; scope<NUM_SCOPES; scope++)
        for(pop=1; pop<NUM_POPS; pop++)
        {
        	/* Which ID is this? */

        	sprintf(arg1,"%s-%1d", scope_types[scope], pop);
        	argl[0] = "popbg";
        	argl[1] = arg1;
         	if ((id = pop_get_id(2,argl)) < 0)
         	{
                	return error(ERROR,
				"Failed to work out ID for %s (%d %d)",
				arg1, scope, pop);
                }

                col = pop * 16;
                row = scope+2;

		switch(pops[id].status_limit)
		{
			case STATUS_LIMIT_NONE:
				mvwprintw(status_window,row,col, "NONE");
				break;

			case STATUS_LIMIT_FWD:
				mvwprintw(status_window,row,col, "  IN");
				break;

			case STATUS_LIMIT_REV:
				mvwprintw(status_window,row,col, " OUT");
				break;

			case STATUS_LIMIT_BOTH:
				mvwprintw(status_window,row,col, "BOTH");
				break;

			default:
				mvwprintw(status_window,row,col, "????");
				break;
		}

		switch(pops[id].status_led)
		{
			case STATUS_LED_OFF:
				mvwprintw(status_window,row,col+5, "OFF");
				break;
			case STATUS_LED_GREEN:
				mvwprintw(status_window,row,col+5, " ON");
				break;
			default:
				mvwprintw(status_window,row,col+5, "???");
				break;
		}

	}
	
        return NOERROR;

} /* pop_background() */
