/************************************************************************/
/* readoif.c								*/
/*									*/
/* Reads and spits out the data in an OIFITS file.			*/
/*									*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*									*/
/* Author: Theo								*/
/* Date: September 2009							*/
/*									*/
/************************************************************************/

#include "../slalib/slalib.h"
#include "../oifitslib/exchange.h"
#include "../charalib/chara.h"
#include "../charaoif/charaoif.h"
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

int main(int argc, char **argv)
{
	oi_array array;
	oi_wavelength wave;
	oi_target targets;
	oi_vis vis;
	oi_vis2 vis2;
	oi_t3 t3;
	fitsfile *fptr;
	fitsfile *fptr2;
	int	status, hdutype;

	/* Check the command line */

	if (argc != 2)
	{
		fprintf(stderr,"usage: %s filename\n", argv[0]);
		exit(-1);
	}

	/* Try and read the file */

	status = 0;
	fits_open_file(&fptr, argv[1], READONLY, &status);
	if (status)
	{
		fprintf(stderr, "Failed to open file %s\n", argv[1]);
		fits_report_error(stderr, status);
		exit(-2);
	}

	/* Read in the target array */

	read_oi_target(fptr, &targets, &status);
	print_oi_target(stdout, targets);

	/* Open a second one so we don't loose our place */

	fits_open_file(&fptr2, argv[1], READONLY, &status);

	/* Read in all VIS tables */

	fits_movabs_hdu(fptr, 1, &hdutype, &status);
	while(!read_next_oi_vis(fptr, &vis, &status))
	{
		if (strlen(vis.arrname) > 0)
		{
			read_oi_array(fptr2, vis.arrname, &array, &status);
		}
		read_oi_wavelength(fptr2, vis.insname, &wave, &status);

		/* Print this stuff out */

		print_oi_vis(stdout, vis);
		print_oi_array(stdout, array);
		print_oi_wavelength(stdout, wave);

		/* Clean up */

		if (!status)
		{
			clear_oi_wavelength(wave);
			if (strlen(vis.arrname) > 0)
				clear_oi_array(array);
			clear_oi_vis(vis);
		}
	}
	if (status!=END_OF_FILE)
	{
		fprintf(stderr,"Failed to reach end of file on VIS tables.\n");
		exit(-2);
	}

	/* Go back to start of file */

	status = 0;
	fits_movabs_hdu(fptr, 1, &hdutype, &status);
	fits_movabs_hdu(fptr2, 1, &hdutype, &status);
	
	/* Read in all VIS2 tables */

	while(!read_next_oi_vis2(fptr, &vis2, &status))
	{
		if (strlen(vis2.arrname) > 0)
		{
			read_oi_array(fptr2, vis2.arrname, &array, &status);
		}
		read_oi_wavelength(fptr2, vis2.insname, &wave, &status);

		/* Print this stuff out */

		print_oi_vis2(stdout, vis2);
		print_oi_array(stdout, array);
		print_oi_wavelength(stdout, wave);

		/* Clean up */

		if (!status)
		{
			clear_oi_wavelength(wave);
			if (strlen(vis.arrname) > 0)
				clear_oi_array(array);
			clear_oi_vis2(vis2);
		}
	}
	if (status!=END_OF_FILE)
	{
		fprintf(stderr,"Failed to reach end of file on VIS2 tables.\n");
		exit(-2);
	}

	/* Go back to start of file */

	status = 0;
	fits_movabs_hdu(fptr, 1, &hdutype, &status);
	fits_movabs_hdu(fptr2, 1, &hdutype, &status);
	
	/* Read in all T3 tables */

	while(!read_next_oi_t3(fptr, &t3, &status))
	{
		if (strlen(t3.arrname) > 0)
		{
			read_oi_array(fptr2, t3.arrname, &array, &status);
		}
		read_oi_wavelength(fptr2, t3.insname, &wave, &status);

		/* Print this stuff out */

		print_oi_t3(stdout, t3);
		print_oi_array(stdout, array);
		print_oi_wavelength(stdout, wave);

		/* Clean up */

		if (!status)
		{
			clear_oi_wavelength(wave);
			if (strlen(vis.arrname) > 0)
				clear_oi_array(array);
			clear_oi_t3(t3);
		}
	}
	if (status!=END_OF_FILE)
	{
		fprintf(stderr,"Failed to reach end of file on T3 tables.\n");
		exit(-2);
	}

	/* That should be all */

	clear_oi_target(targets);
	fits_close_file(fptr, &status);
	fits_close_file(fptr2, &status);

	exit(0);
}
