/************************************************************************/
/* server.c								*/
/*                                                                      */
/* Example server program.						*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY SERVER LIB   				*/
/*                 Based on the CHARA User Interface			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar 		                        */
/* Date   : Oct 2001							*/
/************************************************************************/

#define SERVER

#include "../cliserv/include/cliserv.h"
#include "../include/chara_messages.h"

#include <time.h>

void close_function(void);
int  top_job_0(void);
int  top_job_1(void);
int  top_job_2(void);
int  background_job_1(void);
int  background_job_2(void);

int main(int argc, char **argv)
{
#ifdef DEFINE_PORT
	int	port;

	/* Check the command line */

	if (argc !=2)
	{
		fprintf(stderr,"usage: %s port\n",argv[0]);
		exit(-1);
	}
	sscanf(argv[1],"%d",&port);

	/* Open port */

	if (open_message_socket(port) != NOERROR)
	{
		error(FATAL,"Failed to connect to port %d.\n",port);
	}
#else
	char	s[256];

	if (argc !=2)
	{
		fprintf(stderr,"usage: %s name\n",argv[0]);
		exit(-1);
	}
	sprintf(s, "/chara/theo/chara/control/libs/cliserv/server/server %s",
			argv[1]);
	if (auto_open_message_socket(argv[1],s) != NOERROR)
	{
		error(FATAL,"Failed to connect to port.\n");
	}
#endif
	
	/* Initialise the jobs */

	server_init_jobs();

	/* Setup close function */

	setup_close_down(close_function);

	/* Add any local jobs */

	add_background_job(background_job_1);
	add_background_job(background_job_2);
	add_top_job(top_job_0);
	add_top_job(top_job_1);
	add_top_job(top_job_2);

	/* Set the name */

	set_server_name("Example Server");

	/* 
	 * Had over control 
	 * Alternatively create your own loop, but make sure you call
	 * process_message_socket()
	 * Regularlary. Each call will process a single message 
	 * from a single client.
	 */

	process_jobs();

	/* Should never reach here */

	exit(0);

} /* main() */

/************************************************************************/
/* close_function()							*/
/*									*/
/* Final closing down function.						*/
/************************************************************************/

void close_function(void)
{
	error(MESSAGE,"Goodbye cruel world.");

} /* close_function() */

/************************************************************************/
/* background_job_1()							*/
/************************************************************************/

int  background_job_1(void)
{
	static time_t	last=0;
	time_t		now;
	static int	count = 0;

	count++;
	time(&now);
	if (now - last >= 5)
	{
		last = now;
		error(MESSAGE,"Background 1 %d.",count);
	}

	return NOERROR;
}

/************************************************************************/
/* background_job_2()							*/
/************************************************************************/

int  background_job_2(void)
{
	static time_t	last=0;
	time_t		now;
	static int	count = 0;

	count++;
	time(&now);
	if (now - last >= 10)
	{
		last = now;
		error(MESSAGE,"Background 2 %d.",count);
	}

	return NOERROR;
}

/************************************************************************/
/* top_job_0()							*/
/************************************************************************/

int	top_job_0(void)
{
	int i;

	for(i=0; i<=255; i++)
        {
                if (client_signal_set(i))
                {
                        error(MESSAGE,"Got UI_SIGNAL %d",i);
                        clear_client_signal(i);
                }
	}

	return NOERROR;

} /* top_job_0() */

/************************************************************************/
/* top_job_1()							*/
/************************************************************************/

int  top_job_1(void)
{
	static time_t	last=0;
	time_t		now;
	static int	count = 0;

	count++;
	time(&now);
	if (now - last >= 15)
	{
		last = now;
		error(MESSAGE,"Top 1 %d.",count);
	}

	return NOERROR;
}

/************************************************************************/
/* top_job_2()								*/
/************************************************************************/

int  top_job_2(void)
{
	static time_t	last=0;
	time_t		now;
	static int	count = 0;
	
	count++;
	time(&now);
	if (now - last >= 20)
	{
		last = now;
		error(MESSAGE,"Top 2 %d.",count);
	}

	return NOERROR;
}
