/************************************************************************/
/* status_mirror.c							*/
/*                                                                      */
/* Program that listens to a socket and waits for text to put into a 	*/
/* window. Each line consists of three two. The first two characters    */
/* give a line number, the rest is the string to put in that line.	*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar 		                        */
/* Date   : March 1999							*/
/************************************************************************/

/*
 * Include files
 */

#define FUNCTS

#include "../charaui/src/charaui.h"
#include <netdb.h>
#include <resolv.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>

int main(int argc, char **argv)
{
	WINDOW *status_window;
        struct sockaddr_in server; /* Server address */
	int	sock,sock_fd;
	int	buffer_size;
	char	line[SOCKET_BUFFER];	/* Line buffer */
        struct sockaddr_in connector; 	/* Connector's address */
	unsigned int	len;			/* Lenght of address */
	int	y;
	int	port;
	char 	loop;

	/* Check command line */

	if (argc != 2)
	{
		fprintf(stderr,"usage: %s port\n",argv[0]);
		exit(-1);
	}
	sscanf(argv[1],"%d",&port);

	/* Start up ncurses */

        initscr();      /* Initialise curses */
        cbreak();       /* Raw mode */
        noecho();       /* No character echo */
        nonl();         /* No newline appended to returns */
        intrflush(stdscr,FALSE); /* No flushing on interrupt */

        if ((status_window = newwin(STATUS_LINES, STATUS_COLS, STATUS_BEGIN_Y,
                                  STATUS_BEGIN_X)) == (WINDOW *)ERR)
        {
                fprintf(stderr,"Failed to create window");
        }

        /* Create a socket */

        if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0)
        {
		sock = -1;
		werase(status_window);
                mvwaddstr(status_window,1,0,"Failed to open socket.");
		exit(-1);
        }

        /* Force buffer sizes to buffer_size */

	buffer_size = SOCKET_BUFFER;
        if (setsockopt(sock, SOL_SOCKET, SO_SNDBUF, 
                (char *)&buffer_size, sizeof(buffer_size)) < 0)
        {
                close(sock);
		sock = -1;
		werase(status_window);
                mvwaddstr(status_window,1,0,
			"Failed to change command socket write buffer size.");
		exit(-1);
        }

        if (setsockopt(sock, SOL_SOCKET, SO_RCVBUF, 
                (char *)&buffer_size, sizeof(buffer_size)) < 0)
        {
                close(sock);
		sock = -1;
		werase(status_window);
                mvwaddstr(status_window,1,0,
			"Failed to change command socket read buffer size.");
		exit(-1);
        }

        /* Set up server address */

        server.sin_family = AF_INET;
        server.sin_addr.s_addr = htonl(INADDR_ANY);
        server.sin_port = htons(port);

        /* Now bind the socket to this address */

        if (bind(sock,(struct sockaddr *)&server,sizeof(server)) < 0)
        {
                close(sock);
		sock = -1;
		werase(status_window);
                mvwaddstr(status_window,1,0,"Failed to bind socket.");
		exit(-1);
        }

	/* Create a queue for client connection requests */

        if (listen(sock, 0) < 0)
        {
                close(sock);
		sock = -1;
		werase(status_window);
                mvwaddstr(status_window,1,0,"Failed to listen to socket");
		exit(-1);
        }

	/* We run forever.... */

	werase(status_window);
	while(1)
	{
		/* Now wait until someone connects to us */

		mvwaddstr(status_window,0,0,"Waiting for connection.......");
		wrefresh(status_window);

		while (!char_waiting(sock));

		if ((sock_fd = accept(sock, 
			(struct sockaddr *)&connector,&len)) < 0)
		{
			werase(status_window);
			mvwaddstr(status_window,1,0,
				"Failed to accept socket connection.\n");
			continue;
		}

		/* OK, extract the name */

#warning You need to fix this gethostbyaddr call.
#ifdef JUNK

		if ((hp = gethostbyaddr((char *)&connector.sin_addr,
				len,connector.sin_family)) == NULL)
		{
			werase(status_window);
			mvwaddstr(status_window,1,0,
			"Failed to identify host. Blocking socket.");
			close(sock_fd);
			continue;
		}
		else
		{
			printf("%s",hp->h_name);

			/* Separrate out it's domain part. */

			for(domain=hp->h_name; *domain != 0 &&
					*domain != '.'; domain++);
			if (*domain == '.') ++domain;

			/* Is this OK ? */

			if (strcmp(domain,ALLOWED_DOMAIN))
			{
				close(sock_fd);
				werase(status_window);
				mvwprintw(status_window,1,0,
			"Host %s not in correct domain. Blocking socket.", 
						hp->h_name);
				continue;
			}
		}
#endif

		/* We loop here until the connection is broken */

		loop = TRUE;
		while(loop)
		{
			/* Let's try and read something in. */

			if (socket_gets(sock_fd, line, SOCKET_BUFFER) == 
				(char *)EOF)
			{
				/* Read failed, connection has died */
	
				sock_fd = -1;
				loop = FALSE;
				werase(status_window);
				continue;
			}

			/* Do something with the text */

			sscanf(line,"%2d",&y);
			mvwprintw(status_window,y,0,"%s",line+2);
			wrefresh(status_window);
		}
	}

	/* We never get here */

	exit(0);
}

struct {
                char    *name;
                int     (*function)(int argc, char **argv);
        } functions[] =

        {       {NULL,          NULL},
                {NULL,          NULL}   };
