/************************************************************************/
/* testX.c 								*/
/*									*/
/* Test programme for simple X library.  				*/
/* Part of the simpleX library.						*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : August 1994   						*/
/************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "../nrc/nrc.h"
#include "../nsimpleX/nsimpleX.h"

// this def includes code to test overlay_line() and overlay_rectangle().

#define TEST_OVERLAYS

int main(int argc, char **argv)
{
	Window	theWindow1;
	Window	theWindow2;
	Window	theWindow3;
	Window	theWindow4;
	XImage	*theImage1;
	XImage	*theImage2;
	XImage	*theImage3;
	XImage	*theImage4;
	int	i,j;
	char *picture1;
	char *picture2;
	char *picture3;
	char *picture4;
	float	**values;
	int	npix,pixmult;

	if (argc < 3)
	{
		fprintf(stderr,"usage: %s npix pixmult <display>\n",argv[0]);
		exit(-1);
	}

	sscanf(argv[1],"%d",&npix);
	sscanf(argv[2],"%d",&pixmult);

	if (argc <4)
		i = initX(NULL);
	else
		i = initX(argv[3]);

	if (i<=0)
	{
		fprintf(stderr,"Failed to open display.\n");
		exit(-1);
	}
	infoX();
	printf("Allocated %i colors.\n",i);

	theWindow1 = openWindow("LIN",100,100,npix*pixmult,npix*pixmult);
	theWindow2 = openWindow("LOG",120+npix*pixmult,100,
			npix*pixmult,npix*pixmult);
	theWindow3 = openWindow("EXP",100,140+npix*pixmult,
			npix*pixmult,npix*pixmult);
	theWindow4 = openWindow("STD",120+npix*pixmult,140+npix*pixmult,
			npix*pixmult,npix*pixmult);


	/* Build a test image */

	values = matrix(1,npix,1,npix);

	for(i=1;i<=npix;i++)
	{
		for(j=1;j<=npix;j++)
		{
			values[i][j] = sqrt((double)i*j);
		}
	}
        //for(i=0;i<NUM_COLORS;i++) printf("Color %d = %d\n",i,GreyScale[i]);

	picture1 = make_picture(npix,npix,pixmult,values,LIN);
	picture2 = make_picture(npix,npix,pixmult,values,LOG);
	picture3 = make_picture(npix,npix,pixmult,values,EXP);
	picture4 = make_picture(npix,npix,pixmult,values,STD);

	
	if (picture1 == NULL)
	{
		fprintf(stderr,"Could not create picture 1.\n");
		exit(0);
	}
	if (picture2 == NULL)
	{
		fprintf(stderr,"Could not create picture 2.\n");
		exit(0);
	}
	if (picture3 == NULL)
	{
		fprintf(stderr,"Could not create picture 3.\n");
		exit(0);
	}
	if (picture4 == NULL)
	{
		fprintf(stderr,"Could not create picture 4.\n");
		exit(0);
	}
	
#ifdef TEST_OVERLAYS
	/* Overlay a test line on picture1 */

	overlay_line(npix*pixmult,npix*pixmult,picture1,(npix*pixmult)/4,
			(npix*pixmult)/4, (npix*pixmult)/2, 
			(npix*pixmult)/32, 4, PLOT_GREEN); 

	overlay_line(npix*pixmult,npix*pixmult,picture2,(npix*pixmult)/4,
			(npix*pixmult)/4, (npix*pixmult)/2, 
			1, -4, PLOT_RED); 

	/* Overlay a test rectangle on picture2 */

	overlay_rectangle(npix*pixmult,npix*pixmult, picture3, 
			(npix*pixmult)/4, (npix*pixmult)/4, 3*(npix*pixmult)/4,
			3*(npix*pixmult)/4,1,PLOT_LIGHT_BLUE);
#endif	

        theImage1 = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
			picture1,npix*pixmult,npix*pixmult, theBitmapPad,0);

        theImage2 = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
			picture2,npix*pixmult,npix*pixmult, theBitmapPad,0);

        theImage3 = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
			picture3,npix*pixmult,npix*pixmult, theBitmapPad,0);

        theImage4 = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
			picture4,npix*pixmult,npix*pixmult, theBitmapPad,0);

	if (!theImage1) 
	{
		fprintf(stderr,"Could not create image 1.\n");
		exit(0);
	}
	if (!theImage2) 
	{
		fprintf(stderr,"Could not create image 2.\n");
		exit(0);
	}
	if (!theImage3) 
	{
		fprintf(stderr,"Could not create image 3.\n");
		exit(0);
	}
	if (!theImage4) 
	{
		fprintf(stderr,"Could not create image 4.\n");
		exit(0);
	}

	XPutImage(theDisplay,theWindow1,theGC,theImage1,0,0,0,0,
		npix*pixmult,npix*pixmult);

	XPutImage(theDisplay,theWindow2,theGC,theImage2,0,0,0,0,
		npix*pixmult,npix*pixmult);

	XPutImage(theDisplay,theWindow3,theGC,theImage3,0,0,0,0,
		npix*pixmult,npix*pixmult);

	XPutImage(theDisplay,theWindow4,theGC,theImage4,0,0,0,0,
		npix*pixmult,npix*pixmult);

	/* Try and display it */

        XMapWindow(theDisplay, theWindow1);
        XMapWindow(theDisplay, theWindow2);
        XMapWindow(theDisplay, theWindow3);
        XMapWindow(theDisplay, theWindow4);
        XFlush(theDisplay);

	/* Try and save it */

	make_ppm_P3("LIN_P3.ppm", npix, npix, pixmult, picture1);
	make_ppm_P3("LOG_P3.ppm", npix, npix, pixmult, picture2);
	make_ppm_P3("EXP_P3.ppm", npix, npix, pixmult, picture3);
	make_ppm_P3("STD_P3.ppm", npix, npix, pixmult, picture4);
	make_ppm_P6("LIN_P6.ppm", npix, npix, pixmult, picture1);
	make_ppm_P6("LOG_P6.ppm", npix, npix, pixmult, picture2);
	make_ppm_P6("EXP_P6.ppm", npix, npix, pixmult, picture3);
	make_ppm_P6("STD_P6.ppm", npix, npix, pixmult, picture4);

	nameWindow(theWindow1, "BUM");

	sleep(2);

	XDestroyImage(theImage1);
	XDestroyImage(theImage2);
	XDestroyImage(theImage3);
	XDestroyImage(theImage4);

	free_matrix(values,1,npix,1,npix);

	XDestroyWindow(theDisplay, theWindow1);
	XDestroyWindow(theDisplay, theWindow2);
	XDestroyWindow(theDisplay, theWindow3);
	XDestroyWindow(theDisplay, theWindow4);

	quitX();

	exit(0);
}
