/************************************************************************/
/* testcpk.c                                                            */
/*                                                                      */
/* Test out CPK routines.						*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo                                                        */
/* Date   : March 2003                                                  */
/************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include "../cpk/cpk.h"
#include "../charalib/chara.h"
#include "../astromod/astromod.h"

int main(int argc, char **argv)
{
	struct s_star star;
	double az,el,ref_el,hour_angle;
	double	mjd;
	double	a,b;
	double ut1;
	double  vicrs[3], vdate[3], vobs[3], vtopo[3];
	double cpk_az, cpk_el;
	int year, month, day;
	double press;
	TRANSP pars;
	double r2000, d2000;
	double hours, minutes, seconds;
	char	s[254];
	double	elong, phi;
	char    *tpoints_filename = "/ctrscrut/chara/etc/dome_blank.tps";

	/* Get the input stuff */

	printf("Year      : ");
	scanf("%d", &year);
	printf("Month     : ");
	scanf("%d", &month);
	printf("Day       : ");
	scanf("%d", &day);
	printf("Hours     : ");
	scanf("%lf", &hours);
	printf("Minutes   : ");
	scanf("%lf", &minutes);
	printf("Seconds   : ");
	scanf("%lf", &seconds);
	getchar();
	printf("RA        : ");
	gets(s);
	r2000 = string_to_time(s);
	printf("DEC       : ");
	gets(s);
	d2000 = string_to_angle(s);
	printf("Longitude : ");
	gets(s);
	elong = string_to_angle(s);
	printf("Latitude  : ");
	gets(s);
	phi = string_to_angle(s);
	printf("Pressure  : ");
	scanf("%lf", &press);

	/* Tell us */

	printf ( "\n  RA = %11.5f, Dec = %+9.5f  ICRS\n",
            r2000/M_PI*180.0, d2000/M_PI*180.0 );
	
	/* Do the CPK calculation */

	pars.tmount = ALTAZ;
        switch(cpkModel(tpoints_filename, &pars))
        {
                case 0: break;

                case -1: fprintf(stderr,"Unable to open model file %s.",
                                tpoints_filename);
			exit(-1);

                case -2: fprintf(stderr,"I/O ereror or premature EOF in %s",
                                tpoints_filename);
			exit(-1);

                case -3: fprintf(stderr,"Unsupported term in %s",
                                tpoints_filename);
			exit(-1);

                case -4: fprintf(stderr,"Unknown error in %s",
                                tpoints_filename);
			exit(-1);
        }

        /* OK, get the year, month and day. */

        ut1 = cpkMjd(year, month, day) + cpkTday(hours, minutes, seconds);

        /* Aberation, precession, nutation, refraction and Earth rotation */

        cpkUpda(ut1, press, elong, phi, &pars);

	/* J2000 to vector */

        cpkS2v(r2000, d2000, vicrs);

        /* Convert things to encoders */

        cpkSky2enc(0, &pars, vicrs, ICRS, vdate, vtopo, vobs, &cpk_az, &cpk_el);

	/* Print this */

	printf("CPK RESULTS:\n\n");

	printf ( " Mjd = %.6f\n", ut1);

	cpkV2s ( vdate, &a, &b );
	a *= (180.0/M_PI);
	b *= (180.0/M_PI);
	if ( a < 0.0 ) a += 360.0;
	printf ( "  RA = %11.5f, Dec = %+9.5f  date\n", a, b );

	cpkC2s ( -vtopo[0], vtopo[1], vtopo[2], &a, &b );
	a *= (180.0/M_PI);
	b *= (180.0/M_PI);
	if ( a < 0.0 ) a += 360.0;
	printf ( "  Az = %11.5f, El  = %9.5f  topo\n", a, b );

	cpkC2s ( -vobs[0], vobs[1], vobs[2], &a, &b );
	a *= (180.0/M_PI);
	b *= (180.0/M_PI);
	if ( a < 0.0 ) a += 360.0;
	printf ( "  Az = %11.5f, El  = %9.5f  obs\n", a, b );

	/* Print out this result */

	printf ( "  Az = %11.5f, El  = %9.5f  CPK\n", 
		cpk_az/M_PI*180.0, cpk_el/M_PI*180.0 );

	/* Now the SLALIB based calculations */

	astromod_init(NULL, NULL);
	star.ra = r2000;
	star.dec = d2000;
	star.pm_ra = 0.0;
        star.pm_dec = 0.0;
        star.parallax = 0.0;
        star.radvel = 0.0;
        star.chara_number = 1; /* To fake things out. */

	mjd = astromod_mjd(year, month, day,
			(hours+(minutes+seconds/60.0)/60.0)/12.0*M_PI);

	printf("\nASTROMOD RESULTS:\n\n");

	printf ( " Mjd = %.6f\n", mjd);

	astromod_azel_mjd(star, mjd, &az, &el, &ref_el, &hour_angle);

	printf ( "  Az = %11.5f, El  = %9.5f  topo\n", 
		az/M_PI*180.0, el/M_PI*180.0 );

	printf ( "  Az = %11.5f, El  = %9.5f  obs\n", 
		az/M_PI*180.0, ref_el/M_PI*180.0 );

	printf ( "\n  Az = %11.5f, El  = %9.5f  Delta\n", 
		(cpk_az-az)/M_PI*180.0, (cpk_el-ref_el)/M_PI*180.0 );
	printf ( "\n  Az = %11.5f, El  = %9.5f  Delta (AS)\n", 
		(cpk_az-az)/M_PI*180.0*60.0*60.0, 
		(cpk_el-ref_el)/M_PI*180.0*60.0*60.0 );

	/* That is all */

	exit(0);

} 
