/************************************************************************/
/* testdft.c								*/
/*                                                                      */
/* Program to test the DFT routine.					*/
/************************************************************************/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar			                        */
/* Date   : Feb 2011							*/
/************************************************************************/

#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <time.h>
#include "../nrc/nrc.h"
#include <math.h>
#include "../slalib/slalib.h"
#include "../filter/filter.h"
#include "../rwfits/rwfits.h"
#include "../charalib/chara.h"
#include <stdlib.h>
#include "../cliserv/include/cliserv.h"
#include "../interferometry/interferometry.h"
#include "../clock/clock.h"
#ifdef NOT_AN_APPLE
#include <values.h>
#endif

int main(int argc, char **argv)
{
	float phase12, phase23, phase31, freq12, freq23, freq31;
	int	n;
	float *signal, *dft;
	float re=1.0, im=1.0;
	int	i;

	if (argc != 5)
	{
		fprintf(stderr,"usage: %s n phase12, phase23, phase31\n",
			argv[0]);
		exit(-1);
	}

	sscanf(argv[1],"%d", &n);
	sscanf(argv[2],"%f", &phase12);
	sscanf(argv[3],"%f", &phase23);
	sscanf(argv[4],"%f", &phase31);

	phase12 = phase12/180.0 * M_PI;
	phase23 = phase23/180.0 * M_PI;
	phase31 = phase31/180.0 * M_PI;
	freq12 = (float) n/3;
	freq23 = (float) n/2;
	freq31 = (float) n;

	signal = vector(1, n);

	//printf("#Signal:\n");
	for(i=1; i<=n; i++)
	{
		signal[i] = cos((i-1)/freq12 * 2.0 * M_PI + phase12) +
			    cos((i-1)/freq23 * 2.0 * M_PI + phase23) +
			    cos((i-1)/freq31 * 2.0 * M_PI + phase31);
		//printf("%d %.3f\n", i, signal[i]);
	}

 	dft = dft_fringe(signal, n);

	printf("DFT:\n");
	printf(" 0 %8.4f\n", dft[1]);
	for(i=2; i<n-1; i+=2)
	{
		re=dft[i+1];
		 im=dft[i+2]; 
		printf("%2d %8.4f %8.4f %8.4f %6.1f\n",
		i/2, re, im, 
		sqrt(re*re+im*im),
		atan2(im, re)/M_PI*180.0);
	}
	printf("%2d %8.4f\n", n/2, dft[2]);
	exit(0);

}
