/************************************************************************/
/* testfitgaussian.c 								*/
/*									*/
/* Test programme for simple X library.  				*/
/* Part of the simpleX library.						*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : Feb 2004	   						*/
/************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "../nrc/nrc.h"
#include "../nsimpleX/nsimpleX.h"

#define DISPLAY
int main(int argc, char **argv)
{
	float	amp,x0,y0,c,fwhm;
	float	**pict;
	float	**fit;
	float	**res;
	int	nx,ny;
	int	i,j;
	int	pixmult;
	float	rmsres;
#ifdef DISPLAY
	Window  pictWindow;
	Window  fitWindow;
	Window  resWindow;
        XImage  *theImage;
	char    *picture;
#endif

	if (argc != 8)
	{
		fprintf(stderr,"usage: %s nx ny pixmult, amp x0 y0 fwhm\n",
			argv[0]);
		exit(-1);
	}

	sscanf(argv[1],"%d",&nx);
	sscanf(argv[2],"%d",&ny);
	sscanf(argv[3],"%d",&pixmult);
	sscanf(argv[4],"%f",&amp);
	sscanf(argv[5],"%f",&x0);
	sscanf(argv[6],"%f",&y0);
	sscanf(argv[7],"%f",&fwhm);

	/* Create the fake data */

	pict = matrix(1,nx,1,ny);
	fit = matrix(1,nx,1,ny);
	res = matrix(1,nx,1,ny);
	c = fwhm / 2.354820045;
	for(i=1; i<=nx; i++)
	for(j=1; j<=ny; j++)
	{
	    pict[i][j] = amp * exp(((i-x0)*(i-x0) + (j-y0)*(j-y0))/(-2.0*c*c));
	}
	
	/* Setup X windows and display the data */

#ifdef DISPLAY
	initX(NULL);
	pictWindow = openWindow("DATA",100,100,nx*pixmult,ny*pixmult);
	fitWindow = openWindow("FIT",120+nx*pixmult,100,nx*pixmult,ny*pixmult);
	resWindow = openWindow("DIFF",140+2*(nx*pixmult),100,
				nx*pixmult,ny*pixmult);

	picture = make_picture(nx,ny,pixmult,pict,LIN);
	theImage = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
                        picture,nx*pixmult,ny*pixmult, theBitmapPad,0);
	XPutImage(theDisplay,pictWindow,theGC,theImage,0,0,0,0,
                nx*pixmult,ny*pixmult);
	XMapWindow(theDisplay, pictWindow);
	XDestroyImage(theImage);
	XFlush(theDisplay);
#endif

	/* Do the fit */

	rmsres = fit_2D_gaussian(pict, nx, ny, &amp, &x0, &y0, &c, &fwhm, 
		1.0, 100, 
		SHOW_NOTHING);

	/* Print results */

	printf("Amp    = %f\n",amp);
	printf("X0     = %f\n",x0);
	printf("Y0     = %f\n",y0);
	printf("C      = %f\n",c);
	printf("FWHM   = %f\n",fwhm);
	printf("RMSRES = %f\n",rmsres);

	/* Create the fit and rms images */

	for(i=1; i<=nx; i++)
	for(j=1; j<=ny; j++)
	{
	    fit[i][j] = amp * exp(((i-x0)*(i-x0) + (j-y0)*(j-y0))/(-2.0*c*c));
	    res[i][j] = pict[i][j] - fit[i][j];
	}
	
	/* Display them */

#ifdef DISPLAY
	picture = make_picture(nx,ny,pixmult,fit,LIN);
	theImage = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
                        picture,nx*pixmult,ny*pixmult, theBitmapPad,0);
	XPutImage(theDisplay,fitWindow,theGC,theImage,0,0,0,0,
                nx*pixmult,ny*pixmult);
	XMapWindow(theDisplay, fitWindow);
	XDestroyImage(theImage);

	picture = make_picture(nx,ny,pixmult,res,LIN);
	theImage = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
                        picture,nx*pixmult,ny*pixmult, theBitmapPad,0);
	XPutImage(theDisplay,resWindow,theGC,theImage,0,0,0,0,
                nx*pixmult,ny*pixmult);
	XMapWindow(theDisplay, fitWindow);
	XDestroyImage(theImage);

	XFlush(theDisplay);

	/* Wait for the user to react */

	printf("Hit return to exit.");
	getchar();
#endif

	/* Clean up and go */

	free_matrix(pict,1,nx,1,ny);
	free_matrix(fit,1,nx,1,ny);
	free_matrix(res,1,nx,1,ny);
#ifdef DISPLAY
	XDestroyWindow(theDisplay, pictWindow);
	XDestroyWindow(theDisplay, fitWindow);
	XDestroyWindow(theDisplay, resWindow);
	quitX();
#endif

	exit(0);
}
