/************************************************************************/
/* testmouse.c 								*/
/*									*/
/* Test programme for nsimple X library.  				*/
/* Part of the nsimpleX library.					*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : August 1994   						*/
/************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "../nrc/nrc.h"
#include "../nsimpleX/nsimpleX.h"

int main(int argc, char **argv)
{
	Window	theWindow;
	XImage	*theImage;
	int	i,j;
	char *picture;
	float	**values;
	int	npix,pixmult;
	int	x, y;
	Bool	return_value;

	if (argc < 3)
	{
		fprintf(stderr,"usage: %s npix pixmult <display>\n",argv[0]);
		exit(-1);
	}

	sscanf(argv[1],"%d",&npix);
	sscanf(argv[2],"%d",&pixmult);

	if (argc < 4)
		i = initX(NULL);
	else
		i = initX(argv[3]);

        if (!i)
        {
                fprintf(stderr,"Failed to open display.\n");
		exit(-1);
        }

	infoX();

	theWindow = openWindow("HERE",100,100,npix*pixmult,npix*pixmult);

	/* Build a test image */

	values = matrix(1,npix,1,npix);

	for(i=1;i<=npix;i++)
	{
		for(j=1;j<=npix;j++)
		{
			values[i][j] = sqrt((double)i*j);
		}
	}
			

	picture = make_picture(npix,npix,pixmult,values,LIN);
        theImage = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
			picture,npix*pixmult,npix*pixmult,
			theBitmapPad,0);

	if (!theImage) 
	{
		fprintf(stderr,"Could not create image....\n");
	}

	XPutImage(theDisplay,theWindow,theGC,theImage,0,0,0,0,
		npix*pixmult,npix*pixmult);

	/* Try and display it */

        XMapWindow(theDisplay, theWindow);

        XFlush(theDisplay);

	while(1)
	{
		return_value = get_mouse(theWindow, pixmult, &x, &y);

		printf("return_value = %d\n",return_value);
		printf("image[%d][%d] = %f\n",x,y,values[x][y]);
	}

	XDestroyImage(theImage);
	free_matrix(values,1,npix,1,npix);
	XDestroyWindow(theDisplay, theWindow);

	quitX();

	exit(0);
}
