/************************************************************************/
/* testoif.c								*/
/*									*/
/* Test charaoif, the CHARA OIFITS routines.				*/
/*									*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*									*/
/* Author: Theo								*/
/* Date: September 2009							*/
/*									*/
/************************************************************************/

#include "../slalib/slalib.h"
#include "../oifitslib/exchange.h"
#include "../charalib/chara.h"
#include "../charaoif/charaoif.h"
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

int main(int argc, char **argv)
{
	oi_array array;
	oi_wavelength wave;
	oi_target targets;
	oi_vis vis;
	oi_vis2 vis2;
	oi_t3 t3;
	int	i;
	struct s_star	star;
	double	visamp[3], visamperr[3];
	double	visphi[3], visphierr[3];
	double	vis2data[3], vis2err[3];
	double	t3amp[3], t3amperr[3];
	double	t3phi[3], t3phierr[3];
	int	year, month, day, doy;
	double	mjd;
	double	now;
	int	status = 0;
	fitsfile *fptr;

	/* First get the array information */

	printf("ARRAY      : %s\n",!setup_oi_array(&array)?"No Error":"Error");

	print_oi_array(stdout, array);

	/* Now the wavelength */

	printf("\nWAVELENGTH:\n");
	setup_oi_wavelength(&wave, "CHARA_NIRO");
	printf("Add J band : %s\n",
		!add_oi_wavelength(&wave,J_BAND_CEN, J_BAND_WIDTH)?
			"No Error":"Error");
	printf("Add H band : %s\n",
		!add_oi_wavelength(&wave,H_BAND_CEN, H_BAND_WIDTH)?
			"No Error":"Error");
	printf("Add K band : %s\n",
		!add_oi_wavelength(&wave,K_BAND_CEN, K_BAND_WIDTH)?
			"No Error":"Error");
	print_oi_wavelength(stdout, wave);

	/* Now add a few targets */

	printf("\nTARGETS.\n");
	setup_oi_target(&targets);

	printf("HD 193322  : ");
	if (get_star_by_hd(193322, &star))
	{
		printf("Found.\nHD 193322  : %s\n",
			!add_oi_target(&targets, star)?"No Error":"Error");
	}
	else
		printf("Not found.\n");
	
	printf("HD 198084  : ");
	if (get_star_by_hd(198084, &star))
	{
		printf("Found.\nHD 198084  : %s\n",
			!add_oi_target(&targets, star)?"No Error":"Error");
	}
	else
		printf("Not found.\n");
	
	print_oi_target(stdout, targets);

	/* We will need an MJD */

	get_ut_date(&year, &month, &day, &doy);
	slaCldj(year, month, day, &mjd, &i);
	now = (double)(time(NULL)%(24*60*60));

	mjd += now/(24.0*60.0*60.0);

	printf("\nCOMPLEX VISIBILITY:\n"); 
	setup_oi_vis(&vis, "2009-09-26", array.arrname, wave.insname, 
			wave.nwave);

	for(i=0; i<wave.nwave; i++)
	{
		visamp[i] = (double)i/10.0 + 0.1;
		visamperr[i] = visamp[i] * 0.05;
		visphi[i] = (double)i*120.0;
		visphierr[i] = visphi[i] * 0.05;
	}
	add_oi_vis(&vis, targets.targ[0].target_id, now, mjd, 600.0,
		visamp, visamperr, visphi, visphierr, 100.0, 200.0,
		S1, S2);

	for(i=0; i<wave.nwave; i++)
	{
		visamp[i] = (double)i/5.0 + 0.1;
		visamperr[i] = visamp[i] * 0.05;
		visphi[i] = (double)i*60.0;
		visphierr[i] = visphi[i] * 0.05;
	}
	now += 20;
	mjd += (now/(24.0*60.0*60.0));
	add_oi_vis(&vis, targets.targ[0].target_id, now, mjd, 600.0,
		visamp, visamperr, visphi, visphierr, 300.0, 400.0,
		E1, W2);
	print_oi_vis(stdout, vis);

	printf("\nVISIBILITY SQUARED:\n"); 
	setup_oi_vis2(&vis2, "2009-05-02", array.arrname, wave.insname, 
			wave.nwave);

	for(i=0; i<wave.nwave; i++)
	{
		vis2data[i] = (double)i/20.0 + 0.1;
		vis2err[i] = vis2data[i] * 0.05;
	}
	now += 20;
	mjd += (now/(24.0*60.0*60.0));
	add_oi_vis2(&vis2, targets.targ[1].target_id, now, mjd, 600.0,
		vis2data, vis2err, 500.0, 600.0, W1, W2);

	for(i=0; i<wave.nwave; i++)
	{
		vis2data[i] = (double)i/15.0 + 0.1;
		vis2err[i] = vis2data[i] * 0.05;
	}
	now += 20;
	mjd += (now/(24.0*60.0*60.0));
	add_oi_vis2(&vis2, targets.targ[1].target_id, now, mjd, 600.0,
		vis2data, vis2err, 700.0, 800.0, S1, E2);

	print_oi_vis2(stdout, vis2);

	printf("\nTRIPPLE PRODUCT:\n"); 
	setup_oi_t3(&t3, "2009-10-06", array.arrname, wave.insname, 
			wave.nwave);

	for(i=0; i<wave.nwave; i++)
	{
		t3amp[i] = (double)i/30.0 + 0.1;
		t3amperr[i] = t3amp[i] * 0.05;
		t3phi[i] = (double)i*80.0;
		t3phierr[i] = t3phi[i] * 0.05;
	}
	add_oi_t3(&t3, targets.targ[0].target_id, now, mjd, 700.0,
		t3amp, t3amperr, t3phi, t3phierr, 100.0, 200.0, 300, 400,
		E1, W1, S1);

	for(i=0; i<wave.nwave; i++)
	{
		t3amp[i] = (double)i/25.0 + 0.1;
		t3amperr[i] = t3amp[i] * 0.05;
		t3phi[i] = (double)i*90.0;
		t3phierr[i] = t3phi[i] * 0.05;
	}
	now += 20;
	mjd += (now/(24.0*60.0*60.0));
	add_oi_t3(&t3, targets.targ[0].target_id, now, mjd, 800.0,
		t3amp, t3amperr, t3phi, t3phierr, 500.0, 600.0, 700, 800,
		E1, W1, S1);

	print_oi_t3(stdout, t3);

	/* Save the FITS stuff into a file */

	fits_create_file(&fptr, "CHARA_oif.fit", &status);
	if (status)
	{
		fits_report_error(stderr, status);
		goto cleanup;
	}

	write_oi_target(fptr, targets, &status);
	write_oi_vis(fptr, vis, 1, &status);
	write_oi_vis2(fptr, vis2, 1, &status);
	write_oi_t3(fptr, t3, 1, &status);
	write_oi_array(fptr, array, 1, &status);
	write_oi_wavelength(fptr, wave, 1, &status);

	if (status)
	{
		fits_report_error(stderr, status);
		fits_delete_file(fptr, &status);
	}
	else
		fits_close_file(fptr, &status);

	/* Clean up memory */

cleanup:
	clear_oi_array(array);
	clear_oi_wavelength(wave);
	clear_oi_target(targets);
	clear_oi_vis(vis);
	clear_oi_vis2(vis2);
	clear_oi_t3(t3);

	exit(0);
}
