/************************************************************************/
/* testplot.c 								*/
/*									*/
/* Test programme for simple X library.  				*/
/* Part of the simpleX library.						*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : July 1999   						*/
/************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "../nrc/nrc.h"
#include "../nsimpleX/nsimpleX.h"

int main(int argc, char **argv)
{
	Window	theWindow;
	XImage	*theImage;
	char *picture1;
	char *picture2;
	char *picture3;
	int	width,height;
	float	*data1;
	float	*data2;
	float	*data3;
	int	num_data;
	int	i;
	float	min,max;

	if (argc < 4)
	{
		fprintf(stderr,"usage: %s width height num_data <min max> <display>\n",
			argv[0]);
		exit(-1);
	}

	sscanf(argv[1],"%d",&width);
	sscanf(argv[2],"%d",&height);
	sscanf(argv[3],"%d",&num_data);
	min = max = 0.0;
	
	if (argc > 4)
	{
		sscanf(argv[4],"%f",&min);
	}

	if (argc > 5)
	{
		sscanf(argv[5],"%f",&max);
	}

	/* Create the data */

	data1 = vector(1,num_data);
	data2 = vector(1,num_data);
	data3 = vector(1,num_data);
	for(i=1; i<=num_data; i++) 
	{
		data1[i] = sin((float)i/100*M_PI);
		data2[i] = cos((float)i/100*M_PI);
		data3[i] = cos(((float)i/100+0.5)*M_PI);
	}

        if (argc <6)
                i = initX(NULL);
        else
                i = initX(argv[6]);

        if (i<=0)
        {
                fprintf(stderr,"Failed to open display.\n");
                exit(-1);
        }

	infoX();
	theWindow = openWindow("PLOT TEST",300,100,width,height);

	/* Create the pictures */

	if (!set_plot_color(PLOT_RED))
	{
		fprintf(stderr,"Failed to get color for data1, using WHITE\n");
	}
	picture1 = plot_funct(width,height,data1,num_data,&min,&max);
	printf("Min = %f Max = %f\n",min,max);

	if (!set_plot_color(PLOT_GREEN))
	{
		fprintf(stderr,"Failed to get color for data2, using WHITE\n");
	}
	picture2 = plot_funct(width,height,data2,num_data,&min,&max);
	printf("Min = %f Max = %f\n",min,max);

	if (!set_plot_color(PLOT_DARK_BLUE))
	{
		fprintf(stderr,"Failed to get color for data3, using WHITE\n");
	}
	picture3 = plot_funct(width,height,data3,num_data,&min,&max);
	printf("Min = %f Max = %f\n",min,max);

	if (picture1 == NULL || picture2 == NULL || picture3 == NULL)
	{
		printf("Error making pictures.\n");
		exit(-1);
	}

	/* Display the picture */

        theImage = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
			picture1, width, height, theBitmapPad, 0);

	XPutImage(theDisplay,theWindow,theGC,theImage,0,0,0,0,width,height);

	/* Try and display it */

        XMapWindow(theDisplay, theWindow);
        XFlush(theDisplay);

	/* Wait and then merge next plot */

	printf("SIN\n");
	sleep(5);
	merge_plots(picture1, picture2, width, height);

	/* Display the picture */

        theImage = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
			picture1, width, height, theBitmapPad, 0);

	XPutImage(theDisplay,theWindow,theGC,theImage,0,0,0,0,width,height);

	/* Try and display it */

        XMapWindow(theDisplay, theWindow);
        XFlush(theDisplay);

	/* Wait and then merge next plot */

	printf("COS\n");
	sleep(5);
	merge_plots(picture1, picture3, width, height);

	/* Display the picture */

        theImage = XCreateImage(theDisplay,theVisual,theDepth,ZPixmap,0,
			picture1, width, height, theBitmapPad, 0);

	XPutImage(theDisplay,theWindow,theGC,theImage,0,0,0,0,width,height);

	/* Try and display it */

        XMapWindow(theDisplay, theWindow);
        XFlush(theDisplay);
	
	/* Wait and then clean up */

	printf("SIN and COS and the other one.\n");
	sleep(5);
	XDestroyImage(theImage);
	free_vector(data1,1,num_data);
	free_vector(data2,1,num_data);
	free_vector(data3,1,num_data);
	free(picture2);
	free(picture3);
	XDestroyWindow(theDisplay, theWindow);
	quitX();

	exit(0);
}
