/************************************************************************/
/* testscope.c 								*/
/*									*/
/* Test programme for scope stuff in simple X library.			*/
/* Part of the simpleX library.						*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : November 1994   						*/
/************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include "../nrc/nrc.h"
#include "../nsimpleX/nsimpleX.h"

int main(int argc, char **argv)
{
	int 	i;
	int	xpix,ypix;
	scope	Scope1, Scope2;
	float	*values1, *values2;

	/* Check outthe command line */

	if (argc < 3)
	{
		fprintf(stderr,"usage: %s xpix ypix <display>\n",argv[0]);
		exit(-1);
	}

	sscanf(argv[1],"%d",&xpix);
	sscanf(argv[2],"%d",&ypix);

	/* Initialise */

	if (argc < 4)
		i = initX(NULL);
	else
		i = initX(argv[3]);
        if (!i)
        {
                fprintf(stderr,"Failed to open display.\n");
		exit(-1);
        }

	Scope1 = open_scope("ONE",100,100,xpix,ypix);
	Scope2 = open_scope("TWO",100,120+ypix,xpix,ypix);

	if (add_signal(&Scope1,65535,0,0,-1.0,1.0) != 1)
	{
		close_scope(&Scope1);
		close_scope(&Scope2);
		quitX();
		exit(-2);
	}
		
	if (add_signal(&Scope1,0,65535,0,-1.0,2.0) != 2)
	{
		close_scope(&Scope1);
		close_scope(&Scope2);
		quitX();
		exit(-2);
	}
		
	if (add_signal(&Scope1,0,0,65535,-2.0,1.0) != 3)
	{
		close_scope(&Scope1);
		close_scope(&Scope2);
		quitX();
		exit(-2);
	}

	if (add_signal(&Scope2,65535,0,0,-1.0,1.0) != 1)
	{
		close_scope(&Scope1);
		close_scope(&Scope2);
		quitX();
		exit(-2);
	}
		
	if (add_signal(&Scope2,0,65535,0,-1.0,2.0) != 2)
	{
		close_scope(&Scope1);
		close_scope(&Scope2);
		quitX();
		exit(-2);
	}
		
	if (add_signal(&Scope2,0,0,65535,-2.0,1.0) != 3)
	{
		close_scope(&Scope1);
		close_scope(&Scope2);
		quitX();
		exit(-2);
	}

	values1 = vector(1,3);
	values2 = vector(1,3);

	/* Do something with the dam thing */

	for(i=0;i<100;i++)
	{
		values1[1] = sin((float)i/50*M_PI);
		values1[2] = sin((float)i/100*M_PI);
		values1[3] = sin((float)i/25*M_PI);
		values2[1] = cos((float)i/50*M_PI);
		values2[2] = cos((float)i/100*M_PI);
		values2[3] = cos((float)i/25*M_PI);
		
		update_scope(Scope1,values1);
		update_scope(Scope2,values2);
	}

	/* Clean up */

	sleep(2);
	free_vector(values1,1,3);
	free_vector(values2,1,3);
	close_scope(&Scope1);
	close_scope(&Scope2);
	quitX();

	exit(0);
}
