/************************************************************************/
/* testui.c								*/
/*                                                                      */
/* Test program for CHARAUI.						*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Tony Johnson & Theo ten Brummelaar                          */
/* Date   : Original Version 1990 - ported to Linux 1998                */
/************************************************************************/

#define FUNCTS
#include "../charaui/src/charaui.h"

/* Some prototypes */

int	example(int argc, char **argv);
int	question(int argc, char **argv);
int time_status(void);		/* Example background job below */
int data_status(void);		/* Background job tests data socket */
void goodbye(void) { error(MESSAGE,"Goodbye then."); }
void hello(void) {background_start(0,NULL);\
		  error(WARNING,"Hi there."); }
int background_job(void);
int top_job_job(void);

int	not_implimented(int argc, char **argv)
{
	error(MESSAGE,"Function not yet implimented.");
	while (argc > 0)
	{
		error(MESSAGE,"Arugument %d = %s.",argc,argv[argc-1]);
		argc--;
	}

	if (ask_yes_no("Are you happy about that?","Well?"))
	{
		message(system_window,"Good, I'm done.");
	}
	else
	{
		message(system_window,"Too bad, I'm done.");
	}
	return NOERROR;
}


struct {                       
                char    *name;
                int     (*function)(int argc, char **argv);
        } functions[] = 

	{	{NULL,		NULL},
#include "../charaui/src/std_ui_functs.h"
                {"ni",          not_implimented},
		{"example",	example},
		{"q",		question},
		{NULL,		NULL}	};

void print_usage_message(char *name)
{

        fprintf(stderr,"usage: %s [-flags] name\n",name);
        fprintf(stderr,"Flags:\n");
        fprintf(stderr,"-s\tToggle use of socketmanager (ON)\n");
        fprintf(stderr,"-S\tUse standalone mode (OFF)\n");
	fprintf(stderr,"-n\tUse old socket manger\n");
        fprintf(stderr,"-h\tPrint this message\n");
}

int main(int argc, char **argv)
{
	char *name, *p;
	
	/* Check out the command line */

	name = argv[0];
        while(--argc > 0 && (*++argv)[0] == '-')
        {
             for(p = argv[0]+1; *p != '\0'; p++)
             {
                switch(*p)
                {
                        case 'h': print_usage_message(name);
                                  exit(0);

			case 's': use_socket_manager = !use_socket_manager;
				  break;

			case 'S': standalone = TRUE;
				  break;

			case 'n': use_non_socket_manager = FALSE;

				  break;

			default:  fprintf(stderr,"%s: Unknown flag %c.\n", 
					name, *p);
				  print_usage_message(name);
				  exit(-1);
		}
	     }
	}

	/* DO we have enought arguments? */

	if (argc == 0)
	{
		print_usage_message(name);
		exit(-2);
	}

	/* Title page. */

	ui_clear_screen();
	put_line("");
	center_line("CHARA USER INTERFACE TEST ROUTINE");
	put_line("");
	center_line("The CHARA Array");
	center_line("Center for High Angular Resolution Astronomy");
	center_line("Mount Wilson Observatory, CA 91001, USA");
	put_line("");
	center_line("Telephone: 1-626-796-5405");
	center_line("Fax: 1-626-796-6717");
	center_line("email: theo@chara.gsu.edu");
	center_line("WWW: http://www.chara.gsu.edu");
	put_line("");
	center_line("(C) This executable program is copyright.");
	wait_for_title();

	/* Initialize the user interface */

	TITLE = "TESTUI VERSION 0.0";
	initialise_ui("menus.ini","help.ini",argv[0]);
	set_user_open_function(hello);
	set_user_close_function(goodbye);

	/* Setup background job(s) */

	set_top_job(top_job_job);
	background_add_name(time_status, "time_status");
	background_add_name(data_status, "data_status");
	background_add_name(background_job, "background_job");

	/* Let's go! */

	start_ui(); /* Should never return from here. */

	exit(0);
}

/*
 * Here follows and example background job.
 */

#define clean(y,x)	mvwaddstr(status_window,y,x,\
				"                                       ")

/************************************************************************/
/* time_status()							*/
/* 									*/
/* Displays the time in the status window.				*/
/************************************************************************/

int time_status(void)
{
	time_t current_time;
	struct tm *now;

	/* Display the time */

	time(&current_time);
	now = localtime(&current_time);
	clean(0,0);
	wstandout(status_window);
	mvwaddstr(status_window,0,0,"Local Time : ");
	wstandend(status_window);
	wprintw(status_window,"%2d:%02d:%02d",
		now->tm_hour,now->tm_min,now->tm_sec);

	return NOERROR;

} /* time_status() */

/*
 * String for enumerated type
 */

char	*example_types[] = {
				"Type One  ",
				"Type Two  ",
				"Type Three",
				"Type Four ",
				"Type Five ",
				NULL 
			  };

/************************************************************************/
/* example()								*/
/* 									*/
/* An exmaple function showing varous windows and editing functions.	*/
/************************************************************************/

int example(int argc, char **argv)
{
	int	an_int;
	float	a_float;
	char	a_string[81];
	int	an_enumerate;
#ifdef __NCURSES_H
	/*
	 * Note: Only ncurses has mouse support, and only for xterms 
	 */

        MEVENT  mouse;                  /* A mouse event. */
#endif

	no_socket();

	socket_test_args(3,"one two three");

	/* Clean things up */

	werase(command_window);
	wrefresh(command_window);

	/* Have a look at the commands, just like a C programme */

	*a_string = 0;
	for (an_int=0; an_int<argc; an_int++)
	{
		strcat(a_string,argv[an_int]);
		strcat(a_string," ");
	}

	/* Put it up into the warning/error window call */

	error(WARNING,
		"This is the warning/error window. You typed :\n%s\nLast line.",
		a_string);

	/*
   	 * Introducing the main_window.
  	 */

	active_window = main_window; /* So system knows what you're up to. */
	werase(main_window);
	mvwaddstr(main_window,0,0,"This is the main_window.");
	mvwaddstr(main_window,2,0,
		"You can do what you like in the main_window.");
	wrefresh(main_window);

	/*
	 * Introducing the system_window
	 */

	message(system_window,
		"This is the system_window, it's for messages like this:\n\
Type a key to continue.");

	/*
 	 * Here's how to poll the keyboard.
	 */

	while(!kbhit()) 
	{
		background(); /* Process background jobs while we wait */
	}
	an_int = get_command(); /* Got to clear the character! */
	werase(system_window); /* And clean up the system_window */
	wrefresh(system_window);

	/* 
	 * We can do general editing in any window with line_edit();
	 */

	strcat(a_string,"Like this one");
	mvwaddstr(main_window,4,0,"Like editing a line of text : ");
	if (line_edit(main_window,4,30,a_string,30,STRING,return_true,TRUE) ==
		KEY_ESC) return NOERROR;
	error(MESSAGE,"The string ended up as:\n%s",a_string);
	
	/* 
	 * Introducing the heading and sub_main_windows 
	 */

	active_window = heading_window;
	heading(heading_window,"This is the heading_window.",
		"It's used often for headings obviously.");

	active_window = sub_main_window;
	werase(sub_main_window);
	mvwaddstr(sub_main_window,1,0,"This is the sub_main_window.");
	mvwaddstr(sub_main_window,3,0,
		"In the heading window the first line is always centered.");
	mvwaddstr(sub_main_window,4,0,
		"The second line isn't.");
	mvwaddstr(sub_main_window,5,0,
		"You can do what you like in the sub_main_window.");
	mvwaddstr(sub_main_window,6,5,
		"It's for general purposes (this line at (6,5))");
	wrefresh(sub_main_window);

	message(system_window,"Type a key to continue.");
	while(!kbhit()) background();
	an_int = get_command(); 
	werase(system_window);
	wrefresh(system_window);

	/* Try and edit an enumerated type */

	an_enumerate = 0; /* Set the default */
	if (quick_edit("Example type",example_types[0],
			&an_enumerate,example_types,ENUMERATED) == KEY_ESC)
				return WARNING;
	message(system_window,"You're selection was %s\nType a key to continue.",
		example_types[an_enumerate]);
	while(!kbhit()) background();
	an_int = get_command();

	/* 
	 * Try and edit an string type 
	 * Specify STRING type let's the editor to accept
	 * any value character.
	 */

	sprintf(a_string,"This is the default.    ");
	if (quick_edit("Example string",a_string,
			a_string,NULL,STRING) == KEY_ESC) return NOERROR;
	message(system_window,"You're string is %s\nType a key to continue.",
		a_string);
	while(!kbhit()) background();
	an_int = get_command();
	werase(system_window);
	wrefresh(system_window);

	/* 
	 * Try and edit an integer type 
	 * Specify INTEGER type forces the editor to only accept
	 * numerals as input.
	 */

	an_int = 42; /* Set the default */
	sprintf(a_string,"%d    ",an_int); /* Leave room now */
	if (quick_edit("Example int",a_string,
			a_string,NULL,INTEGER) == KEY_ESC) return WARNING;
	sscanf(a_string,"%d",&an_int);
	message(system_window,"You're integer is %d\nType a key to continue.",
		an_int);
	while(!kbhit()) background();
	an_int = get_command();
	werase(system_window);
	wrefresh(system_window);

	/* 
	 * Try and edit an float type 
	 * Specify FLOAT type forces the editor to only accept
	 * numerals decimal places etc as input.
	 */

	a_float = 3.1415; /* Set the default */
	sprintf(a_string,"%f    ",a_float); /* Leave room now */
	if (quick_edit("Example int",a_string,
			a_string,NULL,FLOAT) == KEY_ESC) return WARNING;
	sscanf(a_string,"%f",&a_float);
	message(system_window,"You're integer is %f\nType a key to continue.",
		a_float);
	while(!kbhit()) background();
	an_int = get_command();
	werase(system_window);
	wrefresh(system_window);

	/* There's even mouse use. */

#ifdef __NCURSES_H
	error(MESSAGE,"There's even mouse support in an Xterm.\
\nAfter clicking on ESC below, left click the mouse or type <ESC>.\n");

	while(TRUE)
	{
		if ((an_int = get_command()) == KEY_ESC)
		{
			break;
		}

		if (an_int != KEY_MOUSE)
		{
			error(WARNING,"No silly the MOUSE!");
			continue;
		}

		getmouse(&mouse);

		if (mouse.bstate != BUTTON1_CLICKED)
		{
			error(WARNING,"LEFT click please!");
			continue;
		}

		error(MESSAGE,"You click it at position (%d,%d).\n\
Note that the mouse works on most things in the interface already.",
			mouse.x, mouse.y);
		break;
	}
#endif

	/* Unless there's a problem, return NOERROR */

	/* You can ask questions too */

	if (ask_yes_no("You can ask questions too like...",
		      "Do you want to report an error?") == YES) 
	{
		return WARNING;
	}

	return NOERROR;

} /* example() */

/************************************************************************/
/* question()								*/
/*									*/
/* Just askes a question and displays the answer.			*/
/************************************************************************/

int question(int argc, char **argv)
{
	char	s[80];

	if (ask_yes_no("I can ask questions.","What do you want to answer."))
	{
		message(system_window,"You said YES.");
	}
	else
	{
		message(system_window,"You said NO.");
	}

	sprintf(s,"This is a long string.");
	quick_edit("Example int",s,s,NULL,FLOAT);

	return NOERROR;

} /* question() */

/************************************************************************/
/* data_status()							*/
/*									*/
/* Tries to read from the data socket. Reports what it gets and sends   */
/* it straight back again.						*/
/************************************************************************/

static int calls = 0;

int data_status(void)
{
	int	len;
	char	s[256];
	time_t current_time;
	static int last_time = 0;

        clean(1,0);
        wstandout(status_window);
        mvwaddstr(status_window,1,0,"Data Ready : ");
        wstandend(status_window);
	if (data_ready())
	{
		wprintw(status_window,"YES");
		len = read_data_fast(s,256); 
        	clean(2,0);
		wstandout(status_window);
		mvwaddstr(status_window,2,0,"Data Read  : ");
		wstandend(status_window);
		wprintw(status_window,"%d",len);
        	clean(3,0);
		wstandout(status_window);
		mvwaddstr(status_window,3,0,"Data is    : ");
		wstandend(status_window);
		s[len]=0;
		wprintw(status_window,"%s",s);
		wrefresh(status_window);
	}
	else
	{
		wprintw(status_window,"NO ");
        	clean(2,0);
		wstandout(status_window);
		mvwaddstr(status_window,2,0,"Data Read  : ");
		wstandend(status_window);
		wprintw(status_window,"%d",0);
        	clean(3,0);
		wstandout(status_window);
		mvwaddstr(status_window,3,0,"Data is    : ");
	}

	time(&current_time);
	if (current_time >= last_time + 5)
	{
		last_time = current_time;
		len = write_data("Time to bug you.\n",
			sizeof("Time to bug you.\n"));
		clean(4,0);
		wstandout(status_window);
		mvwaddstr(status_window,4,0,"Data Write : ");
		wstandend(status_window);
		wprintw(status_window,"%d",len);
	}

	wstandout(status_window);
	mvwaddstr(status_window,5,0,"DATA Calls : ");
	wstandend(status_window);
	wprintw(status_window,"%d",++calls);

        return NOERROR;

} /* data_status() */

/************************************************************************/
/* background_job()							*/
/*									*/
/* Tries to read from the data socket. Reports what it gets and sends   */
/* it straight back again.						*/
/************************************************************************/

static int background_calls = 0;

int background_job(void)
{
	wstandout(status_window);
	mvwaddstr(status_window,6,0,"BkGrd Calls: ");
	wstandend(status_window);
	wprintw(status_window,"%d",++background_calls);

	//sleep(3); 

        return NOERROR;

} /* background_job() */

/************************************************************************/
/* top_job_job()							*/
/*									*/
/* Tries to read from the data socket. Reports what it gets and sends   */
/* it straight back again.						*/
/************************************************************************/

static int top_job_calls = 0;

int top_job_job(void)
{
	int i;

	//sleep(3); 
	wstandout(status_window);
	mvwaddstr(status_window,7,0,"TpJob Calls: ");
	wstandend(status_window);
	wprintw(status_window,"%d",++top_job_calls);

	for(i=0; i<=MAX_SIGNAL; i++)
	{
		if (client_signal_set(i))
		{
			message(system_window,"Got UI_SIGNAL %d",i);
			clear_client_signal(i);
		}
	}

        return NOERROR;

} /* top_job_job() */
