/* Routine to test visbin routine in interferometry library */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <unistd.h>
#include "../charalib/chara.h"
#include "../nrc/nrc.h"
#include "../interferometry/interferometry.h"

int main(int argc, char **argv)
{
	float rho; /* In radians */
	float theta; /* In radians */
	float r; /* 0 - 1 */
	float theta_p; /* In radians */
	float theta_s; /* In radians */
	float u; /* In meters */
	float v; /* In meters */
	float wavelength; /* In meters */
	float vis; /* 0-1 */
	float phase; /* In radians */

	/* Check out the command line */
 
	if (argc != 9)
	{
		fprintf(stderr,"Usage: %s rho(mas) theta(deg) r(0-1) Dp(mas) Ds(mas) U V (m) wavelength(um)\n", argv[0]);
		exit(-1);
	}

	sscanf(argv[1],"%f", &rho);
	sscanf(argv[2],"%f", &theta);
	sscanf(argv[3],"%f", &r);
	sscanf(argv[4],"%f", &theta_p);
	sscanf(argv[5],"%f", &theta_s);
	sscanf(argv[6],"%f", &u);
	sscanf(argv[7],"%f", &v);
	sscanf(argv[8],"%f", &wavelength);

	/* Put into right units */

	rho *= (AS2R/1000.0);
	theta *= (M_PI/180.0);
	theta_p *= (AS2R/1000.0);
	theta_s *= (AS2R/1000.0);
	wavelength *= 1.0e-6;

	/* Do it */

	vis_phase_binary(rho, theta, r, theta_p, theta_s, u, v, wavelength,
                      &vis, &phase);

	printf("Vis = %.4f Phase = %.2f\n", vis, phase/M_PI*180.0);

	exit(0);

}

