/************************************************************************/
/* twilight.c  	                                                        */
/*                                                                      */
/* Look up twilight start and stop limes.				*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo		                                        */
/* Date   : Jan 2008                                                    */
/************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "../charalib/chara.h"
#include "../astromod/astromod.h"

int main(int argc, char **argv)
{
	int    year,month,day,doy;
	double start_local;
        double stop_local;
        double start_ut;
        double stop_ut;
	char   s[81];

	if (argc != 4)
	{
		fprintf(stderr,"usage: %s year month day\n", argv[0]);
		exit(-1);
	}
	sscanf(argv[1],"%d",&year);
	sscanf(argv[2],"%d",&month);
	sscanf(argv[3],"%d",&day);

	doy = astromod_twilight(year, month, day, &start_local,
                &stop_local, &start_ut, &stop_ut);

	if (doy <0)
	{
		fprintf(stderr,"That combination not in data base.\n");
		exit(-2);
	}

	printf("Twilight start local : %s (%.3f)\n",
		time_to_string(start_local,s), start_local);
	printf("Twilight stop local  : %s (%.3f)\n",
		time_to_string(stop_local,s), stop_local);
	printf("Twilight start UT    : %s (%.3f)\n",
		time_to_string(start_ut,s), start_ut);
	printf("Twilight stop UT     : %s (%.3f)\n",
		time_to_string(stop_ut,s), stop_ut);

	exit(0);
}
