/************************************************************************/
/* uibuf.c				                                */
/*                                                                      */
/* Circular buffer control and creation/destruction routines.		*/
/*									*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Tony Johnson & Theo ten Brummelaar                          */
/* Date   : Original Version 1990 - ported to Linux 1998                */
/************************************************************************/

#include "charaui.h"

/************************************************************************/
/* set_up_buffer()							*/
/*									*/
/* Allocates memory for a buffer and sets up pointers.			*/
/* Returns a pointer to a structure of type BUFFER. 			*/
/* Return 0 if we run out of memory space.				*/
/*									*/
/************************************************************************/

BUFFER	*set_up_buffer(int buf_size)
{
	BUFFER	*buf;

	/* First allocate memory for structure */

	if ((buf = (BUFFER *)malloc(sizeof(BUFFER))) == (BUFFER *)NULL)
	{
		return 0;
	}

	/* Now allocate the buffer itself */

	if ((buf->bottom = (unsigned char *)malloc(buf_size+1)) == 
		(unsigned char *)NULL)
	{
		free(buf);
		return 0;
	}

	/* Now set the in and out and top pointers */

	buf->top = buf->bottom + buf_size;
	buf->in = buf->bottom;
	buf->out = buf->top;

	/* That is all */

	return buf;

} /* set_up_buffer() */

/************************************************************************/
/* remove_buffer()							*/
/*									*/
/* Releases memory for a buffer.					*/
/*									*/
/************************************************************************/

void	remove_buffer(BUFFER *buf)
{
	/* First deallocate memory for buffer */

	free(buf->bottom);

	/* Second deallocate memory for structure itself */

	free(buf);
	
} /* remove_buffer() */
