/************************************************************************/
/* uifind.c								*/
/*                                                                      */
/* 	This file contains two functions. 				*/
/*	Find_menu() is for searching the menu array for a 		*/
/*	given menu name.				 		*/
/*	Find_function() is to search the function array for 		*/
/*	a given function name. 						*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Tony Johnson & Theo ten Brummelaar                          */
/* Date   : Original Version 1990 - ported to Linux 1998                */
/************************************************************************/

/*
 * Include files
 */

#include "charaui.h"

/************************************************************************/
/* find_menu()								*/
/*									*/
/* Searches menu array for given menu name. Returns pointer to menu	*/
/* if found, NULL if not found.						*/
/*									*/
/************************************************************************/

struct 	smenu *find_menu(char *menu_name)
{
	/* Internal variables */

	int		loop;		/* Looping counter */
	struct smenu	*menu_pointer;	/* Pointer to menu when found */

	/* Initialise */

	loop = 0;
	menu_pointer = NULL;

	/* Do it */

	while (menus[loop] != NULL)
	{
		if (strcmpi(menu_name, menus[loop]->name) == 0)
		{
			menu_pointer = menus[loop];
			break;
		}
		loop++;
	}

	return menu_pointer;

} /* find_menu() */

/************************************************************************/
/* find_function()							*/
/*									*/
/* Searches function array for given function name. Returns index 	*/
/* to that function in the function array functions[] or NULL if	*/
/* function name does not appear in the list.				*/
/*									*/
/************************************************************************/

int find_function(char *function_name)
{
	/* Internal variables */

	int	loop;			/* Looping counter */
	int	function_index;		/* Index to function when found */

	/* Initialise */

	loop = 1;
	function_index = (int)NULL;

	/* Do it */

	while (functions[loop].name != NULL)
	{
		if (strcmpi(function_name, functions[loop].name) == 0)
		{
			function_index = loop;
			break;
		}
		loop++;
	}

	return function_index;

} /* find_function() */

/************************************************************************/
/* find_script()							*/
/*									*/
/* Searches script list for given script name. Returns pointer 		*/
/* to that script in the list or NULL if				*/
/* script name does not appear in the list.				*/
/*									*/
/************************************************************************/

struct sscripts *find_script(char *script_name)
{
	/* Internal variables */

	struct sscripts	*result,*current;

	/* Initialise */

	result = (struct sscripts *)NULL;

	/* Do it */

        for(current = first_script.next;
            current != &last_script;
            current = current->next)
        {

	
		if (strcmpi(script_name, current->name) == 0)
		{
			result = current;
			break;
		}
	}

	return result;

} /* find_script() */
