/************************************************************************/
/* uimenu.c								*/
/*                                                                      */
/*	Backbone of User Interface. The main menuing system		*/
/*	is contained in this file.  Viz the functions :			*/
/*									*/
/*		menu() - The main recursive routine			*/
/*		put_store_reversed_into_gotonext() - Guess !!!		*/
/*		put_comefrom_into_store() - That's right.		*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Tony Johnson & Theo ten Brummelaar                          */
/* Date   : Original Version 1990 - ported to Linux 1998                */
/************************************************************************/

/*
 * Include files
 */

#include "charaui.h"

/* Globals */

WINDOW	*active_menuwindow = NULL;
struct smenu *menus[MAXMENUS];
int autolist[MAXAUTO]; /*Array of function indexes for auto proceedure */
char autoargs[MAXAUTO][81]; /* Arguments for auto list functions */
struct smenu     *gotonext[MAXLEVEL+1],*comefrom[MAXLEVEL+1],*store[MAXLEVEL+1];
int      ptgoto;         /* Index to gotonext array */
int      ptcome;         /* Index to comefrom array */
int      move_down_menu_tree;/*Flag for menu movement (away from main) */
int      move_up_menu_tree;/* Flag for menu movement (towards main)   */
char     *TITLE;         /* Title shown on screen */

/************************************************************************/
/* menu()								*/
/* Process 4								*/
/*									*/
/* This is the main recusive function that controls (or really IS)	*/
/* the menu system.							*/
/* I'm afraid you will have to look at the design and source to		*/
/* find out more.							*/
/************************************************************************/

void 	menu(
	struct 	smenu	*thismenu,	/* Pointer to current menu */
	struct 	smenu	*lastmenu,	/* Pointer to last menu */
	int	level)			/* Current menu level */
{
	/* Internal variables */

	struct	smenu	*nextmenu;		/* Next menu to be called */
	int		command;		/* Menu command typed by user */
	int		function_pt;		/* Index to functions array */
	WINDOW		*menuwindow;		/* For this menu */

	/* See if the level is not too high */

	if (level > MAXLEVEL) 
	{
		error(WARNING,WARN_14,MAXLEVEL);
		return;
	}

	/* Set up the menu's window */

	active_menuwindow = menuwindow = 
		set_up_menu_window(thismenu, lastmenu, level);

	/* Start of major recursive loop */

	while(TRUE)
	{
		/* Check if we are moving up the menu tree (towards main) */

		if (move_up_menu_tree)
		{
			/* See if we have reached the top (ie in main) */

			if (level == 0)
			{
				/* Now see if we have to move down again */

				if (move_down_menu_tree)
				{
					/* Tell user we are busy */

					working();

					/*Get the previous path out of storage*/

					put_store_reversed_into_gotonext();
					ptgoto = 0;

					/*Set the next menu from the goto path*/

					nextmenu = gotonext[ptgoto++];
				}
				else
				{
					nextmenu = (struct smenu *)NULL;
				}

				/* We must have reached the end of the move up*/

				move_up_menu_tree = FALSE;

				/* Store the path taken up to main */

				comefrom[ptcome++] = NULL;
				put_comefrom_into_store();
			}
			else
			/* We have not reached the top of the menu tree */
			{
				/* Store this part of the path */

				comefrom[ptcome++] = thismenu;

				/* Get out of this menu back to calling menu */

				close_menu_window(menuwindow);
				return;
			}
		}

		/* Check if we are moving down the menu tree (away from main) */

		else if (move_down_menu_tree)
		{
			/*
			 * If we havn't reached the end of the path set the
			 * next menu according to the goto path.
			 */

			if ((nextmenu = gotonext[ptgoto++]) 
						== (struct smenu * )NULL)
			{
				/* Otherwise stop the movement down */

				move_down_menu_tree = FALSE;

				/*and tell the user we are ready for more work*/

				not_working();
			}
		}
		
		else /* This menu is active if we get to here */
		{

			/* Make the screen look like the menu */

			refresh_menu_window(menuwindow, level);
			active_window = menuwindow;


			/* Wait for a menu command */

			switch(command = get_user_input(menuwindow,&nextmenu))
			{
			/* See if it is a special menu key */

			case KEY_HELP : get_menu_help(thismenu);
					nextmenu = (struct smenu *)NULL;
					break;

			case KEY_MENU_UP   : if (level != 0)
					     {
						close_menu_window(menuwindow);
				    	     	return;
					     }
					     nextmenu = (struct smenu *)NULL;
					     break;

			case KEY_SWAP : if (store[0] != (struct smenu *)NULL)
					{
						move_down_menu_tree = TRUE;
						move_up_menu_tree = TRUE;
						ptcome = 0;
					}
					nextmenu = (struct smenu *)NULL;
					break;

			case KEY_RISE : if (level != 0)
					{
						move_up_menu_tree = TRUE;
						ptcome = 0;
					}
					nextmenu = (struct smenu *)NULL;
				    	break;

			/* See if it is a function key */

			case KEY_F(1) :
			case KEY_F(2) :
			case KEY_F(3) :
			case KEY_F(4) :
			case KEY_F(5) :
			case KEY_F(6) :
			case KEY_F(7) :
			case KEY_F(8) :
			case KEY_F(9) : 
			case KEY_F(10) : command = command - KEY_F(1);

				/* Set next menu and function index pointer */

				nextmenu = thismenu->entry[command].pt_menu;
				function_pt = thismenu->entry[command].pt_funct;

				/* If it is a function request call function */

				if (function_pt != (int)NULL) 
					call_function(function_pt, "");
				break;

			/* See if it was bullshit */

			case 0    : nextmenu = (struct smenu *)NULL;
				    break;

			/* See if a menu pointer has been returned by user */

			case MENU : break;

			/* It should never get to this point !! */

			default : error(WARNING,WARN_15);
				  break;
			}
		}

		/* Call the next menu if there is one */

		if (nextmenu != (struct smenu *)NULL && nextmenu != thismenu) 
			menu(nextmenu, thismenu, level+1);

	} /* End of major recursive loop */

} /* menu() [4] */

/************************************************************************/
/* put_store_reversed_into_gotonext()					*/
/*									*/
/* As its name implies this function takes the menu path in the		*/
/* array store and places it reversed into the array gotonext		*/
/* including a terminating NULL at the end.				*/
/************************************************************************/

void 	put_store_reversed_into_gotonext(void)
{
	/* Internal variables */

	struct smenu	**ptstore;	/* Pointer to index store array */
	struct smenu	**ptgotonext;	/* Pointer to index gotonext array */
	int		numentries;	/* Number of entries in store array */

	/* Initialise */

	ptstore = store;
	ptgotonext = gotonext;
	numentries = 0;

	/* Count entries in store */

	while( *ptstore != (struct smenu *)NULL)
	{
		numentries++;
		ptstore++;
	}

	/* Force ptstore to point at last entry */

	ptstore--;

	/* Put list reversed into gotonext */

	for(; numentries > 0; numentries--)
	{
		*ptgotonext++ = *ptstore--;
	}

	/* Add terminating NULL */

	*ptgotonext = (struct smenu *)NULL;

	return;

} /* put_store_reversed_into_gotonext() */

/************************************************************************/
/* put_comefrom_into_store()						*/
/*									*/
/* As the name implies this function puts the menu path held in		*/
/* the comefrom array into the store array including the terminating	*/
/* NULL.								*/
/************************************************************************/

void put_comefrom_into_store(void)
{
	/* Internal variables */

	struct smenu **ptstore;		/* Pointer to index store array */
	struct smenu **ptcomefrom;	/* Pointer to index comefrom array */

	/* Initialise */

	ptstore = store;
	ptcomefrom = comefrom;

	/* Do it */

	while( (*ptstore++ = *ptcomefrom++) != (struct smenu *)NULL)
		;

	return;

} /* put_comefrom_into_store() */
