/************************************************************************/
/* normalize.c 								*/
/*                                                                      */
/* Routines to do Coefficient normalization from microns to phse and	*/
/* visa versa.								*/
/*                                                                      */
/************************************************************************/
/*                                                                      */
/*              THE ZERNIKE LIBRARY CODE.				*/
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : 1st August 1996                                             */
/************************************************************************/

#include <math.h>
#include "zernike.h"

/************************************************************************/
/* r0_normalization()	 						*/
/*									*/
/* Performs normalization for r0 					*/
/* That is, f a set of coefficients is calculated for r0=1m this routine*/
/* normalizes them for a new r0.					*/
/* r0 should be in meters.						*/
/************************************************************************/

void   r0_normalization(float *a,
			 int	maxJ,
			 float r0)
{
	int	i;
	float	norm;

	norm = pow(r0,-5.0/6.0);
	
	for(i=1;i<=maxJ;i++) a[i] *= norm;

} /* r0_normalization() */

/************************************************************************/
/* phase_to_opd()							*/
/*									*/
/* Changes the zernike coefficients from a phase representation to an	*/
/* OPD representation in microns (assuming lambda is in microns). 	*/
/************************************************************************/

void phase_to_opd(float *a, int maxJ, float lambda)
{
	int i;
	float norm;

	norm = lambda/(2.0*M_PI);

	for(i=1;i<=maxJ;i++) a[i] *= norm;

} /* phase_to_opd() */

/************************************************************************/
/* opd_to_phase()							*/
/*									*/
/* Changes the zernike coefficients from an OPD representation to a	*/
/* phase representation. OPD is in microns (assuming that the wavelength*/
/* is also in microns.							*/
/* 									*/
/************************************************************************/

void opd_to_phase(float *a, int maxJ, float lambda)
{
	int i;
	float norm;

	norm = (2.0*M_PI)/lambda;

	for(i=1;i<=maxJ;i++) a[i] *= norm;

} /* opd_to_phase() */
