/************************************************************************/
/* power_spec_phase_grad.c						*/
/*									*/
/* Rouintes to calculate the phase gradient power spectra.		*/
/************************************************************************/
/*                                                                      */
/*              THE ZERNIKE LIBRARY CODE.				*/
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : 1st August 1996                                             */
/************************************************************************/

#include "zernike.h"

/************************************************************************/
/* power_spec_phase_grad()						*/
/*                                                                      */
/* Given the wind speed v(m/s), the frequency f,			*/
/* the aperture radius R(m), the outer sclae length L0 and the angle	*/
/* at which the average phase gradient is calculated this function	*/
/* calculates the value of Wphase_grad(f), the value of power spectrum  */
/* of average phase gradient at that frequency.                         */
/* The parameter calc_time chooses between LONG (Roddier method) and    */
/* SHORT (ten Brummelaar method) calculation times.                     */
/* Calculation is for an r0 of 1 meter.					*/
/************************************************************************/

float power_spec_phase_grad(int    calc_time,
			    float R,
			    float v,
			    float f,
                            float L0,   
			    float epsilon)

{
	int	j;	/* Order of Zernike coef */
	float	result;
	float	multiplier;
	
	zernike_check_setup();

	result = 0;

	for(j=1;j<zernike_max_j;j++)
	{
		if (n_m[j][M] != 1) continue;
		multiplier = zernike(j,1.0,epsilon);
		multiplier *= multiplier;
		result += (multiplier*power_spec_a(calc_time,j,R,v,L0,f));
#ifdef DEBUG
	    printf("j=%d multiplier=%f\n",j,multiplier);
#endif 
	}

#ifdef DEBUG
	printf("\nPOWER_SPEC_PHASE_GRAD(%f,%f)=%f\n",v,f,result);
#endif
	return result;

} /* power_spec_phase_grad() */
