/************************************************************************/
/* setup_zernike.c							*/
/*                                                                      */
/* Routines to setup and clear up the zernike stuff.			*/
/************************************************************************/
/*                                                                      */
/*              THE ZERNIKE LIBRARY CODE.                               */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : 1st August 1996                                             */
/************************************************************************/

#include <time.h>
#include <math.h>
#include "zernike.h"

//#define DEBUG

/* Other Globals */

int     **n_m = NULL; 
int     zernike_max_j = 0;
long int	random_seed = 0;

/************************************************************************/
/* set_up_zernike_calcs()						*/
/*									*/
/* Does a lot of initialisations for the Zernike routines.		*/
/* MUST be called before any other routine in library.			*/
/************************************************************************/

void	set_up_zernike_calcs(int    maxJ)
{
	int	j;
	int	n,m;
	void nrerror();

	if (n_m != NULL) nrerror("Called set_up_zernike_calcs twice!.");

	if (maxJ < 1) nrerror("Maximum J value must be >0.");
		
	/* Memory allocation */

	n_m = imatrix(0,maxJ+1,0,1);

	/* One time calculations */

	zernike_max_j = maxJ;

	/* Fill in the n_m[][] array */

	n_m[0][N] = n_m[0][M] = 0;

	n = 0;
	for(j=1; j<=zernike_max_j; n++)
	{
		for(m=n%2; m<=n && j <=zernike_max_j; m+=2)
		{
			n_m[j][N] = n;
			n_m[j][M] = m;
#ifdef DEBUG
			printf("A %d\t%d\t%d\n",j,n,m);
#endif
#ifdef FINDM1
			if (n_m[j][m] == 1 && zernike(j,1.0,0.0) != 0)
				printf("J=%d\n",j);
#endif
			if (++j>zernike_max_j) break;

			if (m != 0)
			{
				n_m[j][N] = n;
				n_m[j][M] = m;
#ifdef DEBUG
				printf("B %d\t%d\t%d\n",j,n,m);
#endif
#ifdef FINDM1
			if (n_m[j][m] == 1 && zernike(j,1.0,0.0) != 0)
				printf("J=%d\n",j);
#endif
				j++;
			}
		}
	}

        /* Set up seed for random number generators .... */

        random_seed = -1 * time((time_t *)0)%(1<<(sizeof(int)*8-1));

} /* set_up_zernike_calcs() */

/************************************************************************/
/* clean_up_zernike_calcs()                                             */
/*                                                                      */
/* Does a lot of un-initialisations for the Zernike routines.	        */
/* Should always be called once you have finished using these functions.*/
/************************************************************************/

void    clean_up_zernike_calcs(void)
{
	free_imatrix(n_m,0,zernike_max_j+1,0,1);
	n_m = NULL;
}
