/************************************************************************/
/* dispsubph.c 								*/
/*									*/
/* Calculates the value of given phase function over a full aperture	*/
/* and displays the result in an X window.				*/
/*									*/
/* Usage: dispsubph npix pixmult rho theta size a1 a2 a3 a4 a5 ..... aN */
/*									*/
/* Where npix is the number of pixels across the aperture and the a?	*/
/* are Zernike coefficients.						*/
/* 									*/
/************************************************************************/
/*                                        	                      	*/
/* (C) This source code and its associated executable           	*/
/* program(s) are copyright.                                    	*/
/*                                                              	*/
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : August 1993							*/
/************************************************************************/

#include "zernike.h"

int main(int argc,char **argv)
{
	/* Internals */

	int 	maxJ;	/* max order of zernike coef. */
	int	npix;	/* Number of pixels across aperture */
	int	pixmult; /* multiplier for pixels */
	float	*a;
	float	pos_rho;
	float	pos_theta;
	float	size_rho;
	float	in_rho, out_rho;
	int	i;
	aperture_pixel **pixels;
        Window  theWindow;

	/* Analyse the command line */

	if (argc < 9)
	{
		fprintf(stderr,
	"usage: %s npix pixmult rho theta size rhoin rhoout a1 a2 .... aN\n",
		argv[0]);
		exit(-1);
	}

	sscanf(argv[1],"%d",&npix);
	sscanf(argv[2],"%d",&pixmult);
	sscanf(argv[3],"%f",&pos_rho);
	sscanf(argv[4],"%f",&pos_theta);
	pos_theta *= (M_PI/180.0);
	sscanf(argv[5],"%f",&size_rho);
	sscanf(argv[6],"%f",&in_rho);
	sscanf(argv[7],"%f",&out_rho);
	maxJ = argc-8;
#ifdef DEBUG
	printf("maxJ = %d\n",maxJ);
#endif
	if (pixmult < 1 || pixmult*npix > 1024)
	{
		fprintf(stderr,"Pixel multiplier out of range\n");
		exit(-1);
	}

	/* Allocate memory */

	a = vector(0,maxJ);

	/* Read in coefficient data */

	for(i=1; i<=maxJ; i++)
	{
		sscanf(argv[i+7],"%f",a+i);
#ifdef DEBUG
		printf("a[%d] = %f\n",i,a[i]);
#endif
	}

	/* Initialise calculations */

	set_up_zernike_calcs(maxJ);
	pixels = sub_obsc_aperture(npix,maxJ,pos_rho,pos_theta,
			size_rho,in_rho,out_rho);

	/* Set up the X window stuff */

	if (!initX(NULL))
	{
		fprintf(stderr,"Failed to open display.\n");
		exit(-1);
	}
        theWindow = openWindow("Dispsubph",0,0,npix*pixmult,npix*pixmult);

	/* Display the thing */

	display_phase(theWindow, pixels, 2.0*M_PI, a, maxJ, npix, pixmult, LIN);

	/* That is all, clean up */

	printf("Type RETURN to continue....\n");
	scanf("%d",&i);

        XDestroyWindow(theDisplay, theWindow);
	quitX();
	free_sub_obsc_aperture(pixels,npix);
	free_vector(a,0,maxJ);
        clean_up_zernike_calcs();

	exit(0);
}
